﻿
char AppName[]    = { "GSDataPrep" };                       /* application name */
char AppTitle[]   = { "Data Preparation for G-S Model" };   /* caption for window */
char AboutTitle[] = { "GS Data Prep" };                     /* name given in about box */
char AboutDate[]  = { "Sept 2012" };                        /* date given in about box */

#define _CRT_SECURE_NO_WARNINGS

#include "c:\programs\winshell\winshell.inc"


typedef struct
{ int    caseid;          /* case id in 1992 survey */
  int    matchid;         /* case id of partner in 1992 */
  int    birthyear;       /* year of birth */
  int    birthmonth;      /* month of birth */
  int    gender;          /* gender: 1=male, 2=female */
  int    race;            /* race: 1=white, 2=black, 3=hispanic */
  int    ed;              /* education in years */
  int    enjoyspouse;     /* enjoy time with spouse in retirement */
  int    married;         /* indicator if married */
  int    marstat;         /* marital status: 1=mar; 2=lt mar; 3=wid; 4=div; 5=div, mar>10yrs; 6=never mar */
  int    prevmarend;      /* year previous marriage ended; 0 if not previously married */
  int    yearentered;     /* year immigrant entered the country */
  double wealth;          /* wealth in 1992 */
  double buswealth;       /* real estate wealth in 1992 */
  double assetwealth;     /* land, business, financial, and "other" wealth in 1992 */
  double giftamount[11];  /* inheritances, gifts, and life insurance receipt */
  int    giftyear[11];    /* dates of inheritances, gifts, ect. */
  } PERSVARS;

typedef struct
{ struct
  { int    jobind;
    int    startdate;
    int    enddate;
    int    sempl;
    int    occ;
    int    ind;
    int    hours;
    int    weeks;
    double earnings;
    int    hourslimits;
    int    sscoverage;
    int    pencoverage;
    int    layoff;
    } jobvars[9];
  double pencontr;
  int    everworked;
  int    yearfirstworked;
  int    yearlastworked;
  int    lastunknowndate;
  int    ftstartdate;
  int    ftenddate;
  int    ftyears;
  int    nonssjob;
  int    nonssstartdate;
  int    nonssenddate;
  int    nonssyears;
  } JOBHIST;

typedef struct
{ struct
  { int    interview;
    int    intvyear;
    int    intvmonth;
    int    age;
    int    ret;
    int    marstat;
    int    samespouse;
    int    samejob;
    int    layoff;
    int    sempl;
    int    occ;
    int    ind;
    int    hours;
    int    weeks;
    double wage;
    int    hourslimits;
    int    pension;
    int    health;
    double eowamount;
    double eowpens;
    int    eowservice;
    int    eowaccept;
    int    ssdisab;
    double giftamount[3];
    int    giftyear[3];
    int    srret;
    int    ssreceipt;
    int    expretage;
    } wavevars[6];
  } PANELVARS;

typedef struct
{ int    healthage;       /* initial age of health problem                    */
  int    finalretstat;    /* indicator of job status at last survey           */
  int    origjobendage;   /* separation age from 1992 full-time job           */
  int    origjobendsurv;  /* survey of separation from 1992 full-time job     */
  int    newjobstartage;  /* start age of first full-time job after 1992 job  */
  int    impjobendage;    /* imputed separation age from 1992 job             */
  int    layoffage;       /* age of first layoff from full-time job           */
  int    ind;             /* industry of 1992 job (or last prior ft job)      */
  int    occ;             /* occupation of 1992 job (or last prior ft job)    */
  int    ssdiage;         /* starting age of social security disability       */
  int    initage;         /* age at 1992 survey                               */
  int    retstats[13];    /* retirement states for 13 years from initage - 1  */
  double ftwage[45];      /* full-time wages at ages 25-69                    */
  double prwage[20];      /* partial retirement wages at ages 50-69           */
  double secwage[20];     /* full-time wages in reversal jobs at ages 50-69   */
  double sswage[45];      /* social security wages at ages 25-69              */
  double pensben[21];     /* annual pension benefit for retirement at 50-70   */
  int    penstart[21];    /* starting age of pension for retirement at 50-70  */
  double plumpsum[21];    /* lump-sum pension benefit for retirement at 50-70 */
  double eowamount[21];   /* early out window lump sum amounts at 50-70       */
  int    pjobstart;       /* starting age of last pension job                 */
  int    pjobend;         /* last age of last pension job                     */
  int    pjobno;          /* job number of last pension job                   */
  double spwage[45];      /* earnings as a spouse for ages 25-69              */
  double spsswage[45];    /* social security wages as a spouse at ages 25-69  */
  double sppenben;        /* annual pension benefit as a spouse               */
  int    sppenstart;      /* starting age of pension benefit as a spouse      */
  double spplumpsum;      /* lump-sum pension benefit as a spouse             */
  int    sppjobend;       /* last age of last pension job as a spouse         */
  int    spftend;         /* last age of full-time work as a spouse           */
  double aime[21];        /* ss aime for retirement at 50-70                  */
  double pia[21];         /* ss pia for retirement at 50-70                   */
  double ncpens[21];      /* non-covered pensions for retirement at 50-70     */
  double sppia[21];       /* spouse pia for spouse ages 50-70                 */
  double spncpens[21];    /* spouse non-covered pensions for spouse age 50-70 */
  struct
  { int   goodSSstatus;   /* no ambiguity about whether jobs are ss covered    */
    int   goodFTyears;    /* number of ft years available in wave 3/ss record  */
    int   goodearnings;   /* ft earnings available in ss record or self-report */
    int   goodSSearn;     /* no problems with ss earnings if no sr earnings    */
    int   goodCareer;     /* respondent appears to have a career               */
    int   goodAssets;     /* respondent has relatively low business wealth     */
    int   goodPens;       /* respondent with pension has provider record       */
    } goodobs;
  } CALCVARS;

typedef struct
{ PERSVARS persvars;
  JOBHIST jobhist;
  PANELVARS panelvars;
  CALCVARS calcvars;
  } VARS;

typedef struct
{ int   fulltime[3];
  int   jobind[9];
  int   dupjob[9];
  int   startdate[9];
  int   enddate[9];
  int   exper[100];
  int   ten[100];
  int   ft[100];
  int   nonss[100];
  double ptearnbase;
  double earnings[100];
  double ssearnings[100];
  } EARNVARS;

typedef struct
{ int    caseid;
  int    penids[4][2];
  int    jobno;
  int    occ;
  int    ind;
  int    earningscat;
  } PENVARS;


void   mainprog(void);
void   getPersVars(PERSVARS *);
void   getBirthDate(int, int, int *, int *);
int    compbirthdate(const void *, const void *);
void   getJobHist(VARS *);
void   getPanelVars(VARS *);
double wagecalc(int, int, int, int, int);
void   correctErrors(VARS *);
void   getExperience(VARS *, int *, int *);
void   getRetStats(VARS *);
void   calcWageStream(VARS *);
void   calcPastWages(VARS *, EARNVARS *);
void   calcWagesAsSpouse(VARS *, EARNVARS *);
void   calcPotentialWages(VARS *, EARNVARS *);
void   calcPensStream(VARS *);
void   calcLastPens(int *, PENVARS *, VARS *);
void   calcPrevPens(int *, PENVARS *, VARS *);
void   calcPensAsSpouse(int *, PENVARS *, VARS *);
void   penCalc(void *);
int    median(int *, int);
int    comparemed(const void *, const void *);
int    comparefmed(const void *, const void *);
double fmedian(double *, int);
void   initpens(int *, PENVARS *, VARS *);
void   calcSSStreams(VARS *);
double ncpencalc(int, int, double *);
double getSocialSecurityPIA(int, int, int, double *, double *);
void   putRespOutput(VARS *);
void   getSSrec(int, int *, int *, double *);
void   initSSdata(int *, unsigned char *);
void   getSurvivalRates(int, int, double *);
int    eval(int, int);
int    evals(int, char *);
double feval(int, int);
double fevals(int, char *);
void   getvarlocs(int, char *, int *, int *);
void   dictinit(int (*)[2], char (*)[12]);
int    dictcomp(const void *, const void *);


char   rec[57478];
int    printcaseid = 99998;

/*---------------------------------------------------------------------------
			 MAINPROG
			 creates the input data for the structural estimation
  ---------------------------------------------------------------------------*/

void mainprog()
{ int   recno;

  static VARS vars[12652];

  FILE *infile;
  int handle;


  infile = fopen("d:\\data\\hrs92-02.dat", "rb");

  handle = _fileno(infile);

  for (recno = 0; recno < 12652; recno++)
	{ if (recno % 500 == 0)
    eprintf(".");

    _read(handle, rec, 57478);

    if (printcaseid == 99999)
    wnprintf("\n\n%5d", recno);

    getPersVars(&(vars[recno].persvars));
    getJobHist(&vars[recno]);
    getPanelVars(&vars[recno]);
			
    correctErrors(&vars[recno]);

    getRetStats(&vars[recno]);
    calcWageStream(&vars[recno]);

    checkMessages();

    if (vars[recno].persvars.caseid == printcaseid)
    pause(); 
    }

  fclose(infile);

  calcPensStream(vars);
	calcSSStreams(vars);

  putRespOutput(vars);

  }


/*---------------------------------------------------------------------------
			 GET PERS VARS routine
			 gets personal variables
  ---------------------------------------------------------------------------*/

void getPersVars(
  PERSVARS *persvars)        /* structure of personal variables         */

{ int    caseid, matchid, hhid, pn;
  int    gift, giftloc, year;
  int    age, birthmonth, birthyear;
  int    gender, race, ed;
  int    marstat, married, marstart, marno, prevmar, prevmarstart, prevmarend;
  int    srret, enjoyspouse;
  int    foreignborn, yearentered;
  double housewealth, finwealth, landwealth, buswealth, vehwealth, otherwealth;
  double wealth, assetwealth, motorhome, amount;

  int    giftyear[11];
  double giftamount[11];

  static int rangecard[10] = {    250,    707,    1582,     5001,     22361,
										           111805, 500001, 3162278, 31622777, 316227767};
	/* These are the geometric means of the intervals */

  caseid = eval(1, 1);                 /* 1992 case id */
  matchid = eval(1, 3);                /* 1992 case id of spouse/partner */

  if (caseid == printcaseid || printcaseid == 99999)
  wnprintf(" %d %d", caseid, matchid);

  hhid = evals(2, "HHID");             /* HHID */
  pn   = evals(2, "PN");               /* PN */

  getBirthDate(hhid, pn, &birthyear, &birthmonth);

  if (caseid == printcaseid || printcaseid == 99999)
  wnprintf(" bd %d %d", birthmonth, birthyear);

  if (birthyear < 1850 || birthyear > 1992)
  { birthyear  = eval(1, 44);

    if (birthyear > 1992)
    { birthyear = eval(2, 214);

      if (birthyear < 1900 || birthyear > 1992)
      { age = eval(1, 46);

        if (0 < age && age < 99)
        birthyear = 1992 - age;
        else
        { age = eval(2, 104);

          if (0 < age && age < 99)
          birthyear = 1994 - age;
          }
        }
      }
    }

  if (birthmonth < 0 || birthmonth > 12)
  { birthmonth = eval(1, 42);

    if (birthmonth < 1 || birthmonth > 12)
    { birthmonth = eval(2, 212);

      if (birthmonth < 0)
      birthmonth = 99;
      }
    }

  gender = eval(1, 47);

  race = eval(1, 48);

  if      (race == 2) race = 2;            /* black    */
  else if (race == 5) race = 3;            /* hispanic */
  else                race = 1;            /* white    */

  ed = eval(1, 207);

  marstat = eval(1, 225);        /* A10: 1992 marital status */

  if (marstat == 1 || marstat == 7 || marstat == 8)
  { married = 1;
  
    marstart = eval(1, 227);     /* A10a: date marriage started */

    prevmar = eval(1, 239);      /* A11: married before? */

    if (prevmar == 2)
    prevmarend = eval(1, 243);   /* A15: year previous marriage started */
    else prevmarend = 0;
    
    if (prevmarend <= birthyear + 35 || 1993 < prevmarend)
    marstat = 2;                 /* no previous marriages ended after age 35 */
    else marstat = 1;            /* previous marriage ended after age 35 */
    }
  else
  { married = 0;

    if (marstat == 2)            /* partner */
    { prevmar = eval(1, 228);    /* A10b: married before */

      if (prevmar == 1)          /* prior marriage */
      { prevmarstart = eval(1, 231);     /* A10d: date previous marriage started */
        prevmarend   = eval(1, 233);     /* A10e: data previous marriage ended */

        if (1900 < prevmarstart && prevmarstart <= prevmarend + 10 && prevmarend < 1993)
        marstat = 5;
        else marstat = 4;
        }
      else marstat = 6;
      }
    else if (marstat == 3 || marstat == 4)   /* separated or divorced */
    { prevmarstart = eval(1, 236);       /* A10g: date previous marriage started */
      prevmarend   = eval(1, 238);       /* A10h: data previous marriage ended */

      if (1900 < prevmarstart && prevmarstart <= prevmarend + 10 && prevmarend < 1993)
      marstat = 5;
      else marstat = 4;
      }
    else if (marstat == 5)       /* widowed */
    marstat = 3;
    else marstat = 6;            /* never married */

    if (marstat == 4)            /* look for previous 10 year marriages */
    { for (marno = 0; marno < 3; marno++)
      { prevmarstart = eval(1, 241 + 3 * marno);   /* A13: date prevous marriage started */
        prevmarend   = eval(1, 243 + 3 * marno);   /* A15: data prevous marriage ended */

        if (1900 < prevmarstart && prevmarstart <= prevmarend + 10 && prevmarend < 1993)
        marstat = 5;
        }
      }
    }

  srret = eval(1, 4901);            /* K1: self-reported retirement status */
  
  if (srret == 1)
  enjoyspouse = eval(1, 4926);      /* K11d: enjoys retirement with spouse */
  else enjoyspouse = eval(1, 5013); /* K21d: will enjoy retirement with spouse */

  if (enjoyspouse == 1)             /* category: very important */
  enjoyspouse = 1;
  else enjoyspouse = 0;

  if (caseid < 20000)
  { housewealth = eval(1,  605)    /* ranch/farm                  */
	              + eval(1,  612)    /* home on partly-owned ranch  */
	              + eval(1,  619)    /* mobile home site            */
	              + eval(1,  632)    /* mobile home                 */
	              + eval(1,  640)    /* mobile home & site          */
	              + eval(1,  712)    /* house/apartment             */
	              - eval(1,  723)    /* first mortgage              */
	              - eval(1,  730)    /* second mortgage             */
	              - eval(1,  736)    /* third mortgage              */
	              - eval(1,  744)    /* home equity line of credit  */
	              + eval(1,  824)    /* second home                 */
	              - eval(1,  826);   /* mortgage on second home     */

    finwealth   = eval(1, 5218)    /* stocks & mutual funds       */
	              + eval(1, 5222)    /* checking & saving accounts  */
	              + eval(1, 5226)    /* bills & CDs                 */
	              + eval(1, 5230)    /* bonds & bond funds          */
	              - eval(1, 5238)    /* debt                        */
		            + eval(1, 5213);   /* Keoghs & IRAs               */

    landwealth  = eval(1, 5202);   /* real estate                 */
	  vehwealth   = eval(1, 5205);   /* vehicles                    */
    buswealth   = eval(1, 5209);   /* business assets             */
    otherwealth = eval(1, 5234);   /* other assets                */

    motorhome   = eval(1, 839);    /* net value of motor home     */

    if (motorhome != 99996)
    vehwealth = vehwealth + motorhome;

    wealth = housewealth + finwealth + landwealth + vehwealth + buswealth + otherwealth;
    assetwealth = finwealth + landwealth + buswealth + otherwealth;

    /* wnprintf("\n h %7.0f r %7.0f b %7.0f f %7.0f v %7.0f o %7.0f  t %7.0f",
     housewealth, landwealth, buswealth, finwealth, vehwealth, otherwealth, wealth); */

    for (gift = 0; gift < 11; gift++)
    { switch(gift)
      { case  0:  case  1:  case  2:  giftloc = 5307 + 6 * gift;            break;    /* inheritances */
        case  3:                      giftloc = 5321;                       break;
        case  4:  case  5:  case  6:  giftloc = 5326 + 4 * (gift - 4);      break;    /* gifts */
        case  7:                      giftloc = 5336;                       break;
        case  8:  case  9:  case 10:  giftloc = 5340 + 3 * (gift - 8);      break;    /* life insurance */
        }

      if (gift != 3 && gift != 7)
      { year = eval(1, giftloc - 1);

        if (year < 1920 || year > 1993)
        year = -1;
        }
      else year = -1;

      amount = eval(1, giftloc);

      if (9999990 <= amount)
      amount = -1;
      else if (9999980 <= amount)
      amount = rangecard[(int) amount - 9999980];

      giftamount[gift] = amount;
      giftyear[gift] = year;
      }
    }
  else
  { wealth = -1;
    buswealth = -1;
    assetwealth = -1;

    for (gift = 0; gift < 11; gift++)
    { giftamount[gift] = -1;
      giftyear[gift] = -1;
      }
    }

  foreignborn = eval(1, 204);
  yearentered = eval(1, 206);

  if (foreignborn != 5 || yearentered < 1951 || yearentered > 1992)
  yearentered = 0;

  persvars->caseid      = (short int) caseid;
  persvars->matchid     = (short int) matchid;
  persvars->birthyear   = (short int) birthyear;
  persvars->birthmonth  = (char)      birthmonth;
  persvars->gender      = (char)      gender;
  persvars->race        = (char)      race;
  persvars->ed          = (char)      ed;
  persvars->enjoyspouse = (char)      enjoyspouse;
  persvars->married     = (char)      married;
  persvars->marstat     = (short int) marstat;
  persvars->prevmarend  = (short int) prevmarend;
  persvars->wealth      =             wealth;
  persvars->buswealth   =             buswealth;
  persvars->assetwealth =             assetwealth;
  persvars->yearentered = (short int) yearentered;

  for (gift = 0; gift < 11; gift++)
  { persvars->giftamount[gift] =             giftamount[gift];
    persvars->giftyear[gift]   = (short int) giftyear[gift];
    }

  }


/*---------------------------------------------------------------------------
			 GET BIRTH DATE routine
			 gets month and year of birthday from tracker 3.1
  ---------------------------------------------------------------------------*/

typedef struct
{ int   hhid;
  int   pn;
  int   birthyear;
  int   birthmonth;
  } BIRTHSTRUCT;

void getBirthDate(
  int   hhid,               /* Household id */
  int   pn,                 /* Person number */
  int   *birthyear,         /* Year of birth */
  int   *birthmonth)        /* Month of birth */

{ int   recno;
  int   hhidt, pnt;
  int   birthyeart, birthmontht;
  int   recmin, recmax, newrec;

  char  inrec[407];

  FILE *infile;
  int  handle;

  static BIRTHSTRUCT birthstruct[26935];

  static int init = 1;

  if (init == 1)
  { infile = fopen("d:\\datasets\\HRS\\Wave5\\track31.da", "rb");

    if (infile == NULL)
    error("Bad tracker file in getBirthDate");

    handle = _fileno(infile);

    for (recno = 0; recno < 26935; recno++)
    { _read(handle, inrec, 407);

      sscanf(&inrec[ 0], "%6d", &hhidt);
      sscanf(&inrec[ 6], "%3d", &pnt);
      sscanf(&inrec[ 9], "%2d", &birthmontht);
      sscanf(&inrec[11], "%4d", &birthyeart);

      birthstruct[recno].hhid = hhidt;
      birthstruct[recno].pn = (short) pnt;
      birthstruct[recno].birthyear = (short) birthyeart;
      birthstruct[recno].birthmonth = (char) birthmontht;
      }

    fclose(infile);
    
    qsort((void *) birthstruct, 26935, sizeof(BIRTHSTRUCT), compbirthdate);

    init = 0;
    }

  recmin = 0;
  recmax = 26934;

  while (recmin + 1 < recmax)
  { newrec = (recmin + recmax) / 2;

    if (hhid > birthstruct[newrec].hhid
     || (hhid == birthstruct[newrec].hhid && pn > birthstruct[newrec].pn))
    recmin = newrec;
    else recmax = newrec;
    }

  if (recmin > 0)
  { *birthyear = birthstruct[recmax].birthyear;
    *birthmonth = birthstruct[recmax].birthmonth;
    }
  else if (hhid > birthstruct[0].hhid || pn > birthstruct[0].pn)
  { *birthyear = birthstruct[1].birthyear;
    *birthmonth = birthstruct[1].birthmonth;
    }
  else
  { *birthyear = birthstruct[0].birthyear;
    *birthmonth = birthstruct[0].birthmonth;
    }

  }


/*---------------------------------------------------------------------------
			 COMP BIRTH DATE routine
			 compare routine for qsort in the getBirthDate routine
  ---------------------------------------------------------------------------*/

int compbirthdate(
  const void *elem1,
  const void *elem2)

{ if ((* (BIRTHSTRUCT *) elem1).hhid < (* (BIRTHSTRUCT *) elem2).hhid)
  return(-1);
  else if ((* (BIRTHSTRUCT *) elem1).hhid > (* (BIRTHSTRUCT *) elem2).hhid)
  return(1);

  if ((* (BIRTHSTRUCT *) elem1).pn < (* (BIRTHSTRUCT *) elem2).pn)
  return(-1);
  else return(1);

  }


/*---------------------------------------------------------------------------
			 GET JOB HIST routine
			 gets job history variables from 1992 survey
  ---------------------------------------------------------------------------*/

void getJobHist(
  VARS *vars)

{ int    jobno, loc;
  int    curjob, longjob, pensjob, secondpensjob;
  int    sempl, startdate, enddate, hours, weeks, hourslimits;
  int    pencoverage, futurepens, pens, contribpd;
  int    firmclosed, layoff;
  int    everworked, nlfperiod;
  int    govtjob, gtype;
  int    ftstartdate, ftenddate, ftyears, allyears;
  int    nonssjob, nonssstartdate, nonssenddate, nonssyears, ssallyears;
  int    yearfirstworked, yearlastworked, lastunknowndate;

  double contrib, contribpct, contribamt, pencontr;
  double salary, hourly, piecework, tips, profits, earnings;

  double penscontrib[6];

  static int   pencontribloc[6] = { 2911, 3011, 3111, 2937, 3037, 3137};

  struct
  { int    jobind;
    int    startdate;
    int    enddate;
    int    sempl;
    int    occ;
    int    ind;
    int    hours;
    int    weeks;
    double earnings;
    int    hourslimits;
    int    sscoverage;
    int    pencoverage;
    int    layoff;
    } jobvars[9];

  PERSVARS *persvars;
  JOBHIST *jobhist;

  persvars =  &(vars->persvars);
  jobhist = &(vars->jobhist);


  for (jobno = 0; jobno < 9; jobno++)
  { jobvars[jobno].jobind      =  0;
    jobvars[jobno].startdate   = -9;
    jobvars[jobno].enddate     = -9;
    jobvars[jobno].sempl       = -9;
    jobvars[jobno].occ         = -9;
    jobvars[jobno].ind         = -9;
    jobvars[jobno].hours       = -9;
    jobvars[jobno].weeks       = -9;
    jobvars[jobno].hourslimits = -9;
    jobvars[jobno].earnings    = -9;
    jobvars[jobno].sscoverage  = -9;
    jobvars[jobno].pencoverage = -9;
    }

  pencontr = -9;

  curjob = eval(1, 2717);          /* F2: any current work? */

  if (curjob == 1 || curjob == 2)  /* Current job: Section F questions */
  { jobvars[0].jobind = 1;

    sempl = eval(1, 2718);         /* F3: self employed */

	  if (sempl == 2)
	  sempl = 1;
	  else sempl = 0;

    jobvars[0].sempl = sempl;

	  startdate = eval(1, 2816 +  18 * sempl);   /* F23/F33: start date at employer */

	  if (1920 <= startdate && startdate <= 1993)
	  jobvars[0].startdate = startdate;

	  jobvars[0].occ = eval(1, 2720 + 101 * sempl);    /* F5/F6/F27: current occupation */
    jobvars[0].ind = eval(1, 2719 + 101 * sempl);    /* F4/F26: current industry */

	  hours = eval(1, 2722 + 100 * sempl);       /* F8/F28: usual hours per week */
	  weeks = eval(1, 2726 + 97 * sempl);        /* F10/F29: usual weeks per year */

    if (0 < hours && hours < 95)
    jobvars[0].hours = hours;

    if (0 < weeks && weeks <= 52)
    jobvars[0].weeks = weeks;

	  if (sempl == 0)
	  { salary    = wagecalc(1, 2735, 1, hours, weeks);  /* F16a */
		  hourly    = wagecalc(1, 2739, 0, hours, weeks);  /* F16d */
		  piecework = wagecalc(1, 2743, 1, hours, weeks);  /* F16h */
		  tips      = wagecalc(1, 2748, 1, hours, weeks);  /* F16m */

      if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
      jobvars[0].earnings = salary + hourly + piecework + tips;
      }
	  else
	  { salary  = wagecalc(1, 2825, 1, hours, weeks);    /* F30a */
		  profits = wagecalc(1, 2828, 1, hours, weeks);    /* F31a */

      if (salary >= 0 && profits >= 0)
      jobvars[0].earnings = salary + profits;
      }

    hourslimits = eval(1, 2805);     /* F19: would employer allow half-time work? */

    if (hourslimits == 1)
    jobvars[0].hourslimits = 1;
    else jobvars[0].hourslimits = 0;

    if (persvars->caseid == 20897)   /* current job started in 1987 at 60 hours, yet */
    jobvars[0].sscoverage = 0;       /* virtually no ss earnings during this period */
    else jobvars[0].sscoverage = 1;

	  pencoverage = eval(1, 2901 -  63 * sempl);   /* F36/F37: current pension coverage */

	  if (sempl == 0)
	  futurepens = eval(1, 2906);     /* F37e: will become eligible if remains with employer? */
	  else futurepens = 0;

	  if (pencoverage == 1 || futurepens == 1)
	  { jobvars[0].pencoverage = 1;

      for (pens = 0; pens < 6; pens++)
      { loc = pencontribloc[pens];
        contrib = -9;

        contribpct = eval(1, loc);

        if (contribpct < 99996)
        contrib = contribpct / 100.0;
        else
        { contribamt = eval(1, loc + 1);

          if (contribamt < 99998 && jobvars[0].earnings > 0)
          { contribpd = eval(1, loc + 2);

            switch(contribpd)
            { case  2:  contrib = 100 * (52 * contribamt) / jobvars[0].earnings;    break;
              case  3:  contrib = 100 * (26 * contribamt) / jobvars[0].earnings;    break;
              case  4:  contrib = 100 * (12 * contribamt) / jobvars[0].earnings;    break;
              case  5:  contrib = 100 * ( 4 * contribamt) / jobvars[0].earnings;    break;
              case  6:  contrib = 100 *       contribamt  / jobvars[0].earnings;    break;
              }
            }
          }

        penscontrib[pens] = contrib;
        }

      for (pens = 0; pens < 6; pens++)
      { if (penscontrib[pens] >= 0)
        { if (pencontr < 0)
          pencontr = penscontrib[pens];
          else pencontr = pencontr + penscontrib[pens];
          }
        }

      if (pencontr < 0)
      pencontr = 5;                /* default contribution rate is 5% */
      }
	  else jobvars[0].pencoverage = 0;
    }

  everworked = eval(1, 3401);       /* G1: ever worked             */
  yearlastworked = eval(1, 3403);   /* G1a: year last worked       */
  nlfperiod = eval(1, 3404);        /* G2: worked in last 20 years */

  if ((curjob < 1 || curjob > 2) && (everworked == 1 && nlfperiod == 2))
  { jobvars[1].jobind = 1;          /* Section G questions */

    sempl = eval(1, 3405);          /* G3: self employed in last job? */

	  if (sempl == 2)
	  sempl = 1;
	  else sempl = 0;

    jobvars[1].sempl = sempl;

    startdate = eval(1, 3418 + 24 * sempl);     /* G13/G25: year started working for employer */
	  enddate   = yearlastworked;                 /* G1a: year last worked */

	  if (1910 <= startdate && startdate <= 1993)
    jobvars[1].startdate = startdate;

	  if (1910 <= enddate && enddate <= 1993)
    jobvars[1].enddate = enddate;

    jobvars[1].occ = eval(1, 3407 + 25 * sempl);      /* G5/G6/G19: last occupation */
	  jobvars[1].ind = eval(1, 3406 + 25 * sempl);      /* G4/G18: last industry */

	  hours = eval(1, 3408 + 25 * sempl);         /* G7/G20: hours per week */
	  weeks = eval(1, 3411 + 23 * sempl);         /* G9/G21: weeks per year */

    if (0 < hours && hours < 95)
    jobvars[1].hours = hours;

    if (0 < weeks && weeks <= 52)
    jobvars[1].weeks = weeks;

	  if (sempl == 0)          /* employee */
    { earnings = wagecalc(1, 3412, 1, hours, weeks);   /* G10 */

      if (earnings > 0)
      jobvars[1].earnings = earnings;
      }
	  else                        /* self-employed */
	  { salary  = wagecalc(1, 3436, 1, hours, weeks);    /* G22a */
		  profits = wagecalc(1, 3439, 1, hours, weeks);    /* G23a */

      if (salary >= 0 && profits >= 0)
      jobvars[1].earnings = salary + profits;
      }

    hourslimits = eval(1, 3410);                 /* G8a: could work half-time? */

    if (hourslimits == 1)
    jobvars[1].hourslimits = 1;
    else jobvars[1].hourslimits = 0;

    jobvars[1].sscoverage = 1;

	  pencoverage = eval(1, 3430 + 18 * sempl);    /* G17/G29: covered by pension */

	  if (pencoverage == 1)
	  jobvars[1].pencoverage = 1;
    else jobvars[1].pencoverage = 0;

    firmclosed = eval(1, 3422);                  /* G16a: business closed */
    layoff = eval(1, 3423);                      /* G16b: laid off */

    if (firmclosed == 1 || firmclosed == 2 || layoff == 1 || layoff == 2)
    jobvars[1].layoff = 1;
    else jobvars[1].layoff = 0;
	  }

  longjob = eval(1, 3601);                   /* H1: previous 5 year job */

  if (longjob == 1)
  { jobvars[2].jobind = 1;

    sempl = eval(1, 3603);                   /* H3: self employed? */

	  if (sempl == 2)
	  sempl = 1;
	  else sempl = 0;

    jobvars[2].sempl = sempl;

    startdate = eval(1, 3604);               /* H4: date started */
	  enddate   = eval(1, 3607);               /* H7: date separated */

	  if (1920 <= startdate && startdate <= 1993)
    jobvars[2].startdate = startdate;

	  if (1920 <= enddate && enddate <= 1993)
    jobvars[2].enddate = enddate;

    jobvars[2].occ = eval(1, 3606);          /* H5a/H6: occupation */
	  jobvars[2].ind = eval(1, 3605);          /* H5: industry */

    hours = eval(1, 3608);                   /* H8: hours per week */
	  weeks = eval(1, 3609);                   /* H9: weeks per year */

    if (0 < hours && hours < 95)
    jobvars[2].hours = hours;

    if (0 < weeks && weeks <= 52)
    jobvars[2].weeks = weeks;

	  earnings = wagecalc(1, 3610, 1, hours, weeks);     /* H10 */

    if (earnings >= 0)
    jobvars[2].earnings = earnings;

    jobvars[2].sscoverage = 1;

    if (persvars->caseid == 23944)     /* personal service job with acceptable reported */
    jobvars[2].sscoverage = 0;         /* wages but very low social security earnings */
    else jobvars[2].sscoverage = 1;

	  pencoverage = eval(1, 3620);

	  if (pencoverage == 1)
	  jobvars[2].pencoverage = 1;
    else jobvars[2].pencoverage = 0;

    firmclosed = eval(1, 3612);         /* H11a: business closed */
    layoff = eval(1, 3613);             /* H11b: laid off */

    if (firmclosed == 1 || firmclosed == 2 || layoff == 1 || layoff == 2)
    jobvars[2].layoff = 1;
    else jobvars[2].layoff = 0;

    pensjob = eval(1, 3702);

    if (pensjob == 1)          /* pension job */
    { jobvars[3].jobind = 1;

      startdate = eval(1, 3704);           /* H25: year started job */
	    enddate   = eval(1, 3705);           /* H26: year separated */

	    if (1920 <= startdate && startdate <= 1993)
      jobvars[3].startdate = startdate;

	    if (1920 <= enddate && enddate <= 1993)
      jobvars[3].enddate = enddate;

		  earnings = wagecalc(1, 3706, 1, 40, 52);     /* H27 */

      if (earnings >= 0)
      jobvars[3].earnings = earnings;

      jobvars[3].sscoverage = 1;
      jobvars[3].pencoverage = 1;

      secondpensjob = eval(1, 3801);         /* H35: second pension job? */

      if (secondpensjob == 2)
      { jobvars[4].jobind = 1;

        startdate = eval(1, 3804);           /* H25: year started job */
	      enddate   = eval(1, 3805);           /* H26: year separated */

	      if (1920 <= startdate && startdate <= 1993)
        jobvars[4].startdate = startdate;

	      if (1920 <= enddate && enddate <= 1993)
        jobvars[4].enddate = enddate;

		    earnings = wagecalc(1, 3806, 1, 40, 52);     /* H27 */

        if (earnings >= 0)
        jobvars[4].earnings = earnings;

        jobvars[4].sscoverage = 1;
        jobvars[4].pencoverage = 1;
		    }
	    }
	  }

  for (jobno = 0; jobno < 3; jobno++)
  if (0 < jobvars[jobno].hours && 0 < jobvars[jobno].weeks
   && jobvars[jobno].hours * jobvars[jobno].weeks < 100)
  { jobvars[jobno].hours    = -9;
    jobvars[jobno].weeks    = -9;
    jobvars[jobno].earnings = -9;
    }

  for (gtype = 0; gtype < 2; gtype++)      /* 0:state/loc  1:fed */
  { govtjob = eval(1, 3940 + 5 * gtype);       /* H61/H62: employed by government */

	  if (govtjob == 1)
    for (jobno = 0; jobno < 2; jobno++)
	  { startdate = eval(1, 3941 + 2 * jobno + 5 * gtype);     /* H61a/H62a year job started*/
		  enddate   = eval(1, 3942 + 2 * jobno + 5 * gtype);     /* H61a/H62a year separated */

	    if (1920 <= startdate && startdate <= 1993)
      jobvars[5 + 2 * gtype + jobno].startdate = startdate;

	    if (1920 <= enddate && enddate <= 1993)
      jobvars[5 + 2 * gtype + jobno].enddate = enddate;

      if (jobvars[5 + 2 * gtype + jobno].startdate > -9
       || jobvars[5 + 2 * gtype + jobno].enddate > -9)
      { jobvars[5 + 2 * gtype + jobno].jobind = 1;
        jobvars[5 + 2 * gtype + jobno].sscoverage = 1;
        }
		  }
	  }


  /* Get years of other full-time work */

  yearfirstworked = eval(3, 3091);     /* G161: Year first worked 6+ months */
  lastunknowndate = eval(3,  189);     /* Preload: Last year of undocumented job history */
  ftstartdate     = eval(3, 3097);     /* G163: Year first worked full-time */
  ftenddate       = eval(3, 3098);     /* G163: Year last worked full-time */

  ftyears = -9;

  if (ftstartdate < 1920 || ftstartdate > 1996 || ftenddate < 1920 || ftenddate > 1996)
  { allyears = eval(3, 3100);          /* G163: All years response */

	  if (allyears == 1 || allyears == 2)
	  { allyears = eval(3, 3095);        /* G162: full-time & part-time years */

		  if (allyears == 2)
		  { ftstartdate = yearfirstworked; /* G161: Year first worked 6+ months */
		    ftenddate = lastunknowndate;   /* Preload: Last year of undocumented job history */
		    }
		  else
		  { ftyears = eval(3, 3094);       /* G162: Number of full-time & part-time years */

		    if (ftyears < 0 || ftyears > 50)
		    { ftstartdate = eval(3, 3092); /* G162: First full-time or part-time year */
			    ftenddate = eval(3, 3093);   /* G162: Last full-time or part-time year */
			    }
		    }
		  }
    else ftyears = eval(3, 3099);      /* G163: Number of full-time years */
  	}

  if (0 <= ftyears && ftyears <= 50)
  ftenddate = lastunknowndate;         /* Preload: Last year of undocumented job history */
  else ftyears = -9;

  if (ftstartdate < 1920 || ftstartdate > 1996)
  ftstartdate = -9;

  if (ftenddate < 1920 || ftenddate > 1996)
  ftenddate = -9;

  /* Get non-social security jobs */

  nonssstartdate = -9;
  nonssenddate = -9;
  nonssyears = -9;

  nonssjob = eval(3, 3102);            /* G164: Any jobs not covered by ss */

  if (nonssjob == 1)
  { nonssstartdate = eval(3, 3103);    /* G165: Start date of non-ss jobs */
	  nonssenddate   = eval(3, 3104);    /* G165: End date of non-ss jobs */

    if (nonssstartdate < 1920 || nonssstartdate > 1996
     || nonssenddate < 1920 || nonssenddate > 1996)
    { ssallyears = eval(3, 3106);      /* G165: All years response to non-ss jobs */

      if (ssallyears == 2)
		  { nonssstartdate = eval(3, 3091);  /* G161: Year first worked 6+ months */
        nonssenddate = 1996;
        }
      else nonssyears = eval(3, 3105); /* G165: Number of non-ss years */
      }
    }
  else if (nonssjob == 5)
  nonssjob = 0;
  else nonssjob = -9;

  if (nonssstartdate < 1920 || nonssstartdate > 1996)
  nonssstartdate = -9;

  if (nonssenddate < 1920 || nonssenddate > 1996)
  nonssenddate = -9;

  if (nonssyears < 0 || nonssyears > 50)
  nonssyears = -9;


  /* Enter results in jobhist structure */

  for (jobno = 0; jobno < 9; jobno++)
  { jobhist->jobvars[jobno].jobind      = jobvars[jobno].jobind;
    jobhist->jobvars[jobno].startdate   = jobvars[jobno].startdate;
    jobhist->jobvars[jobno].enddate     = jobvars[jobno].enddate;
    jobhist->jobvars[jobno].sempl       = jobvars[jobno].sempl;
    jobhist->jobvars[jobno].occ         = jobvars[jobno].occ;
    jobhist->jobvars[jobno].ind         = jobvars[jobno].ind;
    jobhist->jobvars[jobno].hours       = jobvars[jobno].hours;
    jobhist->jobvars[jobno].weeks       = jobvars[jobno].weeks;
    jobhist->jobvars[jobno].earnings    = jobvars[jobno].earnings;
    jobhist->jobvars[jobno].hourslimits = jobvars[jobno].hourslimits;
    jobhist->jobvars[jobno].sscoverage  = jobvars[jobno].sscoverage;
    jobhist->jobvars[jobno].pencoverage = jobvars[jobno].pencoverage;
    }

  jobhist->pencontr = pencontr;

  jobhist->everworked      = everworked;
  jobhist->yearfirstworked = yearfirstworked;
  jobhist->yearlastworked  = yearlastworked;
  jobhist->lastunknowndate = lastunknowndate;

  jobhist->ftstartdate     = ftstartdate;
  jobhist->ftenddate       = ftenddate;
  jobhist->ftyears         = ftyears;
  jobhist->nonssjob        = nonssjob;
  jobhist->nonssstartdate  = nonssstartdate;
  jobhist->nonssenddate    = nonssenddate;
  jobhist->nonssyears      = nonssyears;

  }


/*---------------------------------------------------------------------------
			 GET PANEL VARS routine
			 gets variables from the 5 waves
  ---------------------------------------------------------------------------*/

void getPanelVars(
  VARS *vars)             /* structure of personal variables         */

{ int    surv, caseid, interview;
  int    famresp, finresp, subhh, moved;
  int    age, intvmonth, intvyear, birthmonth, birthyear;
  int    marstat, prevmarstat, married, newspouse, samespouse;
  int    work, samejob, samejob1, layoff, reasona, reasonb, reasonc, reasond;
  int    sempl, lastsempl, hours, weeks, occ, ind, sameocc, hourslimits, health;
  int    pension, penbranch, pencoverage, futurepens;
  int    eowoffer, eowcount, eownumber, eowservice, eowaccept;
  int    ssdisab, ssdisab1, ssdisab2, ssdisab3, ssdisab4, ssdisab5;
  int    gift, giftreceipt, gifttype, ssreceipt, year;
  int    srret, expretyear, expretage;
  int    retplans1, retplans2, retplans3, retplans4, retplans5, retplans6;
  double salary, hourly, piecework, tips, profits;
  double wage, earnings;
  double amount, eowamount, eowpens, maxamt, maxamtp;

  int    giftyear[3];
  double giftamount[3];
  char   variable[10];

  int    eowpensp[2], eowserv[2], eowaccpt[2];
  double eowamt[2], eowpension[2], eowother[2];

  static double eowmult[6] = { 2000, 52, 26, 12, 24, 1};

  PERSVARS *persvars;
  PANELVARS *panelvars;

  persvars =  &(vars->persvars);
  panelvars = &(vars->panelvars);

  caseid     = persvars->caseid;
  birthyear  = persvars->birthyear;
  birthmonth = persvars->birthmonth;

  for (surv = 0; surv < 6; surv++)
  { switch(surv)
    { case 0:      /* 1992 variables */

        interview = 1;

        intvmonth = eval(1, 23);       /* month of interview, 1992 */
        intvyear = eval(1, 25);        /* year of interview, 1992 */

        marstat = eval(1, 225);        /* A10: 1992 marital status */

        if (marstat == 1 || marstat == 7 || marstat == 8)
        marstat = 1;
        else marstat = 0;

        samespouse = -1;

        work = eval(1, 2717);          /* F2: 1992 any current work? */

        if (work == 1)
        { samejob = 0;
          layoff  = 0;

          sempl = eval(1, 2718);       /* F3: employee or self-employed? */

          if (sempl == 2)              /* self-employed */
          sempl = 1;
          else sempl = 0;

          occ = eval(1, 2720 + 101 * sempl);      /* F5/F6/F27: occupation */
          ind = eval(1, 2719 + 101 * sempl);      /* F4/F26: industry */

          hours = eval(1, 2722 + 100 * sempl);    /* F8/F28: usual weekly hours */
		      weeks = eval(1, 2726 + 97 * sempl);     /* F10/F29: usual weeks/year */

  		    if (sempl == 0)
		      { salary    = wagecalc(1, 2735, 1, hours, weeks);  /* salary */
			  	  hourly    = wagecalc(1, 2739, 0, hours, weeks);  /* hourly     */
			  	  piecework = wagecalc(1, 2743, 1, hours, weeks);  /* piecework  */
				    tips      = wagecalc(1, 2748, 1, hours, weeks);  /* tips, etc. */

            if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
            earnings = salary + hourly + piecework + tips;
            else earnings = -1;
            }
		      else
		      { salary  = wagecalc(1, 2825, 1, hours, weeks);     /* salary  */
				    profits = wagecalc(1, 2828, 1, hours, weeks);     /* profits */

            if (salary >= 0 && profits >= 0)
            earnings = salary + profits;
            else earnings = -1;
            }

          if (earnings > 0 && 0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
          wage = earnings / (hours * weeks);
          else wage = -1;

          hourslimits = eval(1, 2805);     /* F19: would employer allow half-time work? */

          if (hourslimits != 1)
          hourslimits = 0;

		      pencoverage = eval(1, 2901 -  63 * sempl);

		      if (sempl == 0)
		      futurepens = eval(1, 2906);     /* will become eligible if remains with employer */
		      else futurepens = 0;

		      if (pencoverage == 1 || futurepens == 1)
		      pension = 1;
          else pension = 0;
          }
        else
        { samejob = 0;
          sempl = 0;

          reasona = eval(1, 3422);         /* G16a: business closed */
          reasonb = eval(1, 3423);         /* G16b: laid off/let go */

          if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2)
          layoff = 1;
          else layoff = 0;

          hours = 0;
          weeks = 0;
          wage = 0;
					occ = 0;
          ind = 0;
          hourslimits = 0;
          pension = 0;
          }

        health = eval(1, 301);         /* B1: health status */

        if (health == 4 || health == 5)
        health = 1;
        else health = 0;

        ssdisab1 = eval(1, 4709);       /* J123b: Awarded SSDI benefits? */
        ssdisab2 = eval(1, 4713);       /* J123e: Awarded SSDI on appeal? */

        if (ssdisab1 == 1 || ssdisab2 == 1)
        ssdisab = 1;
        else ssdisab = 0;

        for (gift = 0; gift < 3; gift++)
        { giftyear[gift] = -1;
          giftamount[gift] = -1;
          }

        finresp = evals(2, "W1FINR");   /* Wave 1 financial respondent */
        ssreceipt = eval(1, 6001);      /* N41: anyone receive social security */

        if ((finresp == 1 && (ssreceipt == 1 || ssreceipt == 2))
         || (finresp == 0 && (ssreceipt == 1 || ssreceipt == 3)))
        ssreceipt = 1;
        else ssreceipt = 0;

        srret = eval(1, 4901);          /* K1: self reported retirement status */

        if (srret == 3 || srret == 5)
        { expretyear = eval(1, 5001);   /* K13: expected retirement year */

          if (1992 <= expretyear && expretyear <= 2050)
          expretage = min(95, expretyear - birthyear);
          else if (expretyear == 9996)
          expretage = 96;
          else expretage = -1;
          }
        else expretage = -1;

        break;

      case 1:      /* 1994 variables */

        interview = evals(2, "W2MODEIW");
        moved = eval(2, 109);            /* why moved out: 11 = deceased */

        if (interview > 0 && moved != 11)
        { interview = 1;

          intvmonth = eval(2, 56);        /* month of interview, 1994 */
          intvyear = eval(2, 58);         /* year of interview, 1994 */

          marstat = eval(2, 200);        /* A1: marital status */

          if (marstat == 1 || marstat == 7 || marstat == 8)
          { marstat = 1;

            newspouse = eval(2, 112);    /* New spouse */

            if (newspouse != 1)
            samespouse = 1;
            else samespouse = 0;
            }
          else
          { marstat = 0;
            samespouse = 0;
            }

          work  = eval(2, 3316);         /* FA2: 1994 any current work? */
          lastsempl = eval(1, 2718);     /* 1992 F3: employee or self-employed? */

          if (work == 1)
          { sempl = eval(2, 3317);       /* FA3: employee or self-employed? */

            if (sempl == 2)              /* self-employed */
            sempl = 1;
            else sempl = 0;

            if (lastsempl == 1)            /* working for someone else in 1992 */
            { if (sempl == 0)              /* working for someone else in 1994 */
              { samejob = eval(2, 3458);   /* FA19: still working for previous employer? */

                if (samejob == 1 || samejob >= 8)
                { samejob = 1;
                  layoff = 0;
                  }
                else
                { samejob = 0;

                  reasona = eval(2, 3507);     /* FA28: reasons for leaving 1992 job */
                  reasonb = eval(2, 3508);

                  if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2)
                  layoff = 1;
                  else layoff = 0;
                  }
                }
              else                         /* self-employed in 1994 */
              { samejob = 0;

                reasona = eval(2, 4204);   /* FB4: reasons for leaving 1992 employer */
                reasonb = eval(2, 4205);

                if (reasona == 1 || reasona == 2 || reasonb == 2)
                layoff = 1;
                else layoff = 0;
                }
              }
            else if (lastsempl == 2)       /* self-employed in 1992 */
            { if (sempl == 0)              /* working for someone else in 1994 */
              samejob = 0;
              else samejob = 1;

              layoff = 0;
              }
            else                           /* not employed in 1992 */
            { samejob = 0;
              layoff = 0;
              }

            if (sempl == 0)                 /* working for someone else in 1994 */
            { if (lastsempl == 1 && samejob == 1)
              { sameocc = eval(2, 3502);    /* FA21: same occupation */

                if (sameocc == 1)
                occ = panelvars->wavevars[0].occ;
                else if (sameocc == 7)
                occ = evals(2, "W3502B");   /* FA24/FA25: current occupation */
                else occ = eval(2, 3609);   /* FA41b/FA42: current occupation */

                ind = panelvars->wavevars[0].ind;
                }
              else
              { occ = eval(2, 3609);        /* FA41b/FA42: current occupation */
                ind = eval(2, 3608);        /* FA41a: current industry */
                }
              }
            else                            /* self-employed in 1994 */
            { occ = evals(2, "W4312B");     /* FB16: current occupation */
              ind = evals(2, "W4312A");     /* FB15: current industry */
              }

            hours = eval(2, 3617 + 696 * sempl);  /* FA44/FB17: usual weekly hours */
            weeks = eval(2, 3623 + 691 * sempl);  /* FA47/FB18: usual weeks/year */

            if (hours == 0)
            hours = -1;

            if (weeks == 0)
            weeks = -1;

            if (sempl == 0)
		        { salary    = wagecalc(2, 3632, 1, hours, weeks);  /* FA52a */
			  	    hourly    = wagecalc(2, 3637, 0, hours, weeks);  /* FA52f */
				      piecework = wagecalc(2, 3642, 1, hours, weeks);  /* FA52m */
				      tips      = wagecalc(2, 3645, 1, hours, weeks);  /* FA52r */

              if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
              earnings = salary + hourly + piecework + tips;
              else earnings = -1;

              hourslimits = eval(2, 3651);      /* FA55: could work half-time? */

              if (hourslimits != 1)
              hourslimits = 0;

              if (samejob == 1 && panelvars->wavevars[0].pension == 1)
              pencoverage = 1;
              else
              { pencoverage = eval(2, 3748);    /* FA71: pension */
		            futurepens = eval(2, 3753);     /* FA71e: will become eligible */
                }
              }
		        else
		        { salary  = wagecalc(2, 4318, 1, hours, weeks);     /* FB19a */

              profits = eval(2, 4321);

              if (profits == 999995)       /* included in salary */
              profits = 0;
              else profits = wagecalc(2, 4321, 1, hours, weeks);     /* FB20a */

              if (salary >= 0 && profits >= 0)
              earnings = salary + profits;
              else earnings = -1;

              hourslimits = 0;

              if (samejob == 1 && panelvars->wavevars[0].pension == 1)
              pencoverage = 1;
              else
              { pencoverage = eval(2, 4375);    /* FB26: pension coverage */
                futurepens = 0;
                }
              }

            if (earnings > 0 && 0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
            wage = earnings / (hours * weeks);
            else wage = -1;

		        if (pencoverage == 1 || futurepens == 1)
		        pension = 1;
            else if (pencoverage == 5)
            pension = 0;
            else pension = -9;
            }
          else
          { samejob = 0;
            sempl = 0;

            if (lastsempl == 1)            /* working for someone else in 1992 */
            { reasona = eval(2, 4901);     /* FC17: reasons for leaving 1992 job */
              reasonb = eval(2, 4902);

              if (reasona == 1 || reasona == 2 || reasona == 13 || reasonb == 1 || reasonb == 2)
              layoff = 1;
              else layoff = 0;
              }
            else layoff = 0;

            hours = 0;
            weeks = 0;
            wage = 0;
  					occ = 0;
            ind = 0;
            hourslimits = 0;
            pension = 0;
            }

          health = eval(2, 301);         /* B1: health status */

          if (health == 4 || health == 5)
          health = 1;
          else if (health > 0)
          health = 0;
          else health = -1;

          if (work == 1)
          { if (sempl == 0)
            { eowoffer = eval(2, 4066);     /* FA135: Any eow offers? */
              eowcount = eval(2, 4067);     /* FA135a: Number of eow's */

              if (eowoffer == 1)
              { if (eowcount == 0 || eowcount > 95)
                eowcount = 0;
                else if (eowcount > 2)
                eowcount = 2;
                }
              else eowcount = 0;

              if (eowcount >= 1)
              { eowamt[0]     = eval(2, 4078);  /* FA139a: Cash bonus amount */
                eowpension[0] = eval(2, 4079);  /* FA139b: Extra pension benefits */
                eowpensp[0]   = eval(2, 4080);  /* FA139b: Time period for pension benefits */
                eowserv[0]    = eval(2, 4087);  /* FA139e: Additional years of service */
                eowother[0]   = eval(2, 4088);  /* FA139f: Value of other incentives */
                eowaccpt[0]   = eval(2, 4089);  /* FA140:  Accept offer? */
                }

              if (eowcount >= 2)
              { eowamt[1]     = eval(2, 4102);  /* FA139a: Cash bonus amount */
                eowpension[1] = eval(2, 4103);  /* FA139b: Extra pension benefits */
                eowpensp[1]   = eval(2, 4104);  /* FA139b: Time period for pension benefits */
                eowserv[1]    = eval(2, 4111);  /* FA139e: Additional years of service */
                eowother[1]   = eval(2, 4112);  /* FA139f: Value of other incentives */
                eowaccpt[1]   = eval(2, 4113);  /* FA140:  Accept offer? */
                }
              }
            else eownumber = 0;
            }
          else
          { eowoffer = eval(2, 5105);     /* FC53: Any eow offers? */
            eowcount = eval(2, 5106);     /* FC53a: Number of eow's */

            if (eowoffer == 1)
            { if (eowcount == 0 || eowcount > 95)
              eowcount = 0;
              else if (eowcount > 2)
              eowcount = 2;
              }
            else eowcount = 0;

            if (eowcount >= 1)
            { eowamt[0]     = eval(2, 5117);  /* FC57a: Cash bonus amount */
              eowpension[0] = eval(2, 5118);  /* FC57b: Extra pension benefits */
              eowpensp[0]   = eval(2, 5119);  /* FC57b: Time period for pension benefits */
              eowserv[0]    = eval(2, 5126);  /* FC57e: Additional years of service */
              eowother[0]   = eval(2, 5127);  /* FC57f: Value of other incentives */
              eowaccpt[0]   = eval(2, 5128);  /* FC57:  Accept offer? */
              }

            if (eowcount >= 2)
            { eowamt[1]     = eval(2, 5141);  /* FC57a: Cash bonus amount */
              eowpension[1] = eval(2, 5142);  /* FC57b: Extra pension benefits */
              eowpensp[1]   = eval(2, 5143);  /* FC57b: Time period for pension benefits */
              eowserv[1]    = eval(2, 5150);  /* FC57e: Additional years of service */
              eowother[1]   = eval(2, 5151);  /* FC57f: Value of other incentives */
              eowaccpt[1]   = eval(2, 5152);  /* FC58:  Accept offer? */
              }
            }

          ssdisab1 = eval(2, 5259);     /* J18: Still receiving SSDI benefits? */
          ssdisab2 = eval(2, 5278);     /* J26b: Awarded SSDI benefits? */
          ssdisab3 = eval(2, 5282);     /* J26e: Awarded SSDI on appeal? */

          if (ssdisab1 == 1 || ssdisab2 == 1 || ssdisab3 == 1)
          ssdisab = 1;
          else ssdisab = 0;

          for (gift = 0; gift < 3; gift++)
          { giftreceipt = eval(2, 5565 + 7 * gift);    /* K36, K42, K43: any lump sums? */

            if (giftreceipt == 1)
            { gifttype = eval(2, 5566 + 7 * gift);     /* K37: type of gift */

              if (gifttype != 1 && gifttype != 3)      /* not insurance or inheritance */
              giftreceipt = 5;
              }

            if (giftreceipt == 1)
            { year = eval(2, 5568 + 7 * gift);         /* K38: year of gift */

              if (1 <= year && year <= 4)
              year = 1990 + year;
              else year = -1;

              amount = eval(2, 5569 + 7 * gift);       /* K39: amount of gift */

              if (amount > 9999990)
              amount = -1;
              }
            else
            { year   = -1;
              amount =  0;
              }

            giftyear[gift]   = year;
            giftamount[gift] = amount;
            }

          finresp = evals(2, "W2FINR");   /* Wave 1 financial respondent */
          ssreceipt = eval(2, 6254);      /* N23a: who received social security? */

          if ((finresp == 1 && (ssreceipt == 1 || ssreceipt == 3))
           || (finresp == 0 && (ssreceipt == 2 || ssreceipt == 3)))
          ssreceipt = 1;
          else ssreceipt = 0;

          if (work == 1)
          { if (sempl == 0)
            { srret = eval(2, 3983);      /* FA123: self reported retirement status */

              retplans1 = eval(2, 3967);  /* FA122: plan to stop work altogether */
              retplans2 = eval(2, 3968);
              retplans3 = eval(2, 3969);
              retplans4 = eval(2, 3970);

              if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1 || retplans4 == 1)
              { expretage = eval(2, 3975);  /* FA122a: expected age of retirement */

                if (expretage < 25 || expretage > 97)
                expretage = -1;
                else if (expretage == 97)
                { expretyear = eval(2, 3976);  /* FA122a1: expected year of retirement */

                  if (1994 <= expretyear && expretyear <= 2020)
                  expretage = min(95, expretyear - birthyear);
                  else if (expretyear == 9996)
                  expretage = 96;
                  else expretage = -1;
                  }
                }
              else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2 || retplans4 == 2)
              expretage = 96;
              else expretage = -1;
              }
            else
            { srret = eval(2, 4638);      /* FB83: self reported retirement status */

              retplans1 = eval(2, 4622);   /* FB82: plan to stop work altogether */
              retplans2 = eval(2, 4623);
              retplans3 = eval(2, 4624);

              if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1)
              { expretage = eval(2, 4630);    /* FB82a: expected age of retirement */

                if (expretage < 25 || expretage > 97)
                expretage = -1;
                else if (expretage == 97)
                { expretyear = eval(2, 4631);  /* FB82a1: expected year of retirement */

                  if (1994 <= expretyear && expretyear <= 2020)
                  expretage = min(95, expretyear - birthyear);
                  else if (expretyear == 9996)
                  expretage = 96;
                  else expretage = -1;
                  }
                }
              else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2)
              expretage = 96;
              else expretage = -1;
              }
            }
          else
          { srret = eval(2, 5022);    /* FC41: self reported retirement status */
            expretage = -1;
            }
          }
        else interview = 0;

        break;


      case 2:      /* 1996 variables */

        famresp = eval(3, 376);
        finresp = eval(3, 377);

        if (famresp > 0 || finresp > 0)
        { interview = 1;

          intvmonth = eval(3, 391);      /* month of interview, 1996 */
          intvyear = eval(3, 393);       /* year of interview, 1996 */

          marstat  = evals(3, "E256A");    /* marital status */

          if (marstat == 1 || marstat == 2 || marstat == 7)
          { marstat = 1;

            subhh = evals(3, "ESUBHH");
            samespouse = eval(3, 226);    /* CS4: same spouse? */

            if (subhh == 0 && samespouse != 5)
            samespouse = 1;
            else samespouse = 0;
            }
          else
          { marstat = 0;
            samespouse = 0;
            }

          work  = eval(3, 2627);         /* G2: 1996 any current work? */
          lastsempl = eval(3, 185);      /* Preload: prev wave self-employed? */

          if (work == 1)
          { sempl = eval(3, 2628);       /* G3: employee or self-employed? */

            if (sempl == 2)              /* self-employed */
            sempl = 1;
            else sempl = 0;

            if (lastsempl == 1)          /* working for someone else prev wave */
            { if (sempl == 0)            /* working for someone else in 1996 */
              { samejob = eval(3, 2655); /* G19: still working for previous employer? */

                if (samejob == 1 || samejob >= 8)
                samejob = 1;
                else samejob = 0;
                }
              else samejob = 0;
              }
            else if (lastsempl == 2)       /* self-employed prev wave */
            { if (sempl == 0)              /* working for someone else in 1994 */
              samejob = 0;
              else samejob = 1;
              }
            else samejob = 0;              /* not employed in 1992 */

            if (sempl == 0 && lastsempl == 1 && samejob == 1)
            { sameocc = eval(3, 2662);    /* G21: same occupation */

              if (sameocc == 1)
              { if (panelvars->wavevars[1].hours > 0)
                occ = panelvars->wavevars[1].occ;
                else occ = panelvars->wavevars[0].occ;
                }
              else if (sameocc == 7)
              occ = evals(3, "E2666M");   /* G24/G25: current occupation */
              else occ = evals(3, "E2732M");   /* G41b/G42: current occupation */

              if (panelvars->wavevars[1].hours > 0)
              ind = panelvars->wavevars[1].ind;
              else ind = panelvars->wavevars[0].ind;
              }
            else
            { occ = evals(3, "E2732M");   /* G41b/G42: current occupation */
              ind = evals(3, "E2730M");   /* G41a: current industry */
              }

            hours = eval(3, 2736);         /* G44: usual weekly hours */
            weeks = eval(3, 2746);         /* G47: usual weeks/year */

            if (sempl == 0)
		        { salary    = wagecalc(3, 2774, 1, hours, weeks);  /* G56a */
			  	    hourly    = wagecalc(3, 2781, 0, hours, weeks);  /* G56f */
				      piecework = wagecalc(3, 2791, 1, hours, weeks);  /* G56m */
				      tips      = wagecalc(3, 2796, 1, hours, weeks);  /* G56r */

              if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
              earnings = salary + hourly + piecework + tips;
              else earnings = -1;
              }
		        else
		        { salary  = wagecalc(3, 2757, 1, hours, weeks);     /* G52a */
              profits = wagecalc(3, 2761, 1, hours, weeks);     /* G53a */

              if (salary >= 0 && profits >= 0)
              earnings = salary + profits;
              else earnings = -1;
              }

            if (earnings > 0 && 0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
            wage = earnings / (hours * weeks);
            else wage = -1;

            hourslimits = eval(3, 2809);      /* G59: could work half-time? */

            if (hourslimits != 1)
            hourslimits = 0;

            penbranch = eval(3, 2834);        /* G67 branchpoint: Pension in same job last time? */

            if (penbranch == 1 || penbranch == 2)
            pencoverage = 1;
            else
            { pencoverage = eval(3, 2861);    /* G75: pension */
		          futurepens = eval(3, 2866);     /* G75e: will become eligible */
              }

		        if (pencoverage == 1 || futurepens == 1)
		        pension = 1;
            else if (pencoverage == 5)
            pension = 0;
            else pension = -9;
            }
          else
          { samejob = 0;
            sempl = 0;
            hours = 0;
            weeks = 0;
            wage = 0;
					  occ = 0;
            ind = 0;
            hourslimits = 0;
            pension = 0;
            }

          if (lastsempl == 1 && samejob == 0)
          { reasona = evals(3, "E2673M1");   /* G28: reasons for leaving prev wave job */
            reasonb = evals(3, "E2673M2");

            if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2)
            layoff = 1;
            else layoff = 0;
            }
          else layoff = 0;

          health = eval(3, 769);         /* B1: health status */

          if (health == 4 || health == 5)
          health = 1;
          else health = 0;

          eowoffer = eval(3, 3067);     /* G146: Any eow offers? */
          eowcount = eval(3, 3068);     /* G146a: Number of eow's */

          if (eowoffer == 1)
          { if (eowcount == 0 || eowcount > 95)
            eowcount = 0;
            else if (eowcount > 2)
            eowcount = 2;
            }
          else eowcount = 0;

          if (eowcount >= 1)
          { eowamt[0]     =  evals(3, "E3075_1");  /* G150a: Cash bonus amount */
            eowpension[0] = fevals(3, "E3076_1");  /* G150b: Extra pension benefits */
            eowpensp[0]   =  evals(3, "E3077_1");  /* G150c: Time period for pension benefits */
            eowserv[0]    =  evals(3, "E3086_1");  /* G150h: Additional years of service */
            eowother[0]   =  evals(3, "E3087_1");  /* G150j: Value of other incentives */
            eowaccpt[0]   =  evals(3, "E3088_1");  /* G151:  Accept offer? */
            }

          if (eowcount >= 2)
          { eowamt[1]     =  evals(3, "E3075_2");  /* G150a: Cash bonus amount */
            eowpension[1] = fevals(3, "E3076_2");  /* G150b: Extra pension benefits */
            eowpensp[1]   =  evals(3, "E3077_2");  /* G150c: Time period for pension benefits */
            eowserv[1]    =  evals(3, "E3087_2");  /* G150h: Additional years of service */
            eowother[1]   =  evals(3, "E3086_2");  /* G150j: Value of other incentives */
            eowaccpt[1]   =  evals(3, "E3088_2");  /* G151:  Accept offer? */
            }

          ssdisab1 = eval(3, 3580);      /* GD18: Still receiving SSDI benefits? */
          ssdisab2 = eval(3, 3600);      /* GD26b: Awarded SSDI benefits? */
          ssdisab3 = eval(3, 3604);      /* GD26e: Awarded SSDI on appeal? */

          if (ssdisab1 == 1 || ssdisab2 == 1 || ssdisab3 == 1)
          ssdisab = 1;
          else ssdisab = 0;

          for (gift = 0; gift < 3; gift++)
          { giftreceipt = eval(3, 4748 + 7 * gift);    /* J305, J311, J317: any lump sums? */

            if (giftreceipt == 1)
            { gifttype = eval(3, 4749 + 7 * gift);     /* J306, J312, J318: type of gift */

              if (gifttype != 1 && gifttype != 3)      /* not insurance or inheritance */
              giftreceipt = 5;
              }

            if (giftreceipt == 1)
            { year = eval(3, 4751 + 7 * gift);         /* J308, J314, J320: year of gift */

              if (1 <= year && year <= 5)
              year = 1991 + year;
              else year = -1;

              amount = eval(3, 4752 + 7 * gift);       /* J309, J315, J321: amount of gift */

              if (amount > 999996)
              amount = -1;
              }
            else
            { year   = -1;
              amount =  0;
              }

            giftyear[gift]   = year;
            giftamount[gift] = amount;
            }

          ssreceipt = eval(3, 3984);  /* J46: who received social security? */

          if ((finresp == 2 && (ssreceipt == 1 || ssreceipt == 3))
           || (finresp == 3 && (ssreceipt == 2 || ssreceipt == 3)))
          ssreceipt = 1;
          else ssreceipt = 0;

          srret = eval(3, 3039);      /* G134: self reported retirement status */

          retplans1 = evals(3, "E3028M1");  /* G133: plan to stop work altogether */
          retplans2 = evals(3, "E3028M2");
          retplans3 = evals(3, "E3028M3");
          retplans4 = evals(3, "E3028M4");

          if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1 || retplans4 == 1)
          { expretage = eval(3, 3029);  /* G133a: expected age of retirement */

            if (expretage < 25 || expretage > 96)
            expretage = -1;
            else if (expretage == 96)
            { expretyear = eval(3, 3030);  /* G133a1: expected year of retirement */

              if (1996 <= expretyear && expretyear <= 2020)
              expretage = min(95, expretyear - birthyear);
              else expretage = -1;
              }
            else if (expretage == 95)
            expretage = 96;
            }
          else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2 || retplans4 == 2)
          expretage = 96;
          else expretage = -1;
          }
        else interview = 0;

        break;


      case 3:      /* 1998 variables */

        famresp = evals(4, "FFAMR");
        finresp = evals(4, "FFINR");

        if (famresp > 0 || finresp > 0)
        { interview = 1;

          intvmonth = eval(4, 697);      /* month of interview */
          intvyear = eval(4, 699);       /* year of interview */

          marstat  = eval(4, 548);        /* marital status */

          if (marstat == 1)
          { subhh = evals(4, "FSUBHH");
            samespouse = eval(4, 502);  /* CS4: same spouse? */

            if (subhh == 0 && samespouse != 5)
            samespouse = 1;
            else samespouse = 0;
            }
          else
          { marstat = 0;
            samespouse = 0;
            }

          work  = eval(4, 3131);         /* G2: any current work? */
          lastsempl = eval(4, 289);      /* Preload: prev wave self-employed? */

          if (work == 1)
          { sempl = eval(4, 3132);       /* G3: employee or self-employed? */

            if (sempl == 2)              /* self-employed */
            sempl = 1;
            else sempl = 0;

            if (lastsempl == 1)          /* working for someone else prev wave */
            { if (sempl == 0)            /* working for someone else in 1998 */
              { samejob = eval(4, 3158); /* G19a: still working for previous employer? */
                samejob1 = eval(4, 3166); /* G19b: still working for previous employer? */

                if (samejob == 1 || samejob1 == 1 || samejob1 >= 8)
                samejob = 1;
                else samejob = 0;
                }
              else samejob = 0;
              }
            else if (lastsempl == 2)       /* self-employed prev wave */
            { if (sempl == 0)              /* working for someone else in 1998 */
              samejob = 0;
              else samejob = 1;
              }
            else samejob = 0;              /* not employed in 1998 */

            if (sempl == 0 && lastsempl == 1 && samejob == 1)
            { sameocc = eval(4, 3183);     /* G21: same occupation? */

              if (sameocc == 1)
              { if (panelvars->wavevars[2].hours > 0)
                occ = panelvars->wavevars[2].occ;
                else if (panelvars->wavevars[1].hours > 0)
                occ = panelvars->wavevars[1].occ;
                else occ = panelvars->wavevars[0].occ;
              }
              else if (sameocc == 7)
              occ = evals(4, "F3187HM");   /* G24/G25: current occupation */
              else occ = evals(4, "F3255HM");  /*G41b/G42: current occupation */

              if (panelvars->wavevars[2].hours >0)
              ind = panelvars->wavevars[2].ind;
              else if (panelvars->wavevars[1].hours >0)
              ind = panelvars->wavevars[1].ind;
              else ind = panelvars->wavevars[0].ind;
              }
            else
            { occ = evals(4, "F3255HM");   /* G41b/G42: current occupation */
              ind = evals(4, "F3253M");    /* G41a: current industry */
              }

            hours = eval(4, 3259);         /* G44: usual weekly hours */
            weeks = eval(4, 3269);         /* G47: usual weeks/year */

            if (sempl == 0)
		        { salary    = wagecalc(4, 3297, 1, hours, weeks);  /* G56a */
			  	    hourly    = wagecalc(4, 3304, 0, hours, weeks);  /* G56f */
				      piecework = wagecalc(4, 3314, 1, hours, weeks);  /* G56m */
				      tips      = wagecalc(4, 3319, 1, hours, weeks);  /* G56r */

              if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
              earnings = salary + hourly + piecework + tips;
              else earnings = -1;
              }
		        else
		        { salary  = wagecalc(4, 3280, 1, hours, weeks);     /* G52a */
              profits = wagecalc(4, 3284, 1, hours, weeks);     /* G53a */

              if (salary >= 0 && profits >= 0)
              earnings = salary + profits;
              else earnings = -1;
              }

            if (earnings > 0 && 0 < hours && hours <= 168 && 0 < weeks && weeks <= 52)
            wage = earnings / (hours * weeks);
            else wage = -1;

            hourslimits = eval(4, 3332);     /* G59: could work half-time? */

            if (hourslimits != 1)
            hourslimits = 0;

            penbranch = eval(4, 3358);        /* G67 branchpoint: Pension in same job last time? */

            if (penbranch == 1 || penbranch == 2)
            pencoverage = 1;
            else
            { pencoverage = eval(4, 3389);    /* G75: pension */
		          futurepens = eval(4, 3394);     /* G75e: will become eligible */
              }

		        if (pencoverage == 1 || futurepens == 1)
		        pension = 1;
            else if (pencoverage == 5)
            pension = 0;
            else pension = -9;
            }
          else
          { samejob = 0;
            sempl = 0;
            hours = 0;
            weeks = 0;
            wage = 0;
					  occ = 0;
            ind = 0;
            hourslimits = 0;
            pension = 0;
            }

          if (lastsempl == 1 && samejob == 0)
          { reasona = evals(4, "F3194M1");   /* G28: reasons for leaving prev wave job */
            reasonb = evals(4, "F3194M2");
            reasonc = evals(4, "F3194M3");

            if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2
             || reasonc == 1 || reasonc == 2)
            layoff = 1;
            else layoff = 0;
            }
          else layoff = 0;

          health = eval(4, 1097);        /* B1: health status */

          if (health == 4 || health == 5)
          health = 1;
          else health = 0;

          eowoffer = eval(4, 3600);     /* G146: Any eow offers? */
          eowcount = eval(4, 3601);     /* G146a: Number of eow's */

          if (eowoffer == 1)
          { if (eowcount == 0 || eowcount > 95)
            eowcount = 0;
            else if (eowcount > 2)
            eowcount = 2;
            }
          else eowcount = 0;

          if (eowcount >= 1)
          { eowamt[0]     =  evals(4, "F3608_1");  /* G150a: Cash bonus amount */
            eowpension[0] = fevals(4, "F3609_1");  /* G150b: Extra pension benefits */
            eowpensp[0]   =  evals(4, "F3610_1");  /* G150c: Time period for pension benefits */
            eowserv[0]    =  evals(4, "F3621_1");  /* G150h: Additional years of service */
            eowother[0]   =  evals(4, "F3622_1");  /* G150j: Value of other incentives */
            eowaccpt[0]   =  evals(4, "F3623_1");  /* G151:  Accept offer? */
            }

          if (eowcount >= 2)
          { eowamt[1]     =  evals(4, "F3608_2");  /* G150a: Cash bonus amount */
            eowpension[1] = fevals(4, "F3609_2");  /* G150b: Extra pension benefits */
            eowpensp[1]   =  evals(4, "F3610_2");  /* G150c: Time period for pension benefits */
            eowserv[1]    =  evals(4, "F3621_2");  /* G150h: Additional years of service */
            eowother[1]   =  evals(4, "F3622_2");  /* G150j: Value of other incentives */
            eowaccpt[1]   =  evals(4, "F3623_2");  /* G151:  Accept offer? */
            }

          ssdisab1 = eval(4, 4088);     /* GD18: Still receiving SSDI benefits? */
          ssdisab2 = eval(4, 4091);     /* GD18c: Pending SSDI application approved? */
          ssdisab3 = eval(4, 4129);     /* GD26b: Awarded SSDI benefits? */
          ssdisab4 = eval(4, 4133);     /* GD26e: Awarded SSDI on appeal? */

          if (ssdisab1 == 1 || ssdisab2 == 1 || ssdisab3 == 1 || ssdisab4 == 1)
          ssdisab = 1;
          else ssdisab = 0;

          for (gift = 0; gift < 3; gift++)
          { giftreceipt = eval(4, 5508 + 7 * gift);    /* J305, J311, J317: any lump sums? */

            if (giftreceipt == 1)
            { gifttype = eval(4, 5509 + 7 * gift);     /* J306, J312, J318: type of gift */

              if (gifttype != 1 && gifttype != 3 && gifttype != 4)
              giftreceipt = 5;                         /* not insurance, inheritance, or gift */
              }

            if (giftreceipt == 1)
            { year = eval(4, 5511 + 7 * gift);         /* J308, J314, J320: year of gift */

              if (year < 1900 || year > 1999)
              year = -1;

              amount = feval(4, 5512 + 7 * gift);      /* J309, J315, J321: amount of gift */

              if (amount > 9999990)
              amount = -1;
              }
            else
            { year   = -1;
              amount =  0;
              }

            giftyear[gift]   = year;
            giftamount[gift] = amount;
            }

          ssreceipt = eval(4, 4744);  /* J46: who received social security? */

          if (((1 <= finresp && finresp <= 3) && (ssreceipt == 1 || ssreceipt == 3))
           || ((5 <= finresp && finresp <= 7) && (ssreceipt == 2 || ssreceipt == 3)))
          ssreceipt = 1;
          else ssreceipt = 0;

          srret = eval(4, 3570);      /* G134: self reported retirement status */

          if (srret == 3 || srret == 5)
          { retplans1 = evals(4, "F3559M1");  /* G133: plan to stop work altogether */
            retplans2 = evals(4, "F3559M2");
            retplans3 = evals(4, "F3559M3");
            retplans4 = evals(4, "F3559M4");
            retplans5 = evals(4, "F3559M5");
            retplans6 = evals(4, "F3559M6");

            if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1
             || retplans4 == 1 || retplans5 == 1 || retplans6 == 1)
            { expretage = eval(4, 3560);  /* G133a: expected age of retirement */

              if (expretage < 25 || expretage > 96)
              expretage = -1;
              else if (expretage == 96)
              { expretyear = eval(4, 3561);  /* G133a1: expected year of retirement */

                if (1999 <= expretyear && expretyear <= 2020)
                expretage = min(95, expretyear - birthyear);
                else if (expretyear == 9995)
                expretage = 96;
                else expretage = -1;
                }
              else if (expretage == 95)
              expretage = 96;
              }
            else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2
                  || retplans4 == 2 || retplans5 == 2 || retplans6 == 2)
            expretage = 96;
            else expretage = -1;
            }
          else expretage = -1;
          }
        else interview = 0;

        break;


      case 4:      /* 2000 variables */

        famresp = evals(5, "GFAMR");     /* 2000 interview */
        finresp = evals(5, "GFINR");

        if (famresp > 0 || finresp > 0)
        { interview = 1;

          intvmonth = eval(5, 768);      /* month of interview */
          intvyear = eval(5, 770);       /* year of interview */

          marstat  = eval(5, 597);        /* marital status */

          if (marstat == 1)
          { subhh = evals(5, "GSUBHH");
            samespouse = eval(5, 543);  /* CS4: same spouse? */

            if (subhh == 0 && samespouse != 5)
            samespouse = 1;
            else samespouse = 0;
            }
          else
          { marstat = 0;
            samespouse = 0;
            }

          work  = eval(5, 3381);         /* G2: any current work? */
          lastsempl = eval(5, 289);      /* Preload: prev wave self-employed? */

          if (work == 1)
          { sempl = eval(5, 3382);       /* G3: employee or self-employed? */

            if (sempl == 2)              /* self-employed */
            sempl = 1;
            else sempl = 0;

            if (lastsempl == 1)          /* working for someone else prev wave */
            { if (sempl == 0)            /* working for someone else in 2000 */
              { samejob = eval(5, 3416); /* G19b: still working for previous employer? */

                if (samejob == 1 || samejob >= 8)
                samejob = 1;
                else samejob = 0;
                }
              else samejob = 0;
              }
            else if (lastsempl == 2)       /* self-employed prev wave */
            { if (sempl == 0)              /* working for someone else in 2000 */
              samejob = 0;
              else samejob = 1;
              }
            else samejob = 0;              /* not employed in 2000 */

            if (sempl == 0 && lastsempl == 1 && samejob == 1)
            { sameocc = eval(5, 3432);     /* G21: same occupation? */

              if (sameocc == 1)
              { if (panelvars->wavevars[3].hours > 0)
                occ = panelvars->wavevars[3].occ;
                else if (panelvars->wavevars[2].hours > 0)
                occ = panelvars->wavevars[2].occ;
                else if (panelvars->wavevars[1].hours > 0)
                occ = panelvars->wavevars[1].occ;
                else occ = panelvars->wavevars[0].occ;
                }
              else if (sameocc == 7)
              occ = evals(5, "G3436M");    /* G24/G25: current occupation */
              else occ = evals(5, "G3505M");  /* G41b/G42: current occupation */

              if (panelvars->wavevars[3].hours > 0)
              ind = panelvars->wavevars[3].ind;
              else if (panelvars->wavevars[2].hours > 0)
              ind = panelvars->wavevars[2].ind;
              else if (panelvars->wavevars[1].hours > 0)
              ind = panelvars->wavevars[1].ind;
              else ind = panelvars->wavevars[0].ind;
              }
            else
            { occ = evals(5, "G3505M");    /* G41b/G42: current occupation */
              ind = evals(5, "G3503M");    /* G41a: current occupation */
              }

            hours = eval(5, 3509);         /* G44: usual weekly hours */
            weeks = eval(5, 3519);         /* G47: usual weeks/year */

            if (sempl == 0)
		        { salary    = wagecalc(5, 3557, 1, hours, weeks);  /* G56a */
			  	    hourly    = wagecalc(5, 3564, 0, hours, weeks);  /* G56f */
				      piecework = wagecalc(5, 3574, 1, hours, weeks);  /* G56m */
				      tips      = wagecalc(5, 3579, 1, hours, weeks);  /* G56r */

              if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
              earnings = salary + hourly + piecework + tips;
              else earnings = -1;
              }
		        else
		        { salary  = wagecalc(5, 3540, 1, hours, weeks);     /* G52a */
              profits = wagecalc(5, 3544, 1, hours, weeks);     /* G53a */

              if (salary >= 0 && profits >= 0)
              earnings = salary + profits;
              else earnings = -1;
              }

            if (earnings > 0 && 0 < hours && hours <= 168 && 0 < weeks && weeks <= 52)
            wage = earnings / (hours * weeks);
            else wage = -1;

            hourslimits = eval(5, 3592);     /* G59: could work half-time? */

            if (hourslimits != 1)
            hourslimits = 0;

            penbranch = eval(5, 3618);        /* G67 branchpoint: Pension in same job last time? */

            if (penbranch == 1 || penbranch == 2)
            pencoverage = 1;
            else
            { pencoverage = eval(5, 3654);    /* G75: pension */
		          futurepens = eval(5, 3674);     /* G75e: will become eligible */
              }

		        if (pencoverage == 1 || futurepens == 1)
		        pension = 1;
            else if (pencoverage == 5)
            pension = 0;
            else pension = -9;
            }
          else
          { samejob = 0;
            sempl = 0;
            hours = 0;
            weeks = 0;
            wage = 0;
					  occ = 0;
            ind = 0;
            hourslimits = 0;
            pension = 0;
            }

          if (lastsempl == 1 && samejob == 0)
          { reasona = evals(5, "G3444M1");   /* G28: reasons for leaving prev wave job */
            reasonb = evals(5, "G3444M2");
            reasonc = evals(5, "G3444M3");
            reasond = evals(5, "G3444M4");

            if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2
             || reasonc == 1 || reasonc == 2 || reasond == 1 || reasond == 2)
            layoff = 1;
            else layoff = 0;
            }
          else layoff = 0;

          health = eval(5, 1226);        /* B1: health status */

          if (health == 4 || health == 5)
          health = 1;
          else health = 0;

          eowoffer = eval(5, 3889);     /* G146: Any eow offers? */
          eowcount = eval(5, 3890);     /* G146a: Number of eow's */

          if (eowoffer == 1)
          { if (eowcount == 0 || eowcount > 95)
            eowcount = 0;
            else if (eowcount > 2)
            eowcount = 2;
            }
          else eowcount = 0;

          if (eowcount >= 1)
          { eowamt[0]     =  evals(5, "G3897_1");  /* G150a: Cash bonus amount */
            eowpension[0] = fevals(5, "G3898_1");  /* G150b: Extra pension benefits */
            eowpensp[0]   =  evals(5, "G3899_1");  /* G150c: Time period for pension benefits */
            eowserv[0]    =  evals(5, "G3910_1");  /* G150h: Additional years of service */
            eowother[0]   =  evals(5, "G3911_1");  /* G150j: Value of other incentives */
            eowaccpt[0]   =  evals(5, "G3912_1");  /* G151:  Accept offer? */
            }

          if (eowcount >= 2)
          { eowamt[1]     =  evals(5, "G3897_2");  /* G150a: Cash bonus amount */
            eowpension[1] = fevals(5, "G3898_2");  /* G150b: Extra pension benefits */
            eowpensp[1]   =  evals(5, "G3899_2");  /* G150c: Time period for pension benefits */
            eowserv[1]    =  evals(5, "G3910_2");  /* G150h: Additional years of service */
            eowother[1]   =  evals(5, "G3911_2");  /* G150j: Value of other incentives */
            eowaccpt[1]   =  evals(5, "G3912_2");  /* G151:  Accept offer? */
            }

          ssdisab1 = eval(5, 4374);     /* GD18aa: Still receiving SSDI benefits? */
          ssdisab2 = eval(5, 4390);     /* GD18ca: Pending SSDI application approved? */
          ssdisab3 = eval(5, 4403);     /* GD18dd: Awarded SSDI on appeal? */
          ssdisab4 = eval(5, 4459);     /* GD26b: Awarded SSDI benefits? */
          ssdisab5 = eval(5, 4463);     /* GD26e: Awarded SSDI on appeal? */

          if (ssdisab1 == 1 || ssdisab2 == 1 || ssdisab3 == 1 || ssdisab4 == 1 || ssdisab5 == 1)
          ssdisab = 1;
          else ssdisab = 0;

          for (gift = 0; gift < 3; gift++)
          { giftreceipt = eval(5, 5861 + 7 * gift);    /* J305, J311, J317: any lump sums? */

            if (giftreceipt == 1)
            { gifttype = eval(5, 5862 + 7 * gift);     /* J306, J312, J318: type of gift */

              if (gifttype != 1 && gifttype != 3 && gifttype != 4)
              giftreceipt = 5;                         /* not insurance, inheritance, or gift */
              }

            if (giftreceipt == 1)
            { year = eval(5, 5864 + 7 * gift);         /* J308, J314, J320: year of gift */

              if (year < 1900 || year > 2001)
              year = -1;

              amount = feval(5, 5865 + 7 * gift);      /* J309, J315, J321: amount of gift */

              if (amount > 9999990)
              amount = -1;
              }
            else
            { year   = -1;
              amount =  0;
              }

            giftyear[gift]   = year;
            giftamount[gift] = amount;
            }

          ssreceipt = eval(5, 5185);  /* J46: who received social security? */

          if (((finresp == 1 || finresp == 2) && (ssreceipt == 1 || ssreceipt == 3))
           || ((finresp == 5 || finresp == 6) && (ssreceipt == 2 || ssreceipt == 3)))
          ssreceipt = 1;
          else ssreceipt = 0;

          srret = eval(5, 3859);      /* G134: self reported retirement status */

          retplans1 = evals(5, "G3848M1");  /* G133: plan to stop work altogether */
          retplans2 = evals(5, "G3848M2");
          retplans3 = evals(5, "G3848M3");
          retplans4 = evals(5, "G3848M4");

          if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1 || retplans4 == 1)
          { expretage = eval(5, 3849);  /* G133a: expected age of retirement */

            if (expretage < 25 || expretage > 96)
            expretage = -1;
            else if (expretage == 96)
            { expretyear = eval(5, 3850);  /* G133a1: expected year of retirement */

              if (2000 <= expretyear && expretyear <= 2050)
              expretage = min(95, expretyear - birthyear);
              else if (expretyear == 9995)
              expretage = 96;
              else expretage = -1;
              }
            else if (expretage == 95)
            expretage = 96;
            }
          else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2 || retplans4 == 2)
          expretage = 96;
          else expretage = -1;
          }
        else interview = 0;

        break;

      case 5:      /* 2002 variables */

        famresp = evals(6, "HFAMR");         /* 2002 interview */
        finresp = evals(6, "HFINR");

        if (famresp > 0 || finresp > 0)
        { interview = 1;

          intvmonth = evals(6, "HA500");     /* month of interview */
          intvyear = evals(6, "HA501");      /* year of interview */

          prevmarstat = evals(6, "HZ066_R"); /* previous marital status */
          samespouse = evals(6, "HA020");    /* CS4: same spouse? */
          married = evals(6, "HA026");       /* CS7: currently married? */

          if (prevmarstat == 1 || prevmarstat == 3)   /* previously married/partnered */
          { if ((prevmarstat == 1 && samespouse == 1) || samespouse == 3)
            marstat = 1;
            else if (samespouse == 4)        /* previous spouse became partner */
            marstat = 0;
            else                             /* previous spouse/partner not longer spouse/partner */
            { if (married == 1)
              marstat = 1;
              else marstat = 0;
              }
            }
          else                               /* no spouse/partner in previous wave */
          { if (married == 1)
            marstat = 1;
            else marstat = 0;
            }

          if (marstat == 1)
          { subhh = evals(6, "HSUBHH");
            
            if (subhh == 0 && samespouse != 5)
            samespouse = 1;
            else samespouse = 0;
            }
          else
          { marstat = 0;
            samespouse = 0;
            }

          work  = evals(6, "HJ020");         /* G2: any current work? */
          lastsempl = evals(6, "HZ136");     /* Preload: prev wave self-employed? */

          if (work == 1)
          { sempl = evals(6, "HJ021");       /* G3: employee or self-employed? */

            if (sempl == 2)                  /* self-employed */
            sempl = 1;
            else sempl = 0;

            if (lastsempl == 1)              /* working for someone else prev wave */
            { if (sempl == 0)                /* working for someone else in 2002 */
              { samejob = evals(6, "HJ045"); /* G19b: still working for previous employer? */

                if (samejob == 1 || samejob >= 8)
                samejob = 1;
                else samejob = 0;
                }
              else samejob = 0;
              }
            else if (lastsempl == 2)         /* self-employed prev wave */
            { if (sempl == 0)                /* working for someone else in 2002 */
              samejob = 0;
              else samejob = 1;
              }
            else samejob = 0;                /* not employed in 2002 */

            if (sempl == 0 && lastsempl == 1 && samejob == 1)
            { sameocc = evals(6, "HJ058");   /* G21: same occupation? */

              if (sameocc == 1)
              { if (panelvars->wavevars[4].hours > 0)
                occ = panelvars->wavevars[4].occ;
                else if (panelvars->wavevars[3].hours > 0)
                occ = panelvars->wavevars[3].occ;
                else if (panelvars->wavevars[2].hours > 0)
                occ = panelvars->wavevars[2].occ;
                else if (panelvars->wavevars[1].hours > 0)
                occ = panelvars->wavevars[1].occ;
                else occ = panelvars->wavevars[0].occ;
                }
              else if (sameocc == 7)
              occ = evals(6, "HJ062M");      /* G24/G25: current occupation */
              else occ = evals(6, "HJ168M"); /* G41b/G42: current occupation */

              if (panelvars->wavevars[4].hours > 0)
              ind = panelvars->wavevars[4].ind;
              else if (panelvars->wavevars[3].hours > 0)
              ind = panelvars->wavevars[3].ind;
              else if (panelvars->wavevars[2].hours > 0)
              ind = panelvars->wavevars[2].ind;
              else if (panelvars->wavevars[1].hours > 0)
              ind = panelvars->wavevars[1].ind;
              else ind = panelvars->wavevars[0].ind;
              }
            else
            { occ = evals(6, "HJ168M");     /* G41b/G42: current occupation */
              ind = evals(6, "HJ166M");     /* G41a: current occupation */
              }

            hours = evals(6, "HJ172");      /* G44: usual weekly hours */
            weeks = evals(6, "HJ179");      /* G47: usual weeks/year */

            if (sempl == 0)
		        { salary    = wagecalc(6, 206, 1, hours, weeks);  /* G56a */
			  	    hourly    = wagecalc(6, 216, 0, hours, weeks);  /* G56f */
				      piecework = wagecalc(6, 225, 1, hours, weeks);  /* G56m */
				      tips      = wagecalc(6, 230, 1, hours, weeks);  /* G56r */

              if (salary >= 0 && hourly >= 0 && piecework >= 0 && tips >= 0)
              earnings = salary + hourly + piecework + tips;
              else earnings = -1;
              }
		        else
		        { salary  = wagecalc(6, 188, 1, hours, weeks);     /* G52a */
              profits = wagecalc(6, 195, 1, hours, weeks);     /* G53a */

              if (salary >= 0 && profits >= 0)
              earnings = salary + profits;
              else earnings = -1;
              }

            if (earnings > 0 && 0 < hours && hours <= 168 && 0 < weeks && weeks <= 52)
            wage = earnings / (hours * weeks);
            else wage = -1;

            hourslimits = evals(6, "HJ237");    /* G59: could work half-time? */

            if (hourslimits != 1)
            hourslimits = 0;

            penbranch = evals(6, "HJ267");  /* G67 branchpoint: Pension in same job last time? */

            if (penbranch == 1 || penbranch == 2)
            pencoverage = 1;
            else
            { pencoverage = evals(6, "HJ324");  /* G75: pension */
		          futurepens = evals(6, "HJ332");   /* G75e: will become eligible */
              }

		        if (pencoverage == 1 || futurepens == 1)
		        pension = 1;
            else if (pencoverage == 5)
            pension = 0;
            else pension = -9;
            }
          else
          { samejob = 0;
            sempl = 0;
            hours = 0;
            weeks = 0;
            wage = 0;
					  occ = 0;
            ind = 0;
            hourslimits = 0;
            pension = 0;
            }

          if (lastsempl == 1 && samejob == 0)
          { reasona = evals(6, "HJ073M1");   /* G28: reasons for leaving prev wave job */
            reasonb = evals(6, "HJ073M2");
            reasonc = evals(6, "HJ073M3");
            reasond = evals(6, "HJ073M4");

            if (reasona == 1 || reasona == 2 || reasonb == 1 || reasonb == 2
             || reasonc == 1 || reasonc == 2 || reasond == 1 || reasond == 2)
            layoff = 1;
            else layoff = 0;
            }
          else layoff = 0;

          health = evals(6, "HC001");       /* B1: health status */

          if (health == 4 || health == 5)
          health = 1;
          else health = 0;

          eowcount = evals(6, "HJ615");     /* J615: Number of eow's */

          if (eowcount < 1 || eowcount > 2)
          eowcount = 0;

          if (eowcount >= 1)
          { eowamt[0]     =  evals(6, "HJ623_1");  /* J623: Cash bonus amount */
            eowpension[0] = fevals(6, "HJ627_1");  /* J627: Extra pension benefits */
            eowpensp[0]   =  evals(6, "HJ631_1");  /* J631: Time period for pension benefits */
            eowserv[0]    =  evals(6, "HJ642_1");  /* J642: Additional years of service */
            eowother[0]   =  evals(6, "HJ643_1");  /* J643: Value of other incentives */
            eowaccpt[0]   =  evals(6, "HJ644_1");  /* J644:  Accept offer? */
            }

          if (eowcount >= 2)
          { eowamt[1]     =  evals(6, "HJ623_2");  /* J623: Cash bonus amount */
            eowpension[1] = fevals(6, "HJ627_2");  /* J627: Extra pension benefits */
            eowpensp[1]   =  evals(6, "HJ631_2");  /* J631: Time period for pension benefits */
            eowserv[1]    =  evals(6, "HJ642_2");  /* J642: Additional years of service */
            eowother[1]   =  evals(6, "HJ643_2");  /* J643: Value of other incentives */
            eowaccpt[1]   =  evals(6, "HJ644_2");  /* J644:  Accept offer? */
            }

          ssdisab1 = evals(6, "HM030F1");   /* GD18aa: Still receiving SSDI benefits? */
          ssdisab2 = evals(6, "HM032");     /* GD18ca: Pending SSDI application approved? */
          ssdisab3 = evals(6, "HM032Q");    /* GD18dd: Awarded SSDI on appeal? */
          ssdisab4 = evals(6, "HM040E");    /* GD26b: Awarded SSDI benefits? */
          ssdisab5 = evals(6, "HM040I");    /* GD26e: Awarded SSDI on appeal? */

          if (ssdisab1 == 1 || ssdisab2 == 1 || ssdisab3 == 1 || ssdisab4 == 1 || ssdisab5 == 1)
          ssdisab = 1;
          else ssdisab = 0;

          for (gift = 0; gift < 3; gift++)
          { sprintf(variable, "HQ483_%1d", gift + 1);
            giftreceipt = evals(6, variable);   /* J305: any lump sums? */

            if (giftreceipt == 1)
            { sprintf(variable, "HQ484_%1d", gift + 1);
              gifttype = evals(6, variable);    /* J306: type of gift */

              if (gifttype != 1 && gifttype != 3)      /* not insurance or inheritance */
              giftreceipt = 5;
              }

            if (giftreceipt == 1)
            { sprintf(variable, "HQ487_%1d", gift + 1);
              year = evals(6, variable);        /* J308: year of gift */

              if (year < 1900 || year > 2001)
              year = -1;

              sprintf(variable, "HQ488_%1d", gift + 1);
              amount = evals(6, variable);     /* J309: amount of gift */

              if (amount > 9999990)
              amount = -1;
              }
            else
            { year   = -1;
              amount =  0;
              }

            giftyear[gift]   = year;
            giftamount[gift] = amount;
            }

          ssreceipt = evals(6, "HJ478");   /* J478: currently receiving social security? */

          if (ssreceipt != 1)
          ssreceipt = 0;

          srret = evals(6, "HJ578");      /* G134: self reported retirement status */

          retplans1 = evals(6, "HJ566M1");  /* G133: plan to stop work altogether */
          retplans2 = evals(6, "HJ566M2");
          retplans3 = evals(6, "HJ566M3");
          retplans4 = evals(6, "HJ566M4");

          if (retplans1 == 1 || retplans2 == 1 || retplans3 == 1 || retplans4 == 1)
          { expretage = evals(6, "HJ568");  /* G133a: expected age of retirement */

            if (expretage < 25 || expretage > 96)
            expretage = -1;
            else if (expretage == 96)
            { expretyear = evals(6, "HJ569");  /* G133a1: expected year of retirement */

              if (2000 <= expretyear && expretyear <= 2050)
              expretage = min(95, expretyear - birthyear);
              else if (expretyear == 9995)
              expretage = 96;
              else expretage = -1;
              }
            else if (expretage == 95)
            expretage = 96;
            }
          else if (retplans1 == 2 || retplans2 == 2 || retplans3 == 2 || retplans4 == 2)
          expretage = 96;
          else expretage = -1;
          }
        else interview = 0;

        break;

      }

    if (persvars->birthmonth < intvmonth)
    age = intvyear - persvars->birthyear;
    else if (intvmonth < persvars->birthmonth && persvars->birthmonth <= 12)
    age = intvyear - persvars->birthyear - 1;
    else if (persvars->birthmonth == intvmonth)
    { if (surv <= 1)
      { if (surv == 0)
        age = eval(1, 46);                        /* age in 1992 */
        else age = eval(2, 104);                  /* age in 1994 */

        if (age < intvyear - persvars->birthyear - 1
         || age > intvyear - persvars->birthyear)
        age = intvyear - persvars->birthyear - 1;
        }
      else age = intvyear - persvars->birthyear - 1;
      }
    else age = intvyear - persvars->birthyear;

    /* Pension spike for this respondent is based on years of service, so the */
    /* retirement date must be based on year, not age to get the spike right. */
    if (surv == 1 && caseid == 11424)
    age = 1994 - birthyear;

    eowamount  = 0;
    eowpens    = 0;
    eowservice = 0;
    eowaccept  = 0;

    if (surv >= 1)
    { if (surv < 4)
      maxamt = 9999996;
      else maxamt = 99999996;
          
      if (surv < 5)
      maxamtp = 9999996;
      else maxamtp = 99999996;

      for (eownumber = 0; eownumber < eowcount; eownumber++)
      { if (eowamt[eownumber] < 0 || eowamt[eownumber] > maxamt)
        eowamt[eownumber] = 0;

        if (0 < eowpension[eownumber] && eowpension[eownumber] <= maxamtp
         && 0 < eowpensp[eownumber] && eowpensp[eownumber] <= 6)
        eowpension[eownumber] = eowpension[eownumber] * eowmult[eowpensp[eownumber]];
        else eowpension[eownumber] = 0;

        if (eowserv[eownumber] < 0 || eowserv[eownumber] > 20
         || (surv == 5 && eowserv[eownumber] > 7))
        eowserv[eownumber] = 0;

        if (0 < eowother[eownumber] && eowother[eownumber] <= maxamt)
        eowamt[eownumber] = eowamt[eownumber] + eowother[eownumber];

        if (eowaccpt[eownumber] == 1 || eowaccpt[eownumber] == 2)
        eowaccpt[eownumber] = 1;
        else eowaccpt[eownumber] = 0;
        }

      if (eowcount < 2)
      eownumber = eowcount;
      else
      { if (eowaccpt[0] == 1)
        eownumber = 1;
        else if (eowaccpt[1] == 1)
        eownumber = 2;
        else if (eowamt[0] > 0 || eowamt[1] > 0)
        { if (eowamt[0] >= eowamt[1])
          eownumber = 1;
          else eownumber = 2;
          }
        else if (eowpension[0] > 0 || eowpension[1] > 0)
        { if (eowpension[0] >= eowpension[1])
          eownumber = 1;
          else eownumber = 2;
          }
        else if (eowserv[0] > 0 || eowserv[1] > 0)
        { if (eowserv[0] >= eowserv[1])
          eownumber = 1;
          else eownumber = 2;
          }
        else eownumber = 1;
        }

      if (eownumber == 1 || eownumber == 2)
      { eowamount = eowamt[eownumber - 1];
        eowpens   = eowpension[eownumber - 1];
        eowservice = eowserv[eownumber - 1];
        eowaccept  = eowaccpt[eownumber - 1];
        }
      }

    if (interview == 0)
    { intvyear     = -1;
      intvmonth    = -1;
      age          = -1;
      marstat      = -1;
      samespouse   = -1;
      samejob      = -1;
      layoff       = -1;
      sempl        = -1;
      occ          = -1;
      ind          = -1;
      hours        = -1;
      weeks        = -1;
      wage         = -1;
      hourslimits  = -1;
      pension      = -1;
      health       = -1;
      eowamount    = -1;
      eowpens      = -1;
      eowservice   = -1;
      eowaccept    = -1;
      ssdisab      = -1;
      srret        = -1;
      ssreceipt    = -1;
      expretage    = -1;

      for (gift = 0; gift < 3; gift++)
      { giftamount[gift] = -1;
        giftyear[gift]   = -1;
        }
      }
    else if (work <= 0)
    { samejob = -1;
      sempl = -1;
      layoff = -1;
      hours = -1;
      weeks = -1;
      wage = -1;
  	  occ = -1;
      ind = -1;
      hourslimits = -1;
      pension = -1;
      srret = -1;
      expretage = -1;
      }

    if (0 < hours && 0 < weeks && hours * weeks < 100)
    wage  = -1;

    panelvars->wavevars[surv].interview    = interview;
    panelvars->wavevars[surv].intvyear     = intvyear - 1990;
    panelvars->wavevars[surv].intvmonth    = intvmonth;
    panelvars->wavevars[surv].age          = age;
    panelvars->wavevars[surv].marstat      = marstat;
    panelvars->wavevars[surv].samespouse   = samespouse;
    panelvars->wavevars[surv].samejob      = samejob;
    panelvars->wavevars[surv].layoff       = layoff;
    panelvars->wavevars[surv].sempl        = sempl;
    panelvars->wavevars[surv].occ          = occ;
    panelvars->wavevars[surv].ind          = ind;
    panelvars->wavevars[surv].hours        = hours;
    panelvars->wavevars[surv].weeks        = weeks;
    panelvars->wavevars[surv].wage         = wage;
    panelvars->wavevars[surv].hourslimits  = hourslimits;
    panelvars->wavevars[surv].pension      = pension;
    panelvars->wavevars[surv].health       = health;
    panelvars->wavevars[surv].eowamount    = eowamount;
    panelvars->wavevars[surv].eowpens      = eowpens;
    panelvars->wavevars[surv].eowservice   = eowservice;
    panelvars->wavevars[surv].eowaccept    = eowaccept;
    panelvars->wavevars[surv].ssdisab      = ssdisab;
    panelvars->wavevars[surv].srret        = srret;
    panelvars->wavevars[surv].ssreceipt    = ssreceipt;
    panelvars->wavevars[surv].expretage    = expretage;

    for (gift = 0; gift < 3; gift++)
    { panelvars->wavevars[surv].giftamount[gift] = giftamount[gift];
      panelvars->wavevars[surv].giftyear[gift]   = giftyear[gift];
      }

    if (persvars->caseid == printcaseid || printcaseid == 99999)
    { if (surv == 0)
      wnprintf("\ns  ag  mr ss  sj ly se  oc in  hr wk   wg  pn  hl ssd  sr ra  %5d %5d %3d",
       caseid, evals(2, "HHID"), evals(2, "PN"));

      if (surv < 6 && panelvars->wavevars[surv].interview == 1)
      wnprintf("\n%1d  %2d  %2d %2d  %2d %2d %2d  %2d %2d  %2d %2d%6.2f %2d  %2d %2d  %2d %2d",
       surv + 1, panelvars->wavevars[surv].age, panelvars->wavevars[surv].marstat,
       panelvars->wavevars[surv].samespouse, panelvars->wavevars[surv].samejob,
       panelvars->wavevars[surv].layoff, panelvars->wavevars[surv].sempl,
       panelvars->wavevars[surv].occ, panelvars->wavevars[surv].ind,
       panelvars->wavevars[surv].hours, panelvars->wavevars[surv].weeks,
       panelvars->wavevars[surv].wage, panelvars->wavevars[surv].pension,
       panelvars->wavevars[surv].health, panelvars->wavevars[surv].ssdisab,
       panelvars->wavevars[surv].srret, panelvars->wavevars[surv].expretage);
      }
    }

  }


/*---------------------------------------------------------------------------
			 WAGE CALC routine
			 calculates annual wages from the wage structure in the HRS
  ---------------------------------------------------------------------------*/

double wagecalc(
  int wave,                 /* wave number                          */
  int wageloc,              /* variable number of wage variable     */
  int periodflag,           /* flag if wages are by varying periods */
  int hours,                /* usual hours per week                 */
  int weeks)                /* usual weeks per year                 */

{ int    period;
  double wage;

  char   variable[8];

  static double wagemult[6][4] = { { 52, 26, 12,  4},
                                   { 52, 26, 12, 24},
                                   { 52, 26, 12, 24},
                                   { 52, 26, 12, 24},
                                   { 52, 26, 12, 24},
                                   { 52, 26, 12, 24}};

  static int rangecard[10] = {    250,    707,    1582,     5001,     22361,
				 						           111805, 500001, 3162278, 31622777, 316227767};
	  /* These are the geometric means of the intervals */

  if (wave < 3)
  wage = eval(wave, wageloc);        /* wage variable */
  else
  { if (wave < 6)
    wage = feval(wave, wageloc);
    else
    { sprintf(variable, "HJ%3d", wageloc);
      wage = fevals(wave, variable);
      }

    if (wage == -9)
    wage = 0;
    }

  /* wnprintf("\n    %d %d", wageloc, eval(wave, wageloc)); */

  if (wage == 0 || (wave == 1 && wage == 9999996
    && (wageloc == 2748 || wageloc == 2825 || wageloc == 2828)))
  return(0);                         /* no wage from this source */

  if (wage >= 9999990 || (periodflag == 0 &&
    (wave < 6 && wage >= 99994) || (wave == 6 && wage > 999990)))
  return(-1);                        /* unknown wage */

  if (wave == 1 && wage >= 9999980)  /* wage from range card */
  wage = rangecard[(int) wage - 9999980];

  if (periodflag == 1)
  { if (wave < 6)
    period = eval(wave, wageloc + 1);  /* period variable  */
    else
    { if (wageloc == 188 || wageloc == 195 || wageloc == 206 )
      sprintf(variable, "HJ%3d", wageloc + 4);
      else sprintf(variable, "HJ%3d", wageloc + 1);

      period = evals(wave, variable);
      }
    }
  else period = 1;                   /* period is hourly */

  if (period == 1)                   /* hourly wages */
  { if ((0 < hours && hours < 98) && (0 < weeks && weeks < 98))
    if (wave == 1 || (wave == 2 && periodflag == 1))
	  return((wage / 100.0) * hours * weeks);
    else return(wage * hours * weeks);
    /* Note: 1994 hourly wages coded in dollars despite documentation */
	 }
  else if (2 <= period && period <= 5)
  { if (0 < weeks && weeks < 98)
	  return((wage * wagemult[wave - 1][period - 2] * weeks) / 52);
	 }
  else if (period == 6)              /* annual wages */
  return(wage);
  else if (period == 11)             /* daily */
  { if (0 < hours && hours < 98)
    return((5 * wage) / hours);
    }

  return(-1);                        /* hours or weeks missing */

  }


/*---------------------------------------------------------------------------
			 CORRECT ERRORS routine
			 corrects for obvious errors in the raw data
  ---------------------------------------------------------------------------*/

void correctErrors(
  VARS *vars)

{ int   caseid;

  caseid = vars->persvars.caseid;

  switch(caseid)
  { case 10157: vars->jobhist.jobvars[0].earnings = 22000;
                vars->panelvars.wavevars[0].wage  = 10.5769;          break;
                /* Wages in all other years is about $10 per hour, but the
                   reported wage in 1992 is $1.06, which is obviously erroneous. */

    case 10797: vars->jobhist.jobvars[0].hours    = -9;
                vars->jobhist.jobvars[0].weeks    = -9;
                vars->jobhist.jobvars[0].earnings = -9;
                vars->panelvars.wavevars[0].hours = -1;
                vars->panelvars.wavevars[0].weeks = -1;
                vars->panelvars.wavevars[0].wage  = -1;               break;
                /* Job had 32 hours times 52 weeks at a wage of $1.56 per hour.
                   Not clear whether hours are overreported or wage is under-
                   reported. */
    }

  }


/*---------------------------------------------------------------------------
			 GET EXPERIENCE routine
			 gets experience vector
  ---------------------------------------------------------------------------*/

void getExperience(
  VARS  *vars,
  int   *tenure,
  int   *exper)

{ int   jobno, wave, intv, year, index;
  int   birthyear, age, curage;
  int   jobind, startdate, enddate, ftyears;
  int   hours, weeks;
  int   lastwave, lastjobind;
  int   expersum;
  int   matchcode, qc;
  double earnings;

  double earnvec[5], ssearnings[41];

  static double wageind[42] = {
             57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
     80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
    119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
    235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
    345.35, 353.98, 363.61 };
  /* 1951-1992 average weekly earnings           */
  /* Table B41, 1986 ERP and Table B47, 2001 ERP */

  PERSVARS *persvars;
  JOBHIST *jobhist;
  PANELVARS *panelvars;

  persvars =  &(vars->persvars);
  jobhist = &(vars->jobhist);
  panelvars = &(vars->panelvars);

  birthyear = persvars->birthyear;

  memset(exper, 0, 101);

  for (wave = 0; wave <= 4; wave++)
  { intv = panelvars->wavevars[wave].interview;

    if (intv == 1)
    { hours = panelvars->wavevars[wave].hours;
      weeks = panelvars->wavevars[wave].weeks;

      if (hours > 0 || weeks > 0)
      jobind = 1;
      else jobind = 0;

      if (wave == 0)
      { if (jobind == 1)
        { startdate = jobhist->jobvars[0].startdate;

          if (startdate > 0)
          tenure[0] = 1992 - startdate;
          else tenure[0] = -9;

          lastjobind = 1;
          }
        else
        { tenure[0] = -9;
          lastjobind = 0;
          }

        lastwave = 0;
        }
      else
      { if (jobind == 1)
        { curage = (1992 + 2 * wave) - birthyear;

          if (lastjobind == 1)
          { for (age = curage - 2 * (wave - lastwave); age <= curage; age++)
            exper[age] = 1;

            if (tenure[lastwave] >= 0)
            tenure[wave] = tenure[lastwave] + 2 * (wave - lastwave);
            else tenure[wave] = -9;
            }
          else
          { for (age = curage - (wave - lastwave); age < curage; age++)
            exper[age] = 1;

            tenure[wave] = wave - lastwave;
            }

          lastjobind = 1;
          }
        else
        { tenure[wave] = -9;
          lastjobind = 0;
          }

        lastwave = wave;
        }
      }
    else tenure[wave] = -9;
    }

  for (jobno = 0; jobno < 10; jobno++)
  { if (jobno < 9)
    { startdate = jobhist->jobvars[jobno].startdate;

      if (jobno > 0)
      enddate = jobhist->jobvars[jobno].enddate;
      else enddate = 1992;
      }
    else
    { startdate = jobhist->nonssstartdate;
      enddate   = jobhist->nonssenddate;
      }

	 if (1900 <= startdate && startdate <= enddate && enddate <= 1993)
	 for (year = startdate; year <= enddate; year++)
    if (birthyear <= year && year <= birthyear + 100)
	 exper[year - birthyear] = 1;
    }

  /* get experience prior to 92 current/last job from 96 survey */

  startdate = jobhist->ftstartdate;
  enddate   = jobhist->ftenddate;
  ftyears   = jobhist->ftyears;

  getSSrec(persvars->caseid, &matchcode, &qc, ssearnings);

  if ((startdate > 0 && enddate > 0) || ftyears >= 0 || matchcode == 1)
  { if (startdate > 0 && enddate > 0)
    { for (year = startdate; year <= enddate; year++)
      if (birthyear + 16 <= year && year <= birthyear + 100)
	   exper[year - birthyear] = 1;
	   }
    else if (ftyears >= 0)
    { expersum = 0;

	   for (age = 16; age <= enddate - birthyear; age++)
	   if (exper[age] > 0)
	   expersum = expersum + 1;

	   if (expersum < ftyears)
	   for (age = enddate - birthyear; age >= 0; age--)
	   if (expersum < ftyears && exper[age] == 0)
      { exper[age] = 1;
        expersum = expersum + 1;
	     }
	   }
    else
    { memset(earnvec, 0, 5 * sizeof(float));

      for (year = 1951; year < 1992; year++)
      { earnings = ssearnings[year - 1951]
                  * (wageind[1992 - 1951] / wageind[year - 1951]);

        if (earnings < earnvec[0])
        earnvec[0] = earnings;

        for (index = 1; index < 5; index++)
        if (earnvec[index] < earnings)
        { earnvec[index - 1] = earnvec[index];
          earnvec[index] = earnings;
          }
        }

      for (year = 1951; year < 1992; year++)
      if (year >= birthyear)
      { earnings = ssearnings[year - 1951]
         * (wageind[1992 - 1951] / wageind[year - 1951]);

        if (earnings > 0.5 * earnvec[2])   /* higher than 1/2 of median of highest 5 years */
        exper[year - birthyear] = 1;
        }
      }

    expersum = 0;

    for (age = 0; age < 16; age++)
    exper[age] = -9;

    for (age = 16; age <= 2000 - birthyear; age++)
    if (exper[age] == 1)
    { exper[age] = expersum;
      expersum = expersum + 1;
      }
    }
  else
  { for (age = 0; age <= 100; age++)
    exper[age] = -9;
    }

  }


/*---------------------------------------------------------------------------
			 GET RET STATS routine
			 calculates retirement status during survey period
  ---------------------------------------------------------------------------*/

void getRetStats(
  VARS *vars)

{ int   surv, lsurv, lastsurvey, ssdisurv;
  int   age, cage, lage;
  int   initage, goodage;
  int   healthage, healthsurv, nexthealth;
  int   srret, hret;
  int   work, everworked, hours, weeks, annhours;
  int   G3branch, sempl, lastsempl;
  int   intvmonth, intvyear;
  int   lintvmonth, lintvyear;
  int   endmonth, endyear, endage;
  int   endmonth1, endmonth2, endyear1, endyear2;
  int   startmonth, startyear, startage;
  int   interjob, interstart, interend, interstat;
  int   startageint, endageint;
  int   indicator, month, year, badmonths;
  int   var, firstvar, varcount;

  int   ret[6];
  int   lastsurv[6], nextsurv[6];

  char  varname[10], number[6];
  char  workvec[144];

  PERSVARS  *persvars;
  PANELVARS *panelvars;
  CALCVARS  *calcvars;

  persvars  = &(vars->persvars);
  panelvars = &(vars->panelvars);
  calcvars  = &(vars->calcvars);


  /* initialize retirement and job characteristic vectors */

  for (year = 0; year < 13; year++)
  calcvars->retstats[year] = 0;

  calcvars->finalretstat = -1;
  calcvars->origjobendage = 99;
  calcvars->origjobendsurv = 0;
  calcvars->newjobstartage = 99;
  calcvars->impjobendage = 99;
  calcvars->layoffage = 99;
  calcvars->ind = -1;
  calcvars->occ = -1;
  calcvars->ssdiage = 99;


  /* calculate retirement status by survey */

  for (surv = 0; surv < 6; surv++)
  { lastsurv[surv] = -1;
    nextsurv[surv] = -1;
  
    if (panelvars->wavevars[surv].interview == 1)
    { if (surv > 0)
      { lastsurv[surv] = lastsurvey;
        nextsurv[lastsurvey] = surv;
        }

      lastsurvey = surv;
    
      srret = panelvars->wavevars[surv].srret;

      hours = panelvars->wavevars[surv].hours;
      weeks = panelvars->wavevars[surv].weeks;

      if (0 <= hours && hours < 95 && 0 <= weeks && weeks <= 52)
      { annhours = hours * weeks;

        if (annhours == 0)
        hret = 1;                         /* completely retired */
        else if (100 < annhours && (hours <= 25 || annhours <= 1250))
        hret = 3;                         /* partially retired */
        else if (hours > 30 && annhours >= 1560)
        hret = 5;                         /* not retired at all */
        else if (annhours <= 1000)
        hret = 2;                         /* between completely and partially retired */
        else hret = 4;                    /* between partially retired and working */
        }
      else hret = -1;

      if (hret == 1)                      ret[surv] =  1;
      else if (hret == 2)
      { if (srret == 3 || srret == 5)     ret[surv] =  3;
        else                              ret[surv] =  1;
        }
      else if (hret == 3)                 ret[surv] =  3;
      else if (hret == 4)
      { if (srret == 5)                   ret[surv] =  5;
        else                              ret[surv] =  3;
        }
      else if (hret == 5)                 ret[surv] =  5;
      else                                ret[surv] = -1;
      }
    else ret[surv] = -1;

    panelvars->wavevars[surv].ret = ret[surv];
    }


  /* find the last survey before disability */

  ssdisurv = 6;

  for(surv = 5; surv >= 0; surv--)
  { if (panelvars->wavevars[surv].ssdisab == 1)
    ssdisurv = surv;
    }


  /* assign age for initial health problem */

  lastsurvey = 6;
  nexthealth = 0;
  healthage = 99;
  healthsurv = 6;

  for (surv = 5; surv >= 0; surv--)
  { if (panelvars->wavevars[surv].interview == 1)
    { if (panelvars->wavevars[surv].health == 1)
      { if (lastsurvey == 5 || nexthealth == 1)
        { healthage = panelvars->wavevars[surv].age;
          healthsurv = surv;
          }

        nexthealth = 1;
        }
      else nexthealth = 0;

      lastsurvey = surv;
      }
    }

  calcvars->healthage = (char) healthage;


  /* find job characteristics for 1992 job if full-time and not disabled */

  if (ret[0] == 5 && ssdisurv > 0)
  { if (0 < panelvars->wavevars[0].ind && panelvars->wavevars[0].ind <= 13)
    calcvars->ind = panelvars->wavevars[0].ind;

    if (0 < panelvars->wavevars[0].occ && panelvars->wavevars[0].occ <= 17)
    calcvars->occ = panelvars->wavevars[0].occ;
    }


  /* fill in missing status from adjacent surveys */

  for (surv = 0; surv < ssdisurv; surv++)
  { if (ret[surv] == -1)
    { if (lastsurv[surv] >= 0)
      { if (nextsurv[surv] >= 0)
        { if (panelvars->wavevars[surv].samejob == 1)
          { if (panelvars->wavevars[nextsurv[surv]].samejob == 1)
            { if (ret[lastsurv[surv]] == ret[nextsurv[surv]])
              ret[surv] = ret[lastsurv[surv]];
              }
            else ret[surv] = ret[lastsurv[surv]];
            }
          else
          { if (panelvars->wavevars[nextsurv[surv]].samejob == 1)
            ret[surv] = ret[nextsurv[surv]];
            }
          }
        else
        { if (panelvars->wavevars[surv].samejob == 1)      
          ret[surv] = ret[lastsurv[surv]];
          }
        }
      else
      { if (nextsurv[surv] > 0) 
        { if (panelvars->wavevars[nextsurv[surv]].samejob == 1)
          ret[surv] = ret[nextsurv[surv]];
          }
        }  
      } 
    }


  /* determine date status at first wave started */

  work = eval(1, 2717);                         /* F2: any current work? */
  sempl = eval(1, 2718);                        /* F3: self employed */

  if (work == 1)
  { if (sempl == 1)                             /* working for someone else */
    startyear = eval(1, 2816);                  /* F23: date started working for current employer */
    else startyear = eval(1, 2834);             /* F33: date started working at current business */

    startmonth = 12;

    if (startyear < 1910 || startyear > 1993)
    { startyear = -1;
      startmonth = -1;
      }
    }
  else
  { everworked = eval(1, 3401);                 /* G1: ever worked for pay? */
    
    if (everworked == 1)
    { startyear = eval(1, 3403);                /* G1a: date last worked */
      
      if (1991 <= startyear && startyear <= 1993)
      { startmonth = eval(1, 3402);

        if (startmonth < 1 || startmonth > 12)
        startmonth = 12;
        }
      else if (1910 < startyear && startyear <= 1990)
      startmonth = 12;
      else
      { startyear = -1;
        startmonth = -1;
        }
      }
    else
    { startyear = 1990;
      startmonth = 12;
      }
    }


  /* determine age at 1992 interview */

  intvmonth = panelvars->wavevars[0].intvmonth;
  intvyear  = panelvars->wavevars[0].intvyear + 1990;

  if (persvars->birthmonth < intvmonth)
  initage = intvyear - persvars->birthyear;
  else if (intvmonth < persvars->birthmonth && persvars->birthmonth <= 12)
  initage = intvyear - persvars->birthyear - 1;
  else if (persvars->birthmonth == intvmonth)
  { initage = eval(1, 46);                    /* age in 1992 */

    if (initage < intvyear - persvars->birthyear - 1
      || initage > intvyear - persvars->birthyear)
    initage = intvyear - persvars->birthyear;
    }
  else initage = intvyear - persvars->birthyear - 1;

  calcvars->initage = initage;


  /* examine whether first wave status can be projected back one year */

  if (ssdisurv > 0)
  if (startyear < persvars->birthyear + initage - 1
   || (startyear = persvars->birthyear + initage - 1
    && startmonth < persvars->birthmonth))
  calcvars->retstats[0] = ret[0];

  if (ssdisurv > 0)
  if (ret[0] > 0)
  calcvars->finalretstat = ret[0];

  if (ssdisurv == 0)
  calcvars->ssdiage = initage - 1;
  else if (nextsurv[0] == ssdisurv)
  calcvars->ssdiage = initage;
    

  /* examine period between successive waves */

  /* wnprintf("\n ssdisurv %d  initage %d", ssdisurv, initage); */
  for (surv = 1; surv < ssdisurv; surv++)
  { if (panelvars->wavevars[surv].interview == 1)
    { lsurv = lastsurv[surv];

      intvmonth = panelvars->wavevars[surv].intvmonth;
      intvyear  = panelvars->wavevars[surv].intvyear + 1990;
      lintvmonth = panelvars->wavevars[lsurv].intvmonth;
      lintvyear  = panelvars->wavevars[lsurv].intvyear + 1990;


      /* determine age at last interview */

      if (persvars->birthmonth < lintvmonth)
      lage = lintvyear - persvars->birthyear;
      else if (lintvmonth < persvars->birthmonth && persvars->birthmonth <= 12)
      lage = lintvyear - persvars->birthyear - 1;
      else if (persvars->birthmonth == lintvmonth)
      { if (lsurv <= 1)
        { if (lsurv == 0)
          lage = eval(1, 46);                   /* age in 1992 */
          else lage = eval(2, 104);             /* age in 1994 */

          if (lage < lintvyear - persvars->birthyear - 1
           || lage > lintvyear - persvars->birthyear)
          lage = lintvyear - persvars->birthyear;
          }
        else lage = lintvyear - persvars->birthyear;
        }
      else lage = lintvyear - persvars->birthyear;


      /* determine age at current interview */

      if (persvars->birthmonth < intvmonth)
      cage = intvyear - persvars->birthyear;
      else if (intvmonth < persvars->birthmonth && persvars->birthmonth <= 12)
      cage = intvyear - persvars->birthyear - 1;
      else if (persvars->birthmonth == intvmonth)
      { if (surv == 1)
        { cage = eval(2, 104);                  /* age in 1994 */

          if (cage < intvyear - persvars->birthyear - 1
           || cage > intvyear - persvars->birthyear)
          cage = intvyear - persvars->birthyear;
          }
        else cage = intvyear - persvars->birthyear;
        }
      else cage = intvyear - persvars->birthyear - 1;


      /* get date and age of separation from previous wave job */

      endmonth = 99;
      endyear = -1;
      endage = -1;

      if (ret[lsurv] == 3 || ret[lsurv] == 5)
      { if (surv == 1)
        { work = eval(2, 3316);                 /* FA2: any current work? */
          sempl = eval(2, 3317);                /* FA3: self employed */
          lastsempl = eval(1, 2718);            /* F3: self employed */
          
          if (lastsempl == 1)                   /* working for someone else last survey */
          { if (work == 1)                      /* currently working */
            { if (sempl == 1)                   /* working for someone else */
              { endmonth = eval(2, 3503);       /* FA26: date stopped working for previous employer */
                endyear = eval(2, 3504);
                }
              else                              /* currently self employed */
              { endmonth = eval(2, 4200);       /* FB2: date stopped working for previous employer */
                endyear = eval(2, 4201);
                }
              }
            else                                /* not currently working */
            { endmonth = eval(2, 4897);         /* FC16: date stopped working for previous employer */
              endyear = eval(2, 4898);
              }
            }
          else if (lastsempl == 2)              /* self employed last survey */
          { if (sempl == 1)                     /* working for someone else */
            { endmonth = eval(2, 3318);         /* FA5: date stopped working self employed job */
              endyear = eval(2, 3319);
              }
            else if (work != 1)                 /* not currently working */
            { endmonth = eval(2, 4800);         /* FC2: date stopped working self employed job */
              endyear = eval(2, 4801);
              }
            }
          }
        else
        { switch(surv)
          { case  2:
               G3branch = eval(3, 2629);        /* G3 branchpoint */
               endmonth1 = eval(3, 2630);       /* G5: date stopped working self employed job */
               endyear1 = eval(3, 2631);
               endmonth2 = eval(3, 2667);       /* G26: date stopped working for previous employer */
               endyear2 = eval(3, 2668);
               break;

            case  3:
               G3branch = eval(4, 3133);        /* G3 branchpoint */
               endmonth1 = eval(4, 3134);       /* G5: date stopped working self employed job */
               endyear1 = eval(4, 3135);
               endmonth2 = eval(4, 3188);       /* G26: date stopped working for previous employer */
               endyear2 = eval(4, 3189);
               break;

            case  4:
               G3branch = eval(5, 3383);        /* G3 branchpoint */
               endmonth1 = eval(5, 3384);       /* G5: date stopped working self employed job */
               endyear1 = eval(5, 3385);
               endmonth2 = eval(5, 3437);       /* G26: date stopped working for previous employer */
               endyear2 = eval(5, 3438);
               break;

            case  5:
               G3branch = evals(6, "HJ022");    /* J022 branchpoint */
               endmonth1 = evals(6, "HJ023");   /* J023: date stopped working self employed job */
               endyear1 = evals(6, "HJ024");
               endmonth2 = evals(6, "HJ063");   /* J063: date stopped working for previous employer */
               endyear2 = evals(6, "HJ064");
               break;
            }
          
          if (G3branch == 1 || G3branch == 2)   /* working for someone else last survey */
          { endmonth = endmonth2;               /* G63/J063: date stopped working for previous employer */
            endyear = endyear2;
            }
          else if (G3branch == 3)               /* self employed last survey */
          { endmonth = endmonth1;               /* G5/J023: date stopped working self employed job */
            endyear = endyear1;
            }
          }

        if (endmonth < 1 || endmonth > 12)
        endmonth = 99;

        if (endyear < 1990 || endyear > 2003)
        endyear = -1;

        if (endyear > 0)
        { if (endmonth < 99 && persvars->birthmonth < 99)
          { if (persvars->birthmonth <= endmonth)
            endage = endyear - persvars->birthyear;
            else endage = endyear - persvars->birthyear - 1;
            }
          else endage = endyear - persvars->birthyear - 1;
          }
        }
      /* if (surv == 4) wnprintf("   %d %d %d", endmonth, endyear, endage); */


      /* get date and age of hire in current wave job */

      startmonth = 99;
      startyear = -1;
      startage = -1;

      if (ret[surv] == 3 || ret[surv] == 5)
      { if (surv == 1)
        { work = eval(2, 3316);                 /* FA2: any current work? */
          sempl = eval(2, 3317);                /* FA3: self employed */

          { if (work == 1)                      /* currently working */
            { if (sempl == 1)                   /* working for someone else */
              { startmonth = eval(2, 3662);     /* FA59: date started job */
                startyear = eval(2, 3663);
                }
              else                              /* currently self employed */
              { startmonth = eval(2, 4327);     /* FB22: date started job */
                startyear = eval(2, 4328);
                }
              }
            }
          }
        else
        { switch(surv)
          { case  2:
               work = eval(3, 2627);            /* G2: any current work? */
               startmonth = eval(3, 2825);      /* G63: date started job */
               startyear = eval(3, 2826);
               break;

            case  3:
               work = eval(4, 3131);            /* G2: any current work? */
               startmonth = eval(4, 3348);      /* G63: date started job */
               startyear = eval(4, 3349);
               break;

            case  4:
               work = eval(5, 3381);            /* G2: any current work? */
               startmonth = eval(5, 3607);      /* G63: date started job */
               startyear = eval(5, 3608);
               break;

            case  5:
               work = evals(6, "HJ020");        /* J020: any current work? */
               startmonth = evals(6, "HJ248");  /* J248/J249: date started job */
               startyear = evals(6, "HJ249");
               break;
            }

          if (work != 1)
          { startmonth = 99;
            startyear = -1;
            }
          }

        if (startmonth < 1 || startmonth > 12)
        startmonth = 99;

        if (startyear < 1990 || startyear > 2003)
        startyear = -1;

        if (startyear > 0)
        { if (startmonth < 99 && persvars->birthmonth < 99)
          { if (startmonth < persvars->birthmonth)
            startage = startyear - persvars->birthyear - 1;
            else startage = startyear - persvars->birthyear;
            }
          else startage = startyear - persvars->birthyear;
          }
        }


      /* get dates and ages of hiring and separation from intermediate jobs */

      startageint = -1;
      endageint = -1;
      interstat = -1;

      memset(workvec, 0, 144);

      switch(surv)
      { case  1:
           work = eval(2, 3316);              /* FA2: any current work? */
           sempl = eval(2, 3317);             /* FA3: self employed */

           if (work == 1)                     /* currently working */
           { if (sempl == 1)                  /* working for someone else */
             { interjob = eval(2, 3999);      /* FA129: any interim jobs? */
               firstvar = 4001;
               varcount = 23;
               hours = eval(2, 4043);         /* FA132: usual hours */
               }
             else                             /* currently self employed */
             { interjob = eval(2, 4654);      /* FB89: any interim jobs? */
               firstvar = 4656;
               varcount = 33;
               hours = eval(2, 4698);         /* FB92: usual hours */
               }
             }
           else                               /* currently not working */
           { interjob = eval(2, 5038);        /* FC47: any interim jobs? */
             firstvar = 5040;
             varcount = 23;
             hours = eval(2, 5082);           /* FC50: usual hours */
             }
           break;

        case  2:
           interjob = eval(3, 3057);          /* G141: any interim jobs? */
           strcpy(varname, "E3058M");         /* G142: months worked */
           varcount = 32;
           hours = eval(3, 3059);             /* G143: hours worked */
           break;

        case  3:
           interjob = eval(4, 3589);          /* G141: any interim jobs? */
           strcpy(varname, "F3590M");         /* G142: months worked */
           varcount = 31;
           hours = eval(4, 3591);             /* G143: hours worked */
           break;

        case  4:
           interjob = eval(5, 3877);          /* G141: any interim jobs? */
           strcpy(varname, "G3878M");         /* G142: months worked */
           varcount = 27;
           hours = eval(5, 3879);             /* G143: hours worked */
           break;

        case  5:
           interjob = evals(6, "HJ596");      /* J596: any interim jobs? */
           strcpy(varname, "HJ597M");         /* J597: months worked */
           varcount = 27;
           hours = evals(6, "HJ599");         /* J599: hours worked */
           break;
        }

      if (interjob == 1)
      { badmonths = 0;
        
        for (var = 0; var < varcount; var++)
        { if (surv == 1)
          indicator = eval(2, firstvar + var);
          else
          { if (surv == 5 && var < 9)
            strcat(varname, "0");
          
            sprintf(number, "%d", var + 1);
            strcat(varname, number);

            indicator = evals(surv + 1, varname);
            }

          if (surv <= 3)
          { if (1 <= indicator && indicator <= 36)
            workvec[24 * (surv - 1) + indicator - 1] = 1;
            else if (37 <= indicator && indicator <= 39)
            { for (month = 0; month < 12; month++)
              { if ((indicator == 37 && (lintvmonth <= month + 1
                                      || lintvyear < 1992 + 2 * lsurv)) 
                 ||  indicator == 38
                 || (indicator == 39 && (month + 1 <= intvmonth
                                      || intvyear > 1992 + 2 * surv)))
                workvec[24 * (surv - 1) + 12 * (indicator - 37) + month] = 1;
                }
              }
            else if (40 <= indicator && indicator <= 99)
            badmonths = 1;
            }
          else if (surv == 4)
          { if (1 <= indicator && indicator <= 39)
            workvec[72 + indicator - 1] = 1;
            else if (40 <= indicator && indicator <= 55)
            { if (indicator < 44)
              indicator = indicator + 16;

              year = 1992 + (indicator - 43) / 2;
             
              for (month = 0; month < 12; month++)
              { if ((year < 2000 && ((lintvmonth <= month + 1
                                   && lintvyear == year) 
                                   || lintvyear < year)) 
                 || (year == 2000 && (month + 1 <= intvmonth
                                      || intvyear > 2000)))
                workvec[12 * (year - 1992) + month] = 1;
                }
              }
            else if (97 <= indicator && indicator <= 99)
            badmonths = 1;
            }
          else
          { if (1 <= indicator && indicator <= 12)
            workvec[96 + indicator - 1] = 1;
            else if (14 <= indicator && indicator <= 25)
            workvec[96 + indicator - 2] = 1;
            else if (27 <= indicator && indicator <= 38)
            workvec[96 + indicator - 3] = 1;
            else if (60 <= indicator && indicator <= 62)
            workvec[72 + indicator] = 1;
            else if (1 <= indicator && indicator <= 56)
            { if (indicator <= 40)
              year = 1999 + indicator / 13;
              else year = 2000 - (indicator - 40) / 2;

              for (month = 0; month < 12; month++)
              { if ((year < 2002 && ((lintvmonth <= month + 1
                                   && lintvyear == year) 
                                   || lintvyear < year)) 
                 || (year == 2002 && (month + 1 <= intvmonth
                                      || intvyear > 2000)))
                workvec[12 * (year - 1992) + month] = 1;
                }
              }
            else if (97 <= indicator && indicator <= 99)
            badmonths = 1;
            }
          }

        if (badmonths == 0)
        { interstart = -1;
          interend = -1;

          for (month = 0; month < 144; month++)
          { if (workvec[month] == 1)
            { if (interstart == -1)
              interstart = month;

              interend = month;
              }
            }

          month = interstart % 12;
          year = 1992 + interstart / 12;

          if (persvars->birthmonth < 99)
          { if (persvars->birthmonth <= month)
            startageint = year - persvars->birthyear;
            else startageint = year - persvars->birthyear - 1;
            }
          else startageint = year - persvars->birthyear;

          month = interend % 12;
          year = 1992 + interend / 12;

          if (persvars->birthmonth < 99)
          { if (persvars->birthmonth < month)
            endageint = year - persvars->birthyear;
            else endageint = year - persvars->birthyear - 1;
            }
          else endageint = year - persvars->birthyear - 1;
          }
          
        if (0 < hours && hours <= 30)
        interstat = 3;
        else if (30 < hours && (hours <= 95 || (surv > 1 && hours <= 96)
                                            || (surv > 2 && hours <= 168)))
        interstat = 5;
        }


      /* check for inconsistent dates */

      if (0 < startage && startage < endage)
      { endage = -1;
        startage = -1;
        }

      if (0 < startage && startage < lage)
      { goodage = 1;

        for (age = initage; age < lage; age++)
        { if (startage <= age)
          if (initage - 1 <= age && age <= initage + 11)
          if (calcvars->retstats[age - (initage - 1)] > 0)
          goodage = 0;
          }
      
        if (goodage == 1)
        lage = startage;
        else startage = -1;
        }

      if (0 < endage && endage < lage)
      endage = -1;
      
      if (cage < startage || cage < endage)
      startage = -1;

      if (0 < startage && (cage < endageint || startage < endageint))
      { startage = -1;
        endageint = -1;
        }

      if (0 < startageint && startageint < endage)
      { startageint = -1;
        endage = -1;
        }

      if (0 < startageint && startageint < lage)
      { goodage = 1;

        for (age = initage; age < lage; age++)
        { if (startageint <= age)
          if (initage - 1 <= age && age <= initage + 11)
          if (calcvars->retstats[age - (initage - 1)] > 0)
          goodage = 0;
          }
      
        if (goodage == 1)
        lage = startageint;
        else startage = -1;
        }


      /* assign retirement statistics to interval */

      if (ret[lsurv] == ret[surv])
      { for (age = lage; age < cage; age++)
        if (initage - 1 <= age && age <= initage + 11)
        calcvars->retstats[age - (initage - 1)] = ret[lsurv];
        }
      else
      { if (ret[surv] == 3 || ret[surv] == 5)
        { if (panelvars->wavevars[surv].samejob == 1)
          { if (initage - 1 <= lage && lage <= initage + 11 && ret[lsurv] > 0)
            calcvars->retstats[lage - (initage - 1)] = ret[lsurv];
            }
          else
          { if (ret[lsurv] == 3 || ret[lsurv] == 5)
            { if (endage > 0)
              for (age = lage; age < endage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = ret[lsurv];
                }

              if (startage > 0)
              for (age = startage; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = ret[surv];
                }

              if (interjob == 1)
              { if (endage > 0 && startageint > 0)
                for (age = endage; age < startageint; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }

                if (startageint > 0 && endageint > 0 && interstat > 0)
                for (age = startageint; age < endageint; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = interstat;
                  }

                if (endageint > 0 && startage > 0)
                for (age = endageint; age < startage; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }
                }
              else
              { if (startage > 0 && endage > 0)
                for (age = endage; age < startage; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }
                }
              }
            else
            { if (startage > 0)
              for (age = startage; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = ret[surv];
                }

              if (interjob == 1)
              { if (startageint > 0 && ret[lsurv] == 1)
                for (age = lage; age < startageint; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }

                if (startageint > 0 && endageint > 0 && interstat > 0)
                for (age = startageint; age < endageint; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = interstat;
                  }

                if (endageint > 0 && startage > 0)
                for (age = endageint; age < startage; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }
                }
              else
              { if (startage > 0 && ret[lsurv] == 1)
                for (age = lage; age < startage; age++)
                { if (initage - 1 <= age && age <= initage + 11)
                  calcvars->retstats[age - (initage - 1)] = 1;
                  }
                }
              }
            }
          }
        else
        { if (ret[lsurv] == 3 || ret[lsurv] == 5)
          { if (endage > 0)
            for (age = lage; age < endage; age++)
            { if (initage - 1 <= age && age <= initage + 11)
              calcvars->retstats[age - (initage - 1)] = ret[lsurv];
              }

            if (interjob == 1)
            { if (endage > 0 && startageint > 0)
              for (age = endage; age < startageint; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }

              if (startageint > 0 && endageint > 0 && interstat > 0)
              for (age = startageint; age < endageint; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = interstat;
                }

              if (endageint > 0 && ret[surv] == 1)
              for (age = endageint; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }
              }
            else
            { if (endage > 0 && ret[surv] == 1)
              for (age = endage; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }
              }
            }
          else
          { if (interjob == 1)
            { if (startageint > 0 && ret[lsurv] == 1)
              for (age = lage; age < startageint; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }

              if (startageint > 0 && endageint > 0 && interstat > 0)
              for (age = startageint; age < endageint; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = interstat;
                }

              if (endageint > 0 && ret[surv] == 1)
              for (age = endageint; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }
              }
            else
            { if (ret[lsurv] == 1 && ret[surv] == 1)
              for (age = lage; age < cage; age++)
              { if (initage - 1 <= age && age <= initage + 11)
                calcvars->retstats[age - (initage - 1)] = 1;
                }
              }
            }
          }
        }


      /* assign age to first layoff */

      if (ret[lsurv] == 5 && panelvars->wavevars[surv].layoff == 1
        && calcvars->layoffage == 99)
      { if (endage > 0)
        calcvars->layoffage = endage;
        else calcvars->layoffage = lage;
        } 


      /* assign age to end of first full-time job */

      if (calcvars->origjobendage == 99 && ret[0] == 5)
      { if (endage > 0 || startage > 0 || interjob == 1)
        { if (endage > 0)
          calcvars->origjobendage = endage;
          else if (calcvars->layoffage < 99)
          calcvars->origjobendage = calcvars->layoffage;
          else calcvars->origjobendage = lage;

          calcvars->origjobendsurv = surv;
          }
        else if (calcvars->layoffage < 99)
        calcvars->origjobendage = calcvars->layoffage;
        }


      /* assign age to start of last subsequent full-time job */

      if (calcvars->origjobendage < 99 || ret[0] != 5)
      { if (startageint > 0 && interstat == 5)
        calcvars->newjobstartage = startageint;

        if (ret[surv] == 5)
        { if (startage > 0)
          calcvars->newjobstartage = startage;
          else if (calcvars->newjobstartage == 99 || panelvars->wavevars[surv].samejob != 1)
          calcvars->newjobstartage = cage;
          }
        }


      /* assign job characteristics */

      if (ret[surv] == 5)
      { if (calcvars->origjobendage == 99 && ret[0] == 5)
        { if (0 < panelvars->wavevars[surv].ind && panelvars->wavevars[surv].ind <= 13)
          calcvars->ind = panelvars->wavevars[surv].ind;

          if (0 < panelvars->wavevars[surv].occ && panelvars->wavevars[surv].occ <= 17)
          calcvars->occ = panelvars->wavevars[surv].occ;
          }
        }


      /* assign final retirement status */

      if (ret[surv] > 0)
      calcvars->finalretstat = ret[surv];


      /* find ssdi age, if any */

      if (nextsurv[surv] == ssdisurv)
      calcvars->ssdiage = cage;

      if (persvars->caseid == printcaseid || printcaseid == 99999)
      { if (interjob == 1)
        wnprintf("\n %2d %d  %2d %2d %2d %2d %2d %2d   %2d %2d", lsurv, surv, lage, endage,
          startageint, endageint, startage, cage, interjob, panelvars->wavevars[surv].samejob);
        else wnprintf("\n %2d %d  %2d %2d       %2d %2d   %2d %2d", lsurv, surv, lage, endage,
          startage, cage, interjob, panelvars->wavevars[surv].samejob);
        wnprintf("   ");
        for (age = 50; age < 70; age++)
        { if (age == 55 || age == 60 || age == 65) wnprintf(".");
          if (initage - 1 <= age && age <= initage + 11)
          wnprintf("%2d", calcvars->retstats[age - (initage - 1)]);
          else wnprintf(" 0");
          }
        }

      /* if (surv == 1) wnprintf("   %2d %2d %2d %2d %4d %2d %2d",
       eval(2, 3300), eval(2, 3316), eval(2, 3317), eval(2, 4897), eval(2, 4898), work, lastsempl);
      if (surv == 2) wnprintf("   %2d %2d %2d %4d %2d %4d %2d %4d %2d %2d  %2d %2d",
       eval(3, 2627), eval(3, 2629), eval(3, 2630), eval(3, 2630), eval(3, 2667), eval(3, 2668),
       eval(3, 2825), eval(3, 2826), eval(3, 3039), eval(3, 3056), eval(3, 2736), eval(3, 2746));
      if (surv == 3) wnprintf("   %2d %2d %2d %4d %2d %4d %2d",
       eval(4, 3131), eval(4, 3133), eval(4, 3188), eval(4, 3189), eval(4, 3348), eval(4, 3349), eval(4, 3588)); 
      if (surv == 4) wnprintf("   %2d %2d %2d %4d",
       eval(5, 3381), eval(5, 3383), eval(5, 3437), eval(5, 3438)); */
      }
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\n orig %2d %2d", calcvars->origjobendage, calcvars->newjobstartage);
    wnprintf("  %2d  ", initage);

    for (surv = 0; surv < 6; surv++)
    wnprintf(" %2d", ret[surv]);

    wnprintf("  ");
    for (age = initage - 1; age <= initage + 11; age++)
    wnprintf(" %2d", calcvars->retstats[age - (initage - 1)]);
    }

  /* wnprintf("\n %4d %2d", persvars->birthyear, persvars->birthmonth);

  if (panelvars->wavevars[0].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[0].intvyear, panelvars->wavevars[0].intvmonth);
  else wnprintf("        ");

  if (panelvars->wavevars[1].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[1].intvyear, panelvars->wavevars[1].intvmonth);
  else wnprintf("        ");

  if (panelvars->wavevars[2].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[2].intvyear, panelvars->wavevars[2].intvmonth);
  else wnprintf("        ");

  if (panelvars->wavevars[3].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[3].intvyear, panelvars->wavevars[3].intvmonth);
  else wnprintf("        ");

  if (panelvars->wavevars[4].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[4].intvyear, panelvars->wavevars[4].intvmonth);
  else wnprintf("        ");

  if (panelvars->wavevars[5].interview == 1)
  wnprintf("   %2d %2d", panelvars->wavevars[5].intvyear, panelvars->wavevars[5].intvmonth);
  else wnprintf("        ");

  wnprintf("   %2d %2d %2d %2d %2d %2d", ret[0], ret[1], ret[2], ret[3], ret[4], ret[5]);

  if (persvars->gender == 1 && persvars->married == 1)
  for (age = 50; age < 70; age++)
  if (initage - 1 <= age && age <= initage + 11)
  if (calcvars->retstats[age - (initage - 1)] == 1)
  counts[age - 50][0] = counts[age - 50][0] + 1;
  else if (calcvars->retstats[age - (initage - 1)] == 3)
  counts[age - 50][1] = counts[age - 50][1] + 1;
  else if (calcvars->retstats[age - (initage - 1)] == 5)
  counts[age - 50][2] = counts[age - 50][2] + 1;
  
  for (hret = 0; hret < 3; hret++)
  { wnprintf("\n  ");
    for (age = 50; age < 70; age++)
    { if (counts[age - 50][0] + counts[age - 50][1] + counts[age - 50][2] > 0)
      wnprintf("%5.1f", 100 * counts[age - 50][hret] / (counts[age - 50][0] + counts[age - 50][1] + counts[age - 50][2]));
      else wnprintf("     ");
      if ( age % 5 == 4) wnprintf(" ");
      }
    } */

  }


/*---------------------------------------------------------------------------
			 CALC WAGE STREAM routine
			 calculates wage streams
  ---------------------------------------------------------------------------*/

void calcWageStream(
  VARS *vars)

{ EARNVARS earnvars;

  calcPastWages(vars, &earnvars);
  calcWagesAsSpouse(vars, &earnvars);
  calcPotentialWages(vars, &earnvars);

  }


/*---------------------------------------------------------------------------
			 CALC PAST WAGEs routine
			 calculates past wage streams using the first wave job history
  ---------------------------------------------------------------------------*/

void calcPastWages(
  VARS *vars,
  EARNVARS *earnvars)

{ int    job, jobno, govtjob, dupjob;
  int    cohort, gender, race, marstat, ed;
  int    age, matchcode, qc;
  int    hours, weeks, annhours, ft, ftyears;
  int    startdate, enddate, joblength;
  int    gstartdate, genddate, gjoblength;
  int    srearningsflag, commonyears;
  int    nonssjob, nonssstartdate, nonssenddate;
  int    nonssyears, remainingyears;
  int    year, date, yeardiff, earningsyears;
  int    minstartdate, maxenddate;
  int    lastjob, lastjobyear;
  int    tenure, expersum;
  int    firstage, firstyear, lastage, lastyear, everworked;
  int    lastftage, ten, exper;
  int    sempl, occ, ind, pens, healthage;
  int    lastsempl, lastocc, lastind, lastpens;
  int    count, maxcount;
  int    foreignborn, yearentered;
  double earnings, earningssr, ssmaximum;
  double earningstot, earningsave;
  double earningslim, lastearnings, nextearnings;
  double wage, realwage, realwagebase;
  double lnftbase, lnptbase;

  double ssearnvec[41];
  double srearnings[100];

  static double wageind[46] = {                             45.58,  49.00,  50.24,
 	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
  /* 1947-1992 average weekly earnings           */
  /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  static double ssmax[42] = {
			     3600,  3600,  3600,  3600,  4200,  4200,  4200,  4200,  4800, 
    4800,  4800,  4800,  4800,  4800,  4800,  6600,  6600,  7800,  7800,
    7800,  7800,  9000, 10800, 13200, 14100, 15300, 16500, 17700, 22900,
   25900, 29700, 32400, 35700, 37800, 39600, 42000, 43800, 45000, 48000,
   51300, 53400, 55500};
  /* Social Security maximum taxable earnings from Ann Statistical Suppl, 1995 */

  PERSVARS  *persvars;
  JOBHIST   *jobhist;
  PANELVARS *panelvars;
  CALCVARS  *calcvars;

  persvars  = &(vars->persvars);
  jobhist   = &(vars->jobhist);
  panelvars = &(vars->panelvars);
  calcvars  = &(vars->calcvars);

  calcvars->goodobs.goodSSstatus = 1;
  calcvars->goodobs.goodFTyears  = 1;
  calcvars->goodobs.goodearnings = 1;
  calcvars->goodobs.goodSSearn   = 1;

  if (persvars->caseid == 20130)
  calcvars->goodobs.goodearnings = 0;
  /* Respondent is 74 in 1992 and claims to be working until 79, but ss */
  /* earnings end in 1980, when respondent would have been 62.  It is   */
  /* unclear what the true story is.                                    */

  if (persvars->caseid == 23395)
  calcvars->goodobs.goodearnings = 0;
  /* respondent is 71 in 1992 and claims to have been working in current */
  /* job since 1988, but there are no ss earnings in 1990 or 1991.       */

  cohort  = persvars->birthyear;
  gender  = persvars->gender;
  race    = persvars->race;
  ed      = persvars->ed;
  marstat = panelvars->wavevars[0].marstat;

  /* get the social security record if available */

  getSSrec(persvars->caseid, &matchcode, &qc, ssearnvec);

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { if (matchcode == 1)
    for (year = 1952; year < 1992; year++)
    { if (year == 1952) wnprintf("\nss ");
      else if (year % 10 == 2) wnprintf("\n   ");
      wnprintf("%7.0f", ssearnvec[year - 1951]);
      }

    wnprintf("\njst  "); for (jobno = 0; jobno < 9; jobno++) wnprintf("%7d", jobhist->jobvars[jobno].startdate);
    wnprintf("\njend "); for (jobno = 0; jobno < 9; jobno++) wnprintf("%7d", jobhist->jobvars[jobno].enddate);
    wnprintf("\njear "); for (jobno = 0; jobno < 5; jobno++) wnprintf("%7.0f", jobhist->jobvars[jobno].earnings);
    wnprintf("\njhrs "); for (jobno = 0; jobno < 3; jobno++) wnprintf("%7d", jobhist->jobvars[jobno].hours);
    wnprintf("\njwks "); for (jobno = 0; jobno < 3; jobno++) wnprintf("%7d", jobhist->jobvars[jobno].weeks);
    }

  /* see whether there are any self-reported earnings */

  srearningsflag = 0;

  for (jobno = 0; jobno < 5; jobno++)
  { if (jobhist->jobvars[jobno].earnings > 0
     && (jobno == 0 || jobhist->jobvars[jobno].enddate > 0))
    srearningsflag = 1;
    }


  /* calculate which jobs are full-time and part-time */

  for (jobno = 0; jobno < 3; jobno++)
  { if (jobhist->jobvars[jobno].jobind == 1)
    { ft = 1;

      hours = jobhist->jobvars[jobno].hours;
      weeks = jobhist->jobvars[jobno].weeks;

      if (0 <= hours && hours < 95 && 0 <= weeks && weeks <= 52)
      { annhours = hours * weeks;

        if (hours <= 25 || annhours <= 1250)
        ft = 0;                             /* part-time */
        }
      } 
    else ft = -1;

    earnvars->fulltime[jobno] = ft;
    }


  /* find duplicate jobs in fed & state/local jobs */

  for (jobno = 0; jobno < 9; jobno++)
  earnvars->dupjob[jobno] = 0;

  for (jobno = 0; jobno < 5; jobno++)
  if (jobhist->jobvars[jobno].jobind == 1)
  { startdate = jobhist->jobvars[jobno].startdate;
    
    if (jobno > 0)
    enddate = jobhist->jobvars[jobno].enddate;
    else enddate = 1992;

    if (0 < startdate && startdate <= enddate)
    { joblength = enddate - startdate + 1;

      for (govtjob = 5; govtjob < 9; govtjob++)
      if (jobhist->jobvars[govtjob].jobind == 1)
      { gstartdate = jobhist->jobvars[govtjob].startdate;
        genddate = jobhist->jobvars[govtjob].enddate;

        if (0 < gstartdate && gstartdate <= genddate)
        { gjoblength = genddate - gstartdate + 1;
          commonyears = min(enddate, genddate) - max(startdate, gstartdate) + 1;

          if (commonyears >= 0.75 * joblength
           || persvars->caseid == 14850    /* These are farily certain matches even if */
           || persvars->caseid == 23083)   /* common years are slightly less than 75%. */
          earnvars->dupjob[jobno] = govtjob;

          if (commonyears >= 0.75 * gjoblength)
          earnvars->dupjob[govtjob] = jobno;
          }
        }
      }
    }


  /* Identify non social security jobs */

  nonssjob       = jobhist->nonssjob;
  nonssstartdate = jobhist->nonssstartdate;
  nonssenddate   = jobhist->nonssenddate;
  nonssyears     = jobhist->nonssyears;
    
  if (matchcode == 0                /* no social security match */
   || srearningsflag == 0)          /* no self-reported earnings */
  { if (nonssjob != 0)
    { if (nonssstartdate > 0 && nonssenddate > 0)    /* starting & ending dates of non-ss job given */
      { for (jobno = 0; jobno < 9; jobno++)
        if (jobhist->jobvars[jobno].jobind == 1)
        { startdate = jobhist->jobvars[jobno].startdate;
          
          if (jobno > 0)
          enddate = jobhist->jobvars[jobno].enddate;
          else enddate = 1992;

          joblength = enddate - startdate + 1;

          nonssyears = max(startdate, nonssstartdate) - min(enddate, nonssenddate) + 1;

          if (nonssyears >= 0.75 * joblength)
          jobhist->jobvars[jobno].sscoverage = 0;
          }
        }
      else if (nonssyears > 0)                       /* number of years of non-ss jobs given */
      { remainingyears = nonssyears;

        for (jobno = 5; jobno < 9; jobno++)          /* 5-6:state/loc  7-8:fed */
        if (jobhist->jobvars[jobno].jobind == 1)
		    { startdate = jobhist->jobvars[jobno].startdate;
          enddate   = jobhist->jobvars[jobno].enddate;

          if (startdate > 0 && enddate > 0)
          if (jobno <= 6 || startdate <= 1984)        /* state or fed before 1984 */
          { remainingyears = remainingyears - (enddate - startdate + 1);
            jobhist->jobvars[jobno].sscoverage = 0;
            }
          }

        if (remainingyears > 9)                       /* substantial non-ss years remaining */
        calcvars->goodobs.goodSSstatus = 0;
        }
      else                                            /* non-ss jobs indicated, but no dates */
      { for (jobno = 5; jobno < 9; jobno++)           /* 5-6:state/loc  7-8:fed */
        if (jobhist->jobvars[jobno].jobind == 1)
		    { startdate = jobhist->jobvars[jobno].startdate;

          if (jobno <= 6 || startdate <= 1984)        /* state or fed before 1984 */
          jobhist->jobvars[jobno].sscoverage = 0;
          }
        }
      }
    }
  else                              /* with social security match */
  { for (jobno = 0; jobno < 9; jobno++)
    if (jobhist->jobvars[jobno].jobind == 1)
	  { startdate = jobhist->jobvars[jobno].startdate;
		  
      if (jobno > 0)
      enddate = jobhist->jobvars[jobno].enddate;
      else enddate = 1992;

      if (1910 <= startdate && startdate <= enddate && enddate <= 1993)
      { /* find nearest self-reported earnings */

        yeardiff = 99;

        for (job = 0; job < 5; job++)
        if (jobhist->jobvars[jobno].jobind == 1)
        { earnings = jobhist->jobvars[job].earnings;

          if (job > 0)
          date = jobhist->jobvars[job].enddate;
          else date = 1992;

          if (earnings > 0 && date > 0)
          if (abs(enddate - date) < yeardiff)
          { yeardiff = abs(enddate - date);

            if (date < 1947)
            earningssr = earnings * wageind[1992 - 1947] / wageind[1947 - 1947];
            else if (date < 1992)
            earningssr = earnings * wageind[1992 - 1947] / wageind[date - 1947];
            else earningssr = earnings;
            }
          }

        if (yeardiff == 99)
        error("Bad yeardiff in CalcPastWages");

        if (enddate < 1951)
        ssmaximum = ssmax[1951 - 1951] * (wageind[1992 - 1947] / wageind[1947 - 1947]);
	      else
        { if (enddate <= 1992)
          ssmaximum = ssmax[enddate - 1951] * (wageind[1992 - 1947] / wageind[enddate - 1947]);
          else ssmaximum = ssmax[1992 - 1951];
          }

		    if (earningssr > ssmaximum)
		    earningssr = ssmaximum;

        /* see if average ss earnings is at least 60% of self-reported */

		    earningstot = 0;
        earningsyears = 0;

		    for (year = startdate; year <= enddate; year++)
		    if (1951 <= year && year <= 1991)
		    { if (ssearnvec[year - 1951] < ssmax[year - 1951])
          earnings = ssearnvec[year - 1951] * (wageind[1992 - 1947] / wageind[year - 1947]);
          else earnings = earningssr;

          earningstot = earningstot + earnings;
          earningsyears = earningsyears + 1;
          }

        if (earningsyears > 0)
        { earningsave = earningstot / earningsyears;

			    if (earningsave < 0.6 * earningssr)
          jobhist->jobvars[jobno].sscoverage = 0;
          }
        else if (startdate >= 1992)
        { if (nonssstartdate > 0 && nonssenddate > 0)
          { if (nonssstartdate <= 1992 && nonssenddate >= 1992)
            jobhist->jobvars[jobno].sscoverage = 0;
            }
          else if (nonssjob != 0)
          calcvars->goodobs.goodSSstatus = 0;
          }
        }
      }
    }


  /* make ss coverage consistent between duplicate jobs */

  for (jobno = 0; jobno < 5; jobno++)
  { dupjob = earnvars->dupjob[jobno];

    if (jobhist->jobvars[jobno].jobind == 1 && dupjob >= 0)
    jobhist->jobvars[jobno].sscoverage = jobhist->jobvars[dupjob].sscoverage;
    }


  /* Adjust dates of jobs with social security to be consistent with social
  /* security record.  Reconcile dates of duplicate jobs */

  for (jobno = 0; jobno < 9; jobno++)
  if (jobhist->jobvars[jobno].jobind == 1)
  { dupjob = earnvars->dupjob[jobno];

    if (dupjob > 0)
    { startdate = (jobhist->jobvars[jobno].startdate + jobhist->jobvars[dupjob].startdate) / 2;
      minstartdate = min(jobhist->jobvars[jobno].startdate, jobhist->jobvars[dupjob].startdate);

      if (jobno >= 1)
      { enddate = (jobhist->jobvars[jobno].enddate + jobhist->jobvars[dupjob].enddate) / 2;
        maxenddate = max(jobhist->jobvars[jobno].enddate, jobhist->jobvars[dupjob].enddate);
        }
      else enddate = 1992;
      }
    else
    { startdate = jobhist->jobvars[jobno].startdate;
      minstartdate = startdate;

      if (jobno >= 1)
      { enddate = jobhist->jobvars[jobno].enddate;
        maxenddate = enddate;
        }
      else enddate = 1992;
      }

    if (matchcode == 1 && jobhist->jobvars[jobno].sscoverage == 1)
	  if (1910 <= startdate && startdate <= enddate && enddate <= 1993)
    { earningstot = 0;
      earningsyears = 0;

		  for (year = startdate; year <= enddate; year++)
		  if (1951 <= year && year <= 1991)
		  { earnings = ssearnvec[year - 1951]
                  * (wageind[1992 - 1947] / wageind[year - 1947]);

        earningstot = earningstot + earnings;
        earningsyears = earningsyears + 1;
        }

      if (earningsyears > 0)
      { earningsave = earningstot / earningsyears;
        earningslim = 0.5 * earningsave;

        if (startdate >= 1951)
        { earnings = ssearnvec[startdate - 1951]
                    * (wageind[1992 - 1947] / wageind[startdate - 1947]);

          if (minstartdate < startdate && earnings > earningslim)  /* only for duplicate jobs */
		      { startdate = startdate - 1;

            for (year = startdate; year > minstartdate; year--)
            if ( year == startdate)
		        if (1952 < year && year <= 1991)
            { earnings = ssearnvec[year - 1951]
                        * (wageind[1992 - 1947] / wageind[year - 1947]);
              lastearnings = ssearnvec[(year - 1) - 1951]
                            * (wageind[1992 - 1947] / wageind[year - 1947]);

              if ((earnings > earningslim && lastearnings > 0) || lastearnings > earningslim)
              startdate = year - 1;
              }
            }
          else
          { for (year = startdate; year < min(1990, enddate); year++)
            if (year == startdate)
            { earnings = ssearnvec[year - 1951]
                        * (wageind[1992 - 1947] / wageind[year - 1947]);
              nextearnings = ssearnvec[(year + 1) - 1951]
                            * (wageind[1992 - 1947] / wageind[year - 1947]);

              if (earnings < earningslim && (nextearnings < earningslim || earnings == 0))
              startdate = year + 1;
              }
            }
          }

        if (enddate <= 1991)
        { earnings = ssearnvec[enddate - 1951]
                  * (wageind[1992 - 1947] / wageind[year - 1947]);


          if (maxenddate < enddate && earnings > earningslim)   /* only for duplicate jobs */
		      { enddate = enddate + 1;

            for (year = enddate; year < maxenddate; year++)
            if ( year == enddate)
		        if (1951 < year && year <= 1990)
            { earnings = ssearnvec[year - 1951]
                        * (wageind[1992 - 1947] / wageind[year - 1947]);
              nextearnings = ssearnvec[(year + 1) - 1951]
                          * (wageind[1992 - 1947] / wageind[year - 1947]);

              if ((earnings > earningslim && nextearnings > 0) || nextearnings > earningslim)
              enddate = year + 1;
              }
            }
          else
          { for (year = enddate; year > max(1951, startdate - 1); year--)
            if (year == enddate)
            { earnings = ssearnvec[year - 1951]
                        * (wageind[1992 - 1947] / wageind[year - 1947]);
              lastearnings = ssearnvec[(year - 1) - 1951]
                              * (wageind[1992 - 1947] / wageind[year - 1947]);

              if (earnings < earningslim && (lastearnings < earningslim || earnings == 0))
              enddate = year - 1;
              }
            }
          }
        }
      }

    earnvars->startdate[jobno] = startdate;
    earnvars->enddate[jobno] = enddate;

    if (enddate >= startdate)
    earnvars->jobind[jobno] = 1;
    else earnvars->jobind[jobno] = 0;
    }
  else
  { earnvars->jobind[jobno]    =  0;
    earnvars->startdate[jobno] = -9;
    earnvars->enddate[jobno]   = -9;
    }


  /* Find last job */

  lastjobyear = 0;
  lastjob = -9;

  for (jobno = 0; jobno < 5; jobno++)
  if (earnvars->jobind[jobno] == 1)
  { startdate = earnvars->startdate[jobno];
    enddate = earnvars->enddate[jobno];

    if (0 < startdate && startdate <= enddate && enddate <= 1993)
    { if (enddate > 1992)
      enddate = 1992;

      if (jobno <= 1 || (lastjob == -9 || lastjob >= 2 && lastjobyear < enddate))
      { lastjobyear = enddate;
        lastjob = jobno;
        }
      }
    }


  /* Construct tenure, non-social security, and full-time vectors */

  for (age = 0; age <= 100; age++)
  { earnvars->ten[age]   = -1;
    earnvars->ft[age]    = -1;
    earnvars->exper[age] =  0;
    earnvars->nonss[age] =  0;
	 }

  for (jobno = 0; jobno < 8; jobno++)
  if (earnvars->jobind[jobno] == 1)
  { startdate = earnvars->startdate[jobno];
    enddate = earnvars->enddate[jobno];

    if (0 < startdate && startdate <= enddate && enddate <= 1993)
    { if (enddate > lastjobyear)
      enddate = lastjobyear;

      tenure = 0;

      for (year = startdate; year <= enddate; year++)
      { age = year - cohort;

        if (0 <= age && age <= 100)
        if (earnvars->ten[age] < 0)
        { earnvars->ten[age] = tenure;

          if (jobhist->jobvars[jobno].sscoverage == 0)
          earnvars->nonss[age] = 1;
          else if (jobhist->jobvars[jobno].sscoverage == 1)
          earnvars->nonss[age] = 0;

          if (jobno < 3)
          { if (earnvars->fulltime[jobno] == 0)
            earnvars->ft[age] = 0;
            else earnvars->ft[age] = 1;
            }
          else earnvars->ft[age] = 1;
          }

        tenure = tenure + 1;
        }
      }
    
    if (jobno < 3)
    if (earnvars->fulltime[jobno] == 1)
    { if (calcvars->ind == -1)
      if (0 < jobhist->jobvars[jobno].ind && jobhist->jobvars[jobno].ind <= 13)
      calcvars->ind = jobhist->jobvars[jobno].ind;

      if (calcvars->occ == -1)
      if (0 < jobhist->jobvars[jobno].occ && jobhist->jobvars[jobno].occ <= 17)
      calcvars->occ = jobhist->jobvars[jobno].occ;
      }  
    }


  /* Set tenure vector to reflect experience reported in 96 survey */

  if (ed < 12)
  firstage = 19;
  else firstage = ed + 7;

  firstyear = jobhist->ftstartdate;
  lastyear  = jobhist->ftenddate;
  ftyears   = jobhist->ftyears;

  if (1920 <= firstyear && firstyear <= lastyear && lastyear <= 1996)  /* range of full-time */
  { for (year = firstyear; year <= min(lastyear, lastjobyear); year++)  /* years given        */
    if (year <= 1992)
	  { age = year - cohort;

		  if (firstage <= age && earnvars->ten[age] < 0)
		  { earnvars->ten[age] = 0;
        earnvars->ft[age] = 1;
        }
		  }
	  }
  else if (0 <= ftyears && ftyears <= 50)     /* number of full-time years */
  { lastyear = jobhist->lastunknowndate;      /* given, but not range      */

    if (lastyear > 1920)
	  lastage = (lastyear - cohort) - 1;
    else lastage = 1992 - cohort;

	  expersum = 0;

	  for (age = 0; age <= lastage; age++)
	  if (earnvars->ten[age] >= 0)
	  expersum = expersum + 1;

	  if (expersum < ftyears)
	  for (age = min(lastage, lastjobyear - cohort); age >= firstage; age--)
	  if (expersum < ftyears && earnvars->ten[age] < 0)
	  { earnvars->ten[age] = 0;
	    earnvars->ft[age] = 1;
      expersum = expersum + 1;
	    }
	  }
  else                                        /* full-time years not given  */
  { firstyear = jobhist->yearfirstworked;     /* from 1996 survey */
    everworked = jobhist->everworked;         /* from 1992 survey */

	  if (everworked != 5 && firstyear != 9995) /* not (never worked) */
	  { if (gender == 1 || marstat == 0)        /* males and nonmarried females */
	    { if (firstyear < cohort + firstage || firstyear >= 9998)
        firstyear = cohort + firstage;

	      lastyear = jobhist->lastunknowndate;  /* from 1996 survey preload */

        if (lastyear <= 0)
        { if (jobhist->jobvars[0].jobind == 1)
          lastyear = 1992;
          else lastyear = jobhist->yearlastworked;   /* from 1992 survey */
          }

        if (lastyear <= 0)
        { if (gender == 1)
          lastyear = 1981;
          else lastyear = 1971;

          if (matchcode == 0)
          calcvars->goodobs.goodFTyears = 0;
          }

        for (year = firstyear; year <= min(lastyear, lastjobyear); year++)
	      { age = year - cohort;

          if (year <= 1992 && earnvars->ten[age] < 0)
	        { earnvars->ten[age] = 0;
            earnvars->ft[age] = 1;
            }
          }
	      }

      if (matchcode == 0 && gender == 2)
      calcvars->goodobs.goodFTyears = 0;
      }
    }

  earnvars->exper[0] = 0;

  for (age = 1; age <= 1992 - cohort; age++)
  if (earnvars->ten[age] >= 0)
  earnvars->exper[age] = earnvars->exper[age - 1] + 1;
  else earnvars->exper[age] = earnvars->exper[age - 1];

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\nage "); for (age = 25; age < 65; age++) wnprintf("%2d", age);
    wnprintf("\nexp "); for (age = 25; age < 65; age++) wnprintf("%2d", earnvars->exper[age]);
    wnprintf("\nten "); for (age = 25; age < 65; age++) wnprintf("%2d", earnvars->ten[age]);
    wnprintf("\nft  "); for (age = 25; age < 65; age++) wnprintf("%2d", earnvars->ft[age]);
    wnprintf("\nnss "); for (age = 25; age < 65; age++) wnprintf("%2d", earnvars->nonss[age]);
    }


  /* construct full-time self-reported earnings history */

  srearningsflag = 0;

  for (age = 0; age <= 100; age++)
  srearnings[age] = 0;

  for (jobno = 0; jobno < 5; jobno++)
  if (jobhist->jobvars[jobno].jobind == 1)
  { if (jobno < 3)
    ft = earnvars->fulltime[jobno];
    else ft = 1;

    enddate = earnvars->enddate[jobno];

    if (enddate > 0)
    { age = enddate - cohort;

      if (ft == 1 && earnvars->ft[age] == 1 && jobhist->jobvars[jobno].earnings > 0)
      { srearnings[age] = jobhist->jobvars[jobno].earnings;
        srearningsflag = 1;
        }
      }
    }

  if (srearningsflag == 0)
  { lastftage = 0;

    for (age = 0; age < 100; age++)
    if (earnvars->ft[age] == 1)
    lastftage = age;

    if (lastftage > 0)
    { year = cohort + lastftage;

      realwage = 24876.80;        /* median in hrsftwag.log */

      if (year < 1947)
      srearnings[lastftage] = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
      else if (year < 1992)
      srearnings[lastftage] = realwage * wageind[year - 1947] / wageind[1992 - 1947];
      else srearnings[lastftage] = realwage;
      }

    if ((jobhist->jobvars[0].jobind == 1 || jobhist->jobvars[1].jobind == 1)
     && matchcode == 0)
    calcvars->goodobs.goodearnings = 0;
    }

  realwagebase = -99;

  for (age = 99; age >= 0; age--)
  if (realwagebase == -99 && srearnings[age] > 0)
  { year = cohort + age;

    if (year < 1947)
    realwage = srearnings[age] * wageind[1992 - 1947] / wageind[1947 - 1947];
    else if (year < 1992)
    realwage = srearnings[age] * wageind[1992 - 1947] / wageind[year - 1947];
    else realwage = srearnings[age];

    exper = earnvars->exper[age];
    ten   = earnvars->ten[age];

    /* experience/tenure coefficients from e:\mrrc\ssretage\hrsftwag.log */
	  realwagebase = log(realwage)
      - 0.0051772 * exper + 0.0000557 * exper * exper
		  - 0.0180837 * ten   + 0.0002142 * ten * ten;
    }

  for (age = 99; age >= 0; age--)
  if  (earnvars->ft[age] == 1)
  { exper = earnvars->exper[age];
    ten   = earnvars->ten[age];

    if (srearnings[age] > 0)
    { year = cohort + age;

      if (year < 1947)
      realwage = srearnings[age] * wageind[1992 - 1947] / wageind[1947 - 1947];
      else if (year < 1992)
      realwage = srearnings[age] * wageind[1992 - 1947] / wageind[year - 1947];
      else realwage = srearnings[age];

      /* experience/tenure coefficients from e:\mrrc\ssretage\hrsftwag.log */
	    realwagebase = log(realwage)
        - 0.0051772 * exper + 0.0000557 * exper * exper
		    - 0.0180837 * ten   + 0.0002142 * ten * ten;
      }
	  else if (earnvars->ten[age] >= 0)
	  { year = cohort + age;

	    realwage = exp(realwagebase
        + 0.0051772 * exper - 0.0000557 * exper * exper
		    + 0.0180837 * ten   - 0.0002142 * ten * ten);

      if (year < 1947)
      wage = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
      else if (year < 1992)
      wage = realwage * wageind[year - 1947] / wageind[1992 - 1947];
      else wage = realwage;

		  srearnings[age] = wage;
	    }
    }


  /* construct part-time self-reported earnings history */

  lnptbase = -99;

  for (jobno = 0; jobno < 3; jobno++)
  if (jobhist->jobvars[jobno].jobind == 1)
  { if (jobno < 3)
    ft = earnvars->fulltime[jobno];
    else ft = 1;

    enddate = earnvars->enddate[jobno];

    if (enddate > 0)
    { age = enddate - cohort;

      if (ft == 0 && earnvars->ft[age] == 0 && jobhist->jobvars[jobno].earnings > 0)
      { srearnings[age] = jobhist->jobvars[jobno].earnings;

        if (lnptbase == -99)
        { earnings = jobhist->jobvars[jobno].earnings;
          exper = earnvars->exper[age];

          if (enddate < 1947)
          realwage = earnings * wageind[1992 - 1947] / wageind[1947 - 1947];
          else if (enddate < 1992)
          realwage = earnings * wageind[1992 - 1947] / wageind[enddate - 1947];
          else realwage = earnings;

          lnptbase = log(realwage)
            - 0.0103480 * exper + 0.0001875 * exper * exper;
          }
        }
      }
    }

  if (lnptbase == -99)
  { lnftbase = -99;
    lastsempl = -9;
    lastocc   = -9;
    lastind   = -9;
    lastpens  = -9;

    healthage = calcvars->healthage;

    /* find full-time base */

    for (jobno = 0; jobno < 5; jobno++)
    if (lnftbase == -99 && jobhist->jobvars[jobno].jobind == 1)
    { if (jobno < 3)
      ft = earnvars->fulltime[jobno];
      else ft = 1;

      enddate = earnvars->enddate[jobno];

      if (enddate > 0)
      { age = enddate - cohort;

        sempl = jobhist->jobvars[jobno].sempl;
        occ   = jobhist->jobvars[jobno].occ;
        ind   = jobhist->jobvars[jobno].ind;
        pens  = jobhist->jobvars[jobno].pencoverage;

        if (sempl < 0 || sempl > 1)   sempl = lastsempl;
        if (occ < 1 || occ > 17)      occ = lastocc;
        if (ind < 1 || ind > 13)      ind = lastind;
        if (pens < 0 || pens > 1)     pens = lastpens;

        if (ft == 1 && earnvars->ft[age] == 1 && jobhist->jobvars[jobno].earnings > 0)
        { earnings = jobhist->jobvars[jobno].earnings;

          if (enddate < 1947)
          realwage = earnings * wageind[1992 - 1947] / wageind[1947 - 1947];
          else if (enddate < 1992)
          realwage = earnings * wageind[1992 - 1947] / wageind[enddate - 1947];
          else realwage = earnings;

          ten   = earnvars->ten[age];
          exper = earnvars->exper[age];

          lnftbase = log(realwage)         /* from hrsftwag.log */
            - 0.0051772 * exper + 0.0000557 * exper * exper
		        - 0.0180837 * ten   + 0.0002142 * ten * ten;

          if (ed  < 12)             lnftbase = lnftbase + 0.1299299;   /* hsdrop  */
          if (13 <= ed && ed <= 15) lnftbase = lnftbase - 0.1385978;   /* somecol */
          if (ed == 16)             lnftbase = lnftbase - 0.3307412;   /* colgrad */
          if (17 <= ed)             lnftbase = lnftbase - 0.4314858;   /* advdegr */
          if (race == 2)            lnftbase = lnftbase + 0.0594629;   /* black */
          if (race == 3)            lnftbase = lnftbase + 0.0666119;   /* hispanic */
          if (marstat == 1)         lnftbase = lnftbase - 0.0862579;   /* married */
          if (healthage <= age)     lnftbase = lnftbase + 0.0363754;   /* health */
          if (sempl == 1)           lnftbase = lnftbase + 0.0712693;   /* sempl */
          if (ind == 3 || ind == 4) lnftbase = lnftbase - 0.0289258;   /* mfg */
          if (ind == 13)            lnftbase = lnftbase - 0.0557995;   /* govt */
          if (occ == 1 || occ == 2) lnftbase = lnftbase - 0.2266077;   /* mgtprf */
          if (occ == 3 || occ == 4) lnftbase = lnftbase - 0.0331493;   /* whcol */
          if (pens == 1)            lnftbase = lnftbase - 0.2443663;   /* pens */
          }

        if (sempl == 0 || sempl == 1)   lastsempl = sempl;
        if (1 <= occ && occ <= 17)      lastocc = occ;
        if (1 <= ind && ind <= 13)      lastind = ind;
        if (pens == 0 || pens == 1)     lastpens = pens;
        }
      }

    if (lnftbase == -99)
    { lastftage = 0;

      for (age = 99; age >= 0; age--)
      if (lastftage == 0 && srearnings[age] > 0 && earnvars->ft[age] == 1)
      lastftage = age;

      if (lastftage > 0)
      { year = cohort + lastftage;

        if (year < 1947)
        realwage = srearnings[lastftage] * wageind[1992 - 1947] / wageind[1947 - 1947];
        else if (year < 1992)
        realwage = srearnings[lastftage] * wageind[1992 - 1947] / wageind[year - 1947];
        else realwage = srearnings[lastftage];

        ten = earnvars->ten[lastftage];
        exper = earnvars->exper[lastftage];

        lnftbase = log(realwage)
          - 0.0051772 * exper + 0.0000557 * exper * exper
		      - 0.0180837 * ten   + 0.0002142 * ten * ten;

        if (lnftbase < -20 || lnftbase > 20)
        wnprintf("\n %4d %d %d %e %e %e ", persvars->caseid, lastftage,
         cohort, srearnings[lastftage], realwage, lnftbase);
        }
      else lnftbase = log(24876.80);       /* median in hrsftwag.log */

      if (persvars->caseid == printcaseid || printcaseid == 99999)
      { if (lastftage > 0)
        wnprintf("\n lnw %6.3f %d %6.3f", log(realwage), lastftage, lnftbase);
        else wnprintf("\n lnw        %d %6.3f", lastftage, lnftbase);
        }

      if (ed  < 12)             lnftbase = lnftbase + 0.1299299;   /* hsdrop  */
      if (13 <= ed && ed <= 15) lnftbase = lnftbase - 0.1385978;   /* somecol */
      if (ed == 16)             lnftbase = lnftbase - 0.3307412;   /* colgrad */
      if (17 <= ed)             lnftbase = lnftbase - 0.4314858;   /* advdegr */
      if (race == 2)            lnftbase = lnftbase + 0.0594629;   /* black */
      if (race == 3)            lnftbase = lnftbase + 0.0666119;   /* hispanic */
      if (marstat == 1)         lnftbase = lnftbase - 0.0862579;   /* married */
      if (healthage <= age)     lnftbase = lnftbase + 0.0363754;   /* health */
      }

    /* find partial retirement base */

    lnptbase = lnftbase + 0.0332661;           /* from hrsptwag.log */

    if (ed  < 12)             lnptbase = lnptbase - 0.1753805;   /* hsdrop  */
    if (13 <= ed && ed <= 15) lnptbase = lnptbase + 0.0811179;   /* somecol */
    if (ed == 16)             lnptbase = lnptbase + 0.0946980;   /* colgrad */
    if (17 <= ed)             lnptbase = lnptbase + 0.4370315;   /* advdegr */
    if (race == 2)            lnptbase = lnptbase - 0.0266918;   /* black */
    if (race == 3)            lnptbase = lnptbase - 0.0488308;   /* hispanic */
    if (marstat == 1)         lnptbase = lnptbase + 0.0283834;   /* married */
    if (healthage <= age)     lnptbase = lnptbase + 0.0099354;   /* health */

    lnptbase = lnptbase + log(0.5);            /* half-time */

    if (persvars->caseid == printcaseid || printcaseid == 99999)
    wnprintf("\nbases %6.3f %6.3f", lnftbase, lnptbase);
    }

  earnvars->ptearnbase = exp(lnptbase);
  realwagebase = lnptbase;

  for (age = 99; age >= 0; age--)
  if (earnvars->ft[age] == 0)
  { exper = earnvars->exper[age];

    if (srearnings[age] > 0)
    { year = cohort + age;

      if (year < 1947)
      realwage = srearnings[age] * wageind[1992 - 1947] / wageind[1947 - 1947];
      else if (year < 1992)
      realwage = srearnings[age] * wageind[1992 - 1947] / wageind[year - 1947];
      else realwage = srearnings[age];

      /* experience coefficients from e:\mrrc\ssretage\hrsptwag.log */
	    realwagebase = log(realwage)
        - 0.0103480 * exper + 0.0001875 * exper * exper;
      }
	  else
	  { year = cohort + age;

	    realwage = exp(realwagebase
        + 0.0103480 * exper - 0.0001875 * exper * exper);

      if (year < 1947)
      wage = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
      else if (year < 1992)
      wage = realwage * wageind[year - 1947] / wageind[1992 - 1947];
      else wage = realwage;

		  srearnings[age] = wage;
	    }
    }


  /* Form the self-reported and social security earnings vector */

  /* This respondent was born in 1927 and was 64 in the first survey. */
  /* He reported current earnings of $5000, but the social security   */
  /* record is zero for 1991.  Since this was his last observation of */
  /* full time work, the reported earnings are substituted for zero.  */
  if (persvars->caseid == 23468)
  ssearnvec[1991 - 1951] = 5000;

  for (age = 0; age < 100; age++)
  { earnvars->earnings[age] = 0;
    earnvars->ssearnings[age] = 0;

    year = cohort + age;

    if (matchcode == 1)                         /* social security record available */
    { earnvars->earnings[age] = srearnings[age];

      if (1951 <= year && year <= 1991 && earnvars->nonss[age] == 0)
      { if (ssearnvec[year - 1951] < ssmax[year - 1951]
         || srearnings[age] < ssmax[year - 1951])
        earnvars->earnings[age] = ssearnvec[year - 1951];
        }

      if (1951 <= year && year <= 1991)
      earnvars->ssearnings[age] = ssearnvec[year - 1951];
      else if (year == 1992)                    /* impute ss earnings for 1992 */
      { if (earnvars->nonss[age] == 0)
        { if (srearnings[age] < ssmax[year - 1951])
          earnvars->ssearnings[age] = srearnings[age];
          else earnvars->ssearnings[age] = ssmax[year - 1951];
          }
        }
      }
    else
    { earnvars->earnings[age] = srearnings[age];

      if (1951 <= year && year <= 1992 && earnvars->nonss[age] == 0)
      { if (srearnings[age] < ssmax[year - 1951])
        earnvars->ssearnings[age] = srearnings[age];
        else earnvars->ssearnings[age] = ssmax[year - 1951];
        }
      }
    }


  /* if no self-reported earnings, check to see if ss earnings */
  /* are over maximum or if there were non-ss jobs             */

  if (srearningsflag == 0 && matchcode == 1)
  { count = 0;
    maxcount = 0;

    for (year = 1991; year >= 1951; year--)
    if (count < 12 && cohort <= year)
    { if (ssearnvec[year - 1951] > 0.25 * ssmax[year - 1951])
      { if (ssearnvec[year - 1951] == ssmax[year - 1951])
        maxcount = maxcount + 1;

        count = count + 1;
        }
      }

    if (4 * maxcount >= count)
    calcvars->goodobs.goodSSearn = 0;

    for (jobno = 0; jobno < 9; jobno++)
    if (jobhist->jobvars[jobno].sscoverage == 0)
    calcvars->goodobs.goodSSearn = 0;
    }


  /* zero out years before immigrants entered the country */

  if (matchcode == 0)
  { foreignborn = eval(1, 204);
    yearentered = eval(1, 206);

	  if (foreignborn == 5 && 1951 <= yearentered && yearentered <= 1992)
	  { for (age = 1951 - cohort; age < yearentered - cohort; age++)
		  earnvars->ssearnings[age] = 0;
		  }
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { for (year = 1952; year <= 1992; year++)
    { if (year == 1952) wnprintf("\near");
      else if (year != 1992 && year % 10 == 2) wnprintf("\n   ");
      if (year > cohort) wnprintf("%7.0f", earnvars->earnings[year - cohort]);
      }
    for (year = 1952; year <= 1992; year++)
    { if (year == 1952) wnprintf("\nsse");
      else if (year != 1992 && year % 10 == 2) wnprintf("\n   ");
      if (year > cohort) wnprintf("%7.0f", earnvars->ssearnings[year - cohort]);
      }
    }

  }


/*---------------------------------------------------------------------------
			 CALC WAGES AS SPOUSE routine
			 calculates wages for the respondent for use as a spouse
  ---------------------------------------------------------------------------*/

void calcWagesAsSpouse(
  VARS *vars,
  EARNVARS *earnvars)

{ int    age, surv, year;
  int    lastworkage, lastretstat, ftend;
  int    aveage, penage;
  int    jobno, retage;
  int    pjobno, pjobstart, pjobend;
  int    startdate, enddate, tenure, experience;
  double realwage, lnftbase, lnptbase;

  int    exper[100], ten[100], ft[100], nonss[100];
  double earnings[100], ssearnings[100];

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
  /* 1947-1992 average weekly earnings           */
  /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  PERSVARS  *persvars;
  JOBHIST   *jobhist;
  PANELVARS *panelvars;
  CALCVARS  *calcvars;

  persvars  = &(vars->persvars);
  jobhist   = &(vars->jobhist);
  panelvars = &(vars->panelvars);
  calcvars  = &(vars->calcvars);


  /* find the retirement date if still working at last survey */

  retage = 99;

  if (calcvars->finalretstat > 1)
  for (surv = 5; surv >= 0; surv--)
  if (panelvars->wavevars[surv].interview == 1 && retage == 99)
  { if (panelvars->wavevars[surv].expretage == 96)     /* never retire */
    retage = 70;
    else if (25 <= panelvars->wavevars[surv].expretage
     && panelvars->wavevars[surv].expretage < 95)
    retage = panelvars->wavevars[surv].expretage;
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\nexp ret"); for (surv = 0; surv < 6; surv++) wnprintf(" %2d", panelvars->wavevars[surv].expretage);
    wnprintf("\nsamejob"); for (surv = 0; surv < 6; surv++) wnprintf(" %2d", panelvars->wavevars[surv].samejob);
    wnprintf("\nhours  "); for (surv = 0; surv < 6; surv++) wnprintf(" %2d", panelvars->wavevars[surv].hours);
    wnprintf("\nweeks  "); for (surv = 0; surv < 6; surv++) wnprintf(" %2d", panelvars->wavevars[surv].weeks);
    }


  /* extend tenure, experience, social security, and full-time vectors vectors */

  for (age = 0; age < 100; age++)
  { exper[age] = earnvars->exper[age];
    ten[age]   = earnvars->ten[age];
    ft[age]    = earnvars->ft[age];
    nonss[age] = earnvars->nonss[age];
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  wnprintf("    ret %d %d", retage, calcvars->ssdiage);
  
  lastworkage = -1;
  lastretstat = -1;

  for (age = calcvars->initage + 11; age >= calcvars->initage - 1; age--)
  { /* wnprintf(" %2d", calcvars->retstats[age - (calcvars->initage - 1)]); */
    if (calcvars->retstats[age - (calcvars->initage - 1)] > 0)
    { if (calcvars->retstats[age - (calcvars->initage - 1)] == 1)
      lastretstat = -1;
      else if (calcvars->retstats[age - (calcvars->initage - 1)] == 3)
      lastretstat = 0;
      else if (calcvars->retstats[age - (calcvars->initage - 1)] == 5)
      lastretstat = 1;
        
      if (lastworkage == -1 && lastretstat >= 0)
      lastworkage = age;
      }

    if (1992 - persvars->birthyear < age && age < 100)
    ft[age] = lastretstat;
    }

  if (retage < 99)
  { if (lastworkage >= 1992 - persvars->birthyear)
    { if (retage <= lastworkage)
      retage = lastworkage + 1;
      }
    else if (retage < 1992 - persvars->birthyear)
    retage = 1992 - persvars->birthyear;
    }
  else
  { if (lastworkage > 1992 - persvars->birthyear)
    retage = lastworkage + 1;
    else retage = 1992 - persvars->birthyear;

    if (calcvars->finalretstat > 1)
    retage = max(62, retage);
    }

  if (lastworkage > 0 && calcvars->finalretstat > 1)
  { for (age = lastworkage + 1; age < min(retage, calcvars->ssdiage); age++)
    { if (calcvars->finalretstat == 3)
      ft[age] = 0;
      else if (calcvars->finalretstat == 5)
      ft[age] = 1;
      }
    }

  /* if (calcvars->newjobstartage > 1993 - persvars->birthyear)
  jobchange = calcvars->newjobstartage;
  else jobchange = 1993 - persvars->birthyear; */

  for (age = 1993 - persvars->birthyear; age < min(retage, calcvars->ssdiage); age++)
  if (ft[age] >= 0)
  { exper[age] = exper[age - 1] + 1;

    if (age < calcvars->origjobendage || age > calcvars->newjobstartage)
    ten[age] = ten[age - 1] + 1;
    else ten[age] = 0;

    if (age < calcvars->origjobendage - 1)
    nonss[age] = nonss[age - 1];
    else nonss[age] = 0;
    }
  else exper[age] = exper[age - 1];

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\nage "); for (age = 35; age < 75; age++) wnprintf("%2d", age);
    wnprintf("\nexp "); for (age = 35; age < 75; age++) wnprintf("%2d", exper[age]);
    wnprintf("\nten "); for (age = 35; age < 75; age++) wnprintf("%2d", ten[age]);
    wnprintf("\nft  "); for (age = 35; age < 75; age++) wnprintf("%2d", ft[age]);
    wnprintf("\nnss "); for (age = 35; age < 75; age++) wnprintf("%2d", nonss[age]);
    }


  /* calculate wage profile from 1992 survey until retirement */

  for (age = 0; age < 100; age++)
  { earnings[age]   = earnvars->earnings[age];
    ssearnings[age] = earnvars->ssearnings[age];
    }

  lnftbase = -99;

  for (age = 1992 - persvars->birthyear; age >= 0; age--)
  { if (lnftbase == -99 && ft[age] == 1 && earnings[age] > 0)
    { enddate = persvars->birthyear + age;

      if (enddate < 1947)
      realwage = earnings[age] * wageind[1992 - 1947] / wageind[1947 - 1947];
      else if (enddate < 1992)
      realwage = earnings[age] * wageind[1992 - 1947] / wageind[enddate - 1947];
      else realwage = earnings[age];

      tenure = ten[age];
      experience = exper[age];

      lnftbase = log(realwage)                /* from hrsftwag.log */
        - 0.0051772 * experience + 0.0000557 * experience * experience
	      - 0.0180837 * tenure     + 0.0002142 * tenure * tenure;

      if (persvars->caseid == printcaseid || printcaseid == 99999)
      wnprintf("\nbase %d %d %d", age, experience, tenure);
      }
    }

  lnptbase = log(earnvars->ptearnbase);

  for (age = 1993 - persvars->birthyear; age < min(retage, calcvars->ssdiage); age++)
  if (ft[age] >= 0)
  { tenure = ten[age];
    experience = exper[age];

    if (ft[age] == 1)
    realwage = exp(lnftbase
      + 0.0051772 * experience - 0.0000557 * experience * experience
		  + 0.0180837 * tenure     - 0.0002142 * tenure * tenure);
    else realwage = exp(lnptbase
      + 0.0103480 * experience - 0.0001875 * experience * experience);

    earnings[age] = realwage * pow(1.051, age - (1992 - persvars->birthyear));

    if (nonss[age] == 0)
    ssearnings[age] = min(realwage, 55500) * pow(1.051, age - (1992 - persvars->birthyear));
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\nlnptbase %6.3f", lnptbase);
    for (year = 1972; year < 2012; year++)
    { if (year == 1972) wnprintf("\near");
      else if (year % 10 == 2) wnprintf("\n   ");
      if (year > persvars->birthyear) wnprintf("%7.0f", earnings[year - persvars->birthyear]);
      }
    for (year = 1972; year < 2012; year++)
    { if (year == 1972) wnprintf("\nsse");
      else if (year % 10 == 2) wnprintf("\n   ");
      if (year > persvars->birthyear) wnprintf("%7.0f", ssearnings[year - persvars->birthyear]);
      }
    }


  /* calculate last age of full-time work */

  ftend = 0;

  for (age = 0; age < 100; age++)
  if (ft[age] == 1)
  ftend = age;


  /* calculate last age of pension work */

  pjobno = -1;
  pjobstart = 99;
  pjobend = 99;

  for (jobno = 2; jobno >= 0; jobno--)
  { if (earnvars->jobind[jobno] == 1 && jobhist->jobvars[jobno].pencoverage == 1
      && earnvars->fulltime[jobno] == 1)
    { startdate = earnvars->startdate[jobno];
      enddate = earnvars->enddate[jobno];

      if (0 < startdate && (jobno == 0 || (startdate < enddate && enddate <= 1993))
        && startdate <= persvars->birthyear + ftend)
      { pjobno = jobno;
        pjobstart = startdate - persvars->birthyear;

        if (jobno == 0)
        pjobend = min(retage, calcvars->origjobendage) - 1;
        else
        { pjobend = enddate - persvars->birthyear;

          if (pjobend - pjobstart > 3 || ten[pjobend] <= 1)
          { aveage = (pjobstart + pjobend) / 2;
            penage = aveage;

            for (age = aveage + 1; age <= pjobend; age++)
            if (age == penage + 1)
            if (ten[age] = ten[penage] + 1)
            penage = age;
            
            pjobend = penage;
            }
          }

        if (ftend < pjobend)
        pjobend = ftend;
        
        if (pjobend < pjobstart)
        pjobstart = pjobend; 
        }
      }
    }


  for (age = 25; age < 70; age++)
  { calcvars->spwage[age - 25] = earnings[age];
    calcvars->spsswage[age - 25] = ssearnings[age];
    }

  calcvars->pjobno    = pjobno;
  calcvars->pjobstart = pjobstart;
  calcvars->sppjobend = pjobend;
  calcvars->spftend   = ftend;

  }


/*---------------------------------------------------------------------------
			 CALC POTENTIAL WAGES routine
			 calculates wages for continued work
  ---------------------------------------------------------------------------*/

void calcPotentialWages(
  VARS *vars,
  EARNVARS *earnvars)

{ int    age, surv, jobno, year, cohort, birthyear;
  int    lastftage, lastftdate, lastftearnage, lastftstartage;
  int    origjobend, newjobstart;
  int    tenure, experience;
  int    lastptage, lastptearnage, nolimitage;
  int    nearestftjob, nearestftdist;
  int    startdate, enddate;
  int    ed, race, marstat;                                        
  int    sempl, occ, ind, pens, healthage;
  int    pjobend, totftyears, goodCareer;
  double earnings, realwage, lnftbase, lnptbase;

  int    ten[100], exper[100], nonss[100];
  double ftearnings[100], ssearnings[100], secearnings[100], prearnings[100];

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
   /* 1947-1992 average weekly earnings           */
   /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  static double ssmax[42] = {
			     3600,  3600,  3600,  3600,  4200,  4200,  4200,  4200,  4800,
	  4800,  4800,  4800,  4800,  4800,  4800,  6600,  6600,  7800,  7800,
	  7800,  7800,  9000, 10800, 13200, 14100, 15300, 16500, 17700, 22900,
   25900, 29700, 32400, 35700, 37800, 39600, 42000, 43800, 45000, 48000,
   51300, 53400, 55500};
   /* Social Security maximum taxable earnings from Ann Statistical Suppl, 1995 */

  PERSVARS  *persvars;
  JOBHIST   *jobhist;
  PANELVARS *panelvars;
  CALCVARS  *calcvars;

  persvars  = &(vars->persvars);
  jobhist   = &(vars->jobhist);
  panelvars = &(vars->panelvars);
  calcvars  = &(vars->calcvars);


  cohort = persvars->birthyear;

  for (age = 0; age < 100; age++)
  { ten[age] = earnvars->ten[age];
    exper[age] = earnvars->exper[age];
    nonss[age] = earnvars->nonss[age];
    }

  for (age = 0; age < 100; age++)
  { ftearnings[age] = earnvars->earnings[age];
    ssearnings[age] = earnvars->ssearnings[age];
    }


  /* find last full-time age and last full-time earnings in 1992 */

  lastftage = 0;

  for (age = 1992 - cohort; age >= 0; age--)
  if (lastftage == 0 && earnvars->ft[age] == 1)
  lastftage = age;

  lastftearnage = 0;

  for (age = lastftage; age >= 0; age--)
  if (lastftearnage == 0 && earnvars->ft[age] == 1 && earnvars->earnings[age] > 0)
  lastftearnage = age;

  if ((cohort + lastftearnage < 1992 || earnvars->jobind[0] < 1 || earnvars->fulltime[0] < 1)
   && earnvars->ten[lastftearnage] > 0 && earnvars->ft[lastftearnage - 1] == 1
   && earnvars->earnings[lastftearnage] < earnvars->earnings[lastftearnage - 1])
  lastftearnage = lastftearnage - 1;

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  wnprintf("\nlftearn %d", lastftearnage);

  if (lastftearnage == 0)
  { lastftage = 50;
    lastftearnage = 50;
    ten[50] = 0;
    exper[50] = 0;

    realwage = 24876.80;           /* median in hrsftwag.log */

    if (cohort + 50 < 1947)
    earnings = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
    else if (cohort + 50 < 1992)
    earnings = realwage * wageind[cohort + 50 - 1947] / wageind[1992 - 1947];
    else earnings = realwage * pow(1.051, cohort + 50 - 1992);

    ftearnings[50] = earnings;
    ssearnings[50] = earnings;
    }


  /* find any layoffs, accepted eow's or new full-time jobs after 1992 */
  
  origjobend = 99;
  newjobstart = 99;

  if (calcvars->layoffage <= min(calcvars->origjobendage, 98))
  { origjobend = calcvars->layoffage;
    
    if (calcvars->layoffage < calcvars->newjobstartage
     && calcvars->newjobstartage < 99) 
    newjobstart = calcvars->newjobstartage;
    else newjobstart = calcvars->layoffage;
    }
  else if (calcvars->newjobstartage < 99)
  { if (calcvars->origjobendage < 99)
    { origjobend = calcvars->origjobendage;
      newjobstart = calcvars->newjobstartage;
      }
    else
    { origjobend = lastftage;
      newjobstart = max(lastftage, calcvars->newjobstartage);
      }
    }


  /* update potential tenure and experience vectors and calculate ft wages */

  for (age = lastftage + 1; age < 70; age++)
  { exper[age] = exper[age - 1] + 1;

    if (age < origjobend)
    { ten[age] = ten[age - 1] + 1;
      nonss[age] = nonss[age - 1];
      }
    else if (origjobend <= age && age <= newjobstart)
    { ten[age] = 0;
      nonss[age] = 0;
      }
    else
    { ten[age] = ten[age - 1] + 1;
      nonss[age] = 0;
      }
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { wnprintf("\nage "); for (age = 35; age < 75; age++) wnprintf("%2d", age);
    wnprintf("\nexp "); for (age = 35; age < 75; age++) wnprintf("%2d", exper[age]);
    wnprintf("\nten "); for (age = 35; age < 75; age++) wnprintf("%2d", ten[age]);
    }
    
  lastftdate = lastftearnage + cohort;

  if (lastftdate < 1947)
  realwage = ftearnings[lastftearnage] * wageind[1992 - 1947] / wageind[1947 - 1947];
  else if (lastftdate < 1992)
  realwage = ftearnings[lastftearnage] * wageind[1992 - 1947] / wageind[lastftdate - 1947];
  else realwage = ftearnings[lastftearnage];

  tenure = ten[lastftearnage];
  experience = exper[lastftearnage];

  lnftbase = log(realwage)                /* from hrsftwag.log */
    - 0.0051772 * experience + 0.0000557 * experience * experience
    - 0.0180837 * tenure     + 0.0002142 * tenure * tenure;

  for (age = lastftearnage + 1; age < 70; age++)
  if (age >= lastftage || earnvars->ft[age] == 1)
  { year = cohort + age;

    tenure = ten[age];
    experience = exper[age];

    realwage = exp(lnftbase
      + 0.0051772 * experience - 0.0000557 * experience * experience
		  + 0.0180837 * tenure     - 0.0002142 * tenure * tenure);

    if (year <= 1992)
    { if (year < 1947)
      ftearnings[age] = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
      else if (year < 1992)
      ftearnings[age] = realwage * wageind[year - 1947] / wageind[1992 - 1947];
      else ftearnings[age] = realwage;

      if (year >= 1951 && nonss[age] == 0)
      ssearnings[age] = min(ftearnings[age], ssmax[year - 1951]);
      }
    else
    { ftearnings[age] = realwage * pow(1.051, age - (1992 - cohort));

      if (nonss[age] == 0)
      ssearnings[age] = min(realwage, 55500) * pow(1.051, age - (1992 - cohort));
      }
    }


  /* find earnings in full-time jobs started after a reversal */

  for (age = 50; age < 70; age++)
  { if (0 <= ten[age] && ten[age] <= 42)
    { tenure = ten[age];
      secearnings[age] = ftearnings[age] / exp(0.0180837 * tenure - 0.0002142 * tenure * tenure);
      }
    else secearnings[age] = ftearnings[age];
    }


  /* find beginning of job if last ft job allowed pt work */

  nolimitage = 99;

  if (newjobstart < 99)
  lastftstartage = newjobstart;
  else
  { lastftstartage = 69;

    for (age = 69; age > 0; age--)
    if (age == lastftstartage && ten[age] == ten[age - 1] + 1)
    lastftstartage = age - 1;
    }

  if (earnvars->jobind[1] == 1)
  { enddate = earnvars->enddate[1];
    birthyear = persvars->birthyear;

    if (1900 <= birthyear && birthyear < enddate && enddate <= 1993)
    { age = enddate - birthyear;

      if (lastftstartage < age && age <= lastftage)
      if (earnvars->ft[age] == 1)
      { if (jobhist->jobvars[1].hourslimits == 1)
        nolimitage = lastftstartage;
        }
      }
    }

  for (surv = 0; surv <= 5; surv++)
  { age = panelvars->wavevars[surv].age;

    if (panelvars->wavevars[surv].interview == 1)
    if (lastftstartage <= age && age <= calcvars->ssdiage)
    if (panelvars->wavevars[surv].ret == 5)
    if (panelvars->wavevars[surv].hours > 30)
    { if (panelvars->wavevars[surv].hourslimits == 1)
      nolimitage = lastftstartage;
      else nolimitage = 99;
      }
    }


  /* find partial retirement wage base */

  lastptage = 0;

  for (age = 1992 - cohort; age > lastftage; age--)
  if (lastptage == 0 && earnvars->ft[age] == 0)
  lastptage = age;

  lastptearnage = 0;

  for (age = lastptage; age > lastftage; age--)
  if (lastptearnage == 0 && earnvars->ft[age] == 0 && earnvars->earnings[age] > 0)
  lastptearnage = age;

  if (earnvars->ten[lastptearnage] > 0 && earnvars->ft[lastptearnage - 1] == 1
   && earnvars->earnings[lastptearnage] < earnvars->earnings[lastptearnage - 1])
  lastptearnage = lastptearnage - 1;

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  wnprintf("\nlptearn %d", lastptearnage);

  if (lastptearnage > 0)
  { earnings = earnvars->earnings[lastptearnage];
    experience = earnvars->exper[lastptearnage];

    if (persvars->caseid == printcaseid || printcaseid == 99999)
    wnprintf(" %6.0f %2d", earnings, experience);

    if (cohort + lastptearnage < 1947)
    realwage = earnings * wageind[1992 - 1947] / wageind[1947 - 1947];
    else if (cohort + lastptearnage < 1992)
    realwage = earnings * wageind[1992 - 1947] / wageind[cohort + lastptearnage - 1947];
    else realwage = earnings;

    lnptbase = log(realwage)
      - 0.0103480 * experience + 0.0001875 * experience * experience;
    }
  else
  { nearestftjob = -1;
    nearestftdist = 99;

    for (jobno = 2; jobno >= 0; jobno--)
    if (jobhist->jobvars[jobno].jobind == 1 && earnvars->fulltime[jobno] == 1)
    { startdate = earnvars->startdate[jobno];
      enddate = earnvars->enddate[jobno];

      if (1920 <= startdate && startdate <= cohort + lastftearnage
       && cohort + lastftearnage <= enddate && enddate <= 1993)
      { nearestftjob = jobno;
        nearestftdist = 0;
        }
      else if (cohort + lastftearnage <= startdate && startdate <= 1993)
      { if (startdate - (cohort + lastftearnage) <= nearestftdist)
        { nearestftjob = jobno;
          nearestftdist = startdate - (cohort + lastftearnage);
          }
        }
      else if (1920 <= enddate && enddate <= cohort + lastftearnage)
      { if (cohort + lastftearnage - enddate <= nearestftdist)
        { nearestftjob = jobno;
          nearestftdist = cohort + lastftearnage - enddate;
          }
        }
      }

    if (nearestftjob >= 0)
    { sempl = jobhist->jobvars[nearestftjob].sempl;
      occ   = jobhist->jobvars[nearestftjob].occ;
      ind   = jobhist->jobvars[nearestftjob].ind;
      pens  = jobhist->jobvars[nearestftjob].pencoverage;
      }
    else
    { sempl = -9;
      occ   = -9;
      ind   = -9;
      pens  = -9;
      }

    ed = persvars->ed;
    race = persvars->race;
    marstat = panelvars->wavevars[0].marstat;
    healthage = calcvars->healthage;

    /* find full-time base */

    earnings = ftearnings[lastftearnage];

    if (cohort + lastftearnage < 1947)
    realwage = earnings * wageind[1992 - 1947] / wageind[1947 - 1947];
    else if (cohort + lastftearnage < 1992)
    realwage = earnings * wageind[1992 - 1947] / wageind[cohort + lastftearnage - 1947];
    else realwage = earnings;

    tenure     = earnvars->ten[lastftearnage];
    experience = earnvars->exper[lastftearnage];

    lnftbase = log(realwage)         /* from hrsftwag.log */
      - 0.0051772 * experience + 0.0000557 * experience * experience
	   - 0.0180837 * tenure     + 0.0002142 * tenure * tenure;

    if (ed  < 12)             lnftbase = lnftbase + 0.1299299;   /* hsdrop  */
    if (13 <= ed && ed <= 15) lnftbase = lnftbase - 0.1385978;   /* somecol */
    if (ed == 16)             lnftbase = lnftbase - 0.3307412;   /* colgrad */
    if (17 <= ed)             lnftbase = lnftbase - 0.4314858;   /* advdegr */
    if (race == 2)            lnftbase = lnftbase + 0.0594629;   /* black */
    if (race == 3)            lnftbase = lnftbase + 0.0666119;   /* hispanic */
    if (marstat == 1)         lnftbase = lnftbase - 0.0862579;   /* married */
    if (healthage <= age)     lnftbase = lnftbase + 0.0363754;   /* health */
    if (sempl == 1)           lnftbase = lnftbase + 0.0712693;   /* sempl */
    if (ind == 3 || ind == 4) lnftbase = lnftbase - 0.0289258;   /* mfg */
    if (ind == 13)            lnftbase = lnftbase - 0.0557995;   /* govt */
    if (occ == 1 || occ == 2) lnftbase = lnftbase - 0.2266077;   /* mgtprf */
    if (occ == 3 || occ == 4) lnftbase = lnftbase - 0.0331493;   /* whcol */
    if (pens == 1)            lnftbase = lnftbase - 0.2443663;   /* pens */

    /* find partial retirement base */

    lnptbase = lnftbase + 0.0332661;           /* from hrsptwag.log */

    if (ed  < 12)             lnptbase = lnptbase - 0.1753805;   /* hsdrop  */
    if (13 <= ed && ed <= 15) lnptbase = lnptbase + 0.0811179;   /* somecol */
    if (ed == 16)             lnptbase = lnptbase + 0.0946980;   /* colgrad */
    if (17 <= ed)             lnptbase = lnptbase + 0.4370315;   /* advdegr */
    if (race == 2)            lnptbase = lnptbase - 0.0266918;   /* black */
    if (race == 3)            lnptbase = lnptbase - 0.0488308;   /* hispanic */
    if (marstat == 1)         lnptbase = lnptbase + 0.0283834;   /* married */
    if (healthage <= age)     lnptbase = lnptbase + 0.0099354;   /* health */

    lnptbase = lnptbase + log(0.5);            /* half-time */

    if (persvars->caseid == printcaseid || printcaseid == 99999)
    wnprintf("\nbases %6.3f %6.3f", lnftbase, lnptbase);
    }


  /* impute partial retirement wages */

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  wnprintf("\nbases: %6.3f %d %6.3f %d", lnftbase, lastftearnage, lnptbase, lastptearnage);

  for (age = 50; age < 70; age++)
  { if (age < nolimitage)
    { year = cohort + age;
    
      experience = exper[age];

      realwage = exp(lnptbase
        + 0.0103480 * experience - 0.0001875 * experience * experience);

      if (year <= 1992)
      { if (year < 1947)
        prearnings[age] = realwage * wageind[1947 - 1947] / wageind[1992 - 1947];
        else if (year < 1992)
        prearnings[age] = realwage * wageind[year - 1947] / wageind[1992 - 1947];
        else prearnings[age] = realwage;
        }
      else prearnings[age] = realwage * pow(1.051, age - (1992 - cohort));
      }
    else prearnings[age] = 0.5 * ftearnings[age];
    }

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  { for (age = 40; age < 70; age++)
    { if (age == 40) wnprintf("\nft ");
      else if (age % 10 == 0 && age > 40) wnprintf("\n   ");
      wnprintf("%7.0f", ftearnings[age]);
      }
    for (age = 50; age < 70; age++)
    { if (age == 50) wnprintf("\nss ");
      else if (age % 10 == 0 && age > 50) wnprintf("\n   ");
      wnprintf("%7.0f", ssearnings[age]);
      }
    for (age = 50; age < 70; age++)
    { if (age == 50) wnprintf("\npr ");
      else if (age % 10 == 0 && age > 50) wnprintf("\n   ");
      wnprintf("%7.0f", prearnings[age]);
      }
    }

  /* limit pr earnings to 50% of ft earnings */

  for (age = 50; age <= 69; age++)
  if (prearnings[age] > 0.5 * ftearnings[age])
  prearnings[age] = 0.5 * ftearnings[age];


  /* calculate termination date, if any, from last ft pension job */

  pjobend = calcvars->sppjobend;

  for (age = pjobend; age <= 68; age++)
  if (age == pjobend && ten[age] + 1 == ten[age + 1])
  pjobend = pjobend + 1;

  if (pjobend == 69)
  pjobend = 99;


  /* check for good full-time career */

  totftyears = 0;
  lastftage = 0;

  for (age = min(40, 1982 - cohort); age <= 1992 - cohort; age++)
  if (earnvars->ft[age] == 1 && ftearnings[age] > 0)
  { totftyears = totftyears + 1;
    lastftage = age;
    }

  if (totftyears <= 0.5 * (lastftage - min(40, 1982 - cohort) + 1) || totftyears < 5)
  goodCareer = 0;
  else goodCareer = 1;

  if ((cohort + 50 <= 1992 && lastftage <= 49)
   || (cohort + 50 > 1992 && earnvars->fulltime[0] != 1))
  goodCareer = 0;

  for (age = 25; age < 70; age++)
  { calcvars->ftwage[age - 25] = ftearnings[age];
    calcvars->sswage[age - 25] = ssearnings[age];
    }

  for (age = 50; age < 70; age++)
  { calcvars->prwage[age - 50] = prearnings[age];
    calcvars->secwage[age - 50] = secearnings[age];
    }


  if (origjobend < 99)
  calcvars->impjobendage = origjobend - 1;
  else calcvars->impjobendage = 99;

  calcvars->pjobend = pjobend;
  calcvars->goodobs.goodCareer = goodCareer;

  if (persvars->caseid == printcaseid || printcaseid == 99999)
  wnprintf(" C %d", goodCareer);

  }


/*---------------------------------------------------------------------------
			 CALC PENS STREAM routine
			 calculates pension amounts
  ---------------------------------------------------------------------------*/

void calcPensStream(
  VARS *vars)

{ int   recno, age;

  static int   idlocs[30000];
  PENVARS      penvars[5000];

  initpens(idlocs, penvars, vars);

  for (recno = 0; recno < 12652; recno++)
  { for (age = 50; age <= 70; age++)
    { vars[recno].calcvars.pensben[age - 50]  =  0;
      vars[recno].calcvars.penstart[age - 50]  = 99;
      vars[recno].calcvars.plumpsum[age - 50] =  0;
      vars[recno].calcvars.eowamount[age - 50] = 0;
      }

    vars[recno].calcvars.sppenben   =  0;
    vars[recno].calcvars.sppenstart = 99;
    vars[recno].calcvars.spplumpsum =  0;

    vars[recno].calcvars.goodobs.goodPens = 1;
    }

  calcLastPens(idlocs, penvars, vars);
  calcPrevPens(idlocs, penvars, vars);
  calcPensAsSpouse(idlocs, penvars, vars);

  }


/*---------------------------------------------------------------------------
			 CALC LAST PENS routine
			 calculates potential pension amounts on last full-time job
  ---------------------------------------------------------------------------*/

void calcLastPens(
  int     *idlocs,
  PENVARS *penvars,
  VARS    *vars)

{ int    age, recno, survey, penrec, penno, year;
  int    lines, nlines;
  int    pjobno, pjobstart, pjobend;
  int    caseid, codeid, seqno, penid, idloc;
  int    occ, ind, earningscat, enddate;
  int    pencount, matchlevel;
  int    llimocc, llimind, llimear;
  int    ulimocc, ulimind, ulimear;
  int    birthyear, spousebirthyear, matchid;
  int    gender, hireyear, retyear;
  int    hours, weeks, annhours;
  int    bentype, benage;
  int    codeidp, seqnop, caseidp, agep;
  int    lastcaseid;
  int    origjobendsurv, eowserv, eowaccept;
  double earnings, earningsp, lastearnings, survearnings;
  double wgrowth, contrib, wlinear, wquadr;
  double ftenure, fbenage, fretyear;
  double curben, vestedben, presval, ssbase;
  double eowamount, eowpension;

  int    thread1[2], thread2[2];
  int    benagev[21], samplevector[1250];
  double benamtv[21], lumpsumv[21], presvalv[21];
  double surv[120];

  char  inputstring[250];

  void  *pendatabuffer, *arglist1, *arglist2;

  typedef struct
  { int    caseid;
    int    benage[21];
    double benamt[21];
    double lumpsum[21];
    int    goodPens;
    } PENDATA;

  PENDATA *pendata;

  static int lindgroup[14] = { 0,  1,  1,  3,  3,  5,  6,  6,  6,  6,  6,  6, 12, 12};
  static int uindgroup[14] = { 0,  2,  2,  4,  4,  5, 11, 11, 11, 11, 11, 11, 12, 13};

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
 	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
  /* 1947-1992 average weekly earnings           */
  /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  FILE *infile, *outfile, *partic, *dbfile, *dcfile, *penfile, *datafile;
  int  handle;


  pendatabuffer = malloc(7500 * sizeof(PENDATA));
  pendata = (PENDATA *) pendatabuffer;

  if (_access("pendatal.pen", 0) < 0)
  { partic = fopen("partic.lst", "wt");
    dbfile = fopen("partic.dbl", "wt");
    dcfile = fopen("partic.dcl", "wt");

    nlines = 0;

    for (recno = 0; recno < 12652; recno++)
    { birthyear = vars[recno].persvars.birthyear;
      pjobno    = vars[recno].calcvars.pjobno;
      pjobstart = vars[recno].calcvars.pjobstart;
      pjobend   = vars[recno].calcvars.pjobend + 1;

      if (birthyear + pjobend > 1990)
      if (0 <= pjobno && pjobno <= 2)
      { caseid = vars[recno].persvars.caseid;

        penid = -1;

        for (penrec = 0; penrec < 5000; penrec++)
        { if (penvars[penrec].caseid == caseid)
          penid = penrec;
          }

        if (penid < 0)
        { occ = vars[recno].jobhist.jobvars[pjobno].occ;
          ind = vars[recno].jobhist.jobvars[pjobno].ind;
          earnings = vars[recno].jobhist.jobvars[pjobno].earnings;
          enddate  = vars[recno].jobhist.jobvars[pjobno].enddate;

          if (ind < 1 || ind >13)
          ind = 0;

               if (occ <= 0)   occ =  0;
          else if (occ <  3)   occ =  1;  /* Managers & Prof */
          else if (occ <  5)   occ =  2;  /* White Collar    */
          else if (occ < 18)   occ =  3;  /* Blue Collar     */
          else                 occ =  0;

          if (pjobno == 0)
          enddate = 1992;

          if (1900 < enddate && enddate <= 1992)
          { if (enddate < 1947)
            earnings = earnings * (wageind[1992 - 1947] / wageind[1947 - 1947]);
            else if (enddate < 1992)
            earnings = earnings * (wageind[1992 - 1947] / wageind[enddate - 1947]);

			      if (earnings < 10000)    earningscat = 1;
    		    else if (earnings < 20000)    earningscat = 2;
	   	      else if (earnings < 30000)    earningscat = 3;
		        else if (earnings < 40000)    earningscat = 4;
			      else if (earnings < 50000)    earningscat = 5;
			      else                          earningscat = 6;
            }
          else earningscat = -1;

          for (penrec = 0; penrec < 1250; penrec++)
          samplevector[penrec] = -1;

          pencount = 0;

          for (matchlevel = 0; matchlevel < 4; matchlevel ++)
          if (pencount == 0)
          { llimocc = occ;             ulimocc = occ;
            llimind = ind;             ulimind = ind;
            llimear = earningscat;     ulimear = earningscat;

	         switch(matchlevel)
	         { case  0:                                                          break;
             case  1:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];   break;
             case  2:  llimear = llimear - 1;      ulimear = ulimear + 1;      break;
             case  3:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];
                       llimear = llimear - 1;      ulimear = ulimear + 1;      break;
		         }

	         for (penrec = 0; penrec < 5000; penrec++)
           if ( penvars[penrec].caseid > 0
            && ((llimocc <= penvars[penrec].occ && penvars[penrec].occ <= ulimocc) || occ == 0)
	          && ((llimind <= penvars[penrec].ind && penvars[penrec].ind <= ulimind) || ind == 0)
	          && ((llimear <= penvars[penrec].earningscat && penvars[penrec].earningscat <= ulimear) || earningscat == -1))
            { samplevector[pencount] = penrec;
              pencount = pencount + 1;

              if (pencount >= 1250)
              error("Too many pensions for sample in calcLastPens");
              }
            }

          if (pencount == 0)
          error("No matching pensions in calcLastPens");

          penid = samplevector[recno % pencount];
          }

        matchid   = vars[recno].persvars.matchid;
        gender    = vars[recno].persvars.gender;
        hireyear  = birthyear + pjobstart;
        retyear   = birthyear + pjobend;
        hours     = vars[recno].jobhist.jobvars[pjobno].hours;
        weeks     = vars[recno].jobhist.jobvars[pjobno].weeks;

        if (pjobno == 0)
        contrib = vars[recno].jobhist.pencontr;
        else contrib = 5;

        if (0 < matchid && matchid < 30000)
        { spousebirthyear = vars[idlocs[matchid]].persvars.birthyear;

          if (spousebirthyear > 1993)
          if (gender == 1)
          spousebirthyear = birthyear + 2;
          else spousebirthyear = birthyear - 2;
          }
        else spousebirthyear = 0;

        if (hireyear <= birthyear + 15)
        hireyear = birthyear + 25;

        if (retyear > birthyear + 70)
        retyear = birthyear + 70;

        if (0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
        annhours = hours * weeks;
        else if (0 < hours && hours < 95)
        annhours = 52 * hours;
        else if (0 < weeks && weeks < 52)
        annhours = 40 * weeks;
        else annhours = 2080;

        if (1992 < retyear)
        survearnings = vars[recno].calcvars.ftwage[(1992 - birthyear) - 25];
        else survearnings = vars[recno].calcvars.ftwage[((retyear - 1) - birthyear) - 25]
                             * pow(1.051, 1992 - (retyear - 1));

        for (penno = 0; penno < 4; penno++)
        { codeid = penvars[penid].penids[penno][0];
          seqno  = penvars[penid].penids[penno][1];

          if (codeid > 0 && seqno > 0)
          { fprintf(partic, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 %5.2f 0 0\n",
              caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
              annhours, survearnings, contrib);

            if (codeid < 3000)
            penfile = dbfile;
            else penfile = dcfile;

            fprintf(penfile, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 %5.2f 0 0\n",
              caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
              annhours, survearnings, contrib);

            if (caseid == printcaseid || printcaseid == 99999)
            wnprintf("\n%5d %2d %4d %4d", caseid, pjobend, hireyear, retyear);

            fprintf(penfile, "%4d %4d", hireyear, retyear);

            lastearnings = 0;

            for (year = hireyear; year <= retyear; year++)
            { if ((year - hireyear) % 10 == 0)
              fprintf(penfile, "\n");

              if (year < retyear)
              { if (year - birthyear < 25)
                earnings = vars[recno].calcvars.ftwage[25 - 25]
                            * pow(1.051, year - birthyear - 25);
                else if (year - birthyear < 70)
                earnings = vars[recno].calcvars.ftwage[(year - birthyear) - 25];
                else earnings = vars[recno].calcvars.ftwage[69 - 25]
                                 * pow(1.051, year - birthyear - 69);
                }
              else earnings = 1.051 * lastearnings;

              if (earnings < 1)
              earnings = 1;

              fprintf(penfile, "%6.0f ", earnings);

              lastearnings = earnings;
              }

            fprintf(penfile, "-1\n");
            nlines = nlines + 1;
            }
          }
		    }
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);


    /* run pension provider program */

    thread1[0] = 1;      /* thread number */
    thread2[0] = 2;

    thread1[1] = 1;      /* calling subroutine number */
    thread2[1] = 1;

    arglist1 = (void *) &thread1;
    arglist2 = (void *) &thread2;

    _beginthread(penCalc, 65536, arglist1);
    _beginthread(penCalc, 65536, arglist2);

    while(thread1[0] > 0 || thread2[0] > 0)
    checkMessages();


    /* process results */

    partic = fopen("partic.lst", "rt");
    dbfile = fopen("penout.dbl", "rt");
    dcfile = fopen("penout.dcl", "rt");

    pendatabuffer = malloc(7500 * sizeof(PENDATA));
    pendata = (PENDATA *) pendatabuffer;

    caseidp    = 0;
    codeidp    = 0;
    seqnop     = 0;
    lastcaseid = 0;

    pencount = 0;

    for (lines = 0; lines <= nlines; lines++)
    { if(fgets(inputstring, 250, partic) != NULL)
      if (strlen(inputstring) >= 15)
      { sscanf(inputstring, "%d %d %d %d %d %d %d %d %d %f %f %f %f %f",
         &caseid, &codeid, &seqno, &spousebirthyear, &gender, &birthyear, &hireyear, &retyear,
         &annhours, &earnings, &wgrowth, &contrib, &wlinear, &wquadr);
        }

      if (lines == 0 || lines == nlines || caseid != lastcaseid)
      { if (lines > 0)
        { for (age = 50; age <= 70; age++)
          { pendata[pencount].caseid = lastcaseid;
            pendata[pencount].benage[age - 50] = benagev[age - 50];
            pendata[pencount].benamt[age - 50] = benamtv[age - 50];
            pendata[pencount].lumpsum[age - 50] = lumpsumv[age - 50];
            }

          pendata[pencount].goodPens = 0;

          for (penrec = 0; penrec < 5000; penrec++)
          { if (penvars[penrec].caseid == lastcaseid)
            pendata[pencount].goodPens = 1;
            }

          pencount = pencount + 1;

          if (pencount >= 7500)
          error("Too many pensions in calcLastPens");
          }

        for (age = 50; age <= 70; age++)
        { benagev[age - 50] = 99;
          benamtv[age - 50] = 0;
          lumpsumv[age - 50] = 0;
          }
        }

      if (lines < nlines)
      { if (codeid < 3000)
        datafile = dbfile;
        else datafile = dcfile;

        for (age = hireyear - birthyear; age <= 76; age++)
        { fgets(inputstring, 250, datafile);

          sscanf(inputstring, "%d %d %d %f %d %f %f %f %f %d %f %f %f",
           &caseidp, &codeidp, &seqnop, &fretyear, &agep, &ftenure, &earningsp, &ssbase,
           &curben, &bentype, &vestedben, &fbenage, &presval);

          if (caseid != caseidp || codeid != codeidp || seqno != seqnop || age != agep)
          { wnprintf("\ncaseid %5d %5d", caseid, caseidp);
            wnprintf("\ncodeid %5d %5d", codeid, codeidp);
            wnprintf("\nseqno  %5d %5d", seqno,  seqnop );
            wnprintf("\nage    %5d %5d", age,    agep   );
            error("Mismatch in pension file in calcLastPens");
            }

          if (vestedben < 0 || presval < 0) wnprintf("\n %5d %4d %4d  %4d %4d %2d",
            caseid, codeid, seqno, birthyear, hireyear, age);

          if (50 <= age && age <= 70)
          { if (codeid < 3000)
            { benage = (int) fbenage;

              if (benage < age)
              benage = age;

              if (benage == age)
              { benagev[age - 50] = age;
                
                if (curben > vestedben)
                benamtv[age - 50] = curben;               /* in benage dollars */
                else benamtv[age - 50] = vestedben;
                }
              else
              { if (curben * pow(1.09, benage - age) > vestedben)
                { benagev[age - 50] = age;
                  benamtv[age - 50] = curben;
                  }
                else
                { benagev[age - 50] = benage;
                  benamtv[age - 50] = vestedben;
                  }
                }

              presvalv[age - 50] = presval;
              }
            else
            { presval = presval * pow(1.063, (birthyear + age) - 1992);
 
              getSurvivalRates(birthyear, gender, surv);

              if (birthyear + age > 1992)
              presval = presval * (surv[1992 - birthyear] / surv[age]);

              lumpsumv[age - 50] = lumpsumv[age - 50] + presval;    /* in retyear dollars */
              }
            }
          }

        /* penrec = 0;
        for (age = max(50, hireyear - birthyear) + 1; age <= 70; age++)
        { if (benamtv[age - 50] < benamtv[(age - 1) - 50]
           && presvalv[age - 50] < presvalv[(age - 1) - 50]) penrec = 1;
          if (lumpsumv[age - 50] < lumpsumv[(age - 1) - 50]) penrec = 1;
          }
        if (penrec == 1)
        { wnprintf("\ncaseid %4d %5d %4d %4d", pencount, caseid, codeid, seqno);
          wnprintf("\n    "); for (age = 50; age <= 70; age++) wnprintf("     %2d", age);
          wnprintf("\n    "); for (age = 50; age <= 70; age++) wnprintf("%7.0f", benamtv[age - 50]);
          wnprintf("\n    "); for (age = 50; age <= 70; age++) wnprintf("%7.0f", lumpsumv[age - 50]);
          } */

        lastcaseid = caseid;
        }
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);

    wnprintf("\nCount for last pensions: %d", pencount);
    pause();

    outfile = fopen("pendatal.pen", "wb");
    handle = _fileno(outfile);

    _write(handle, &pencount, sizeof(int));
    _write(handle, pendata, pencount * sizeof(PENDATA));

    fclose(outfile);
    }
  else
  { infile = fopen("pendatal.pen", "rb");
    handle = _fileno(infile);

    _read(handle, &pencount, sizeof(int));
    _read(handle, pendata, pencount * sizeof(PENDATA));

    fclose(infile);

    wnprintf("\nCount for last pensions: %d", pencount);
    }


  /* place data in VARS structure */

  for (penno = 0; penno < pencount; penno++)
  { caseid = pendata[penno].caseid;
    idloc = idlocs[caseid];

    pjobend = vars[idloc].calcvars.pjobend + 1;
    
    if (pjobend < 50)
    pjobend = 50;

    for (age = 50; age <= 70; age++)
    { if (age <= pjobend)
      { vars[idloc].calcvars.pensben[age - 50]  = pendata[penno].benamt[age - 50];
        vars[idloc].calcvars.penstart[age - 50] = pendata[penno].benage[age - 50];
        vars[idloc].calcvars.plumpsum[age - 50] = pendata[penno].lumpsum[age - 50];
        }
      else 
      { vars[idloc].calcvars.pensben[age - 50]  = pendata[penno].benamt[pjobend - 50];
        vars[idloc].calcvars.penstart[age - 50] = pendata[penno].benage[pjobend - 50];
        vars[idloc].calcvars.plumpsum[age - 50] = pendata[penno].lumpsum[pjobend - 50];
        }
      }

    vars[idlocs[caseid]].calcvars.goodobs.goodPens = pendata[penno].goodPens;
    }

  for (recno = 0; recno < 12652; recno++)
  { origjobendsurv = vars[recno].calcvars.origjobendsurv;
    
    for (survey = 1; survey <= origjobendsurv; survey++)
    if (vars[recno].panelvars.wavevars[survey].interview == 1)
    { eowamount  = vars[recno].panelvars.wavevars[survey].eowamount;
      eowpension = vars[recno].panelvars.wavevars[survey].eowpens;
      eowserv    = vars[recno].panelvars.wavevars[survey].eowservice;
      eowaccept  = vars[recno].panelvars.wavevars[survey].eowaccept;
      
      if (eowamount > 0 || eowpension > 0 || eowserv > 0)
      { if (survey < origjobendsurv)
        age = vars[recno].panelvars.wavevars[survey].age - 1;
        else if (eowaccept < 1)
        age = vars[recno].calcvars.origjobendage - 1;
        else age = vars[recno].calcvars.origjobendage;

        if (vars[recno].persvars.caseid == printcaseid || printcaseid == 99999)
        { wnprintf("\n caseid %5d", vars[recno].persvars.caseid);
          wnprintf("\n%5d %1d %1d %6.0f %5.0f %2d %1d %2d %2d %2d", recno, survey, origjobendsurv,
            eowamount, eowpension, eowserv, eowaccept, age, min(vars[recno].calcvars.pjobend + 1, 99),
            vars[recno].calcvars.newjobstartage);

          for (age = 50; age < 70; age++)
          { if (age == 60) wnprintf("\n                                    ");
            wnprintf("%6.0f", vars[recno].calcvars.ftwage[age - 25]);
            }

          for (age = 50; age < 70; age++)
          { if (age == 50 || age == 60) wnprintf("\n                                    ");
            wnprintf("%6.0f", vars[recno].calcvars.pensben[age - 50]);
            }
          }

        if (50 <= age && age <= 70)
        { if (eowamount > 0)
          vars[recno].calcvars.eowamount[age - 50]
           = vars[recno].calcvars.eowamount[age - 50] + eowamount;

          if (eowpension > 0)
          vars[recno].calcvars.pensben[age - 50]
            = vars[recno].calcvars.pensben[age - 50] + eowpension;

          if (eowserv > 0)
          vars[recno].calcvars.pensben[age - 50]
           = max(vars[recno].calcvars.pensben[min(age + eowserv, 70) - 50]
                   / pow(1.051, min(eowserv, 70 - age)),
                 vars[recno].calcvars.pensben[age - 50]
                  * (double) ((age + eowserv) - vars[recno].calcvars.pjobstart)
                   / max(age - vars[recno].calcvars.pjobstart, 5));
          }
        }
      }
    }

  free(pendatabuffer);

  }


/*---------------------------------------------------------------------------
			 CALC PREV PENS routine
			 calculates pension amount from previous full-time jobs
  ---------------------------------------------------------------------------*/

void calcPrevPens(
  int     *idlocs,
  PENVARS *penvars,
  VARS *vars)

{ int    age, recno, penrec, penno, year;
  int    lines, nlines;
  int    pjobno, pjobstart, pjobend;
  int    caseid, idloc, penloc;
  int    occ, ind, earningscat, enddate;
  int    pencount, matchlevel, skipno;
  int    llimocc, llimind, llimear;
  int    ulimocc, ulimind, ulimear;
  int    birthyear, matchid, spousebirthyear;
  int    gender, hireyear, retyear;
  int    hours, weeks, annhours;
  int    codeid, seqno;
  int    bentype, benage;
  int    codeidp, seqnop, caseidp, genderp;
  int    lastcaseid, lastpenrec;
  double earnings, earningsp, lastearnings, survearnings;
  double wgrowth, contrib, wlinear, wquadr;
  double fspousebirthyear, fbirthyear, fhireyear, fretyear;
  double fage, ftenure, fbenage;
  double replrate, generosity, repl30;
  double benamt, presval, ben65;

  int    thread1[2], thread2[2];
  int    idvector[21], benagev[21], samplevector[1250];
  double benamtv[21], lumpsumv[21];
  double surv[120];

  char   inputstring[250];

  void   *arglist1, *arglist2;

  typedef struct
  { int    caseid;
    int    benage;
    double benamt;
    double lumpsum;
    } PENDATA;

  PENDATA pendata[1000];

  static int lindgroup[14] = { 0,  1,  1,  3,  3,  5,  6,  6,  6,  6,  6,  6, 12, 12};
  static int uindgroup[14] = { 0,  2,  2,  4,  4,  5, 11, 11, 11, 11, 11, 11, 12, 13};

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
   /* 1947-1992 average weekly earnings           */
   /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  FILE *infile, *outfile, *partic, *dbfile, *dcfile, *penfile, *datafile;
  int   handle;

  if (_access("pendatap.pen", 0) < 0)
  { partic = fopen("partic.pst", "wt");
    dbfile = fopen("partic.dbp", "wt");
    dcfile = fopen("partic.dcp", "wt");

    nlines = 0;

    for (recno = 0; recno < 12652; recno++)
    { birthyear = vars[recno].persvars.birthyear;
      pjobno    = vars[recno].calcvars.pjobno;
      pjobstart = vars[recno].calcvars.pjobstart;
      pjobend   = vars[recno].calcvars.pjobend + 1;

      if (birthyear + pjobend <= 1990)
      if (0 <= pjobno && pjobno <= 2)
      { caseid = vars[recno].persvars.caseid;

        penloc = -1;

        for (penrec = 0; penrec < 5000; penrec++)
        { if (penvars[penrec].caseid == caseid)
          penloc = penrec;
          }

        if (penloc >= 0)
        { idvector[0] = penloc;

          for (idloc = 1; idloc < 21; idloc++)
          idvector[idloc] = -1;
          }
        else
        { occ = vars[recno].jobhist.jobvars[pjobno].occ;
          ind = vars[recno].jobhist.jobvars[pjobno].ind;
          earnings = vars[recno].jobhist.jobvars[pjobno].earnings;
          enddate  = vars[recno].jobhist.jobvars[pjobno].enddate;

          if (ind < 1 || ind >13)
          ind = 0;

               if (occ <= 0)   occ =  0;
          else if (occ <  3)   occ =  1;  /* Managers & Prof */
          else if (occ <  5)   occ =  2;  /* White Collar    */
          else if (occ < 18)   occ =  3;  /* Blue Collar     */
          else                 occ =  0;

          if (pjobno == 0)
          enddate = 1992;

          if (1900 < enddate && enddate <= 1992)
          { if (enddate < 1947)
            earnings = earnings * (wageind[1992 - 1947] / wageind[1947 - 1947]);
            else if (enddate < 1992)
            earnings = earnings * (wageind[1992 - 1947] / wageind[enddate - 1947]);

			      if (earnings < 10000)    earningscat = 1;
    		    else if (earnings < 20000)    earningscat = 2;
	   	      else if (earnings < 30000)    earningscat = 3;
		        else if (earnings < 40000)    earningscat = 4;
			      else if (earnings < 50000)    earningscat = 5;
			      else                          earningscat = 6;
            }
          else earningscat = -1;

          for (penrec = 0; penrec < 1250; penrec++)
          samplevector[penrec] = -1;

          pencount = 0;

          for (matchlevel = 0; matchlevel < 4; matchlevel ++)
          if (pencount == 0)
          { llimocc = occ;             ulimocc = occ;
            llimind = ind;             ulimind = ind;
            llimear = earningscat;     ulimear = earningscat;

	          switch(matchlevel)
	          { case  0:                                                          break;
              case  1:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];   break;
              case  2:  llimear = llimear - 1;      ulimear = ulimear + 1;      break;
              case  3:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];
                        llimear = llimear - 1;      ulimear = ulimear + 1;      break;
		          }

	          for (penrec = 0; penrec < 5000; penrec++)
            if ( penvars[penrec].caseid > 0
             && ((llimocc <= penvars[penrec].occ && penvars[penrec].occ <= ulimocc) || occ == 0)
	           && ((llimind <= penvars[penrec].ind && penvars[penrec].ind <= ulimind) || ind == 0)
	           && ((llimear <= penvars[penrec].earningscat && penvars[penrec].earningscat <= ulimear) || earningscat == -1))
            { samplevector[pencount] = penrec;
              pencount = pencount + 1;

              if (pencount >= 1250)
              error("Too many pensions for sample in calcPrevPens");
              }
            }

          if (pencount == 0)
          error("No matching pensions in calcPrevPens");

          if (pencount > 21)
          { skipno = (pencount / 21);

            for (penrec = 0; penrec < 21; penrec++)
            idvector[penrec] = samplevector[skipno * penrec];
            }
          else
          { for (penrec = 0; penrec < 21; penrec++)
            if (penrec < pencount)
            idvector[penrec] = samplevector[penrec];
            else idvector[penrec] = -1;
            }
          }

        matchid   = vars[recno].persvars.matchid;
        gender    = vars[recno].persvars.gender;
        hireyear  = birthyear + pjobstart;
        retyear   = birthyear + pjobend;
        hours     = vars[recno].jobhist.jobvars[pjobno].hours;
        weeks     = vars[recno].jobhist.jobvars[pjobno].weeks;

        if (0 < matchid && matchid < 30000)
        { spousebirthyear = vars[idlocs[matchid]].persvars.birthyear;

          if (spousebirthyear > 1993)
          if (gender == 1)
          spousebirthyear = birthyear + 2;
          else spousebirthyear = birthyear - 2;
          }
        else spousebirthyear = 0;

        if (hireyear <= birthyear + 15)
        hireyear = birthyear + 25;

        if (retyear > birthyear + 70)
        retyear = birthyear + 70;

        if (0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
        annhours = hours * weeks;
        else if (0 < hours && hours < 95)
        annhours = 52 * hours;
        else if (0 < weeks && weeks < 52)
        annhours = 40 * weeks;
        else annhours = 2080;

        if (1992 < retyear)
        survearnings = vars[recno].calcvars.ftwage[(1992 - birthyear) - 25];
        else if ((retyear - 1) - birthyear < 25)
        survearnings = vars[recno].calcvars.ftwage[25 - 25]
                        * pow(1.051, ((retyear - 1) - birthyear - 25));
        else survearnings = vars[recno].calcvars.ftwage[((retyear - 1) - birthyear) - 25]
                             * pow(1.051, 1992 - (retyear - 1));

        for (penrec = 0; penrec < 21; penrec++)
        if (idvector[penrec] >= 0)
        { for (penno = 0; penno < 4; penno++)
          { codeid = penvars[idvector[penrec]].penids[penno][0];
            seqno  = penvars[idvector[penrec]].penids[penno][1];

            if (codeid > 0 && seqno > 0)
            { fprintf(partic, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 5 0 0 %2d\n",
                caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
                annhours, survearnings, penrec);

              if (codeid < 3000)
              penfile = dbfile;
              else penfile = dcfile;

              fprintf(penfile, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 5 0 0\n",
                caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
                annhours, survearnings);

              fprintf(penfile, "%4d %4d", hireyear, retyear);

              lastearnings = 0;

              for (year = hireyear; year <= retyear; year++)
              { if ((year - hireyear) % 10 == 0)
                fprintf(penfile, "\n");

                if (year < retyear)
                { if (year - birthyear < 25)
                  earnings = vars[recno].calcvars.ftwage[25 - 25]
                              * pow(1.051, year - birthyear - 25);
                  else if (year - birthyear < 70)
                  earnings = vars[recno].calcvars.ftwage[(year - birthyear) - 25];
                  else earnings = vars[recno].calcvars.ftwage[69 - 25]
                                   * pow(1.051, year - birthyear - 69);

                  if (year == retyear - 1)
                  earnings = max(earnings, 1.051 * lastearnings);
                  }
                else earnings = 1.051 * lastearnings;

                if (earnings < 1)
                earnings = 1;

                fprintf(penfile, "%6.0f ", earnings);

                lastearnings = earnings;
                }

              fprintf(penfile, "-1\n");
              nlines = nlines + 1;
              }
            }
          }
		    }
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);


    /* run pension provider program */

    thread1[0] = 1;      /* thread number */
    thread2[0] = 2;

    thread1[1] = 2;      /* calling subroutine number */
    thread2[1] = 2;

    arglist1 = (void *) &thread1;
    arglist2 = (void *) &thread2;

    _beginthread(penCalc, 65536, arglist1);
    _beginthread(penCalc, 65536, arglist2);

    while(thread1[0] > 0 || thread2[0] > 0)
    checkMessages();
    

    /* process results */

    partic = fopen("partic.pst", "rt");
    dbfile = fopen("penout.dbp", "rt");
    dcfile = fopen("penout.dcp", "rt");

    caseidp    = 0;
    codeidp    = 0;
    seqnop     = 0;
    lastcaseid = 0;
    lastpenrec = 0;

    pencount = 0;

    for (lines = 0; lines <= nlines; lines++)
    { if(fgets(inputstring, 250, partic) != NULL)
      if (strlen(inputstring) >= 15)
      { sscanf(inputstring, "%d %d %d %d %d %d %d %d %d %f %f %f %f %f %d",
         &caseid, &codeid, &seqno, &spousebirthyear, &gender, &birthyear, &hireyear, &retyear,
         &annhours, &earnings, &wgrowth, &contrib, &wlinear, &wquadr, &penrec);
        }

      if (codeid < 3000)
      datafile = dbfile;
      else datafile = dcfile;

      if (fgets(inputstring, 250, datafile) != NULL)
      if (strlen(inputstring) >= 15)
      { sscanf(inputstring, "%d %d %d %d %f %f %f %f %f %f %f %d %f %f %f %f %f %f %f",
         &caseidp, &codeidp, &seqnop, &genderp, &fspousebirthyear, &fbirthyear,
         &fhireyear, &fretyear, &earningsp, &fage, &ftenure, &bentype, &fbenage,
         &replrate, &generosity, &repl30, &benamt, &presval, &ben65);
        }

      if (benamt < 0) wnprintf("\n %5d %4d %4d  %4d %4d %4d",
        caseid, codeid, seqno, birthyear, hireyear, retyear);

      if (caseid != caseidp || codeid != codeidp || seqno != seqnop)
      { wnprintf("\n%5d %5d %4d %4d %4d %4d %2d", caseid, caseidp, codeid, codeidp, seqno, seqnop, penrec);
        error("Mismatch between partic and penout in calcPrevPens");
        }

      if (lines == 0 || lines == nlines || caseid != lastcaseid)
      { if (lines > 0)
        { pendata[pencount].caseid = lastcaseid;
          pendata[pencount].benage = median(benagev, lastpenrec + 1);
          pendata[pencount].benamt = fmedian(benamtv, lastpenrec + 1);
          pendata[pencount].lumpsum = fmedian(lumpsumv, lastpenrec + 1);

          pencount = pencount + 1;

          if (pencount >= 1000)
          error("Too many pensions in calcPrevPens");
          }

        for (penno = 0; penno < 21; penno++)
        { benagev[penno] = 99;
          benamtv[penno] = 0;
          lumpsumv[penno] = 0;
          }
        }

      if (lines < nlines)
      { if (codeid < 3000)
        { benage = (int) fbenage;

          if (benage < retyear- birthyear)
          benage = retyear - birthyear;

          if (benage < benagev[penrec])
          { benagev[penrec] = benage;
            benamtv[penrec] = benamt;             /* in benage dollars */
            }
          }
        else
        { presval = presval * pow(1.063, retyear - 1992);

          getSurvivalRates(birthyear, gender, surv);

          if (retyear > 1992)
          presval = presval * (surv[1992 - birthyear] / surv[retyear - birthyear]);

          lumpsumv[penrec] = lumpsumv[penrec] + presval;    /* in retyear dollars */
          }

        lastcaseid   = caseid;
        lastpenrec   = penrec;
        }
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);

    wnprintf("\nCount for previous pensions: %d", pencount);

    outfile = fopen("pendatap.pen", "wb");
    handle = _fileno(outfile);

    _write(handle, &pencount, sizeof(int));
    _write(handle, pendata, pencount * sizeof(PENDATA));

    fclose(outfile);
    }
  else
  { infile = fopen("pendatap.pen", "rb");
    handle = _fileno(infile);

    _read(handle, &pencount, sizeof(int));
    _read(handle, pendata, pencount * sizeof(PENDATA));

    fclose(infile);

    wnprintf("\nCount for previous pensions: %d", pencount);
    }


  /* place data in VARS structure */

  for (penno = 0; penno < pencount; penno++)
  { caseid = pendata[penno].caseid;
    pjobend = vars[idlocs[caseid]].calcvars.pjobend;

    for (age = 50; age <= 70; age++)
    { if (age <= pjobend)
      { vars[idlocs[caseid]].calcvars.pensben[age - 50]  = 0;
        vars[idlocs[caseid]].calcvars.penstart[age - 50] = 99;
        vars[idlocs[caseid]].calcvars.plumpsum[age - 50] = 0;
        }
      else
      { vars[idlocs[caseid]].calcvars.pensben[age - 50]  = pendata[penno].benamt;
        vars[idlocs[caseid]].calcvars.penstart[age - 50] = pendata[penno].benage;
        vars[idlocs[caseid]].calcvars.plumpsum[age - 50] = pendata[penno].lumpsum;
        }
      }
    }

  }


/*---------------------------------------------------------------------------
			 CALC PENS AS SPOUSE routine
			 calculates pension amount as a spouse (with given retirement date)
  ---------------------------------------------------------------------------*/

void calcPensAsSpouse(
  int     *idlocs,
  PENVARS *penvars,
  VARS    *vars)

{ int    recno, penrec, penno, year;
  int    lines, nlines;
  int    pjobno, pjobstart, pjobend;
  int    caseid, idloc, penloc;
  int    occ, ind, earningscat, enddate;
  int    pencount, matchlevel, skipno;
  int    llimocc, llimind, llimear;
  int    ulimocc, ulimind, ulimear;
  int    birthyear, matchid, spousebirthyear;
  int    gender, hireyear, retyear;
  int    hours, weeks, annhours;
  int    codeid, seqno;
  int    bentype, benage;
  int    codeidp, seqnop, caseidp, genderp;
  int    lastcaseid, lastpenrec;
  double earnings, earningsp, lastearnings, survearnings;
  double wgrowth, contrib, wlinear, wquadr;
  double fspousebirthyear, fbirthyear, fhireyear, fretyear;
  double fage, ftenure, fbenage;
  double replrate, generosity, repl30;
  double benamt, presval, ben65;

  int    thread1[2], thread2[2];
  int    idvector[21], benagev[21], samplevector[1250];
  double benamtv[21], lumpsumv[21];
  double surv[120];

  char   inputstring[250];

  void   *arglist1, *arglist2;

  typedef struct
  { int    caseid;
    int    benage;
    double benamt;
    double lumpsum;
    } PENDATA;

  PENDATA pendata[7500];

  static int lindgroup[14] = { 0,  1,  1,  3,  3,  5,  6,  6,  6,  6,  6,  6, 12, 12};
  static int uindgroup[14] = { 0,  2,  2,  4,  4,  5, 11, 11, 11, 11, 11, 11, 12, 13};

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
   /* 1947-1992 average weekly earnings           */
   /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  FILE *infile, *outfile, *partic, *dbfile, *dcfile, *penfile, *datafile;
  int   handle;


  if (_access("pendatas.pen", 0) < 0)
  { partic = fopen("partic.sp", "wt");
    dbfile = fopen("partic.dbs", "wt");
    dcfile = fopen("partic.dcs", "wt");

    nlines = 0;

    for (recno = 0; recno < 12652; recno++)
    { pjobno    = vars[recno].calcvars.pjobno;
      pjobstart = vars[recno].calcvars.pjobstart;
      pjobend   = vars[recno].calcvars.sppjobend;

      if (0 <= pjobno && pjobno <= 2)
      { caseid = vars[recno].persvars.caseid;

        penloc = -1;

        for (penrec = 0; penrec < 5000; penrec++)
        { if (penvars[penrec].caseid == caseid)
          penloc = penrec;
          }

        if (penloc >= 0)
        { idvector[0] = penloc;

          for (idloc = 1; idloc < 21; idloc++)
          idvector[idloc] = -1;
          }
        else
        { occ = vars[recno].jobhist.jobvars[pjobno].occ;
          ind = vars[recno].jobhist.jobvars[pjobno].ind;
          earnings = vars[recno].jobhist.jobvars[pjobno].earnings;
          enddate  = vars[recno].jobhist.jobvars[pjobno].enddate;

          if (ind < 1 || ind >13)
          ind = 0;

               if (occ <= 0)   occ =  0;
          else if (occ <  3)   occ =  1;  /* Managers & Prof */
          else if (occ <  5)   occ =  2;  /* White Collar    */
          else if (occ < 18)   occ =  3;  /* Blue Collar     */
          else                 occ =  0;

          if (pjobno == 0)
          enddate = 1992;

          if (1900 < enddate && enddate <= 1992)
          { if (enddate < 1947)
            earnings = earnings * (wageind[1992 - 1947] / wageind[1947 - 1947]);
            else if (enddate < 1992)
            earnings = earnings * (wageind[1992 - 1947] / wageind[enddate - 1947]);

			      if (earnings < 10000)    earningscat = 1;
    		    else if (earnings < 20000)    earningscat = 2;
	   	      else if (earnings < 30000)    earningscat = 3;
		        else if (earnings < 40000)    earningscat = 4;
			      else if (earnings < 50000)    earningscat = 5;
			      else                          earningscat = 6;
            }
          else earningscat = -1;

          for (penrec = 0; penrec < 1250; penrec++)
          samplevector[penrec] = -1;

          pencount = 0;

          for (matchlevel = 0; matchlevel < 4; matchlevel ++)
          if (pencount == 0)
          { llimocc = occ;             ulimocc = occ;
            llimind = ind;             ulimind = ind;
            llimear = earningscat;     ulimear = earningscat;

	          switch(matchlevel)
	          { case  0:                                                          break;
              case  1:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];   break;
              case  2:  llimear = llimear - 1;      ulimear = ulimear + 1;      break;
              case  3:  llimind = lindgroup[ind];   ulimind = uindgroup[ind];
                        llimear = llimear - 1;      ulimear = ulimear + 1;      break;
		          }

	          for (penrec = 0; penrec < 5000; penrec++)
            if ( penvars[penrec].caseid > 0
             && ((llimocc <= penvars[penrec].occ && penvars[penrec].occ <= ulimocc) || occ == 0)
	           && ((llimind <= penvars[penrec].ind && penvars[penrec].ind <= ulimind) || ind == 0)
	           && ((llimear <= penvars[penrec].earningscat && penvars[penrec].earningscat <= ulimear) || earningscat == -1))
            { samplevector[pencount] = penrec;
              pencount = pencount + 1;

              if (pencount >= 1250)
              error("Too many pensions for sample in calcPensAsSpouse");
              }
            }

          if (pencount == 0)
          error("No matching pensions in calcPensAsSpouse");

          if (pencount > 21)
          { skipno = (pencount / 21);

            for (penrec = 0; penrec < 21; penrec++)
            idvector[penrec] = samplevector[skipno * penrec];
            }
          else
          { for (penrec = 0; penrec < 21; penrec++)
            if (penrec < pencount)
            idvector[penrec] = samplevector[penrec];
            else idvector[penrec] = -1;
            }
          }

        birthyear = vars[recno].persvars.birthyear;
        matchid   = vars[recno].persvars.matchid;
        gender    = vars[recno].persvars.gender;
        hireyear  = birthyear + pjobstart;
        retyear   = birthyear + pjobend + 1;
        hours     = vars[recno].jobhist.jobvars[pjobno].hours;
        weeks     = vars[recno].jobhist.jobvars[pjobno].weeks;

        if (pjobno == 0)
        contrib = vars[recno].jobhist.pencontr;
        else contrib = 5;

        if (0 < matchid && matchid < 30000)
        { spousebirthyear = vars[idlocs[matchid]].persvars.birthyear;

          if (spousebirthyear > 1993)
          if (gender == 1)
          spousebirthyear = birthyear + 2;
          else spousebirthyear = birthyear - 2;
          }
        else spousebirthyear = 0;

        if (hireyear <= birthyear + 15)
        hireyear = birthyear + 25;

        if (retyear > birthyear + 70)
        retyear = birthyear + 70;

        if (0 < hours && hours < 95 && 0 < weeks && weeks <= 52)
        annhours = hours * weeks;
        else if (0 < hours && hours < 95)
        annhours = 52 * hours;
        else if (0 < weeks && weeks < 52)
        annhours = 40 * weeks;
        else annhours = 2080;

        if (1992 < retyear)
        survearnings = vars[recno].calcvars.spwage[(1992 - birthyear) - 25];
        else if (retyear - birthyear < 25)
        survearnings = vars[recno].calcvars.spwage[25 - 25] * pow(1.051, retyear - birthyear - 25);
        else survearnings = vars[recno].calcvars.spwage[((retyear - 1) - birthyear) - 25]
                             * pow(1.051, 1992 - (retyear - 1));

        for (penrec = 0; penrec < 21; penrec++)
        if (idvector[penrec] >= 0)
        { for (penno = 0; penno < 4; penno++)
          { codeid = penvars[idvector[penrec]].penids[penno][0];
            seqno  = penvars[idvector[penrec]].penids[penno][1];

            if (codeid > 0 && seqno > 0)
            { fprintf(partic, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 %5.2f 0 0 %2d\n",
                caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
                annhours, survearnings, contrib, penrec);

              if (codeid < 3000)
              penfile = dbfile;
              else penfile = dcfile;

              fprintf(penfile, "%5d %4d %4d %4d %1d %4d %4d %4d %4d %6.0f 0 %5.2f 0 0\n",
                caseid, codeid, seqno, spousebirthyear, gender, birthyear, hireyear, retyear,
                annhours, survearnings, contrib);

              fprintf(penfile, "%4d %4d", hireyear, retyear);

              lastearnings = 0;

              for (year = hireyear; year <= retyear; year++)
              { if ((year - hireyear) % 10 == 0)
                fprintf(penfile, "\n");

                if (year < retyear)
                { if (year - birthyear < 25)
                  earnings = vars[recno].calcvars.spwage[25 - 25]
                              * pow(1.051, year - birthyear - 25);
                  else if (year - birthyear < 70)
                  earnings = vars[recno].calcvars.spwage[(year - birthyear) - 25];
                  else earnings = vars[recno].calcvars.spwage[69 - 25]
                                   * pow(1.051, year - birthyear - 69);

                  if (year == retyear - 1)
                  earnings = max(earnings, 1.051 * lastearnings);
                  }
                else earnings = 1.051 * lastearnings;

                if (earnings < 1)
                earnings = 1;

                fprintf(penfile, "%6.0f ", earnings);

                lastearnings = earnings;
                }

              fprintf(penfile, "-1\n");
              nlines = nlines + 1;
              }
            }
          }
		    }
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);


    /* run pension provider program */

    thread1[0] = 1;      /* thread number */
    thread2[0] = 2;

    thread1[1] = 3;      /* calling subroutine number */
    thread2[1] = 3;

    arglist1 = (void *) &thread1;
    arglist2 = (void *) &thread2;

    _beginthread(penCalc, 65536, arglist1);
    _beginthread(penCalc, 65536, arglist2);

    while(thread1[0] > 0 || thread2[0] > 0)
    checkMessages();


    /* process results */

    partic = fopen("partic.sp", "rt");
    dbfile = fopen("penout.dbs", "rt");
    dcfile = fopen("penout.dcs", "rt");

    caseidp    = 0;
    codeidp    = 0;
    seqnop     = 0;
    lastcaseid = 0;
    lastpenrec = 0;

    pencount = 0;

    for (lines = 0; lines <= nlines; lines++)
    { if(fgets(inputstring, 250, partic) != NULL)
      if (strlen(inputstring) >= 15)
      { sscanf(inputstring, "%d %d %d %d %d %d %d %d %d %f %f %f %f %f %d",
         &caseid, &codeid, &seqno, &spousebirthyear, &gender, &birthyear, &hireyear, &retyear,
         &annhours, &earnings, &wgrowth, &contrib, &wlinear, &wquadr, &penrec);
         }

      if (codeid < 3000)
      datafile = dbfile;
      else datafile = dcfile;

      if (fgets(inputstring, 250, datafile) != NULL)
      if (strlen(inputstring) >= 15)
      { sscanf(inputstring, "%d %d %d %d %f %f %f %f %f %f %f %d %f %f %f %f %f %f %f",
         &caseidp, &codeidp, &seqnop, &genderp, &fspousebirthyear, &fbirthyear,
         &fhireyear, &fretyear, &earningsp, &fage, &ftenure, &bentype, &fbenage,
         &replrate, &generosity, &repl30, &benamt, &presval, &ben65);
         }

      if (benamt < 0) wnprintf("\n %5d %4d %4d  %4d %4d %4d",
        caseid, codeid, seqno, birthyear, hireyear, retyear);

      if (caseid != caseidp || codeid != codeidp || seqno != seqnop)
      { wnprintf("\n%5d %5d %4d %4d %4d %4d %2d", caseid, caseidp, codeid, codeidp, seqno, seqnop, penrec);
        error("Mismatch between partic and penout in calcPensAsSpouse");
        }

      if (lines == nlines || caseid != lastcaseid)
      { if (lines > 0)
        { pendata[pencount].caseid = lastcaseid;
          pendata[pencount].benage = median(benagev, lastpenrec + 1);
          pendata[pencount].benamt = fmedian(benamtv, lastpenrec + 1);
          pendata[pencount].lumpsum = fmedian(lumpsumv, lastpenrec + 1);
          }

        for (penno = 0; penno < 21; penno++)
        { benagev[penno] = 99;
          benamtv[penno] = 0;
          lumpsumv[penno] = 0;
          }

        if (lines > 0)
        { pencount = pencount + 1;

          if (pencount >= 7500)
          error("Too many pensions in calcPensAsSpouse");
          }
        }

      if (codeid < 3000)
      { benage = (int) fbenage;

        if (benage < retyear - birthyear)
        benage = retyear - birthyear;

        if (benage < benagev[penrec])
        { benagev[penrec] = benage;
          benamtv[penrec] = benamt;             /* in benage dollars */
          }
        }
      else
      { presval = presval * pow(1.063, retyear - 1992);

        getSurvivalRates(birthyear, gender, surv);

        if (retyear > 1992)
        presval = presval * (surv[1992 - birthyear] / surv[retyear - birthyear]);

        lumpsumv[penrec] = lumpsumv[penrec] + presval;    /* in retyear dollars */
        }

      lastcaseid   = caseid;
      lastpenrec   = penrec;
      }

    fclose(partic);
    fclose(dbfile);
    fclose(dcfile);

    wnprintf("\nCount for pensions as spouse: %d", pencount);

    outfile = fopen("pendatas.pen", "wb");
    handle = _fileno(outfile);

    _write(handle, &pencount, sizeof(int));
    _write(handle, pendata, pencount * sizeof(PENDATA));

    fclose(outfile);
    }
  else
  { infile = fopen("pendatas.pen", "rb");
    handle = _fileno(infile);

    _read(handle, &pencount, sizeof(int));
    _read(handle, pendata, pencount * sizeof(PENDATA));

    fclose(infile);

    wnprintf("\nCount for pensions as spouse: %d", pencount);
    }


  /* place data in VARS structure */

  for (penno = 0; penno < pencount; penno++)
  { caseid = pendata[penno].caseid;

    vars[idlocs[caseid]].calcvars.sppenstart = pendata[penno].benage;
    vars[idlocs[caseid]].calcvars.sppenben   = pendata[penno].benamt;
    vars[idlocs[caseid]].calcvars.spplumpsum = pendata[penno].lumpsum;
    }

  }


/*---------------------------------------------------------------------------
			 PEN CALC routine
			 runs the pension program
  ---------------------------------------------------------------------------*/

void penCalc(
  void *arglist)

{ int   *threadno;
  char  runtype;
  char  pentype[5];
  char  suffix[5];
  char  filename[24];

  FILE  *outfile;

  threadno = (int *) arglist;

  switch(threadno[0])
  { case  1: switch(threadno[1])
             { case 1:  strcpy(suffix, "dbl");    break;
               case 2:  strcpy(suffix, "dbp");    break;
               case 3:  strcpy(suffix, "dbs");    break;
               }

             strcpy(pentype, "DB");                         break;
    case  2: switch(threadno[1])
             { case 1:  strcpy(suffix, "dcl");    break;
               case 2:  strcpy(suffix, "dcp");    break;
               case 3:  strcpy(suffix, "dcs");    break;
               }

             strcpy(pentype, "DC");                         break;
    }

  switch(threadno[1])
  { case  1: runtype = '3';            break;
    case  2: runtype = '1';            break;
    case  3: runtype = '1';            break;
    }

  strcpy(filename, "calc-");
  strcat(filename, suffix);
  strcat(filename, ".bat");

  outfile = fopen(filename, "w");

  fprintf(outfile, "md calc-%s", suffix);
  fprintf(outfile, "\ncd calc-%s", suffix);
  fprintf(outfile, "\ncopy ..\\partic.%s partic", suffix);
  fprintf(outfile, "\ncopy ..\\param");
  fprintf(outfile, "\ncopy d:\\programs\\hrspensions\\program\\Custom\\Dpmi16bi.ovl");
  fprintf(outfile, "\ncopy d:\\programs\\hrspensions\\program\\Custom\\Rtm.exe");
  fprintf(outfile, "\ncopy d:\\programs\\hrspensions\\program\\Custom\\Calc6c%s.exe", pentype);
  fprintf(outfile, "\nCalc6c%s %c partic param penout.dat penout.tab penout.err", pentype, runtype);
  fprintf(outfile, "\ndel fn*.dat");
  fprintf(outfile, "\ncopy penout.dat ..\\penout.%s", suffix);
  fprintf(outfile, "\ncopy penout.tab ..\\pentab.%s", suffix);
  fprintf(outfile, "\ncopy penout.err ..\\penerr.%s", suffix);
  fprintf(outfile, "\ndel Calc6c%s.exe", pentype);
  fprintf(outfile, "\ndel partic");
  fprintf(outfile, "\ndel param");
  fprintf(outfile, "\ndel penout*.*");
  fprintf(outfile, "\ndel Dpmi16bi.ovl");
  fprintf(outfile, "\ndel Rtm.exe");
  fprintf(outfile, "\ncd ..");
  fprintf(outfile, "\nrd calc-%s", suffix);

  fclose(outfile);

  system(filename);

  _unlink(filename);

  while (threadno[0] != 0)
  threadno[0] = 0;

  _endthread();

  }


/*---------------------------------------------------------------------------
			 MEDIAN routine
			 calculates the median of a vector of integers
  ---------------------------------------------------------------------------*/

int median(
  int   *inputvector,
  int   count)

{ int   penno, highpen;

  qsort(inputvector, count, sizeof(int), comparemed);

  highpen = 0;

  for (penno = 0; penno < count; penno++)
  if (inputvector[penno] < 99)
  highpen = penno + 1;

  if (highpen > 0)
  { if (highpen % 2 == 1)
    return(inputvector[(highpen - 1) / 2]);
    else return((inputvector[(highpen / 2) - 1] + inputvector[highpen / 2]) / 2);
    }
  else return(99);

  }


/*---------------------------------------------------------------------------
			 COMPARE MED routine
			 compares two observations for quick sort routine in median
  ---------------------------------------------------------------------------*/

int comparemed(
  const void *elem1,
  const void *elem2)

{ if (* (int *) elem1 < * (int *) elem2)
  return(-1);
  else return(1);

  }


/*---------------------------------------------------------------------------
			 FMEDIAN routine
			 calculates the median of a vector of floating point numbers
  ---------------------------------------------------------------------------*/

double fmedian(
  double *inputvector,
  int    count)

{ int    penno, lowpen;
  double pensamt;

  qsort(inputvector, count, sizeof(double), comparefmed);

  lowpen = 0;

  for (penno = 0; penno < count; penno++)
  if (inputvector[penno] == 0)
  lowpen = penno + 1;

  if (count - lowpen > 0)
  { if ((count - lowpen) % 2 == 1)
    pensamt = inputvector[(lowpen + count - 1) / 2];
    else pensamt = (inputvector[((lowpen + count) / 2) - 1]
                   + inputvector[(lowpen + count) / 2]) / 2;

    pensamt = (pensamt * (count - lowpen)) / count;
    }
  else pensamt = 0;

  return(pensamt);

  }


/*---------------------------------------------------------------------------
			 COMPARE FMED routine
			 compares two observations for quick sort routine in fmedian
  ---------------------------------------------------------------------------*/

int comparefmed(
  const void *elem1,
  const void *elem2)

{ if (* (double *) elem1 < * (double *) elem2)
  return(-1);
  else return(1);

  }


/*---------------------------------------------------------------------------
			 INIT PENS routine
			 reads pension plan districution from NLS
  ---------------------------------------------------------------------------*/

void initpens(
  int     *idlocs,
  PENVARS *penvars,
  VARS    *vars)

{ int    jobno, recno, planno;
  int    hours, weeks;
  int    recordcount, pencount;
  int    caseid, codeid, seqno;
  int    occ, ind, earningscat, enddate;
  double earnings;

  char  filename[60];
  char  linestring[120];

  static double wageind[46] = {                             45.58,  49.00,  50.24,
	  50.13,  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
   /* 1947-1992 average weekly earnings           */
   /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  void *penidbuffer;
  int  (*penids)[4][2];

  FILE  *infile, *outfile;
  int   handle;

  for (caseid = 0; caseid < 30000; caseid++)
  idlocs[caseid] = -1;

  memset(penvars, 0, 5000 * sizeof(PENVARS));

  for (recno = 0; recno < 12652; recno++)
  { caseid = vars[recno].persvars.caseid;

    if (0 < caseid && caseid < 30000)
    idlocs[caseid] = recno;
    }

  pencount = 0;

  if (_access("penmatch.dat", 0) < 0)
  { penidbuffer = malloc(sizeof(int[30000][4][2]));
    penids = (int (*)[4][2]) penidbuffer;

    for (jobno = 0; jobno < 3; jobno++)
    { memset(penids, 0, sizeof(int[30000][4][2]));

      strcpy(filename, "c:\\programs\\hrspensions\\input\\");

      switch(jobno)
      { case  0:  strcat(filename, "partic-F.all");
                  recordcount = 3882;                   break;
        case  1:  strcat(filename, "partic-G.all");
                  recordcount = 1202;                   break;
        case  2:  strcat(filename, "partic-H.all");
                  recordcount = 1327;                   break;
        }

      infile = fopen(filename, "rt");

      for (recno = 0; recno < recordcount; recno++)
      { fgets(linestring, 120, infile);
	      sscanf(&linestring[ 0], "%d", &caseid);
        sscanf(&linestring[ 6], "%d", &codeid);
	      sscanf(&linestring[11], "%d", &seqno);

        if (jobno >= 1)
        fgets(linestring, 120, infile);

        if (codeid ==   56 || codeid ==   77 || codeid ==   82 || codeid ==   94
         || codeid ==  106 || codeid ==  158 || codeid ==  163 || codeid ==  196
         || codeid ==  224 || codeid ==  225 || codeid ==  274 || codeid ==  300
         || codeid ==  340 || codeid ==  358 || codeid ==  365 || codeid ==  411
         || codeid ==  426 || codeid ==  454 || codeid ==  458 || codeid ==  479
         || codeid ==  487 || codeid ==  488 || codeid ==  499 || codeid ==  509
         || codeid ==  557 || codeid ==  573 || codeid ==  581 || codeid ==  582
         || codeid ==  584 || codeid ==  640 || codeid ==  627 || codeid ==  636
         || codeid ==  653 || codeid ==  654 || codeid == 2531 || codeid == 2602
         || codeid == 2612 || codeid == 2625 || codeid == 3270 || codeid == 4503
         || codeid == 5104)
        penids[caseid][0][0] = -1;
        else if (penids[caseid][0][0] >= 0)
	      { if (0 <= caseid && caseid < 30000)
	        {      if (penids[caseid][0][0] == 0) planno = 0;
		        else if (penids[caseid][1][0] == 0) planno = 1;
		        else if (penids[caseid][2][0] == 0) planno = 2;
		        else if (penids[caseid][3][0] == 0) planno = 3;
		        else
            { wnprintf("\ncaseid %5d %4d %4d %4d %4d", caseid, penids[caseid][0][0],
                 penids[caseid][1][0], penids[caseid][2][0], penids[caseid][3][0]);
              error("Too many pensions for HRS individual in initpens");
              }

            penids[caseid][planno][0] = codeid;
            penids[caseid][planno][1] = seqno;
            }
		      }
	      }

      fclose(infile);

      for (caseid = 0; caseid < 30000; caseid++)
      { if (penids[caseid][0][0] > 0
         && vars[idlocs[caseid]].jobhist.jobvars[jobno].jobind == 1)
        { hours = vars[idlocs[caseid]].jobhist.jobvars[jobno].hours;
          weeks = vars[idlocs[caseid]].jobhist.jobvars[jobno].weeks;

          if (hours < 0 || weeks < 0 || hours > 25 || hours * weeks > 1250)
          { penvars[pencount].caseid = (short int) caseid;

            for (planno = 0; planno < 4; planno++)
            { penvars[pencount].penids[planno][0] = (short int) penids[caseid][planno][0];
              penvars[pencount].penids[planno][1] = (short int) penids[caseid][planno][1];
              }

            penvars[pencount].jobno = (char) jobno;

            occ      = vars[idlocs[caseid]].jobhist.jobvars[jobno].occ;
            ind      = vars[idlocs[caseid]].jobhist.jobvars[jobno].ind;
            earnings = vars[idlocs[caseid]].jobhist.jobvars[jobno].earnings;
            enddate  = vars[idlocs[caseid]].jobhist.jobvars[jobno].enddate;

            if (ind < 1 || ind >13)
            ind = 0;

                 if (occ <= 0)   occ =  0;
            else if (occ <  3)   occ =  1;  /* Managers & Prof */
            else if (occ <  5)   occ =  2;  /* White Collar    */
            else if (occ < 18)   occ =  3;  /* Blue Collar     */
            else                 occ =  0;

            if (jobno == 0)
            enddate = 1992;

            if (1900 < enddate && enddate <= 1992)
            { if (enddate < 1947)
              earnings = earnings * (wageind[1992 - 1947] / wageind[1947 - 1947]);
              else if (enddate < 1992)
              earnings = earnings * (wageind[1992 - 1947] / wageind[enddate - 1947]);

				           if (earnings < 10000)    earningscat = 1;
    			    else if (earnings < 20000)    earningscat = 2;
	   		      else if (earnings < 30000)    earningscat = 3;
		   	      else if (earnings < 40000)    earningscat = 4;
			        else if (earnings < 50000)    earningscat = 5;
			        else                          earningscat = 6;
              }
            else earningscat = -1;

            penvars[pencount].occ         = (char) occ;
            penvars[pencount].ind         = (char) ind;
            penvars[pencount].earningscat = (char) earningscat;

            pencount = pencount + 1;
            }
          }
        }
      }

    wnprintf("\npencount %d", pencount);

    free(penidbuffer);

    outfile = fopen("penmatch.dat", "wb");
    handle = _fileno(outfile);

    _write(handle, &pencount, sizeof(int));
    _write(handle, penvars, pencount * sizeof(PENVARS));

    fclose(outfile);
    }
  else
  { infile = fopen("penmatch.dat", "rb");
    handle = _fileno(infile);

    _read(handle, &pencount, sizeof(int));
    _read(handle, penvars, pencount * sizeof(PENVARS));

    fclose(infile);
    }


  for (recno = 0; recno < pencount; recno++)
  if (vars[recno].persvars.caseid == printcaseid || printcaseid == 99999)
  wnprintf("\n%5d %4d %4d %4d %4d %1d %2d %2d %1d", penvars[recno].caseid,
   penvars[recno].penids[0][0], penvars[recno].penids[1][0],
   penvars[recno].penids[2][0], penvars[recno].penids[3][0],
   penvars[recno].jobno, penvars[recno].occ, penvars[recno].ind, penvars[recno].earningscat);

  }


/*---------------------------------------------------------------------------
			 CALC SS STREAM routine
			 calculates social security benefit amounts
  ---------------------------------------------------------------------------*/

void calcSSStreams(
  VARS *vars)

{ int    recno;
  int    age, year, nonssyears;
  int    caseid, matchid, matchloc;
  int    birthyear, gender, windfallflag;
  int    spbirthyear, spgender;
  double ssmaximum;
  double aime, pia, ncpens;
  double spaime, sppia, spncpens;

  double nonssearnings[45], spnonssearns[45];

  static int idlocs[30000];

  double *earnings, *ssearnings, *spearnings, *spssearns;

  static double ssmax[42] = {
		    	 3600,  3600,  3600,  3600,  4200,  4200,  4200,  4200,  4800,
	  4800,  4800,  4800,  4800,  4800,  4800,  6600,  6600,  7800,  7800,
	  7800,  7800,  9000, 10800, 13200, 14100, 15300, 16500, 17700, 22900,
   25900, 29700, 32400, 35700, 37800, 39600, 42000, 43800, 45000, 48000,
   51300, 53400, 55500};
   /* Social Security maximum taxable earnings from Ann Statistical Suppl, 1995 */


  /* find vector of locations by caseid's */

  for (caseid = 0; caseid < 30000; caseid++)
  idlocs[caseid] = -1;

  for (recno = 0; recno < 12652; recno++)
  { caseid = vars[recno].persvars.caseid;

    if (0 < caseid && caseid < 30000)
    idlocs[caseid] = recno;
    }


  for (recno = 0; recno < 12652; recno++)
  { birthyear  = vars[recno].persvars.birthyear;
    gender     = vars[recno].persvars.gender;
    earnings   = vars[recno].calcvars.ftwage;
    ssearnings = vars[recno].calcvars.sswage;

    nonssyears = 0;

    for (age = 25; age < 70; age++)
    nonssearnings[age - 25] = 0;

    for (age = 25; age < 50; age++)
    { year = birthyear + age;

      if (year >= 1951)
      { if (year <= 1992)
        ssmaximum = ssmax[year - 1951];
        else ssmaximum = ssmax[1992 - 1951] * pow(1.051, year - 1992);

        if (ssearnings[age - 25] < earnings[age - 25]
         && ssearnings[age - 25] < 0.999 * ssmaximum)
        nonssearnings[age - 25] = earnings[age - 25] - ssearnings[age - 25];
        else nonssearnings[age - 25] = 0;
        }
      else nonssearnings[age - 25] = 0;

      if (nonssearnings[age - 25] > 0)
      nonssyears = nonssyears + 1;
      }

		for (age = 50; age <= 70; age++)
    { year = birthyear + age;

      if (nonssyears >= 5)
      windfallflag = 1;
      else windfallflag = 0;

      if (year >= 1951)
      { pia = getSocialSecurityPIA(birthyear, age, windfallflag, ssearnings, &aime);

        ncpens = ncpencalc(birthyear, gender, nonssearnings);

        vars[recno].calcvars.aime[age - 50]   = aime;
        vars[recno].calcvars.pia[age - 50]    = pia;
        vars[recno].calcvars.ncpens[age - 50] = ncpens;

        if (vars[recno].persvars.caseid == printcaseid || printcaseid == 99999)
        if (58 <= age && age <= 68)
        wnprintf("%6.0f", pia);
        }
      else
      { vars[recno].calcvars.aime[age - 50]   = 0;
        vars[recno].calcvars.pia[age - 50]    = 0;
        vars[recno].calcvars.ncpens[age - 50] = 0;

        if (vars[recno].persvars.caseid == printcaseid || printcaseid == 99999)
        if (58 <= age && age <= 68)
        wnprintf("%6.0f", pia);
        }

      if (age < 70)
      { if (year >= 1951)
        { if (year <= 1992)
          ssmaximum = ssmax[year - 1951];
          else ssmaximum = ssmax[1992 - 1951] * pow(1.051, year - 1992);

          if (ssearnings[age - 25] < earnings[age - 25]
             && ssearnings[age - 25] < 0.999 * ssmaximum)
          nonssearnings[age - 25] = earnings[age - 25] - ssearnings[age - 25];
          else nonssearnings[age - 25] = 0;
          }
        else nonssearnings[age - 25] = 0;

        if (nonssearnings[age - 25] > 0)
        nonssyears = nonssyears + 1;
        }
      }

    /* look for spouse and calculate spouse pia */

		matchid = vars[recno].persvars.matchid;

    if (0 < matchid && matchid < 30000)
    matchloc = idlocs[matchid];
    else matchloc = -1;

    if (matchloc >= 0)
    { spbirthyear = vars[matchloc].persvars.birthyear;
      spgender    = vars[matchloc].persvars.gender;
      spearnings  = vars[matchloc].calcvars.spwage;
      spssearns   = vars[matchloc].calcvars.spsswage;

      nonssyears = 0;

      for (age = 25; age < 70; age++)
      spnonssearns[age - 25] = 0;

      for (age = 25; age < 50; age++)
      { year = spbirthyear + age;

        if (year >= 1951)
        { if (year <= 1992)
          ssmaximum = ssmax[year - 1951];
          else ssmaximum = ssmax[1992 - 1951] * pow(1.051, year - 1992);

          if (spssearns[age - 25] < spearnings[age - 25]
           && spssearns[age - 25] < 0.999 * ssmaximum)
          spnonssearns[age - 25] = spearnings[age - 25] - spssearns[age - 25];
          else spnonssearns[age - 25] = 0;
          }
        else spnonssearns[age - 25] = 0;

        if (spnonssearns[age - 25] > 0)
        nonssyears = nonssyears + 1;
        }

      for (age = 50; age <= 70; age++)
      { year = spbirthyear + age;

        if (nonssyears >= 5)
        windfallflag = 1;
        else windfallflag = 0;

        if (year >= 1951)
        { sppia = getSocialSecurityPIA(spbirthyear, age, windfallflag, spssearns, &spaime);

          spncpens = ncpencalc(spbirthyear, spgender, spnonssearns);

          vars[recno].calcvars.sppia[age - 50]    = sppia;
          vars[recno].calcvars.spncpens[age - 50] = spncpens;
          }
        else
        { vars[recno].calcvars.sppia[age - 50]    = 0;
          vars[recno].calcvars.spncpens[age - 50] = 0;
          }

        if (age < 70)
        { if (year >= 1951)
          { if (year <= 1992)
            ssmaximum = ssmax[year - 1951];
            else ssmaximum = ssmax[1992 - 1951] * pow(1.051, year - 1992);

            if (spssearns[age - 25] < spearnings[age - 25]
             && spssearns[age - 25] < 0.999 * ssmaximum)
            spnonssearns[age - 25] = spearnings[age - 25] - spssearns[age - 25];
            else spnonssearns[age - 25] = 0;
            }
          else spnonssearns[age - 25] = 0;

          if (spnonssearns[age - 25] > 0)
          nonssyears = nonssyears + 1;
          }
        }
      }
    else
    { for (age = 50; age <= 70; age++)
      { vars[recno].calcvars.sppia[age - 50]    = 0;
        vars[recno].calcvars.spncpens[age - 50] = 0;
        }
      }

    if (vars[recno].persvars.caseid == printcaseid || printcaseid == 99999)
    { wnprintf("\npia  "); for (age = 58; age <= 68; age++) wnprintf("%6.0f", vars[recno].calcvars.pia[age - 50]);
      wnprintf("\nsppia"); for (age = 58; age <= 68; age++) wnprintf("%6.0f", vars[recno].calcvars.sppia[age - 50]);
      wnprintf("\nncpen"); for (age = 58; age <= 68; age++) wnprintf("%6.0f", vars[recno].calcvars.ncpens[age - 50]);
      wnprintf("\nspncp"); for (age = 58; age <= 68; age++) wnprintf("%6.0f", vars[recno].calcvars.spncpens[age - 50]);
      }

    checkMessages();
    }

  }


/*---------------------------------------------------------------------------
			 NON-COVERED PENSION CALC routine
			 calculates pensions for non-covered jobs
  ---------------------------------------------------------------------------*/

double ncpencalc(
  int    birthyear,          /* cohort of participant            */
  int    gender,             /* gender of participant            */
  double *earnings)          /* vector of non-ss annual earnings */

{ int    age, lastage;
  int    year, yearcount, lowyear;
  double pv55, pv62, disc;
  double avewage, pen;

  double highwages[3], surv[120];

  static double cpiind[42] = {
			     26.0,  26.5,  26.7,  26.9,  26.8,  27.2,  28.1,  28.9,  29.1,
	  29.6,  29.9,  30.2,  30.6,  31.0,  31.5,  32.4,  33.4,  34.8,  36.7,
	  38.8,  40.5,  41.8,  44.4,  49.3,  53.8,  56.9,  60.6,  65.2,  72.6,
	  82.4,  90.9,  96.5,  99.6, 103.9, 107.6, 109.6, 113.6, 118.3, 124.0,
	 130.7, 136.2, 140.3};
   /* 1951-1992 CPI-U, Table B-59, 1995 ERP */

  static double inflationRate[29] = {
			   2.9,  2.8,  2.8,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,
	 3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,
	 3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0};
   /* CPI inflation rate, 1992-2020, in Table II.D1, p. 59, in the   */
   /* 1993 Annual Report of the Board of Trustees of Social Security */
   /* static double inflationRate[29] = {
			 2.9,  2.8,  2.5,  3.1,  3.2,  3.3,  3.4,  3.5,  3.7,
	 3.9,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,
	 4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0};
     CPI inflation rate, 1992-2020, in Table II.D1, p. 56, in the
     1995 Annual Report of the Board of Trustees of Social Security */

  /* Calculations assume full benefit at age 62 of 1.2% times highest three   */
  /* year salary average times years of service.  Benefits can start at 55,   */
  /* reduced actuarially.  Once started, benefits are adjusted for inflation. */


  lastage = 0;
  yearcount = 0;
  lowyear = 0;

  for (year = 0; year < 3; year++)
  highwages[year] = 0;

  for (age = 25; age < 70; age++)
  { if (earnings[age - 25] > 0)
    { if (highwages[1] < highwages[0])
      { if (highwages[2] < highwages[1])
        lowyear = 2;
        else lowyear = 1;
        }
      else
      { if (highwages[2] < highwages[0])
        lowyear = 2;
        else lowyear = 0;
        }
      }

    if (earnings[age - 25] > highwages[lowyear])
    highwages[lowyear] = earnings[age - 25];

    lastage = age;
    yearcount = yearcount + 1;
    }

  avewage = (highwages[0] + highwages[1] + highwages[2]) / 3;

  if(yearcount > 0)
  { pen = 0.012 * avewage * yearcount;                        /* full pension */

    if (lastage < 62)                                         /* early retirement reduction */
    { getSurvivalRates(birthyear, gender, surv);

      pv55 = 0;
      pv62 = 0;
      disc = 1.0;

      for (age = 55; age < 120; age++)
      { if (age >= lastage)
        pv55 = pv55 + surv[age] * disc;

        if (age >= 62)
        pv62 = pv62 + surv[age] * disc;

        disc = disc / 1.063;
        }

      pen = pen * (pv62 / pv55);
      }

    if (birthyear + max(lastage + 1, 55) < 1992)     /* express pension in 1992 $ */
    pen = pen * (cpiind[1992 - 1951] / cpiind[birthyear + max(lastage + 1, 55) - 1951]);
    else
    { for (year = 1992; year < birthyear + max(lastage + 1, 55); year++)
      { if (year < 2020)
        pen = pen / (1 + 0.01 * inflationRate[year - 1992]);
        else pen = pen / (1 + 0.01 * inflationRate[2020 - 1992]);
        }
      }

    pen = 0.6667 * pen;       /* only 2/3 of pension is used in ss calculations */
    }
  else pen = 0;

  /* wnprintf("\n   %d %d %d %d %d %5.0f %5.0f %5.0f %5.0f",
   gender, cohort, hireyear, endyear, retyear, highwages[0], highwages[1],
    highwages[2], pen); */

  return(pen);
  }


/*---------------------------------------------------------------------------
			 GET SOCIAL SECURITY PIA routine
			 gets Social Security primary insurance amounts
  ---------------------------------------------------------------------------*/

double getSocialSecurityPIA(
  int    birthyear,
  int    piaage,
  int    windfallflag,
  double *earnings,
  double *ssaime)

{ int    age;
  int    year, year60, year62;
  int    yearindx, replacementyear;
  int    coverage, coverageyears;
  double wage, aime, pia;
  double bend1, bend2, firstbracketpct;
  double infladj, bendindex;

  double ssearningsvec[35];

  static double cpiind[42] = {
			     26.0,  26.5,  26.7,  26.9,  26.8,  27.2,  28.1,  28.9,  29.1,
	  29.6,  29.9,  30.2,  30.6,  31.0,  31.5,  32.4,  33.4,  34.8,  36.7,
	  38.8,  40.5,  41.8,  44.4,  49.3,  53.8,  56.9,  60.6,  65.2,  72.6,
	  82.4,  90.9,  96.5,  99.6, 103.9, 107.6, 109.6, 113.6, 118.3, 124.0,
	 130.7, 136.2, 140.3};
   /* 1951-1992 CPI-U, Table B-59, 1995 ERP */

  static double wageind[42] = {
		     	  57.86,  60.65,  63.76,  64.52,  67.72,  70.74,  73.33,  75.08,  78.78,
	  80.67,  82.60,  85.91,  88.46,  91.33,  95.45,  98.82, 101.84, 107.73, 114.61,
   119.83, 127.31, 136.90, 145.39, 154.76, 163.53, 175.45, 189.00, 203.70, 219.91,
   235.10, 255.20, 267.26, 280.70, 292.86, 299.09, 304.85, 312.50, 322.02, 334.24,
   345.35, 353.98, 363.61};
   /* 1951-1992 average weekly earnings           */
   /* Table B41, 1986 ERP and Table B45, 1995 ERP */

  static double inflationRate[29] = {
		   	  2.9,  2.8,  2.8,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,
	  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,
	  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0,  3.0};
    /* CPI inflation rate, 1992-2020, in Table II.D1, p. 59, in the   */
    /* 1993 Annual Report of the Board of Trustees of Social Security */
    /* static double inflationRate[29] = {
			    2.9,  2.8,  2.5,  3.1,  3.2,  3.3,  3.4,  3.5,  3.7,
	  3.9,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,
	  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0,  4.0};
     CPI inflation rate, 1992-2020, in Table II.D1, p. 56, in the
     1995 Annual Report of the Board of Trustees of Social Security */

  static double wGrowthRate[29] = {
			   4.0,  4.2,  4.8,  5.2,  4.9,  4.9,  4.8,  4.8,  4.9,
	 4.9,  5.0,  5.0,  5.0,  5.0,  4.9,  4.9,  4.9,  4.9,  4.8,
	 4.8,  4.8,  4.8,  4.8,  4.8,  4.7,  4.7,  4.7,  4.7,  4.7};
   /* Wage growth rate, 1992-2020, in Table II.D1, p. 59, in the 1993 */
   /* Annual Report of the Board of Trustees of Social Security    */
   /* static double wGrowthRate[29] = {
			   5.4,  1.3,  3.5,  4.0,  4.1,  4.3,  4.1,  4.2,  4.5,
	 4.7,  4.8,  4.9,  5.1,  5.1,  5.1,  5.1,  5.1,  5.1,  5.1,
	 5.1,  5.1,  5.1,  5.1,  5.1,  5.0,  5.0,  5.0,  5.0,  5.0};
     Wage growth rate, 1992-2020, in Table II.D1, p. 56, in the 1995
     Annual Report of the Board of Trustees of Social Security    */


  coverageyears = 0;

  for (age = max(25, 1951 - birthyear); age < min(70, piaage); age++)
  { year = birthyear + age;

    if (earnings[age - 25] > 0)
    if (windfallflag == 0 || year < 1990)
	  coverageyears = coverageyears + 1;
    else
    { wage = earnings[age - 25];

		  if (year <= 1992)
	  	wage = wage * wageind[1992 - 1951] / wageind[year - 1951];
      else
      { for (yearindx = 1992; yearindx < birthyear + age; yearindx++)
		    { if (yearindx < 2020)
		      wage = wage / (1 + 0.01 * wGrowthRate[yearindx - 1992]);
		      else wage = wage / (1 + 0.01 * wGrowthRate[2020 - 1992]);
          }
        }

      if (wage >= 10350)
	    coverageyears = coverageyears + 1;
      }
    }

  if (coverageyears >= 10)
  coverage = 1;
  else coverage = 0;

  if (coverage == 1)
  { for (year = 0; year < 35; year++)
    ssearningsvec[year] = 0;

    for (age = max(25, 1951 - birthyear); age < min(70, piaage); age++)
    { replacementyear = 0;

	    for (year = 1; year < 35; year++)
	    if (ssearningsvec[year] < ssearningsvec[replacementyear])
	    replacementyear = year;

	    year = birthyear + age;

      wage = earnings[age - 25];

	    if (age < 60)                  /* index wages to age 60 */
	    { year60  = birthyear + 60;

		    if (year <= 1992)
		    wage = wage * wageind[min(1992, year60) - 1951] / wageind[year - 1951];

		    for (yearindx = max(1992, year); yearindx < year60; yearindx++)
		    { if (yearindx < 2020)
		      wage = wage * (1 + 0.01 * wGrowthRate[yearindx - 1992]);
		      else wage = wage * (1 + 0.01 * wGrowthRate[2020 - 1992]);
          }
        }

	    if (wage > ssearningsvec[replacementyear])
	    ssearningsvec[replacementyear] = wage;
	    }

    /* for (year = 0; year < 35; year++)
    { if (year % 10 == 0) wnprintf("\n");
	    wnprintf(" %6.0f", ssearningsvec[year]);
	    } */

    aime = 0;

    for (year = 0; year < 35; year++)
    aime = aime + ssearningsvec[year];

    /* wnprintf("\n aime %5.0f", (aime / 35)); */

    aime = aime / (35 * 12);              /* nominal aime */

    /* calculate nominal pia bend points */

    year62  = birthyear + 62;

    if (year62 < 1992)                  /* index bend points */
    { if (year62 < 1951)
	    bendindex = wageind[1951 - 1951] / wageind[1992 - 1951];
	    else bendindex = wageind[year62 - 1951] / wageind[1992 - 1951];
	    }
    else
    { bendindex = 1;

	    for (year = 1992; year < year62; year++)
	    if (year < 2020)
	    bendindex = bendindex * (1 + 0.01 * wGrowthRate[year - 1992]);
	    else bendindex = bendindex * (1 + 0.01 * wGrowthRate[2020 - 1992]);
	    }

    bend1 =  387 * bendindex;          /* 1992 bend points */
    bend2 = 2333 * bendindex;

    if (windfallflag == 1 && coverageyears < 30)
    { if (coverageyears > 20)
      firstbracketpct = 0.4 + 0.05 * (coverageyears - 20);
      else firstbracketpct = 0.4;
      }
    else firstbracketpct = 0.9;

    if (aime < bend1)                  /* nominal pia bend points */
    pia = firstbracketpct * aime;
    else if (aime < bend2)
    pia = firstbracketpct * bend1 + 0.32 * (aime - bend1);
    else pia = firstbracketpct * bend1 + 0.32 * (bend2 - bend1) + 0.15 * (aime - bend2);

    aime = aime * 12;                /* nominal annual aime */
    pia = pia * 12;                  /* nominal annual pia */

    /* wnprintf(" age62 pia %5.0f", pia); */

    if (year62 < 1992)
    { if (year62 < 1951)
	    infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
	    else infladj = cpiind[1992 - 1951] / cpiind[year62 - 1951];
	    }
    else
    { infladj = 1;

	    for (year = 1992; year < year62; year++)
	    if (year < 2020)
	    infladj = infladj / (1 + 0.01 * inflationRate[year - 1992]);
	    else infladj = infladj / (1 + 0.01 * inflationRate[2020 - 1992]);
	    }


    *ssaime = aime * infladj;          /* aime in 1992 dollars */
    pia = pia * infladj;               /* pia in 1992 dollars */
    }
  else
  { *ssaime = 0;
    pia = 0;
    }

  /* wnprintf("  p %d %5.0f", piaage, pia); pause(); */
  return(pia);

  }


/*---------------------------------------------------------------------------
			 PUT RESP OUTPUT routine
			 write the output which will be the input data for the maximization
  ---------------------------------------------------------------------------*/

void putRespOutput(
  VARS *vars)

{ int    surv, recno, finloc, matchloc;
  int    age, initage, year;
  int    outputcount;
  int    caseid, matchid;
  int    marstat, samemarstat, lastyear;
  int    spouseinterview, spouseFTyears;
  int    penage, ssdiage, ssdiyear;
  int    inherno, giftyear;
  double giftamount, infladj;

  int idlocs[30000];

  typedef struct
  { short  origobs;         /* observation number in original data set            */
    short  caseid;          /* caseid number in original data set                 */
    char   marstat;         /* 1992 marital status                                */
    char   race;            /* race/ethnicity                                     */
    short  birthyear;       /* year of birth                                      */
    short  spbirthyear;     /* year of spouse's birth                             */
    char   gender;          /* gender                                             */
    char   retage;          /* age of actual or expected retirement from ft job   */
    char   spretage;        /* respondent age when spouse can collect ss benefits */
    char   healthage;       /* initial age of health problem                      */
    char   penjobend;       /* age at potential last year of pension job          */
    char   origjobend;      /* age of separation from original job                */
    char   ind;             /* industry of original job                           */
    char   occ;             /* occupation of original job                         */
    char   age[6];          /* age at the survey dates                            */
    char   ret[6];          /* retirement status at the survey dates              */
    char   retv[20];        /* retirement status vector for ages 50-69            */
    char   ssreceipt[6];    /* social security recipiency status for survey dates */

                                    /* all amounts are in 1992$ */
    float  wealth;          /* wealth in 1992                                     */
    float  assetwealth;     /* land, business, financial & "other" wealth in 1992 */
    float  ftwage[45];      /* full-time wages at ages 25-69                      */
    float  prwage[20];      /* partial retirement wages at ages 50-69             */
    float  secwage[20];     /* full-time wages in reversal jobs at ages 50-69     */
    float  sswage[45];      /* social security wages at ages 25-69                */
    float  spwage[45];      /* wages of spouse at spouse ages 25-69               */
    float  spsswage[45];    /* social security wages at ages 25-69                */
    float  pensben[21];     /* annual pension benefit for retirement at 50-70     */
    float  penstart[21];    /* starting age of pension for retirement at 50-70    */
    float  plumpsum[21];    /* lump-sum pension benefit for retirement at 50-70   */
    float  eowamount[21];   /* early out window lump sum amounts at 50-70         */
    float  projpenben;      /* actual/projected pension benefit                   */
    char   projpstart;      /* actual/projected starting age of pension benefit   */
    char   projpenjobend;   /* actual/projected age at last year of pension job   */
    float  projplumpsum;    /* actual/projected lump-sum pension benefit          */
    float  sppensben;       /* annual pension benefit for spouse                  */
    char   sppenstart;      /* starting spouse age of pension                     */
    char   sppjobend;       /* spouse age at last year of spouse pension job      */
    float  spplumpsum;      /* amount of spouse pension lump sum                  */
    float  aime[21];        /* aime at ages 50-70                                 */
    float  pia[21];         /* pia at ages 50-70                                  */
    float  ncpens[21];      /* noncovered pensions for retirement at 50-70        */
    float  sppia[21];       /* spouse pia at spouse ages 50-70                    */
    float  spncpens[21];    /* noncovered spouse pens for spouse retirement 50-70 */
    float  othincome[75];   /* inheritances for ages 25-99                        */
    
    struct
    { char  spouseinterview;
      char  goodCareer;
      char  goodSSstatus;
      char  goodFTyears;
      char  goodearnings;
      char  goodSSearn;
      char  goodAssets;
      char  goodPens;
      char  spouseFTyears;
      } goodobs;
    } DATA;

  void  *databuffer;
  DATA  *data;

  static double cpiind[42] = {
		  	  26.0,  26.5,  26.7,  26.9,  26.8,  27.2,  28.1,  28.9,  29.1,
	 29.6,  29.9,  30.2,  30.6,  31.0,  31.5,  32.4,  33.4,  34.8,  36.7,
	 38.8,  40.5,  41.8,  44.4,  49.3,  53.8,  56.9,  60.6,  65.2,  72.6,
	 82.4,  90.9,  96.5,  99.6, 103.9, 107.6, 109.6, 113.6, 118.3, 124.0,
	130.7, 136.2, 140.3};
  /* 1951-1992 CPI-U, Table B-59, 1995 ERP */

  FILE *outfile;
  int  handle;


  /* find vector of locations by caseid's */

  for (caseid = 0; caseid < 30000; caseid++)
  idlocs[caseid] = -1;

  for (recno = 0; recno < 12652; recno++)
  { caseid = vars[recno].persvars.caseid;
    
    if (0 < caseid && caseid < 30000)
    idlocs[caseid] = recno;
    }


  /* start processing output file */

  databuffer = malloc(sizeof(int) + 13000 * sizeof(DATA));

  outputcount = 0;

  for (recno = 0; recno < 12652; recno++)
  { data = (DATA *) &((char *) databuffer)[sizeof(int) + outputcount * sizeof(DATA)];

    marstat = vars[recno].persvars.marstat;


    /* determine last survey of unchanged marital status from 1992 status */

    if (marstat == 1 || marstat == 2)
    { if (vars[recno].panelvars.wavevars[1].interview == 1
		   && vars[recno].panelvars.wavevars[1].marstat == 1
		   && vars[recno].panelvars.wavevars[1].samespouse != 0)
      { if (vars[recno].panelvars.wavevars[2].interview == 1
		     && vars[recno].panelvars.wavevars[2].marstat == 1
		     && vars[recno].panelvars.wavevars[2].samespouse != 0)
        { if (vars[recno].panelvars.wavevars[3].interview == 1
		       && vars[recno].panelvars.wavevars[3].marstat == 1
		       && vars[recno].panelvars.wavevars[3].samespouse != 0)
          { if (vars[recno].panelvars.wavevars[4].interview == 1
		         && vars[recno].panelvars.wavevars[4].marstat == 1
		         && vars[recno].panelvars.wavevars[4].samespouse != 0)
            { if (vars[recno].panelvars.wavevars[5].interview == 1
		           && vars[recno].panelvars.wavevars[5].marstat == 1
		           && vars[recno].panelvars.wavevars[5].samespouse != 0)
              samemarstat = 5;
              else samemarstat = 4;
					    }
            else samemarstat = 3;
					  }
          else samemarstat = 2;
				  }
        else samemarstat = 1;
			  }
      else samemarstat = 0;
      }
    else
    { if (vars[recno].panelvars.wavevars[1].interview == 1
		   && vars[recno].panelvars.wavevars[1].marstat == 0)
      { if (vars[recno].panelvars.wavevars[2].interview == 1
		     && vars[recno].panelvars.wavevars[2].marstat == 0)
        { if (vars[recno].panelvars.wavevars[3].interview == 1
		       && vars[recno].panelvars.wavevars[3].marstat == 0)
          { if (vars[recno].panelvars.wavevars[4].interview == 1
		         && vars[recno].panelvars.wavevars[4].marstat == 0)
            { if (vars[recno].panelvars.wavevars[5].interview == 1
		           && vars[recno].panelvars.wavevars[5].marstat == 0)
              samemarstat = 5;
              else samemarstat = 4;
					    }
            else samemarstat = 3;
					  }
          else samemarstat = 2;
				  }
        else samemarstat = 1;
			  }
      else samemarstat = 0;
      }


    /* determine if business assets are too large */

    vars[recno].calcvars.goodobs.goodAssets = 1;

    caseid = vars[recno].persvars.caseid;

    if (vars[recno].persvars.married != 1 || (10000 <= caseid && caseid < 20000))
    finloc = recno;
    else
    { matchid = vars[recno].persvars.matchid;

      if (10000 <= matchid && matchid < 30000)
      finloc = idlocs[matchid];
      else finloc = -1;
      }

    if (finloc > 0)
    if (vars[finloc].persvars.buswealth > 0.5 * max(0, vars[finloc].persvars.assetwealth))
    vars[recno].calcvars.goodobs.goodAssets = 0;


    /* determine whether spouse variables are good */

    spouseinterview = 1;
    spouseFTyears = 1;
    
    if (marstat == 1 || marstat == 2)
    { matchid = vars[recno].persvars.matchid;

      if (10000 <= matchid && matchid < 30000)
      { matchloc = idlocs[matchid];

        if (idlocs[matchid] >= 0)
        { if (vars[matchloc].calcvars.goodobs.goodFTyears == 0)
          spouseFTyears = 0;
          }
        else spouseinterview = 0;
        }
      else spouseinterview = 0;
      }

    data->origobs       = (short) recno;
    data->caseid        = (short) vars[recno].persvars.caseid;
    data->marstat       = (char)  marstat;
    data->race          = (char)  vars[recno].persvars.race;
    data->birthyear     = (short) vars[recno].persvars.birthyear;
    data->gender        = (char)  vars[recno].persvars.gender;
    data->retage        = (char)  vars[recno].calcvars.spftend + 1;
    data->healthage     = (char)  vars[recno].calcvars.healthage;
    data->penjobend     = (char)  vars[recno].calcvars.pjobend;
    data->projpenjobend =         vars[recno].calcvars.sppjobend;
    data->origjobend    = (char)  vars[recno].calcvars.impjobendage;
    data->ind           = (char)  vars[recno].calcvars.ind;
    data->occ           = (char)  vars[recno].calcvars.occ;

    data->goodobs.spouseinterview = (char) spouseinterview;
    data->goodobs.goodCareer      = (char) vars[recno].calcvars.goodobs.goodCareer;
    data->goodobs.goodSSstatus    = (char) vars[recno].calcvars.goodobs.goodSSstatus;
    data->goodobs.goodFTyears     = (char) vars[recno].calcvars.goodobs.goodFTyears;
    data->goodobs.goodearnings    = (char) vars[recno].calcvars.goodobs.goodearnings;
    data->goodobs.goodSSearn      = (char) vars[recno].calcvars.goodobs.goodSSearn;
    data->goodobs.goodAssets      = (char) vars[recno].calcvars.goodobs.goodAssets;
    data->goodobs.goodPens        = (char) vars[recno].calcvars.goodobs.goodPens;
    data->goodobs.spouseFTyears   = (char) spouseFTyears;

    /* last year where observed marital status is unchanged from 1992 */

    lastyear = vars[recno].panelvars.wavevars[samemarstat].intvyear + 1990;

    /* find ssdi year, if any */

    ssdiage = vars[recno].calcvars.ssdiage;

    if (ssdiage < 99)
    ssdiyear = vars[recno].persvars.birthyear + ssdiage;
    else ssdiyear = 9999;

    /* retirement vector */

    for (age = 50; age < 70; age++)
    { year = data->birthyear + age;

      initage = vars[recno].calcvars.initage;
          
      if (initage - 1 <= age && age <= initage + 11 && year < lastyear && year < ssdiyear)
      data->retv[age - 50] = (char) (vars[recno].calcvars.retstats[age - (initage - 1)]);
      else data->retv[age - 50] = 0;
      }

    /* age and ss receipt status during the survey years */

    for (surv = 0; surv < 6; surv++)
    { data->age[surv] = (char) vars[recno].panelvars.wavevars[surv].age;
      data->ret[surv] = (char) vars[recno].panelvars.wavevars[surv].ret;
      data->ssreceipt[surv] = (char) vars[recno].panelvars.wavevars[surv].ssreceipt;
      }

    /* determine the financial respondent */

    caseid = vars[recno].persvars.caseid;

    if (vars[recno].persvars.married != 1 || (10000 <= caseid && caseid < 20000))
    finloc = recno;
    else
    { matchid = vars[recno].persvars.matchid;

      if (0 <= matchid && matchid < 30000)
      finloc = idlocs[matchid];
      else finloc = -1;
      }

    /* determine wealth figures */

    if (finloc >= 0)
    { data->wealth = (float) vars[finloc].persvars.wealth;
      data->assetwealth = (float) vars[finloc].persvars.assetwealth;
      }
    else
    { data->wealth = (float) 9999999999;
      data->assetwealth = (float) 9999999999;
      }


		/* get earnings amounts in 1992 $ */

    for (age = 25; age < 70; age++)
    { year = vars[recno].persvars.birthyear + age;

      if (year < 1951)
      infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
      else if (year <= 1992)
      infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
      else infladj = pow(1.04, 1992 - year);

      data->ftwage[age - 25] = (float) (infladj * vars[recno].calcvars.ftwage[age - 25]);
      data->sswage[age - 25] = (float) (infladj * vars[recno].calcvars.sswage[age - 25]);
       
      if (age >= 50)
      { data->prwage[age - 50] = (float) (infladj * vars[recno].calcvars.prwage[age - 50]);
        data->secwage[age - 50] = (float) (infladj * vars[recno].calcvars.secwage[age - 50]);
        }
      }


    /* get pension amounts in 1992 $ */

    for (age = 50; age <= 70; age++)
		{ if (vars[recno].calcvars.penstart[age - 50] < 99)
      { penage = vars[recno].calcvars.penstart[age - 50];

        year = vars[recno].persvars.birthyear + penage;

        if (year < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (year <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
        else infladj = pow(1.04, 1992 - year);

        data->pensben[age - 50] = (float) (infladj * vars[recno].calcvars.pensben[age - 50]);
        data->penstart[age - 50] = (char) penage;
        }
      else
      { data->pensben[age - 50] = 0;
        data->penstart[age - 50] = 99;
        }

      if (vars[recno].calcvars.plumpsum[age - 50] > 0)
      { year = vars[recno].persvars.birthyear + age;

        if (year < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (year <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
        else infladj = pow(1.04, 1992 - year);

        data->plumpsum[age - 50] = (float) (infladj * vars[recno].calcvars.plumpsum[age - 50]);
        data->eowamount[age - 50] = (float) (infladj * vars[recno].calcvars.eowamount[age - 50]);
        }
      else
      { data->plumpsum[age - 50] = 0;
        data->eowamount[age - 50] = 0;
        }
      }

    if (vars[recno].calcvars.sppenstart < 99)
    { data->projpstart = (char) vars[recno].calcvars.sppenstart;

      year = data->birthyear + data->projpstart;

      if (year < 1951)
      infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
      else if (year <= 1992)
      infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
      else infladj = pow(1.04, 1992 - year);

      data->projpenben = (float) (infladj * vars[recno].calcvars.sppenben);
      }
    else
    { data->projpstart = 99;
      data->projpenben = 0;
      }

    if (vars[recno].calcvars.spplumpsum > 0)
    { year = data->birthyear + vars[recno].calcvars.sppjobend + 1;

      if (year < 1951)
      infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
      else if (year <= 1992)
      infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
      else infladj = pow(1.04, 1992 - year);

      data->projplumpsum = (float) (infladj * vars[recno].calcvars.spplumpsum);
      }
    else data->projplumpsum = 0;


    /* get social security amounts in 1992 $ */

    for (age = 50; age <= 70; age++)
    { data->aime[age - 50] = (float) vars[recno].calcvars.aime[age - 50];
      data->pia[age - 50] = (float) vars[recno].calcvars.pia[age - 50];
      data->ncpens[age - 50] = (float) vars[recno].calcvars.ncpens[age - 50];
      }


    /* inheritances and other income */

    for (age = 25; age < 100; age++)
    data->othincome[age - 25] = 0;

    for (inherno = 0; inherno < 11; inherno++)
    { giftamount = vars[finloc].persvars.giftamount[inherno];
      giftyear   = vars[finloc].persvars.giftyear[inherno];

      if (giftamount > 0)
      if (1920 <= giftyear && giftyear <= 1992)
      { if (giftyear < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (giftyear <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[giftyear - 1951];
        else infladj = pow(1.04, 1992 - giftyear);

        age = giftyear - vars[recno].persvars.birthyear;

        if (age < 25)
        age = 25;
        else if (age >= 100)
        age = 99;

        data->othincome[age - 25] = (float) (data->othincome[age - 25] + infladj * giftamount);
        }
      }

    for (surv = 1; surv < 6; surv++)
    if (vars[recno].panelvars.wavevars[surv].interview == 1)
    { for (inherno = 0; inherno < 3; inherno++)
      { giftamount = vars[recno].panelvars.wavevars[surv].giftamount[inherno];
        giftyear   = vars[recno].panelvars.wavevars[surv].giftyear[inherno];

        if (giftamount > 0)
        if (1920 <= giftyear && giftyear <= 1992)
        { if (giftyear < 1951)
          infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
          else if (giftyear <= 1992)
          infladj = cpiind[1992 - 1951] / cpiind[giftyear - 1951];
          else infladj = pow(1.04, 1992 - giftyear);

          age = giftyear - vars[recno].persvars.birthyear;

          if (age < 25)
          age = 25;
          else if (age >= 100)
          age = 99;

          data->othincome[age - 25] = (float) (data->othincome[age - 25] + infladj * giftamount);
          }
        }
      }


    /* get spouse variables if married */

    if ((marstat == 1 || marstat == 2) && (10000 <= matchid && matchid < 30000))
    { data->spbirthyear = (short) vars[matchloc].persvars.birthyear;
      data->spretage = (char) vars[matchloc].calcvars.spftend + 1;

      if (data->spretage < 62)
      data->spretage = 62;
      else if (data->spretage > 70)
      data->spretage = 70;


  	  /* get earnings amounts in 1992 $ */

      for (age = 25; age < 70; age++)
      { year = data->spbirthyear + age;

        if (year < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (year <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
        else infladj = pow(1.04, 1992 - year);

        data->spwage[age - 25] = (float) (infladj * vars[matchloc].calcvars.spwage[age - 25]);
        data->spsswage[age - 25] = (float) (infladj * vars[matchloc].calcvars.spsswage[age - 25]);
        }


      /* get pension amounts in 1992 $ */

      if (vars[matchloc].calcvars.sppenstart < 99)
      { data->sppenstart = (char) vars[matchloc].calcvars.sppenstart;

        year = data->spbirthyear + data->sppenstart;

        if (year < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (year <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
        else infladj = pow(1.04, 1992 - year);

        data->sppensben = (float) (infladj * vars[matchloc].calcvars.sppenben);
        }
      else
      { data->sppenstart = 99;
        data->sppensben = 0;
        }

      if (vars[matchloc].calcvars.spplumpsum > 0)
      { year = data->spbirthyear + vars[matchloc].calcvars.sppjobend + 1;

        if (year < 1951)
        infladj = cpiind[1992 - 1951] / cpiind[1951 - 1951];
        else if (year <= 1992)
        infladj = cpiind[1992 - 1951] / cpiind[year - 1951];
        else infladj = pow(1.04, 1992 - year);

        data->sppjobend = vars[matchloc].calcvars.sppjobend;
        data->spplumpsum = (float) (infladj * vars[matchloc].calcvars.spplumpsum);
        }
      else
      { data->sppjobend = 99;
        data->spplumpsum = 0;
        }

      /* get social security amounts in 1992 $ */

      for (age = 50; age <= 70; age++)
      { data->sppia[age - 50] = (float) vars[recno].calcvars.sppia[age - 50];
        data->spncpens[age - 50] = (float) vars[recno].calcvars.spncpens[age - 50];
        }
      }
    else
    { data->spbirthyear = 9999;
      data->spretage = 0;

      for (age = 25; age < 70; age++)
      data->spwage[age - 25] = 0;

      data->sppenstart = 99;
      data->sppensben = 0;
      data->sppjobend = 99;
      data->spplumpsum = 0;

      for (age = 50; age <= 70; age++)
      { data->sppia[age - 50] = 0;
        data->spncpens[age - 50] = 0;
        }
      } 


    /* write output and increment counter */

    outputcount = outputcount + 1;

    if (outputcount > 13000)
    error("Too many output records");
			
    checkMessages();
    }

  wnprintf("\nRespondent output records: %d", outputcount);

  memcpy(databuffer, &outputcount, sizeof(int));

  outfile = fopen("GSRetirementData.dat", "wb");
  handle = _fileno(outfile);

  _write(handle, databuffer, sizeof(int) + outputcount * sizeof(DATA));

  fclose(outfile);

  free(databuffer);

  }


/*---------------------------------------------------------------------------
			 GET SS REC routine
			 gets a social security record
  ---------------------------------------------------------------------------*/

void getSSrec(
  int    codeid,             /* hrs identifier                     */
  int    *matchcode,         /* flag for record in SS file         */
  int    *qc,                /* total quarters of coverage         */
  double *ssearnings)        /* vector of social security earnings */

{ int recno, year;
  int earnings;

  unsigned char *earnrec;

  static int init = 0;
  static int sslocs[30000];
  static unsigned char ssdata[9539 * 42];

  if (init == 0)
  { initSSdata(sslocs, ssdata);
	 init = 1;
	 }

  recno = sslocs[codeid];

  if (recno >= 0)
  { *matchcode = 1;

	  earnrec = &ssdata[42 * recno];

	  *qc = earnrec[0];

	  for (year = 1951; year <= 1991; year++)
	  { if (year < 1966)
		  earnings = earnrec[1 + (year - 1951)] & 0x3F;
		  else if (year < 1980)
		  earnings = earnrec[1 + (year - 1951)];
		  else earnings = 4 * (earnrec[1 + (year - 1980)] & 0xC0)
							          +  earnrec[1 + (year - 1951)];

		  ssearnings[year - 1951] = 100 * earnings;
		  }
	  }
  else
  { *matchcode = 0;
	  *qc = 0;

	  for (year = 1951; year <= 1991; year++)
	  ssearnings[year - 1951] = 0;
	  }

  }


/*---------------------------------------------------------------------------
			 INIT SS DATA routine
			 reads social security earnings records from file
  ---------------------------------------------------------------------------*/

void initSSdata(
  int  *sslocs,             /* locations by hrs identifiers     */
  unsigned char *ssdata)    /* social security earnings records */

{ int recno;
  int codeid, year;
  int qc, totqc;
  int earnings;

  unsigned char *earnrec;
  
  char ssrec[444];

  FILE *infile, *outfile, *ssfile;
  int  handle, sshandle;

  for (codeid = 0; codeid < 30000; codeid++)
  sslocs[codeid] = -1;

  if (_access("ssdata.dat", 0) < 0)
  { for (codeid = 0; codeid < 30000; codeid++)
	  sslocs[codeid] = -1;

	  ssfile = fopen("ssaear3.da", "rb");

	  if (ssfile == NULL)
	  error("Social Security earnings file not found");

	  sshandle = _fileno(ssfile);

	  for (recno = 0; recno < 9539; recno++)
	  { _read(sshandle, ssrec, 444);

		  sscanf(&ssrec[437], "%5d", &codeid);

      if (0 < codeid && codeid < 30000)
		  sslocs[codeid] = recno;
      else wnprintf(" c%d", codeid);

		  earnrec = &ssdata[42 * recno];

		  sscanf(&ssrec[428], "%2d", &totqc);     /* 1947-50 quarters of coverage */

		  for (year = 1951; year <= 1991; year++)
		  { sscanf(&ssrec[9 + (year - 1951)], "%1d", &qc);

		    totqc = totqc + qc;
		    }

		  earnrec[0] = (char) totqc;

		  for (year = 1951; year <= 1991; year++)
		  { sscanf(&ssrec[50 + 9 * (year - 1951)], "%5d", &earnings);

		    if (earnings == -1)
		    earnings = 0;

		    earnings = earnings / 100;

		    if (year > 1979)
		    { if (earnings > 511)
			    { earnings = earnings - 512;
				    earnrec[1 + (year - 1980)] =
				      (unsigned char) (earnrec[1 + (year - 1980)] | 0x80);
				    }
			    else if (earnings > 255)
			    { earnings = earnings - 256;
				    earnrec[1 + (year - 1980)] =
				      (unsigned char) (earnrec[1 + (year - 1980)] | 0x40);
				    }
			    }

		    earnrec[1 + (year - 1951)] = (unsigned char) earnings;
		    }
		  }

	  fclose(ssfile);

    outfile = fopen("ssdata.dat", "wb");
    handle = _fileno(outfile);

    _write(handle, sslocs, 30000 * sizeof(int));
    _write(handle, ssdata, 9539 * 42 * sizeof(char));

    fclose(outfile);
	  }
  else
  { infile = fopen("ssdata.dat", "rb");
    handle = _fileno(infile);

    _read(handle, sslocs, 30000 * sizeof(int));
    _read(handle, ssdata, 9539 * 42 * sizeof(char));

    fclose(infile);
 	  }

  }


/*---------------------------------------------------------------------------
			 EVAL routine
			 gets a numeric-named variable from the HRS records
  ---------------------------------------------------------------------------*/

int eval(
  int  wave,
  int  varno)

{ char varname[9];

  switch(wave)
  { case  1:  varname[0] = 'V';      break;
    case  2:  varname[0] = 'W';      break;
    case  3:  varname[0] = 'E';      break;
    case  4:  varname[0] = 'F';      break;
    case  5:  varname[0] = 'G';      break;
    case  6:  varname[0] = 'H';      break;
    }

  sprintf(&varname[1], "%d", varno);

  return(evals(wave, varname));

  }


/*---------------------------------------------------------------------------
			 EVALS routine
			 gets a character-named variable from the HRS records
  ---------------------------------------------------------------------------*/

typedef struct
{ int  wave;
  char varname[8];
  int  loc;
  int  len;
  } VARDEF;


int evals(
  int  wave,
  char *vararg)

{ int loc, len, value;

  static char blanks[16] = "               ";

  getvarlocs(wave, vararg, &loc, &len);

  if (rec[loc] > 0)
  { if (memcmp(&rec[loc], blanks, len) != 0)
    switch(len)
    { case  1:  sscanf(&rec[loc],  "%1d", &value);     break;
	   case  2:  sscanf(&rec[loc],  "%2d", &value);     break;
	   case  3:  sscanf(&rec[loc],  "%3d", &value);     break;
	   case  4:  sscanf(&rec[loc],  "%4d", &value);     break;
	   case  5:  sscanf(&rec[loc],  "%5d", &value);     break;
	   case  6:  sscanf(&rec[loc],  "%6d", &value);     break;
	   case  7:  sscanf(&rec[loc],  "%7d", &value);     break;
	   case  8:  sscanf(&rec[loc],  "%8d", &value);     break;
	   case  9:  sscanf(&rec[loc],  "%9d", &value);     break;
	   /* case 10:  sscanf(&rec[loc], "%10d", &value);     break;
	   case 11:  sscanf(&rec[loc], "%11d", &value);     break;
	   case 12:  sscanf(&rec[loc], "%12d", &value);     break;
	   case 13:  sscanf(&rec[loc], "%13d", &value);     break;
	   case 14:  sscanf(&rec[loc], "%14d", &value);     break;
	   case 15:  sscanf(&rec[loc], "%15d", &value);     break; */
     default:  wnprintf("\nwave %d name %s loc %d len %d", wave, vararg, loc, len);
               error("eval length too long");         break;
      }
    else return(-9);
	 }
  else return(-9999);

  return(value);

  }


/*---------------------------------------------------------------------------
			 FEVAL routine
			 gets a numeric-named floating point variable from the HRS records
  ---------------------------------------------------------------------------*/

double feval(
  int  wave,
  int  varno)

{ char varname[9];

  switch(wave)
  { case  1:  varname[0] = 'V';      break;
    case  2:  varname[0] = 'W';      break;
    case  3:  varname[0] = 'E';      break;
    case  4:  varname[0] = 'F';      break;
    case  5:  varname[0] = 'G';      break;
    case  6:  varname[0] = 'H';      break;
    }

  sprintf(&varname[1], "%d", varno);

  return(fevals(wave, varname));

  }


/*---------------------------------------------------------------------------
			 FEVALS routine
			 gets a character-named floating point variable from the HRS records
  ---------------------------------------------------------------------------*/


double fevals(
  int  wave,
  char *vararg)

{ int loc, len;
  double value;

  static char blanks[16] = "               ";

  getvarlocs(wave, vararg, &loc, &len);

  if (rec[loc] > 0)
  { if (memcmp(&rec[loc], blanks, len) != 0)
    switch(len)
    { case  1:  sscanf(&rec[loc],  "%1lf", &value);     break;
	   case  2:  sscanf(&rec[loc],  "%2lf", &value);     break;
	   case  3:  sscanf(&rec[loc],  "%3lf", &value);     break;
	   case  4:  sscanf(&rec[loc],  "%4lf", &value);     break;
	   case  5:  sscanf(&rec[loc],  "%5lf", &value);     break;
	   case  6:  sscanf(&rec[loc],  "%6lf", &value);     break;
	   case  7:  sscanf(&rec[loc],  "%7lf", &value);     break;
	   case  8:  sscanf(&rec[loc],  "%8lf", &value);     break;
	   case  9:  sscanf(&rec[loc],  "%9lf", &value);     break;
	   case 10:  sscanf(&rec[loc], "%10lf", &value);     break;
	   case 11:  sscanf(&rec[loc], "%11lf", &value);     break;
	   case 12:  sscanf(&rec[loc], "%12lf", &value);     break;
	   case 13:  sscanf(&rec[loc], "%13lf", &value);     break;
	   case 14:  sscanf(&rec[loc], "%14lf", &value);     break;
	   case 15:  sscanf(&rec[loc], "%15lf", &value);     break;
     default:  error("feval length too long");         break;
      }
    else return(-9);
	 }
  else return(-9999);

  return(value);

  }


/*---------------------------------------------------------------------------
			 GET VAR LOCS routine
			 gets the location and length of the variable in the HRS record
  ---------------------------------------------------------------------------*/

void getvarlocs(
  int  wave,
  char *vararg,
  int  *loc,
  int  *len)

{ int   charloc;
  int   varno, varmin, varmax, newvar;

  char  var[8];

  static int varlims[6][2];
  static char varlocs[25519][12];

  static int init = 1;

  if (init == 1)
  { dictinit(varlims, varlocs);
	  init = 0;
	  }

  memcpy(var, vararg, 8);

  for (charloc = 1; charloc < 8; charloc++)
  if (var[charloc - 1] == ' ' || var[charloc] == '\0')
  var[charloc] = ' ';

  varmin = varlims[wave - 1][0];
  varmax = varlims[wave - 1][1];

  while (varmin + 1 < varmax)
  { newvar = (varmin + varmax) / 2;

    if (strncmp(var, varlocs[newvar], 8) > 0)
    varmin = newvar;
    else varmax = newvar;
    }

  if (varmin == varlims[wave][0])
  { if (strncmp(var, varlocs[varmin], 8) == 0)
    varno = varmin;
    else varno = varmax;
    }
  else varno = varmax;

  if (strncmp(var, varlocs[varno], 8) != 0)
  { wnprintf("\nVariable %8.8s in wave %1d not in dictionary", var, wave);
	  pause();
	  exit(0);
    }

  *loc = * (unsigned short *) &varlocs[varno][ 8];
  *len = * (unsigned short *) &varlocs[varno][10];

  }


/*---------------------------------------------------------------------------
			 DICT INIT routine
			 gets the HRS dictionaries to locate the variables
  ---------------------------------------------------------------------------*/

void dictinit(
  int  (*varlims)[2],
  char (*varlocs)[12])

{ int  wave, loc, len, var, varno;
  char linestring[120];

  void *varbuffer;
  VARDEF *vardefs;

  FILE *dictfile;

  varbuffer = malloc(25519 * sizeof(VARDEF));
  vardefs = (VARDEF *) varbuffer;

  memset(varlocs, 0, 25519 * 12);
  memset(vardefs, 0, 25519 * sizeof(VARDEF));

  dictfile = fopen("d:\\data\\hrs92-02.dic", "rt");

  if (dictfile == NULL)
  { wnprintf("\nMissing dictionary file");
    pause();
    exit(0);
    }

  var = 0;

  while(fgets(linestring, 120, dictfile) != NULL)
  { sscanf(&linestring[ 0], "%1d", &wave);
    sscanf(&linestring[67], "%5d", &loc);
	  sscanf(&linestring[73], "%3d", &len);

    vardefs[var].wave = wave;
    vardefs[var].loc  = loc - 1;
    vardefs[var].len  = len;

    strncpy(vardefs[var].varname, &linestring[2], 8);

    var = var + 1;
	 }

  fclose(dictfile);

  qsort(vardefs, var, sizeof(VARDEF), dictcomp);

  memset(varlims, 0, 10 * sizeof(int));

  for (varno = 0; varno < var; varno++)
  { strncpy(varlocs[varno], vardefs[varno].varname, 8);

    * (unsigned short *) &varlocs[varno][ 8] = (unsigned short) (vardefs[varno].loc);
	  * (unsigned short *) &varlocs[varno][10] = (unsigned short) (vardefs[varno].len);

    for (wave = 0; wave < 6; wave++)
    { if (vardefs[varno].wave - 1 < wave)
      varlims[wave][0] = varno + 1;

      if (vardefs[varno].wave - 1 <= wave)
      varlims[wave][1] = varno;
      }
    }

  }


/*---------------------------------------------------------------------------
			 DICT COMP routine
			 compare routine for qsort in the dictinit routine
  ---------------------------------------------------------------------------*/

int dictcomp(
  const void *elem1,
  const void *elem2)

{ if ((* (VARDEF *) elem1).wave < (* (VARDEF *) elem2).wave)
  return(-1);
  else if ((* (VARDEF *) elem1).wave > (* (VARDEF *) elem2).wave)
  return(1);

  if (strncmp((* (VARDEF *) elem1).varname, (* (VARDEF *) elem2).varname, 8) < 0)
  return(-1);
  if (strncmp((* (VARDEF *) elem1).varname, (* (VARDEF *) elem2).varname, 8) > 0)
  return(1);

  if ((* (VARDEF *) elem1).loc < (* (VARDEF *) elem2).loc)
  return(-1);
  else return(1);

  }


/*---------------------------------------------------------------------------
			 GET SURVIVAL RATES routine
			 gets cohort and gender specific survival rates
  ---------------------------------------------------------------------------*/

void getSurvivalRates(
  int cohort,                 /* year of birth            */
  int gender,                 /* sex of respondent        */
  double *surv)               /* vector of survival rates */

{ int filenumber;
  int age, coh, gen, line;
  int agecheck, survprob;

  char stringline[151];

  static int init = 1;
  static double survtable[91][2][120];

  static int startcohort[3] = { 0, 16, 56};
  static int endcohort[3]   = {15, 55, 90};
  static char infilename[3][12] = {"coh0015.95t", "coh1655.95t", "coh5690.95t"};

  FILE *infile, *outfile;
  int inhandle, outhandle;


  if (init == 1)
  { if (_access("SurvTabs.dat", 0) < 0)
	  { for (filenumber = 0; filenumber < 3; filenumber++)
		  { infile = fopen(infilename[filenumber], "r");

        if (infile == NULL)
        error("File not found in getSurvivalRates");

        for (coh = startcohort[filenumber]; coh <= endcohort[filenumber]; coh++)
        for (gen = 0; gen < 2; gen++)
        for (age = 0; age < 120; age++)
        { if (age % 40 == 0)
          for (line = 0; line < 8; line++)
          fgets(stringline, 150, infile);

          fgets(stringline, 150, infile);

          sscanf(&stringline[1], "%3d", &agecheck);
          if (age != agecheck)
          error("Bad input in getSurvivalRates");

          sscanf(&stringline[17], "%6d", &survprob);
          survtable[coh][gen][age] = survprob;

          if (age % 5 == 4)
          fgets(stringline, 150, infile);
          }

        fclose(infile);
        }

      outfile = fopen("SurvTabs.dat", "wb");
      outhandle = _fileno(outfile);
      _write(outhandle, survtable, sizeof(survtable));
      fclose(outfile);
      }
    else
    { infile = fopen("SurvTabs.dat", "r");
      inhandle = _fileno(infile);
      _read(inhandle, survtable, sizeof(survtable));
      fclose(infile);
      }

    init = 0;
    }

  if (cohort < 1900 || cohort > 1990 || gender < 1 || gender > 2)
  { wnprintf("\nBad cohort or gender: cohort %d  gender %d", cohort, gender);
    error("Bad cohort or gender in getSurvivalRates");
    }

  for (age = 0; age < 120; age++)
  surv[age] = survtable[cohort - 1900][gender - 1][age];

  }

