﻿char AppName[]    = { "GSModel" };                                    /* application name */
char AppTitle[]   = { "Gustman-Steinmeier Retirement Model" };        /* caption for window */
char AboutTitle[] = { "GS Retirement Model" };                        /* name given in about box */
char AboutDate[]  = { "Sept 2012" };                                  /* date given in about box */

#define _CRT_SECURE_NO_WARNINGS

#ifdef _WIN64
#include "winshell64.inc"
#else
#include "winshell.inc"
#endif

#include "time.h"
#include "winbase.h"

#define nthreads 1
#define nparms 8

#define maxobs 2500

#define ntcatlim 6
#define nacatlim 40
#define necatlim 17
#define nlcatlim 10
#define npcatlim 4
#define nrcatlim 17
#define nscatlim 4

#define nmoments sizeof(MOMENTSTRUCT) / sizeof(double)

typedef struct
{ double sige;                   /* standard deviation of epsilon, husband */
  double erho;                   /* correlation of husband's epsilon after retirement */
  double beta0;                  /* beta vector: constant */
  double beta1;                  /* beta vector: age */
  double beta2;                  /* beta vector: own health */
  double gamma0;                 /* gamma vector for partial retirement: constant */
  double gamma1;                 /* gamma vector for partial retirement: age */
  double alpha;                  /* consumption parameter */
  } PARM;

typedef struct
{ char age[13];
  char pr[5];
  char lowinc[5];
  char highinc[5];
  char health[5];
  char healthpr[5];
  char reversal[5];
  } MOBSSTRUCT;

typedef struct
{ double age[13];
  double pr[5];
  double lowinc[5];
  double highinc[5];
  double health[5];
  double healthpr[5];
  double reversal[5];
  } MOMENTSTRUCT;

typedef struct
{ int    mode;
  int    obs;

  double sige;
  double erho;
  double beta0;
  double beta1;
  double beta2;
  double gamma0;
  double gamma1;
  double alpha;

  /* the following quantities are calculated in calcPrep */

  int    birthyear;
  int    spbirthyear;

  int    firstage;
  int    lastage;

  int    earlypenage;                  /* first age with immediate pension */
  int    delayedpenage;                /* age at which delayed pensions can be collected */
  int    contribstartage;              /* age at which pension contributions start */
  int    penjobend;                    /* age at potential last year of pension job */
  int    sserage;                      /* social security early entitlement age */
  int    nrage;                        /* social security normal retirement age */
  int    spnrage;                      /* social security normal retirement age for spouse */

  double realrate[75];                 /* actual returns 1926-2000 from ibbotson */

  double realrateg;                    /* geometric mean of real interest rate */
  double realratea;                    /* arithmatic mean of real interest rate */
  double realratesd;                   /* standard deviation of real interest rate */
  double inflrate;                     /* inflation rate */
  double penadjrate;                   /* fraction by which pensions are adjusted for inflation */
  double delayedret;                   /* social security delayed retirement rate */
  double spdelayedret;                 /* social security delayed retirement rate for spouse */

  int    ntcats;                       /* number of partial retirement categories */
  int    nacats;                       /* number of asset categories */
  int    necats;                       /* number of epsilon categories */
  int    nlcats;                       /* number of pension lump sum categories */
  int    npcats;                       /* number of pension value categories */
  int    nrcats;                       /* number of return categories */
  int    nscats;                       /* number of social security value categories */

  double tcats[ntcatlim][20];          /* utility of partial retirement categories, 50-69 */
  double acats[nacatlim];              /* asset categories */
  double ecats[necatlim];              /* leisure preference categories */
  double lcats[nlcatlim];              /* pension lump sum categories */
  double pcats[npcatlim];              /* pension annual benefit categories */

  double mainwage[45];                 /* 25-69 */
  double secwage[20];                  /* 50-69 */
  double prwage[20];                   /* 50-69 */
  double eowamount[21];                /* 50-70, early out window amount */

  double initialpcat[21];              /* 50-70 */
  double initialscat[21];              /* 50-70 */
  double dccontrib[45];                /* 25-69 */

  double ssbenf[nscatlim];            /* full own benefits at scat ages */
  double ssbenm[45][2];               /* 25-69, surv states; main job */
  double ssbens[20][2][nscatlim];     /* 50-69, surv states; return to work */
  double ssbenp[20][2][nscatlim];     /* 50-69, surv states; partial retirement */
  double ssbenr[50][3][nscatlim];     /* 50-99, surv states; full retirement */
  double othincome[75][3];            /* 25-99, surv states */

  double benchanges[8][2][nscatlim];  /* 62-69, surv states; return to work */
  double benchangep[8][2][nscatlim];  /* 62-69, surv states; partial retirement */

  double srate[2][75];                 /* resp/spouse, 25-99 */
  double transrate[3][75];             /* surv states, 25-99 */
  double etrans[necatlim][necatlim];   /* necats, necats */
  double rprob[nrcatlim];              /* nrcats */
  double rtransa[nacatlim][nrcatlim];  /* nacats, nrcats */

  double rtransl[45][nlcatlim][nrcatlim];  /* 25-69 */

  double utilr[necatlim][20];          /* necats, 50-69 */

  /* the following matrices are used in rhocalc and associated routines only */

  double income[3][75];                /* surv states, 25-99 */
  double cons[75][3][nacatlim];        /* 25-75, surv states */

  /* the following matrices are used in calcModel only */

         /* cons, mu & tu in following period */
  double marguf[3][2][npcatlim][nscatlim][nacatlim];  /* surv states, pendelay */
  double totuf[3][2][npcatlim][nscatlim][nacatlim];   /* surv states, pendelay */

         /* current cons, mu & tu */
  double marguc[3][2][npcatlim][nscatlim][nacatlim];  /* surv states, pendelay */
  double totuc[3][2][npcatlim][nscatlim][nacatlim];   /* surv states, pendelay */

         /* mu & tu in following period, given current main job */
  double margufw[2][nlcatlim][necatlim][nacatlim];    /* surv states */
  double totufw[2][nlcatlim][necatlim][nacatlim];     /* surv states */

         /* mu & tu in following period, given not currently in main job */
  double margufr[3][2][npcatlim][nscatlim][necatlim][nacatlim];  /* surv states, pendelay */
  double totufr[3][2][npcatlim][nscatlim][necatlim][nacatlim];   /* surv states, pendelay */

         /* adjusted mu & tu in following period, given current main job */
  double cmatw[2][nlcatlim][necatlim][nacatlim];      /* surv states */
  double tumatw[2][nlcatlim][necatlim][nacatlim];     /* surv states */

         /* adjusted mu & tu in following period, given not currently in main job */
  double cmatr[3][2][npcatlim][nscatlim][necatlim][nacatlim];    /* surv states, pendelay */
  double tumatr[3][2][npcatlim][nscatlim][necatlim][nacatlim];   /* surv states, pendelay */

         /* current mu & tu if currently in main job */
  double margucw[2][nlcatlim][necatlim][nacatlim];    /* surv states */
  double totucw[2][nlcatlim][necatlim][nacatlim];     /* surv states */

         /* current mu & tu if currently not in main job */
  double margucr[3][3][2][npcatlim][nscatlim][necatlim][nacatlim];  /* 0=secjob 1=pr 2=ret, surv states, pendelay */
  double totucr[3][3][2][npcatlim][nscatlim][necatlim][nacatlim];   /* 0=secjob 1=pr 2=ret, surv states, pendelay */

  /* the following matrices are calculated in calcModel and passed to getMoments */

  double totuw[45][nlcatlim][necatlim][nacatlim];                   /* 25-69 */
  double totud[20][3][2][npcatlim][nscatlim][necatlim][nacatlim];   /* 50-69, 0=secjob 1=pr 2=ret, pendelay */

  double savw[45][nlcatlim][necatlim][nacatlim];                    /* 25-69 */
  double savd[20][3][2][npcatlim][nscatlim][necatlim][nacatlim];    /* 50-69, 0=secjob 1=pr 2=ret, pendelay */
  double savr[30][2][npcatlim][nscatlim][nacatlim];                 /* 70-99, pendelay */

  int    mjchoice[20][nlcatlim][necatlim][nacatlim];                /* 50-69 */
  int    retchoice[20][2][npcatlim][nscatlim][necatlim][nacatlim];  /* 50-69, pendelay */

  /* the following variables and vectors are used only in getMoments */

  double everreturnft;
  double everreturnpt;
  double everreturn;

  double ret0vec[21];
  double ret1vec[21];
  double ret2vec[21];
  double returnft[21];
  double returnpt[21];
  double newpt[21];
  double reversal[5];
  } CALCSTRUCT;

typedef struct
{ int    origobs;         /* observation number in original data set            */
  int    caseid;          /* caseid number in original data set                 */
  int    birthyear;       /* year of birth                                      */
  int    spbirthyear;     /* year of spouse's birth                             */
  int    gender;          /* gender                                             */
  int    age[6];          /* age in each survey                                 */
  int    ret[6];          /* retirement status in each survey                   */
  int    retv[20];        /* retirement status vector for ages 50-69            */
  int    spretage;        /* respondent age when spouse can collect ss benefits */
  int    healthage;       /* initial age of health problem                      */
  int    penjobend;       /* age at potential last year of pension job          */
                                  /* all amounts are in 1992$ */
  double totalwealth;     /* wealth in 1992                                     */
  double assetwealth;     /* land, business, financial & "other" wealth in 1992 */
  double ftwage[45];      /* full-time wages at ages 25-69                      */
  double prwage[20];      /* partial retirement wages at ages 50-69             */
  double secwage[20];     /* full-time wages in reveral jobs at ages 50-69      */
  double spwage[45];      /* wages of spouse at spouse ages 25-69               */
  double pensben[21];     /* annual pension benefit for retirement at 50-70     */
  int    penstart[21];    /* starting age of pension for retirement at 50-70    */
  double plumpsum[21];    /* lump-sum pension benefit for retirement at 50-70   */
  double eowamount[21];   /* early out window lump sum amounts at 50-70         */
  double sppensben;       /* annual pension benefit for spouse                  */
  int    sppenstart;      /* starting spouse age of pension                     */
  double spplumpsum;      /* amount of spouse pension lump sum;                 */
  double pia[21];         /* pia at ages 50-69                                  */
  double ncpens[21];      /* noncovered pensions for retirement at 50-70        */
  double sppia[21];       /* spouse pia at spouse ages 50-70                    */
  double spncpens[21];    /* noncovered spouse pens for spouse retirement 50-70 */
  double othincome[75];   /* inheritances for ages 25-99                        */

  int    inputobs;        /* observation number in the input file               */
  int    inccat;          /* category of lifetime income (1=low, 2=med, 3=high) */
  } DATA;

typedef struct
{ int    mode;                    /* 0: test; 1: iteration; 2: estimation; 3: simulation */
  int    newalpha;                /* new value of alpha */
  char   nmat[maxobs][nmoments];
  double mvec[nthreads][nmoments];
  double wmat[nthreads][nmoments * (nmoments + 1) / 2];
  double rvec[maxobs];
  double ret0mat[nthreads][21];
  double ret1mat[nthreads][21];
  double ret2mat[nthreads][21];
  double returnft[nthreads][21];
  double returnpt[nthreads][21];
  double newpt[nthreads][21];
  double everreturnft[nthreads];
  double everreturnpt[nthreads];
  double everreturn[nthreads];
  } ARGLIST;


void   inpt(int *nobs);
void   test(void);
void   setData(void);
double iterate(int nobs);
void   qmin(int nobs);
void   covar(double *var, double *wind, double *dmat);
void   inv(double *a, int n, double eps, int *err);
double f(double *mvec, double *winv, int nobs, int sw);
void   sim(int nobs);
void   threadprocess(void *voidthreadno);
void   indivmodel(void *voidthreadno);
void   calcPrep(int threadno);
double rhocalc(int threadno);
void   incomecalc(int, CALCSTRUCT *, DATA *);
void   conscalc(double, CALCSTRUCT *, double);
double wealthcalc(CALCSTRUCT *, DATA *);
double adjben(int, int, CALCSTRUCT *);
void   sscalc(int threadno);
void   calcModel(int threadno, int tcat);
void   getMoments(int threadno, int tcat);
void   getSurvivalRates(int birthyear, int gender, double *surv);
double erf(double x);
double frand(int threadno);
double gaussdev(int threadno);
void   randreset(int threadno, int seed);

int    prntlevel = 0;           /* flag for printing intermediate results */

int    localthreadmin, localthreadmax;
int    threadstate[nthreads];
int    threadobs[nthreads];

CALCSTRUCT *calcStructv;
ARGLIST     arglist;
DATA        datav[maxobs];
PARM        parm;


/*---------------------------------------------------------------------------
			 MAINPROG
			 estimates parameters for model with variable time preference
  ---------------------------------------------------------------------------*/

void mainprog()
{ int    nobs, threadno, runmode, count;
  int    startmonth, startday, starthour, startmin;

  double q;
  double surv[120];

  static DATA data[12652];

  void   *calcStructbuffer;

  static int threadindex[48] = { 0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                                10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                                20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                                30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                                40, 41, 42, 43, 44, 45, 46, 47 };

  INT64  longtime;
  tm     newtime;


  /* wnprintf("%d", sizeof(CALCSTRUCT)); pause(); */

  localthreadmin = -1;
  localthreadmax = -1;
  
  while (localthreadmin < 0 || localthreadmax < localthreadmin || nthreads <= localthreadmax)
  wnscanf("Input range of threads", "%d %d", &localthreadmin, &localthreadmax);

  runmode = 0;              /* 0: test; 1: iterate; 2: estimate; 3: simulate */

  if (runmode > 0 && localthreadmin == 0)
  inpt(&nobs);              /* get sample budget sets */

  /* pause if desired */

  startmonth = -1;  /* 0-11 */
  startday = 1;    /* 1-31 */
  starthour = 5;   /* 0-23 */
  startmin = 15;   /* 0-59 */

  if (startmonth > 0)
  { eprintf("\nWaiting until month %d day %d, hour %d, minute %d\n", startmonth, startday, starthour, startmin);
  
    _time64(&longtime);
    _localtime64_s(&newtime, &longtime);
  
    while(newtime.tm_mon < startmonth || newtime.tm_mday < startday
       || newtime.tm_hour < starthour || newtime.tm_min < startmin)
    { Sleep(1000);
      _time64(&longtime);
      _localtime64_s(&newtime, &longtime);
      eprintf("\rCurrently month %d day %d, hour %d, minute %d", newtime.tm_mon, newtime.tm_mday, newtime.tm_hour, newtime.tm_min);
      }
    }

  /* allocate buffer for calcStruct */

  calcStructbuffer = malloc((localthreadmax - localthreadmin + 1) * sizeof(CALCSTRUCT));

  if (calcStructbuffer == NULL)
  error("calcStruct buffer allocation failed");
  
  calcStructv = (CALCSTRUCT *) calcStructbuffer;

  /* initialize survival tables */

  getSurvivalRates(1950, 1, surv);

  /* start calculation threads */

  for (threadno = 0; threadno < nthreads; threadno++)
  { threadstate[threadno] = 0;

    if (localthreadmin == 0)
    { if(threadno <= localthreadmax)
      _beginthread(indivmodel, 1048576, (void *) &threadindex[threadno]);
      else _beginthread(threadprocess, 1048576, (void *) &threadindex[threadno]);
      }
    else if (localthreadmin <= threadno && threadno <= localthreadmax)
    _beginthread(threadprocess, 104856, (void *) &threadindex[threadno]);
    }

  /* pause the main program indefinitely on satellite computers */
  if (0 < localthreadmin)
  { count = 0;
    
    while (count < 1000000000)
    { checkMessages();
      Sleep(1000);
      count = count + 1;
      }
    }

  /* assign initial parameter values */

  parm.sige   =   5.8700;  /* sige / beta[1]: std dev of leisure preference random effect */
  parm.erho   =   0.7100;  /* erho: autocorrelation parameter for epsilon */
  parm.beta0  =  -9.8000;  /* beta[0]: constant term in utility fn */
  parm.beta1  =   0.0670;  /* beta[1]: coefficient of age - 62 */
  parm.beta2  =   5.7000;  /* beta[2] / beta[1]: coefficient of health variable */
  parm.gamma0 =  -3.6200;  /* gamma[0]: constant term in pr density function */
  parm.gamma1 =   0.1600;  /* gamma[1]: coefficient of age in pr density function */
  parm.alpha  =  -0.1600;  /* alpha: exponent of consumption */

  if (runmode == 0)
  { memset(datav, 0, sizeof(DATA));
    test();
    }
  else if (runmode == 1)
  q = iterate(nobs);
  else
  { if (runmode == 2)
    qmin(nobs);

    sim(nobs);
    }

  for (threadno = 0; threadno < nthreads; threadno++)
  threadstate[threadno] = -1;
  
  free(calcStructbuffer);

  }


/*---------------------------------------------------------------------------
			 INPT routine
			 obtains the input
  ---------------------------------------------------------------------------*/

void inpt(
  int  *nobs)                 /* number of observations         */

{ int    obs, origobs, totobs;
  int    loc;
  int    deletionreason;
  int    age, flength, tabs;
  int    surv, ret;
  double wealth;

  int    badobsvec[13];
  double  ret1[20], ret2[20], ret3[20], cret1[20], cret2[20], tot[20];

  typedef struct
  { short  origobs;         /* observation number in original data set            */
    short  caseid;          /* caseid number in original data set                 */
    char   marstat;         /* 1992 marital status                                */
    char   race;            /* race/ethnicity                                     */
    short  birthyear;       /* year of birth                                      */
    short  spbirthyear;     /* year of spouse's birth                             */
    char   gender;          /* gender                                             */
    char   retage;          /* age of actual or expected retirement from ft job   */
    char   spretage;        /* spouse age when spouse can collect ss benefits     */
    char   healthage;       /* initial age of health problem                      */
    char   penjobend;       /* age at potential last year of pension job          */
    char   origjobend;      /* age of separation from original job                */
    char   ind;             /* industry of original job                           */
    char   occ;             /* occupation of original job                         */
    char   age[6];          /* age at the survey dates                            */
    char   ret[6];          /* retirement status at the survey dates              */
    char   retv[20];        /* retirement status vector for ages 50-69            */
    char   ssreceipt[6];    /* social security recipiency status for survey dates */

                                    /* all amounts are in 1992$ */
    float  wealth;          /* wealth in 1992                                     */
    float  assetwealth;     /* land, business, financial & "other" wealth in 1992 */
    float  ftwage[45];      /* full-time wages at ages 25-69                      */
    float  prwage[20];      /* partial retirement wages at ages 50-69             */
    float  secwage[20];     /* full-time wages in reversal jobs at ages 50-69     */
    float  sswage[45];      /* social security wages at ages 25-69                */
    float  spwage[45];      /* wages of spouse at spouse ages 25-69               */
    float  spsswage[45];    /* social security wages at ages 25-69                */
    float  pensben[21];     /* annual pension benefit for retirement at 50-70     */
    float  penstart[21];    /* starting age of pension for retirement at 50-70    */
    float  plumpsum[21];    /* lump-sum pension benefit for retirement at 50-70   */
    float  eowamount[21];   /* early out window lump sum amounts at 50-70         */
    float  projpenben;      /* actual/projected pension benefit                   */
    char   projpstart;      /* actual/projected starting age of pension benefit   */
    char   projpenjobend;   /* actual/projected age at last year of pension job   */
    float  projplumpsum;    /* actual/projected lump-sum pension benefit          */
    float  sppensben;       /* annual pension benefit for spouse                  */
    char   sppenstart;      /* starting spouse age of pension                     */
    char   sppjobend;       /* spouse age at last year of spouse pension job      */
    float  spplumpsum;      /* amount of spouse pension lump sum                  */
    float  aime[21];        /* aime at ages 50-70                                 */
    float  pia[21];         /* pia at ages 50-70                                  */
    float  ncpens[21];      /* noncovered pensions for retirement at 50-70        */
    float  sppia[21];       /* spouse pia at spouse ages 50-70                    */
    float  spncpens[21];    /* noncovered spouse pens for spouse retirement 50-70 */
    float  othincome[75];   /* inheritances for ages 25-99                        */
    
    struct
    { char  spouseinterview;
      char  goodCareer;
      char  goodSSstatus;
      char  goodFTyears;
      char  goodearnings;
      char  goodSSearn;
      char  goodAssets;
      char  goodPens;
      char  spouseFTyears;
      } goodobs;
    } ORIGDATA;


  DATA *data;
  ORIGDATA *origdata;
  void *origdatabuffer, *inbuffer;

  FILE *infile;
  int  handle;


  tabs = 1;

  /* read in data */

  if (_access("GSRetirementData.dat", 0) < 0)
  error("Input file not found");

  infile = fopen("GSRetirementData.dat", "rb");
  handle = _fileno(infile);

  flength = _filelength(handle);

  inbuffer = malloc(flength);

  _read(handle, inbuffer, flength); 

  fclose(infile);

  origobs = * (int *) inbuffer;

  origdatabuffer = malloc(flength - sizeof(int));
  origdata = (ORIGDATA *) origdatabuffer;
  
  memmove(origdata, &((char *) inbuffer)[sizeof(int)], origobs * sizeof(ORIGDATA));
  
  free(inbuffer);


  /* choose observations of interest */

  *nobs = 0;
  totobs = 0;

  memset(badobsvec, 0, sizeof(int[13]));

  for (obs = 0; obs < origobs; obs++)
  { totobs = totobs + 1;

    /* create original observation structure */

    loc = 0;
    deletionreason = 0;

    if (origdata[obs].goodobs.spouseFTyears    == 0)   deletionreason = 11;
    if (origdata[obs].goodobs.goodPens         == 0)   deletionreason = 10;
    if (origdata[obs].goodobs.goodAssets       == 0)   deletionreason =  9;
    if (origdata[obs].goodobs.goodSSearn       == 0)   deletionreason =  8;
    if (origdata[obs].goodobs.goodearnings     == 0)   deletionreason =  7;
    if (origdata[obs].goodobs.goodFTyears      == 0)   deletionreason =  6;
    if (origdata[obs].goodobs.goodSSstatus     == 0)   deletionreason =  5;
    if (origdata[obs].goodobs.goodCareer       == 0)   deletionreason =  4;
    if (origdata[obs].goodobs.spouseinterview  == 0)   deletionreason =  3;
    if (origdata[obs].marstat                  != 2)   deletionreason =  2;
    if (origdata[obs].gender                   != 1)   deletionreason =  1;

    if (deletionreason > 0)
    badobsvec[deletionreason] = badobsvec[deletionreason] + 1;

    if (deletionreason == 0)
    { data = &datav[*nobs];

      data->origobs      = origdata[obs].origobs;
      data->caseid       = origdata[obs].caseid;
      data->birthyear    = origdata[obs].birthyear;
      data->spbirthyear  = origdata[obs].spbirthyear;
      data->gender       = origdata[obs].gender;
      data->spretage     = origdata[obs].spretage;
      data->healthage    = origdata[obs].healthage;
      data->penjobend    = origdata[obs].penjobend;
      data->totalwealth  = origdata[obs].wealth;
      data->assetwealth  = origdata[obs].assetwealth;
      data->sppensben    = origdata[obs].sppensben;
      data->sppenstart   = origdata[obs].sppenstart;
      data->spplumpsum   = origdata[obs].spplumpsum;

      for (surv = 0; surv < 6; surv++)
      { data->age[surv]  = origdata[obs].age[surv];
        data->ret[surv]  = origdata[obs].ret[surv];
        }

      for (age = 25; age < 70; age++)
      { data->ftwage[age - 25]   = origdata[obs].ftwage[age - 25];
        data->spwage[age - 25]   = origdata[obs].spwage[age - 25];
        }

      for (age = 50; age < 70; age++)
      { data->retv[age - 50]    = origdata[obs].retv[age - 50];
        data->prwage[age - 50]  = origdata[obs].prwage[age - 50];
        data->secwage[age - 50] = origdata[obs].secwage[age - 50];
        }

      for (age = 50; age <= 70; age++)
      { data->pensben[age - 50]   = origdata[obs].pensben[age - 50];
        data->penstart[age - 50]  = (int) origdata[obs].penstart[age - 50];
        data->plumpsum[age - 50]  = origdata[obs].plumpsum[age - 50];
        data->eowamount[age - 50] = origdata[obs].eowamount[age - 50];
        data->pia[age - 50]       = origdata[obs].pia[age - 50];
        data->ncpens[age - 50]    = origdata[obs].ncpens[age - 50];
        data->sppia[age - 50]     = origdata[obs].sppia[age - 50];
        data->spncpens[age - 50]  = origdata[obs].spncpens[age - 50];
        }
               
      for (age = 25; age < 100; age++)
      data->othincome[age - 25] = origdata[obs].othincome[age - 25];

      data->inputobs = obs;

      *nobs = *nobs + 1;

      if (*nobs > maxobs)
      error("Too many observations");
      }
    }

  free(origdatabuffer);


  /* print table of deletions */

  wnprintf("\n\n                 Reasons for Deletions of Observations");
  wnprintf("\n\n                                                 Obs       Obs");
  wnprintf(  "\n                                               Dropped     Left");


  for (deletionreason = 0; deletionreason < 12; deletionreason++)
  if (deletionreason == 0 || badobsvec[deletionreason] > 0)
  { switch(deletionreason)
    { case  0:  wnprintf("\nTotal sample individuals                    ");   break;
      case  1:  wnprintf("\nNot specified gender                        ");   break;
      case  2:  wnprintf("\nNot specified marital status                ");   break;
      case  3:  wnprintf("\nSpouse not interviewed if married           ");   break;
      case  4:  wnprintf("\nNot a career worker                         ");   break;
      case  5:  wnprintf("\nAmbiguity about whether jobs are ss covered ");   break;
      case  6:  wnprintf("\nFT years unavailable in wave 3 or ss record ");   break;
      case  7:  wnprintf("\nNo FT earnings in ss record or self report  ");   break;
      case  8:  wnprintf("\nNo sr earnings, and ss earnings over limit  ");   break;
      case  9:  wnprintf("\nRelatively large business assets            ");   break;
      case 10:  wnprintf("\nNo Pension Provider record in last job      ");   break;
      case 11:  wnprintf("\nFT years unavailable for spouse             ");   break;
      }

    if (deletionreason == 0)
    wnprintf("              %5d", totobs);
    else
    { totobs = totobs - badobsvec[deletionreason];
      wnprintf("   %5d      %5d", badobsvec[deletionreason], totobs);
      }
    }


  /* process observations */

  for (obs = 0; obs < *nobs; obs++)
  { data = &datav[obs];

    if (data->totalwealth < 0)
    data->totalwealth = 0;

    if (data->assetwealth < 0)
    data->assetwealth = 0;

    /* calculate lifetime wealth category */

    wealth = 0;

    for (age = 25; age < 62; age++)
    wealth = wealth + data->ftwage[age - 25] + data->spwage[age - 25]
            + data->othincome[age - 25];

    wealth = wealth + data->plumpsum[age - 50] + data->spplumpsum;

         if (wealth < 1250000)   data->inccat = 1;
    else if (wealth < 1900000)   data->inccat = 2;
    else                         data->inccat = 3;
    }

  if (tabs == 1)
  { /* sample tabulations of retirement status */

    memset(ret1, 0, sizeof(double[20]));
    memset(ret2, 0, sizeof(double[20]));
    memset(ret3, 0, sizeof(double[20]));

    for (obs = 0; obs < *nobs; obs++)
    for (age = 50; age < 70; age++)
    { ret = datav[obs].retv[age - 50];

      if (ret == 5)
      ret1[age - 50] = ret1[age - 50] + 1;
      else if (ret == 3)
      ret2[age - 50] = ret2[age - 50] + 1;
      else if (ret == 1)
      ret3[age - 50] = ret3[age - 50] + 1;
      }

    wnprintf("\n\n      Observed Retirement Percentages\n");

    for (age = 50; age < 70; age++)
    { tot[age - 50] = ret1[age - 50] + ret2[age - 50] + ret3[age - 50];

      ret1[age - 50] = 100 * ret1[age - 50] / tot[age - 50];
      ret2[age - 50] = 100 * ret2[age - 50] / tot[age - 50];
      ret3[age - 50] = 100 * ret3[age - 50] / tot[age - 50];

      cret1[age - 50] = 100 - ret1[age - 50];
      cret2[age - 50] = ret3[age - 50];

      if (age == 50)
      { ret1[age - 50] = cret1[age - 50];
        ret2[age - 50] = cret2[age - 50];
        }
      else
      { ret1[age - 50] = cret1[age - 50] - cret1[(age - 1) - 50];
        ret2[age - 50] = cret2[age - 50] - cret2[(age - 1) - 50];
        }

      wnprintf("\n %2d  %5.1f %5.1f   %5.1f %5.1f   %4.0f", age,
       ret1[age - 50], ret2[age - 50], cret1[age - 50], cret2[age - 50], tot[age - 50]);
      }

    wnprintf("\nTotal number of observed respondents: %d", *nobs);
    }

  }


/*---------------------------------------------------------------------------
			 TEST routine
			 runs model for a test individual
  ---------------------------------------------------------------------------*/

void test()

{ int    nobs, age;
  int    firstage, lastage;
  double *rho;

  double ret0mat[21], ret1mat[21], ret2mat[21];
  double returnft[21], returnpt[21], newpt[21];
  double everreturnft, everreturnpt, everreturn;

  
  setData();

  nobs = 1;

  arglist.mode         = 0;
  arglist.newalpha     = 1;
  
  rho = &(arglist.rvec[0]);

  memset(arglist.nmat,         0, nobs * sizeof(char[nmoments]));
  memset(arglist.mvec,         0, sizeof(double[nthreads][nmoments]));
  memset(arglist.wmat,         0, sizeof(double[nthreads][nmoments * (nmoments + 1) / 2]));
  memset(arglist.ret0mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.ret1mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.ret2mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.returnft,     0, sizeof(double[nthreads][21]));
  memset(arglist.returnpt,     0, sizeof(double[nthreads][21]));
  memset(arglist.newpt,        0, sizeof(double[nthreads][21]));
  memset(arglist.everreturnft, 0, sizeof(double[nthreads]));
  memset(arglist.everreturnpt, 0, sizeof(double[nthreads]));
  memset(arglist.everreturn,   0, sizeof(double[nthreads]));

  threadobs[0] = nobs - 1;
  threadstate[0] = 1;

  while(threadstate[0] > 0)
  checkMessages();

  /* calculate results */

  firstage = calcStructv[nobs - 1].firstage;
  lastage = calcStructv[nobs - 1].lastage;

  for (age = firstage; age <= lastage; age++)
  { ret0mat[age - 50]  = 100 * arglist.ret0mat[0][age - 50];
    ret1mat[age - 50]  = 100 * arglist.ret1mat[0][age - 50];
    ret2mat[age - 50]  = 100 * arglist.ret2mat[0][age - 50];
    returnft[age - 50] = 100 * arglist.returnft[0][age - 50];
    returnpt[age - 50] = 100 * arglist.returnpt[0][age - 50];
    newpt[age - 50]    = 100 * arglist.newpt[0][age - 50];
    }
      
  everreturnft = 100 * arglist.everreturnft[0];
  everreturnpt = 100 * arglist.everreturnpt[0];
  everreturn   = 100 * arglist.everreturn[0];

  /* print tabulations */


  wnprintf("\n\n                Percent                   Percent");
  wnprintf(  "\n            Pseudo-Retiring               Retired");
  wnprintf(  "\n           From                       From");
  wnprintf(  "\nAge       FT Work  Completely        FT Work  Completely");
  wnprintf(  "\n");

  wnprintf("\n %2d                                  %5.1f     %5.1f",
   firstage, ret1mat[firstage - 50], ret2mat[firstage - 50]);

  for (age = firstage + 1; age <= lastage; age++)
  wnprintf("\n %2d       %5.1f    %5.1f             %5.1f     %5.1f", age,
   ret1mat[age - 50] - ret1mat[(age - 1) - 50], ret2mat[age - 50] - ret2mat[(age - 1) - 50],
   ret1mat[age - 50], ret2mat[age - 50]);


  wnprintf("\n\n\n                       Percent    Percent                          Percent Newly");
  wnprintf(    "\n                         in        Newly                   Percent   Returned");
  wnprintf(    "\n        Percent        FT Work    Returned       Percent    Newly   to PT Work");
  wnprintf(    "\nAge     in Main         after       to             in        in     Previously");
  wnprintf(    "\n          Job          Retiring   FT Work        PT Work   PT Work   Retired");
  wnprintf(    "\n");

  wnprintf("\n %2d      %5.1f                                   %5.1f",
   firstage, 100 - ret0mat[firstage - 50], ret1mat[firstage - 50] - ret2mat[firstage - 50]);

  for (age = firstage + 1; age <= lastage; age++)
  wnprintf("\n %2d      %5.1f          %5.1f     %5.1f          %5.1f     %5.1f     %5.1f", age,
   100 - ret0mat[age - 50], ret0mat[age - 50] - ret1mat[age - 50], returnft[age - 50],
   ret1mat[age - 50] - ret2mat[age - 50], newpt[age - 50], returnpt[age - 50]);

  wnprintf("\n\nPercent returning to full time work after full or partial retirement: %5.1f", everreturnft);
  wnprintf("\nPercent returning to part time work after full retirement: %5.1f", everreturnpt);
  wnprintf("\nPercent returning to full time work after full or partial retirement");
  wnprintf("\n  or returning to part time work after full retirement: %5.1f", everreturn);

  }


/*---------------------------------------------------------------------------
			 SET DATA
			 prepares data structure for hypothetical individual
  ---------------------------------------------------------------------------*/

void setData()

{ int    age, surv;

  static int    ages[6]   =  { 58, 60, 62, 64, 66, 68 };
  static int    rets[6]  =  {  5,  5,  1,  1,  1,  1 };
  static int    retv[20] =  { 0, 0, 0, 0, 0, 0, 0, 5, 5, 5, 5, 0, 1, 1, 1, 1, 1, 1, 1, 0 };

  DATA   *data;
  
  
  data = &datav[0];

  data->inputobs = 0;                             /* observation number in input file                   */

  /* husband's variables */

  data->origobs     =     0;                      /* observation number in original data set            */
  data->caseid      = 10001;                      /* caseid number in original data set                 */
  data->birthyear   =  1931;                      /* year of birth                                      */
  data->spbirthyear =  1933;                      /* year of spouse's birth                             */
  data->gender      =     1;                      /* gender                                             */
  data->healthage   =    99;                      /* initial age of health problem                      */
  data->penjobend   =    99;                      /* age at potential last year in pension job          */

  for (surv = 0; surv < 6; surv++)
  { data->age[surv] =  ages[surv];                /* age at the survey dates                            */
    data->ret[surv] = rets[surv];                 /* retirement status at survey dates                  */
    }

  for (age = 50; age < 70; age++)
  data->retv[age - 50] = retv[age - 50];          /* retirement status at ages 50-70                  */

  for (age = 25; age < 70; age++)
  data->ftwage[age - 25] = 30000;                 /* full-time wages at ages 25-69                      */

  for (age = 50; age < 70; age++)
  { data->secwage[age - 50] = 18000;              /* full-time wages in reversal jobs at ages 50-69     */
    data->prwage[age - 50]  =  6000;              /* partial retirement wages at ages 50-69             */
    }

  for (age = 25; age < 70; age++)
  { if (age < 62)
    data->spwage[age - 25] = 10000;               /* full-time wages at ages 25-69                      */
    else data->spwage[age - 25] = 0;
    }

  for (age = 50; age <= 70; age++)
  { data->pensben[age - 50]   =     0;            /* annual pension benefit for retirement at 50-70     */
    data->penstart[age - 50]  =    99;            /* starting age of pension for retirement at 50-70    */
    data->plumpsum[age - 50]  =     0;            /* lump-sum pension benefit for retirement at 50-70   */
    data->eowamount[age - 50] =     0;            /* early out window lump sum amounts at 50-70         */
    data->pia[age - 50]       = 10000;            /* pia at ages 50-69                                  */
    data->ncpens[age - 50]    =     0;            /* noncovered pensions for retirement at 50-70        */
    data->sppia[age - 50]     =  4000;            /* pia at ages 50-69, wife                            */
    data->spncpens[age - 50]  =     0;            /* noncovered pensions for retirement at 50-70, wife  */
    }

  for (age = 50; age <= 70; age++)
  { data->pensben[age - 50] = 4000 + 200 * (age - 50);   /* annual pension benefit for retirement at 50-70 */

    if (age < 60)
    { data->pensben[age - 50] = data->pensben[age - 50] / exp(0.0248 * (60 - 60) + 0.05 * (60 - age));
      data->penstart[age - 50] = 60 ;             /* starting age of pension for retirement at 50-70    */
      }
    else data->penstart[age - 50] =  age;
    
    data->plumpsum[age - 50] = 500 * (age - 40);  /* lump-sum pension benefit for retirement at 50-70 */
    }

  data->sppensben = 700;                          /* annual pension benefit, wife   */
  data->sppenstart = 62;                          /* starting age of pension, wife  */
  data->spplumpsum = 0;                           /* lump-sum pension benefit, wife */

  for (age = 50; age <= 70; age++)
  { data->pia[age - 50] = 10000 + 100 * (age - 65);
    data->sppia[age - 50] = 4000 + 25 * (age - 65);
    }

  data->totalwealth =  10000;                     /* wealth in 1992                                                        */
  data->assetwealth =  10000;                     /* land, business, financial & "other" wealth in 1992 (excludes housing) */

  for (age = 25; age < 100; age++)
  data->othincome[age - 25] = 0;                  /* inheritances for ages 25-99                        */

  data->inputobs = 0;
  data->inccat = 0;

  }


/*---------------------------------------------------------------------------
			 ITERATE
			 performs one evaluation of the objective function
  ---------------------------------------------------------------------------*/

double iterate(
  int    nobs)             /* number of observations */

{ int    k;
  int    threadno, age;

  double q;
  double everreturnft, everreturnpt, everreturn;

  double ret1mat[21], ret2mat[21];

  double mvec[nmoments];
  double winv[nmoments * (nmoments + 1) / 2];

  double *cparm;


  arglist.mode = 1;

  cparm = (double *) &parm;

  for (k = 0; k < nparms; k++)
  { if (k % 10 == 0)
    eprintf("\n    ");
    
    eprintf(" %6.3f", cparm[k]);
    }

  q = f(mvec, winv, nobs, 1);

  /* calculate results */

  for (age = 50; age < 70; age++)
  { ret1mat[age - 50]  = 0;
    ret2mat[age - 50]  = 0;
      
    for (threadno = 0; threadno < nthreads; threadno++)
    { ret1mat[age - 50]  = ret1mat[age - 50] + arglist.ret1mat[threadno][age - 50];
      ret2mat[age - 50]  = ret2mat[age - 50] + arglist.ret2mat[threadno][age - 50];
      }

    ret1mat[age - 50]  = 100 * ret1mat[age - 50] / nobs;
    ret2mat[age - 50]  = 100 * ret2mat[age - 50] / nobs;
    }

  everreturnft = 0;
  everreturnpt = 0;
  everreturn   = 0;

  for (threadno = 0; threadno < nthreads; threadno++)
  { everreturnft = everreturnft + arglist.everreturnft[threadno];
    everreturnpt = everreturnpt + arglist.everreturnpt[threadno];
    everreturn   = everreturn + arglist.everreturn[threadno];
    }

  everreturnft = 100 * everreturnft / nobs;
  everreturnpt = 100 * everreturnpt / nobs;
  everreturn   = 100 * everreturn / nobs;

  /* print tabulations */

  wnprintf("\n\n                Percent                   Percent");
  wnprintf(  "\n            Pseudo-Retiring               Retired");
  wnprintf(  "\n           From                       From");
  wnprintf(  "\nAge       FT Work  Completely        FT Work  Completely");
  wnprintf(  "\n");

  wnprintf("\n 50                                  %5.1f     %5.1f", ret1mat[50 - 50], ret2mat[50 - 50]);

  for (age = 51; age < 70; age++)
  wnprintf("\n %2d       %5.1f    %5.1f             %5.1f     %5.1f", age,
   ret1mat[age - 50] - ret1mat[(age - 1) - 50], ret2mat[age - 50] - ret2mat[(age - 1) - 50],
   ret1mat[age - 50], ret2mat[age - 50]);

  wnprintf("\n\nPercent returning to full time work after full or partial retirement: %5.1f", everreturnft);
  wnprintf("\nPercent returning to part time work after full retirement: %5.1f", everreturnpt);
  wnprintf("\nPercent returning to full time work after full or partial retirement");
  wnprintf("\n  or returning to part time work after full retirement: %5.1f", everreturn);

  return(q);

  }


/*---------------------------------------------------------------------------
			 QMIN
			 minimizes the q criterion for GMM estimates
  ---------------------------------------------------------------------------*/

void qmin(
  int    nobs)                     /* number of observations */

{ int    i, k, l, m;
  int    iter, sameparms;
  int    intindex;
  double incr, newincr;
  double v, v0, v1, v2;
  double q, q0, q1, q2;
  double dq0, dq1, dv0, dv1, dv02, dv12;
  double se, t;

  double lparm[nparms];

  double *cparm;

  static double var[nparms * (nparms + 1) / 2];
  static double mvec[nmoments];
  static double mvec0[nmoments];
  static double mvec1[nmoments];
  static double mvec2[nmoments];
  static double winv[nmoments * (nmoments + 1) / 2];
  static double dmat[nmoments * nparms];

  static double intervals[4][8] = {
    { 0.25,  0.05, 0.250, 0.025,  0.25,  0.25,  0.10,  0.05 },
    { 0.25,  0.05, 0.025, 0.025,  0.25,  0.25,  0.10,  0.05 },
    { 0.05,  0.02, 0.005, 0.005,  0.05,  0.05,  0.03,  0.02 },
    { 0.01,  0.01, 0.001, 0.001,  0.01,  0.01,  0.01,  0.01 }};


  arglist.mode = 2;

  cparm = (double *) &parm;

  for (intindex = 0; intindex < 4; intindex++)
  { sameparms = 0;

    for (iter = 0; iter < 10; iter++)
    if (sameparms == 0)
    { for (k = 0; k < nparms; k++)
      lparm[k] = cparm[k];

      q1 = f(mvec1, winv, nobs, 1);
      wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
        cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q1);

      for (k = 0; k < nparms; k++)
      { switch(k)
        { case  0: m = 2;      break;
          case  1: m = 3;      break;
          case  2: m = 7;      break;
          case  3: m = 0;      break;
          case  4: m = 4;      break;
          case  5: m = 1;      break;
          case  6: m = 5;      break;
          case  7: m = 6;      break;
          }

        v1 = cparm[m];

        incr = intervals[intindex][m];

        v0 = v1 - incr;
        cparm[m] = v0;

        q0 = f(mvec0, winv, nobs, 0);
        wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
          cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q0);

        newincr = incr;

        if (q0 < q1)
        { while (q0 < q1)
          { v2 = v1;
            q2 = q1;
            v1 = v0;
            q1 = q0;

            memcpy(mvec2, mvec1, sizeof(double[nmoments]));
            memcpy(mvec1, mvec0, sizeof(double[nmoments]));

            v0 = v1 - newincr;
            cparm[m] = v0;
            newincr = 2 * newincr;

            q0 = f(mvec0, winv, nobs, 0);
            wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
              cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q0);
            }
          }
        else
        { v2 = v1 + newincr;
          cparm[m] = v2;

          q2 = f(mvec2, winv, nobs, 0);
          wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
            cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q2);

          while (q2 <= q1)
          { v0 = v1;
            q0 = q1;
            v1 = v2;
            q1 = q2;

            memcpy(mvec0, mvec1, sizeof(double[nmoments]));
            memcpy(mvec1, mvec2, sizeof(double[nmoments]));

            v2 = v1 + newincr;
            cparm[m] = v2;
            newincr = 2 * newincr;

            q2 = f(mvec2, winv, nobs, 0);
            wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
              cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q2);
            }
          }

        while (v1 - v0 > 1.01 * incr || v2 - v1 > 1.01 * incr)
        { dv0 = v1 - v0;
          dv1 = v2 - v1;
          dq0 = q0 - q1;
          dq1 = q2 - q1;

          dv02 = v1 * v1 - v0 * v0;
          dv12 = v2 * v2 - v1 * v1;

          v = (dv12 * dq0 + dv02 * dq1) / (2 * (dv1 * dq0 + dv0 * dq1));

          if (v2 - v1 <= 1.01 * incr || (v < v1 && v1 - v0 > 1.01 * incr))
          { v = (v0 + v1) / 2;
            cparm[m] = v;

            q = f(mvec, winv, nobs, 0);
            wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
              cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q);

            if (q < q1)
            { v2 = v1;
              q2 = q1;
              v1 = v;
              q1 = q;

              memcpy(mvec2, mvec1, sizeof(double[nmoments]));
              memcpy(mvec1, mvec, sizeof(double[nmoments]));
              }
            else
            { v0 = v;
              q0 = q;

              memcpy(mvec0, mvec, sizeof(double[nmoments]));
              }
            }
          else
          { v = (v1 + v2) / 2;
            cparm[m] = v;

            q = f(mvec, winv, nobs, 0);
            wnprintf("\n     %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f  %9.3f",
              cparm[0], cparm[1], cparm[2], cparm[3], cparm[4], cparm[5], cparm[6], cparm[7], q);

            if (q < q1)
            { v0 = v1;
              q0 = q1;
              v1 = v;
              q1 = q;

              memcpy(mvec0, mvec1, sizeof(double[nmoments]));
              memcpy(mvec1, mvec, sizeof(double[nmoments]));
              }
            else
            { v2 = v;
              q2 = q;

              memcpy(mvec2, mvec, sizeof(double[nmoments]));
            }
          }
        }

        for (l = 0; l < nmoments; l++)
        dmat[nmoments * m + l] = (mvec2[l] - mvec0[l]) / (2 * incr);

        cparm[m] = v1;
        }

      sameparms = 1;

      for (k = 0; k < nparms; k++)
      if (cparm[k] != lparm[k])
      sameparms = 0;
      }
    }

  covar(var, winv, dmat);

  wnprintf("\n\n          coeff          s.e.           t");

  k = -1;
  for (i = 0; i < nparms; i++)
  { k = k + i + 1;
    se = sqrt(var[k]);
    t = cparm[i] / se;

    wnprintf("\n%3i%14.6f%14.6f%12.4f", i, cparm[i], se, t);
    }

  wnprintf("\n\nnobs: %8d", nobs);
  wnprintf("\nq: %11.3f", q1);

  }


/*---------------------------------------------------------------------------
			 COVAR routine
			 Calculates the variance-covariance matrix of the estimates
  ---------------------------------------------------------------------------*/

void covar(
  double *var,
  double *winv,
  double *dmat)

{ int    k, k1, k2, l, l1, l2;
  int    err;
  double eps, x;

  static double dtmp[nmoments * nparms];

  /* calculate variance matrix as G' winv G */

  for (k = 0; k < nparms; k++)         /* calculate G' times winv */
  for (l1 = 0; l1 < nmoments; l1++)
  { x = 0;

    l = (l1 * (l1 + 1)) / 2 - 1;

    for (l2 = 0; l2 < nmoments; l2++)
    { if (l2 <= l1)
      l = l + 1;
      else l = l + l2;

      x = x + winv[l] * dmat[nmoments * k + l2];
      }

    dtmp[nmoments * k + l1] = x;
    }


  /* calculate (G'winv) times G */

  k = -1;

  for (k1 = 0; k1 < nparms; k1++)
  for (k2 = 0; k2 <= k1; k2++)
  { k = k + 1;

    x = 0;

    for (l = 0; l < nmoments; l++)
    x = x + dtmp[nmoments * k1 + l] * dmat[nmoments * k2 + l];

    var[k] = x;
    }

  eps = 0.0000001;

  inv(var, nparms, eps, &err);

  if (err != 0)
  wnprintf("\nvar matrix not positive definite in covar");

  }


/*---------------------------------------------------------------------------
			 INV routine
			 Inverts a triangular matrix in place
  ---------------------------------------------------------------------------*/

void inv(
  double *a,
  int    n,
  double eps,
  int    *err)

{ int    i, j, k, l, m;
  int    hor, ind, ipiv, kpiv, ver;
  double tol;
  double invr, piv, sum, w;


  if (n <= 0)
  { *err = -1;
    return;
    }
  else *err = 0;

  /* factor a into t such that a = t't */

  kpiv = -1;
  for (k = 1; k <= n; k++)
  { kpiv = kpiv + k;
    ind = kpiv;

    tol = eps * a[kpiv];

    if (tol < 0)
    tol = - tol;

    for (i = k; i <= n; i++)
    { sum = 0;
      for (l = 1; l < k; l++)
      sum = sum + a[kpiv - l] * a[ind - l];

      sum = a[ind] - sum;

      if (i == k)
      { if (sum - tol <= 0)
        { if (sum <= 0)
          { *err = - (max(k, 2) - 1);
            return;
            }
          else if (*err == 0)
          *err = k - 1;
          }

        piv = sqrt(sum);
        a[kpiv] = piv;
        piv = 1 / piv;
        }
      else a[ind] = sum * piv;

      ind = ind + i;
      }
    }

  /* invert t */

  ipiv = n * (n + 1) / 2 - 1;
  ind = ipiv;

  for (i = 1; i <= n; i++)
  { invr = 1 / a[ipiv];
    a[ipiv] = invr;
    m = n;
    j = ind;

    for (k = 1; k < i; k++)
    { w = 0;
      m = m - 1;
      hor = ipiv;
      ver = j;

      for (l = n - i + 1; l <= m; l++)
      { ver = ver + 1;
        hor = hor + l;
        w = w + a[ver] * a[hor];
        }

      a[j] = - w * invr;
      j = j - m;
      }

    ipiv = ipiv - m;
    ind = ind - 1;
    }

  /* calculate a inverse from t inverse */

  for (i = 1; i <= n; i++)
  { ipiv = ipiv + i;
    j = ipiv;

    for (k = i; k <= n; k++)
    { w = 0;

      hor = j;
      for (l = k; l <= n; l++)
      { w = w + a[hor] * a[hor + k - i];
        hor = hor + l;
        }

      a[j] = w;
      j = j + k;
      }
    }

  }


/*---------------------------------------------------------------------------
			 F routine
			 Evaluates the q-statistic for the moments
  ---------------------------------------------------------------------------*/

double f(
  double *mvec,               /* moment vector           */
  double *winv,               /* inverst of w matrix     */
  int    nobs,                /* number of observations  */
  int    sw)                  /* switch to evaluate winv */

{ int    k, l, l1, l2;
  int    threadindex, threadno, threadcheck;
  int    obs, surv, err;
  int    age, age0, age1;
  int    ret, ret0, ret1;
  double q, eps;

  double *cparm;

  static char   *buffer;
  static double *rvec;
  static int    init = 1;
  static double alpha = 2;

  DATA       *data;
  MOBSSTRUCT *mobsStruct;

  FILE *tempfile;


  memset(arglist.mvec,         0, sizeof(double[nthreads][nmoments]));
  memset(arglist.wmat,         0, sizeof(double[nthreads][nmoments * (nmoments + 1) / 2]));
  memset(arglist.ret0mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.ret1mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.ret2mat,      0, sizeof(double[nthreads][21]));
  memset(arglist.returnft,     0, sizeof(double[nthreads][21]));
  memset(arglist.returnpt,     0, sizeof(double[nthreads][21]));
  memset(arglist.newpt,        0, sizeof(double[nthreads][21]));
  memset(arglist.everreturnft, 0, sizeof(double[nthreads]));
  memset(arglist.everreturnpt, 0, sizeof(double[nthreads]));
  memset(arglist.everreturn,   0, sizeof(double[nthreads]));

  cparm = (double *) &parm;
  wnprintf("\n"); for (k = 0; k < nparms; k++) wnprintf(" %6.3f", cparm[k]);

  if (init == 1)
  { init = 0;

    memset(arglist.nmat, 0, nobs * nmoments);

    /* get moment observations */

    for (obs = 0; obs < nobs; obs++)
    { data = &datav[obs];
      mobsStruct = (MOBSSTRUCT *) &(arglist.nmat[obs][0]);

      /* for (age = 54; age <= 66; age++)   * using retirement status at all ages *
      { ret = data[obs].retv[age - 50]; */

      for (surv = 0; surv < 6; surv++)
      { age = data->age[surv];
        ret = data->ret[surv];

        if (54 <= age && age <= 66 && 1 <= ret && ret <= 5)
        { mobsStruct->age[age - 54] = 1;

               if (age == 55)      k =  0;
          else if (age == 58)      k =  1;
          else if (age == 60)      k =  2;
          else if (age == 62)      k =  3;
          else if (age == 65)      k =  4;
          else                     k = -1;

          if (k >= 0)
          {                                  mobsStruct->pr[k]       = 1;
            if (data->inccat == 1)           mobsStruct->lowinc[k]   = 1;
            if (data->inccat == 3)           mobsStruct->highinc[k]  = 1;
            if (data->healthage <= age)      mobsStruct->health[k]   = 1;
            if (data->healthage <= age)      mobsStruct->healthpr[k] = 1;
            }
          }
        }

      for (surv = 0; surv < 5; surv++)
      { age0 = data->age[surv];
        ret0 = data->ret[surv];
        age1 = data->age[surv + 1];
        ret1 = data->ret[surv + 1];

        if (54 <= age0 && age0 <= 66 && 1 <= ret0 && ret0 <= 5
         && 54 <= age1 && age1 <= 66 && 1 <= ret1 && ret1 <= 5)
        mobsStruct->reversal[surv] = 1;
        }
      /* wnprintf("\n%4d", obs); for (l = 0; l < nmoments; l++) wnprintf(" %1d", arglist.nmat[obs][l]); */
      }
    }

  if (parm.sige <= 0.00001 || parm.erho <= -0.99999 || parm.erho >= 0.99999)
  { for (l = 0; l < nmoments; l++)
    mvec[l] = 0;

    return(9e20);
    }

  if (parm.alpha != alpha)
  { arglist.newalpha = 1;
    alpha = parm.alpha;
    }
  else arglist.newalpha = 0;

  /* call threads to fill mvec, dmat and smat matrices */

  eprintf("\n");

  for (obs = 0; obs < nobs; obs++)
  { threadno = -1;
    
    while (threadno < 0)
    { checkMessages();

      for (threadindex = 0; threadindex < nthreads; threadindex++)  
      if (threadno < 0)
      if (threadstate[threadindex] == 0)
      threadno = threadindex;
      }

    threadobs[threadno] = obs;

    if (arglist.mode != 4)
    { if (obs < nobs - 1)
      { eprintf("\r%5d  ", obs);
    
        for (threadindex = 0; threadindex < nthreads; threadindex++)
        if (threadindex <= 1 || (22 <= threadindex && threadindex <= 25) || threadindex >= 38)
        eprintf("%5d", threadobs[threadindex]);
        }
      else
      { eprintf("\r");
        for (threadindex = 0; threadindex < nthreads; threadindex++)
        eprintf("%5d", threadobs[threadindex]);
        }
      }
    else
    { wnprintf("\n%4d", threadobs[0]);
    
      for (threadindex = 1; threadindex < nthreads; threadindex++)
      wnprintf("%5d", threadobs[threadindex]);
      }

    threadstate[threadno] = 1;
    }

  threadcheck = 0;

  while(threadcheck == 0)
  { checkMessages();

    threadcheck = 1;
  
    for (threadno = 0; threadno < nthreads; threadno++)  
    if (threadstate[threadno] > 0)
    threadcheck = 0;
    }

  /* consolidate results */

  for (l = 0; l < nmoments; l++)
  { mvec[l] = 0;
  
    for (threadno = 0; threadno < nthreads; threadno++)
    mvec[l] = mvec[l] + arglist.mvec[threadno][l];
    }

  wnprintf("\n mvec  ");
  for (l = 0; l < nmoments; l++)
  { if (l > 0 && l % 10 == 0)
    wnprintf("\n       ");
    wnprintf("%7.3f", mvec[l]);
    }

  if (arglist.mode < 1)
  return(0);

  if (sw > 0)
  { for (k = 0; k < nmoments * (nmoments + 1) / 2; k++)
    { winv[k] = 0;
    
      for (threadno = 0; threadno < nthreads; threadno++)
      winv[k] = winv[k] + arglist.wmat[threadno][k];
      }

    /* calculate inverse of w */

    eps = 0.0000001;
    inv(winv, nmoments, eps, &err);

    if (err != 0)
    wnprintf("\nw matrix not positive definite in f; err %d", err);
    }

  /* calculate q statistic */

  q = 0;
  l = -1;

  for (l1 = 0; l1 < nmoments; l1++)
  for (l2 = 0; l2 <= l1; l2++)
  { l = l + 1;

    if (l1 == l2)
    q = q + mvec[l1] * winv[l] * mvec[l2];
    else q = q + 2 * mvec[l1] * winv[l] * mvec[l2];
    }

  tempfile = fopen("HealthModelEst.tmp", "at");

  fprintf(tempfile, "\n\n%9.4f   ", q);

  for (k = 0; k < nparms; k++)
  fprintf(tempfile, "%6.3f", cparm[k]);

  fprintf(tempfile, "\n mvec  ");
  for (l = 0; l < nmoments; l++)
  { if (l > 0 && l % 10 == 0)
    fprintf(tempfile, "\n       ");
    fprintf(tempfile, "%7.3f", mvec[l]);
    }

  fclose(tempfile);

  return(q);

  }


/*---------------------------------------------------------------------------
			 SIM
			 simulates the model with the estimated parameters
  ---------------------------------------------------------------------------*/

void sim(
  int    nobs)                   /* number of observations */

{ int    i, j;
  int    age, cat;
  int    threadindex, threadno, threadcheck;
  int    obs;
  double everreturnft, everreturnpt, everreturn;

  int    rhocats[21];
  double ret0mat[21], ret1mat[21], ret2mat[21];
  double returnft[21], returnpt[21], newpt[21];


  arglist.mode     = 3;
  arglist.newalpha = 1;

  memset(arglist.ret0mat, 0, sizeof(double[nthreads][21]));
  memset(arglist.ret1mat, 0, sizeof(double[nthreads][21]));
  memset(arglist.ret2mat, 0, sizeof(double[nthreads][21]));
  memset(arglist.returnft, 0, sizeof(double[nthreads][21]));
  memset(arglist.returnpt, 0, sizeof(double[nthreads][21]));
  memset(arglist.newpt, 0, sizeof(double[nthreads][21]));
  memset(arglist.everreturnft, 0, sizeof(double[nthreads]));
  memset(arglist.everreturnpt, 0, sizeof(double[nthreads]));
  memset(arglist.everreturn, 0, sizeof(double[nthreads]));

  /* call threads to fill mvec, dmat and smat matrices */

  eprintf("\n");

  for (obs = 0; obs < nobs; obs++)
  { threadno = -1;
    
    while (threadno < 0)
    { checkMessages();

      for (threadindex = 0; threadindex < nthreads; threadindex++)  
      if (threadno < 0)
      if (threadstate[threadindex] == 0)
      threadno = threadindex;
      }

    threadobs[threadno] = obs;
    threadstate[threadno] = 1;

    eprintf("\r%5d  ", obs);
    
    for (threadindex = 0; threadindex < nthreads; threadindex++)
    if (threadindex <= 1 || (22 <= threadindex && threadindex <= 25) || threadindex >= 38)
    eprintf("%5d", threadobs[threadindex]);
    }

  threadcheck = 0;

  while(threadcheck == 0)
  { checkMessages();

    threadcheck = 1;
  
    for (threadno = 0; threadno < nthreads; threadno++)  
    if (threadstate[threadno])
    threadcheck = 0;
    }


  /* consolidate results */

  for (age = 50; age < 70; age++)
  { ret0mat[age - 50]  = 0;
    ret1mat[age - 50]  = 0;
    ret2mat[age - 50]  = 0;
    returnft[age - 50] = 0;
    returnpt[age - 50] = 0;
    newpt[age - 50]    = 0;
  
    for (threadno = 0; threadno < nthreads; threadno++)
    { ret0mat[age - 50]  = ret0mat[age - 50] + arglist.ret0mat[threadno][age - 50];
      ret1mat[age - 50]  = ret1mat[age - 50] + arglist.ret1mat[threadno][age - 50];
      ret2mat[age - 50]  = ret2mat[age - 50] + arglist.ret2mat[threadno][age - 50];
      returnft[age - 50] = returnft[age - 50] + arglist.returnft[threadno][age - 50];
      returnpt[age - 50] = returnpt[age - 50] + arglist.returnpt[threadno][age - 50];
      newpt[age - 50]    = newpt[age - 50] + arglist.newpt[threadno][age - 50];
      }
    
    ret0mat[age - 50]  = 100 * ret0mat[age - 50] / nobs;
    ret1mat[age - 50]  = 100 * ret1mat[age - 50] / nobs;
    ret2mat[age - 50]  = 100 * ret2mat[age - 50] / nobs;
    returnft[age - 50] = 100 * returnft[age - 50] / nobs;
    returnpt[age - 50] = 100 * returnpt[age - 50] / nobs;
    newpt[age - 50]    = 100 * newpt[age - 50] / nobs;
    }

  everreturnft = 0;
  everreturnpt = 0;
  everreturn   = 0;

  for (threadno = 0; threadno < nthreads; threadno++)
  { everreturnft = everreturnft + arglist.everreturnft[threadno];
    everreturnpt = everreturnpt + arglist.everreturnpt[threadno];
    everreturn   = everreturn + arglist.everreturn[threadno];
    }

  everreturnft = 100 * everreturnft / nobs;
  everreturnpt = 100 * everreturnpt / nobs;
  everreturn   = 100 * everreturn / nobs;

  /* calculate and print tabulations */

  wnprintf("\n\n                Percent                   Percent");
  wnprintf(  "\n            Pseudo-Retiring               Retired");
  wnprintf(  "\n           From                       From");
  wnprintf(  "\nAge       FT Work  Completely        FT Work  Completely");
  wnprintf(  "\n");

  wnprintf("\n 50                                  %5.1f     %5.1f",
   ret1mat[50 - 50], ret2mat[50 - 50]);

  for (age = 51; age < 70; age++)
  wnprintf("\n %2d       %5.1f    %5.1f             %5.1f     %5.1f", age,
   ret1mat[age - 50] - ret1mat[(age - 1) - 50], ret2mat[age - 50] - ret2mat[(age - 1) - 50],
   ret1mat[age - 50], ret2mat[age - 50]);


  wnprintf("\n\n\n                       Percent    Percent                          Percent Newly");
  wnprintf(    "\n                         in        Newly                   Percent   Returned");
  wnprintf(    "\n        Percent        FT Work    Returned       Percent    Newly   to PT Work");
  wnprintf(    "\nAge     in Main         after       to             in        in     Previously");
  wnprintf(    "\n          Job          Retiring   FT Work        PT Work   PT Work   Retired");
  wnprintf(    "\n");

  wnprintf("\n 50      %5.1f                                   %5.1f",
   100 - ret0mat[50 - 50], ret1mat[50 - 50] - ret2mat[50 - 50]);

  for (age = 51; age < 70; age++)
  wnprintf("\n %2d      %5.1f          %5.1f     %5.1f          %5.1f     %5.1f     %5.1f", age,
   100 - ret0mat[age - 50], ret0mat[age - 50] - ret1mat[age - 50], returnft[age - 50],
   ret1mat[age - 50] - ret2mat[age - 50], newpt[age - 50], returnpt[age - 50]);

  wnprintf("\n\nPercent returning to full time work after full or partial retirement: %5.1f", everreturnft);
  wnprintf("\nPercent returning to part time work after full retirement: %5.1f", everreturnpt);
  wnprintf("\nPercent returning to full time work after full or partial retirement");
  wnprintf("\n  or returning to part time work after full retirement: %5.1f", everreturn);

  /* distribution of time preferences */

  wnprintf("\n\nDistribution of time preference rates\n");

  memset(rhocats, 0, sizeof(int[21]));

  for (obs = 0; obs < nobs; obs++)
  { if (arglist.rvec[obs] < 0)
    rhocats[0] = rhocats[0] + 1;
    else if (arglist.rvec[obs] > 1)
    rhocats[20] = rhocats[20] + 1;
    else
    { cat = (int) (20 * arglist.rvec[obs]);
      rhocats[cat] = rhocats[cat] + 1;
      }
    }

  for (i = 0; i < 5; i++)
  { wnprintf("\n");

    for (j = 0; j < 5; j++)
    if (j < 4 || i == 0)
    wnprintf("   %5.2f %5d", (0.05 * (5 * j + i)), rhocats[5 * j + i]);
    }

  }


/*---------------------------------------------------------------------------
			 THREAD PROCESS routine
			 starts the thread on satellite computers
  ---------------------------------------------------------------------------*/

void threadprocess(
  void *voidthreadno)                /* threadno */

{ int    obs, threadno, count, err;

  char   filename[40], sthreadno[4];
  
  FILE  *passfile;
  int passhandle;

  typedef struct
  { int    obs;
    int    mode;
    int    newalpha;

    PARM   parm;
    DATA   data;

    char   nmat[nmoments];
    double mvec[nmoments];
    double wmat[nmoments * (nmoments + 1) / 2];
    double rho;
    double ret0mat[21];
    double ret1mat[21];
    double ret2mat[21];
    double returnft[21];
    double returnpt[21];
    double newpt[21];
    double everreturnft;
    double everreturnpt;
    double everreturn;
    } ARGBUFFER;

  ARGBUFFER argbuffer;


  threadno = * (int *) voidthreadno;

  if (localthreadmin == 0)          /* program on main computer */
  { while(threadstate[threadno] == 0 || threadstate[threadno] == 1)
    { while(threadstate[threadno] == 0)
      { checkMessages();
        Sleep(1000);
        }
    
      if (threadstate[threadno] == 1)
      { obs = threadobs[threadno];

        argbuffer.obs      = obs;
        argbuffer.mode     = arglist.mode;
        argbuffer.newalpha = arglist.newalpha;
        argbuffer.rho      = arglist.rvec[obs];

        memcpy(&argbuffer.parm,         &parm,                           sizeof(PARM));
        memcpy(&argbuffer.data,         &datav[obs],                     sizeof(DATA));
        memcpy(argbuffer.nmat,          arglist.nmat[obs],               sizeof(char[nmoments]));
        memcpy(argbuffer.mvec,          arglist.mvec[threadno],          sizeof(double[nmoments]));
        memcpy(argbuffer.wmat,          arglist.wmat[threadno],          sizeof(double[nmoments * (nmoments + 1) / 2]));
        memcpy(argbuffer.ret0mat,       arglist.ret0mat[threadno],       sizeof(double[21]));
        memcpy(argbuffer.ret1mat,       arglist.ret1mat[threadno],       sizeof(double[21]));
        memcpy(argbuffer.ret2mat,       arglist.ret2mat[threadno],       sizeof(double[21]));
        memcpy(argbuffer.returnft,      arglist.returnft[threadno],      sizeof(double[21]));
        memcpy(argbuffer.returnpt,      arglist.returnpt[threadno],      sizeof(double[21]));
        memcpy(argbuffer.newpt,         arglist.newpt[threadno],         sizeof(double[21]));

        argbuffer.everreturnft = arglist.everreturnft[threadno];
        argbuffer.everreturnpt = arglist.everreturnpt[threadno];
        argbuffer.everreturn   = arglist.everreturn[threadno];

        strcpy(filename, "k:\\thread");
        sprintf(sthreadno, "%d", threadno);
        strcat(filename, sthreadno);
        strcat(filename, ".dat");

        passfile = fopen(filename, "wb");
        passhandle = _fileno(passfile);
        
        count = _write(passhandle, &argbuffer, sizeof(ARGBUFFER));
        
        fclose(passfile);

        strcpy(filename, "k:\\thread");
        sprintf(sthreadno, "%d", threadno);
        strcat(filename, sthreadno);
        strcat(filename, ".sat");

        passfile = fopen(filename, "wb");
        fclose(passfile);

        while(_access(filename, 0) >= 0)
        Sleep(1000);

        strcpy(filename, "k:\\thread");
        sprintf(sthreadno, "%d", threadno);
        strcat(filename, sthreadno);
        strcat(filename, ".dat");

        passfile = fopen(filename, "rb");
        passhandle = _fileno(passfile);
        
        _read(passhandle, &argbuffer, sizeof(ARGBUFFER));
        
        fclose(passfile);
        
        arglist.rvec[obs] = argbuffer.rho;

        memcpy(arglist.mvec[threadno],          argbuffer.mvec,          sizeof(double[nmoments]));
        memcpy(arglist.wmat[threadno],          argbuffer.wmat,          sizeof(double[nmoments * (nmoments + 1) / 2]));
        memcpy(arglist.ret0mat[threadno],       argbuffer.ret0mat,       sizeof(double[21]));
        memcpy(arglist.ret1mat[threadno],       argbuffer.ret1mat,       sizeof(double[21]));
        memcpy(arglist.ret2mat[threadno],       argbuffer.ret2mat,       sizeof(double[21]));
        memcpy(arglist.returnft[threadno],      argbuffer.returnft,      sizeof(double[21]));
        memcpy(arglist.returnpt[threadno],      argbuffer.returnpt,      sizeof(double[21]));
        memcpy(arglist.newpt[threadno],         argbuffer.newpt,         sizeof(double[21]));

        arglist.everreturnft[threadno] = argbuffer.everreturnft;
        arglist.everreturnpt[threadno] = argbuffer.everreturnpt;
        arglist.everreturn[threadno]   = argbuffer.everreturn;
        
        threadstate[threadno] = 0;
        }
      }
    }
  else                               /* program on satellite computer */
  { count = 0;
  
    while (count < 1000000000)
    { strcpy(filename, "\\\\maincomputer\\k\\thread");
      sprintf(sthreadno, "%d", threadno);
      strcat(filename, sthreadno);
      strcat(filename, ".sat");

      while (_access(filename, 0) < 0)
      Sleep(1000 + 3 * threadno);

      strcpy(filename, "\\\\maincomputer\\k\\thread");
      sprintf(sthreadno, "%d", threadno);
      strcat(filename, sthreadno);
      strcat(filename, ".dat");

      passfile = fopen(filename, "rb");
      passhandle = _fileno(passfile);
       
      _read(passhandle, &argbuffer, sizeof(ARGBUFFER));

      fclose(passfile);

      threadstate[threadno] = 1;
      
      obs = argbuffer.obs;
      threadobs[threadno] = obs;

      if (obs == localthreadmin)
      { arglist.mode     = argbuffer.mode;
        arglist.newalpha = argbuffer.newalpha;
        memcpy(&parm, &argbuffer.parm, sizeof(PARM));
        }
      else if (localthreadmin < obs && obs <= localthreadmax)
      Sleep(5000);

      arglist.rvec[obs] = argbuffer.rho;

      memcpy(&datav[obs],                     &argbuffer.data,         sizeof(DATA));
      memcpy(arglist.nmat[obs],               argbuffer.nmat,          sizeof(char[nmoments]));
      memcpy(arglist.mvec[threadno],          argbuffer.mvec,          sizeof(double[nmoments]));
      memcpy(arglist.wmat[threadno],          argbuffer.wmat,          sizeof(double[nmoments * (nmoments + 1) / 2]));
      memcpy(arglist.ret0mat[threadno],       argbuffer.ret0mat,       sizeof(double[21]));
      memcpy(arglist.ret1mat[threadno],       argbuffer.ret1mat,       sizeof(double[21]));
      memcpy(arglist.ret2mat[threadno],       argbuffer.ret2mat,       sizeof(double[21]));
      memcpy(arglist.returnft[threadno],      argbuffer.returnft,      sizeof(double[21]));
      memcpy(arglist.returnpt[threadno],      argbuffer.returnpt,      sizeof(double[21]));
      memcpy(arglist.newpt[threadno],         argbuffer.newpt,         sizeof(double[21]));

      arglist.everreturnft[threadno] = argbuffer.everreturnft;
      arglist.everreturnpt[threadno] = argbuffer.everreturnpt;
      arglist.everreturn[threadno]   = argbuffer.everreturn;

      indivmodel(voidthreadno);

      argbuffer.rho      = arglist.rvec[obs];

      memcpy(argbuffer.mvec,          arglist.mvec[threadno],          sizeof(double[nmoments]));
      memcpy(argbuffer.wmat,          arglist.wmat[threadno],          sizeof(double[nmoments * (nmoments + 1) / 2]));
      memcpy(argbuffer.ret0mat,       arglist.ret0mat[threadno],       sizeof(double[21]));
      memcpy(argbuffer.ret1mat,       arglist.ret1mat[threadno],       sizeof(double[21]));
      memcpy(argbuffer.ret2mat,       arglist.ret2mat[threadno],       sizeof(double[21]));
      memcpy(argbuffer.returnft,      arglist.returnft[threadno],      sizeof(double[21]));
      memcpy(argbuffer.returnpt,      arglist.returnpt[threadno],      sizeof(double[21]));
      memcpy(argbuffer.newpt,         arglist.newpt[threadno],         sizeof(double[21]));

      argbuffer.everreturnft = arglist.everreturnft[threadno];
      argbuffer.everreturnpt = arglist.everreturnpt[threadno];
      argbuffer.everreturn   = arglist.everreturn[threadno];

      strcpy(filename, "\\\\maincomputer\\k\\thread");
      sprintf(sthreadno, "%d", threadno);
      strcat(filename, sthreadno);
      strcat(filename, ".dat");

      passfile = fopen(filename, "wb");
      passhandle = _fileno(passfile);
        
      err = _write(passhandle, &argbuffer, sizeof(ARGBUFFER));
        
      fclose(passfile);

      strcpy(filename, "\\\\maincomputer\\k\\thread");
      sprintf(sthreadno, "%d", threadno);
      strcat(filename, sthreadno);
      strcat(filename, ".sat");

      _unlink(filename);
 
      count = count + 1;
      }  
    }
    
  }


/*---------------------------------------------------------------------------
			 INDIV MODEL
          calculates the moments of individual observations
  ---------------------------------------------------------------------------*/

void indivmodel(
  void *voidthreadno)                /* threadno */

{ int    threadno, obs;
  int    firstage, lastage;
  int    mode, surv, newalpha, tcat;
  int    age, ret;

  double *rvec;

  DATA   *data;
  CALCSTRUCT *calcStruct;


  threadno = * (int *) voidthreadno;

  while(threadstate[threadno] == 0 || threadstate[threadno] == 1)
  { while(threadstate[threadno] == 0)
    checkMessages();

    if (threadstate[threadno] == 1)
    { obs      = threadobs[threadno];

      newalpha = arglist.newalpha;
      mode     = arglist.mode;

      rvec     = &(arglist.rvec[obs]);

      calcStruct = &calcStructv[threadno - localthreadmin];
      memset(calcStruct, 0, sizeof(CALCSTRUCT));

      calcStruct->mode = mode;
      calcStruct->obs = obs;

      calcStruct->sige   = parm.sige * parm.beta1;
      calcStruct->erho   = parm.erho;
      calcStruct->gamma0 = parm.gamma0;
      calcStruct->gamma1 = parm.gamma1;
      calcStruct->beta0  = parm.beta0 + 10 * (parm.alpha + 1);   /* to enhance stability */;
      calcStruct->beta1  = parm.beta1;
      calcStruct->beta2  = parm.beta2 * parm.beta1;
      calcStruct->alpha  = parm.alpha;
      
      data = &datav[obs];      

      if (mode == 0)
      { firstage = 50;
        lastage  = 69;
        }
      else if (mode == 1 || mode == 2)
      { firstage = 99;
        lastage = 0;

        for (surv = 0; surv < 6; surv++)
        { age = data->age[surv];
          ret = data->ret[surv];

          if (54 <= age && age <= 66 && 1 <= ret && ret <= 5)
          { if (age < firstage)
            firstage = age;

            if (age > lastage)
            lastage = age;
            }
          }

        if (firstage > lastage)
        { if (localthreadmin > 0)
          return;
          }
        else
        { firstage = firstage - 1;

          if (lastage < 60)
          lastage = lastage + 2;
          else lastage = lastage + 3;
          }
        }
      else
      { firstage = 50;
        lastage = 69;
        }

      if (firstage <= lastage)
      { calcStruct->firstage = firstage;
        calcStruct->lastage = lastage;

        randreset(threadno, 20 * obs);

        calcPrep(threadno);

        if (newalpha == 1)
        *rvec = rhocalc(threadno);

        for (tcat = 0; tcat < ntcatlim; tcat++)
        { calcModel(threadno, tcat);
          getMoments(threadno, tcat);
          }
        }

      if (localthreadmin == 0)
      threadstate[threadno] = 0;
      else threadstate[threadno] = 2;
      }
    }

  }


/*---------------------------------------------------------------------------
			 CALC PREP routine
			 calculates utility, savings, and job choices for the model
  ---------------------------------------------------------------------------*/

void calcPrep(
  int threadno)                 /* threadno */

{ int    obs;
  int    age, spage, firstage, lastage;
  int    acat, ecat, lcat, pcat, rcat, tcat, vcat;
  int    birthyear, spbirthyear;
  int    somepens, somezero;
  int    earlypenage, delayedpenage, contribstartage;
  double beta0, beta1, beta2, erho, gamma0, gamma1, sige;
  double earnings, totearnings;
  double condmean, stddev;
  double minpen, maxpen, adjpen, sppensben;
  double inflrate, penadjrate;
  double assets, ramean, rgmean, rstddev, bX;
  double contribrate, accum, maxaccum, adjfactor;
  double percentile, exponent, delta;

  int    penstart[21];
  double rnorm[nrcatlim];
  double pensben[21];
  double surv[120], spsurv[120];

  double *acats, *ecats, *lcats, *pcats;
  double *mainwage, *secwage, *prwage;
  double *plumpsum, *eowamount, *dccontrib;
  double *initialpcat, *rprob, *realrate;
  double (*tcats)[20];
  double (*othincome)[3];
  double (*rtransa)[nrcatlim];
  double (*rtransl)[nlcatlim][nrcatlim];
  double (*srate)[75], (*transrate)[75], (*etrans)[necatlim];
  double (*utilr)[20];

  static double realratev[75] = {             9.14,  22.65,  24.40,  -2.09,  -5.11,
    -11.90,   7.40,  26.62,  -2.23,  21.15,  16.17, -20.46,  18.58,   0.55,  -5.58,
    -15.45,   1.15,  10.04,   8.04,  16.60, -22.04,  -6.05,   0.56,  11.99,  10.61,
      6.64,   9.11,  -0.12,  27.52,  16.07,   1.29,  -6.65,  20.35,   5.72,   0.59,
     13.79,  -4.03,  11.22,   8.82,   6.13,  -6.05,  10.45,   3.17,  -7.59,   0.03,
      6.39,   8.08, -13.03, -21.60,  14.64,  10.18,  -8.06,  -2.54,   0.58,   8.75,
     -4.62,  13.45,  11.49,   4.36,  17.22,  12.01,   0.57,   7.31,  15.72,  -3.86,
     15.63,   2.89,   4.39,  -0.59,  20.15,  10.54,  18.09,  15.48,   9.57,  -4.31 };
    /* 1926-2000 return data from file ibbotson.xls */
    /* ibbotson valuation edition yearbook 2001     */

  DATA       *data;
  CALCSTRUCT *calcStruct;


  calcStruct = &calcStructv[threadno - localthreadmin];
  obs = calcStruct->obs;
  data = &datav[obs];

  firstage = calcStruct->firstage;
  lastage  = calcStruct->lastage;

  tcats = calcStruct->tcats;
  acats = calcStruct->acats;
  ecats = calcStruct->ecats;
  lcats = calcStruct->lcats;
  pcats = calcStruct->pcats;

  mainwage = calcStruct->mainwage;
  secwage  = calcStruct->secwage;
  prwage   = calcStruct->prwage;

  initialpcat = calcStruct->initialpcat;

  plumpsum = data->plumpsum;
  dccontrib = calcStruct->dccontrib;
  eowamount = calcStruct->eowamount;

  srate = calcStruct->srate;
  transrate = calcStruct->transrate;
  etrans = calcStruct->etrans;
  rprob = calcStruct->rprob;
  rtransa = calcStruct->rtransa;
  rtransl = calcStruct->rtransl;
  realrate = calcStruct->realrate;

  othincome = calcStruct->othincome;

  utilr = calcStruct->utilr;

  calcStruct->inflrate = 0.040;    /* inflation rate */
  calcStruct->penadjrate = 0.38;   /* Fraction of inflation that is offset by pension increases */

  calcStruct->realrateg  = 0.0474;  /* geometric mean of real returns */
  calcStruct->realratea  = 0.0533;  /* arithmatic mean of real returns */
  calcStruct->realratesd = 0.1105;  /* standard deviation of real returns */
  /* return data from file ibbotson.xls, ibbotson valuation edition yearbook 2001 */

  inflrate   = calcStruct->inflrate;
  penadjrate = calcStruct->penadjrate;

  ramean = calcStruct->realratea;
  rgmean = calcStruct->realrateg;
  rstddev = calcStruct->realratesd;

  memcpy(realrate, realratev, sizeof(double[75]));

  calcStruct->ntcats = ntcatlim;
  calcStruct->nacats = nacatlim;
  calcStruct->necats = necatlim;
  calcStruct->nlcats = nlcatlim;
  calcStruct->npcats = npcatlim;
  calcStruct->nrcats = nrcatlim;
  calcStruct->nscats = nscatlim;


  /* get survival rates and transition rates */

  birthyear = data->birthyear;
  spbirthyear = data->spbirthyear;

  calcStruct->birthyear = birthyear;
  calcStruct->spbirthyear = spbirthyear;

  getSurvivalRates(birthyear, data->gender, surv);
  getSurvivalRates(spbirthyear, 3 - data->gender, spsurv);

  for (age = 0; age < 62; age++)  /* Treat survival as certain until age 62 */
  { surv[age] = surv[62];
    spsurv[age] = spsurv[62];
    }

  for (age = 25; age < 100; age++)
  { spage = (age + data->birthyear) - data->spbirthyear;

    if (surv[age] > 0 && surv[age + 1] > 0)
    srate[0][age - 25] = surv[age + 1] / surv[age];
    else srate[0][age - 25] = 0.001;

    if (spage < 1)
    srate[1][age - 25] = 1;
    else if (spage < 119)
    { if (spsurv[spage] > 0 && spsurv[spage + 1] > 0)
      srate[1][age - 25] = spsurv[spage + 1] / spsurv[spage];
      else srate[1][age - 25] = 0.001;
      }
    else srate[1][age - 25] = 0.001;

    transrate[0][age - 25] = srate[0][age - 25] * srate[1][age - 25];         /* remain married */
    transrate[1][age - 25] = srate[0][age - 25] * (1 - srate[1][age - 25]);   /* respondent remains */
    transrate[2][age - 25] = (1 - srate[0][age - 25]) * srate[1][age - 25];   /* spouse remains */
    }


  /* copy earnings amounts to calcStruct */

  for (age = 25; age < 70; age++)
  mainwage[age - 25] = data->ftwage[age - 25];

  for (age = 50; age < 70; age++)
  secwage[age - 50] = data->secwage[age - 50];

  for (age = 50; age < 70; age++)
  prwage[age - 50] = data->prwage[age - 50];


  /* set up categories for assets */

  earnings = 0;

  for (age = 25; age <= min(69, 1992 - data->birthyear); age++)
  { earnings = earnings * (1 + rgmean);

    earnings = earnings + mainwage[age - 25] + data->othincome[age - 25];

    if (data->spbirthyear > 0)
    { spage = age + (data->birthyear - data->spbirthyear);

      if (25 <= spage && spage < 70)
      earnings = earnings + data->spwage[spage - 25];

      if (data->sppenstart == spage)
      earnings = earnings + data->spplumpsum;
      }
    }

  totearnings = earnings;

  earnings = data->plumpsum[70 - 50];

  for (age = 70; age > max(25, 1992 - data->birthyear); age--)
  { earnings = earnings / (1 + rgmean);

    earnings = earnings + data->ftwage[age - 25] + data->othincome[age - 25];

    if (data->spbirthyear > 0)
    { spage = age + (data->birthyear - data->spbirthyear);

      if (25 <= spage && spage < 70)
      earnings = earnings + data->spwage[spage - 25];

      if (data->sppenstart == spage)
      earnings = earnings + data->spplumpsum;
      }
    }

  acats[calcStruct->nacats - 1] = max(totearnings, earnings);

  for (acat = calcStruct->nacats - 2; acat >= 4; acat--)
  acats[acat] = acats[acat + 1] / 1.25;

  for (acat = 3; acat >= 0; acat--)
  acats[acat] = acat * acats[4] / 4;


  /* set up categories for epsilon */

  ecats[0] = -3;
  ecats[calcStruct->necats - 1] = 3;

  for (ecat = 1; ecat < calcStruct->necats - 1; ecat++)
  ecats[ecat] = ecats[0] + ecat * (ecats[calcStruct->necats - 1] - ecats[0])
                 / (calcStruct->necats - 1);


  /* calculate transition probabilities for epsilon categories */

  erho = calcStruct->erho;

  stddev = sqrt(1 - erho * erho);

  for (ecat = 0; ecat < necatlim; ecat++)
  { condmean = erho * ecats[ecat];
	 etrans[ecat][0] = 0;

	 for (tcat = 1; tcat < necatlim; tcat++)
	 { etrans[ecat][tcat]
			= erf((((ecats[tcat] + ecats[tcat - 1]) / 2) - condmean) / stddev);
		etrans[ecat][tcat - 1] = etrans[ecat][tcat] - etrans[ecat][tcat - 1];
		}

	 etrans[ecat][necatlim - 1] = 1 - etrans[ecat][necatlim - 1];
	 }


  /* set up normalized values for rates of return */

  rnorm[0] = -2;
  rnorm[calcStruct->nrcats - 1] =  2;

  for (rcat = 1; rcat < calcStruct->nrcats - 1; rcat++)
  rnorm[rcat] = rnorm[0] + rcat * (rnorm[calcStruct->nrcats - 1] - rnorm[0])
                 / (calcStruct->nrcats - 1);


  /* calculate probabilities for rates of return */

  rprob[0] = 0;

  for (rcat = 1; rcat < nrcatlim; rcat++)
  { rprob[rcat] = erf((rnorm[rcat] + rnorm[rcat - 1]) / 2);
	 rprob[rcat - 1] = rprob[rcat] - rprob[rcat - 1];
	 }

  rprob[nrcatlim - 1] = 1 - rprob[nrcatlim - 1];


  /* calculate asset transitions for rates of return */

  for (acat = 0; acat < nacatlim; acat++)
  for (rcat = 0; rcat < nrcatlim; rcat++)
  { if (acat == 0)
    rtransa[acat][rcat] = 0;
    else
    { assets = (1 + ramean + rnorm[rcat] * rstddev) * acats[acat];

      tcat = 0;

      while(tcat < calcStruct->nacats - 1 && assets > acats[tcat])
      tcat = tcat + 1;

      if (tcat > 0)
      tcat = tcat - 1;

      rtransa[acat][rcat] = tcat + (assets - acats[tcat]) / (acats[tcat + 1] - acats[tcat]);
      }
    }


  /* find out if eligible for pensions */

  somepens = 0;
  somezero = 0;

  for (age = firstage; age <= lastage + 1; age++)
  { if (data->pensben[age - 50] > 0)
    somepens = 1;
    else somezero = 1;
    }

  /* adjust pensions if necessary */

  if (somepens > 0)
  { earlypenage = 0;   /* find first age pension can be immediately collected */

    for (age = firstage; age <= lastage + 1; age++)
    { pensben[age - 50] = data->pensben[age - 50];
      penstart[age - 50] = data->penstart[age - 50];

      if (earlypenage == 0 && age == penstart[age - 50])
      earlypenage = age;
      }

    if (earlypenage == 0)
    earlypenage = lastage + 2;

    if (earlypenage > firstage)
    { delayedpenage = 99;    /* find earliest age deferred vested benefits can be collected */

      for (age = firstage; age < earlypenage; age++)
      { if (penstart[age - 50] < delayedpenage)
        delayedpenage = penstart[age - 50];
        }

      if (delayedpenage < earlypenage || delayedpenage == 99)
      delayedpenage = earlypenage;
      }
    else delayedpenage = earlypenage;

    for (age = firstage; age < earlypenage; age++)     /* adjust deferred vested benefits to delayedpenage */
    { if (penstart[age - 50] != delayedpenage)
      pensben[age - 50] = pensben[age - 50] * adjben(penstart[age - 50], delayedpenage, calcStruct);
      }

    for (age = earlypenage; age <= lastage + 1; age++)    /* adjust if not currently collectable */
    { if (penstart[age - 50] != age)
      pensben[age - 50] = pensben[age - 50] * adjben(penstart[age - 50], age, calcStruct);
      }
    }
  else
  { earlypenage = 99;
    delayedpenage = 99;
    }

  calcStruct->earlypenage = earlypenage;
  calcStruct->delayedpenage = delayedpenage;


  /* set up categories for pensions */

  /* The pensions associated with the pcats categories are assumed to decline */
  /* by the rate inflrate * (1 - penadj) every year to reflect that pensions  */
  /* are incompletely indexed.  The 0 and 3 categories are the lowest and       */
  /* highest pensions available.  The amounts of pcats[0] and pcats[3] are    */
  /* as of age 50, as if the individual could collect them at that age.       */

  if (somepens == 0)
  { calcStruct->npcats = 1;
    pcats[0] = 0;

    for (age = firstage; age <= lastage + 1; age++)
    initialpcat[age - 50] = 0;
    }
  else
  { maxpen = 0;
    minpen = 0;

    for (age = firstage; age <= lastage + 1; age++)
    if (pensben[age - 50] > 0)
    { adjpen = pensben[age - 50] * exp(inflrate * (1 - penadjrate) * (penstart[age - 50] - 50));

      if (minpen == 0 || adjpen < minpen)
      minpen = adjpen;

      if (adjpen > maxpen)
      maxpen = adjpen;
      }

    if (somezero == 1)
    { pcats[0] = 0;
      pcats[1] = 0.99 * minpen;
      pcats[calcStruct->npcats - 1] = 1.01 * maxpen;

      for (pcat = 2; pcat < calcStruct->npcats - 1; pcat++)
      pcats[pcat] = pcats[1] * pow((pcats[calcStruct->npcats - 1] / pcats[1]),
                                    ((double) (pcat - 1) / (double) (calcStruct->npcats - 2)));
      }
    else
    { pcats[0] = 0.99 * minpen;
      pcats[calcStruct->npcats - 1] = 1.01 * maxpen;

      for (pcat = 1; pcat < calcStruct->npcats - 1; pcat++)
      pcats[pcat] = pcats[0] * pow((pcats[calcStruct->npcats - 1] / pcats[0]),
                                    ((double) pcat / (double) (calcStruct->npcats - 1)));
      }

    for (age = firstage; age <= lastage + 1; age++)
    { if (pensben[age - 50] > 0)
      { adjpen = pensben[age - 50] * exp(inflrate * (1 - penadjrate) * (penstart[age - 50] - 50));

        pcat = 0;

        while(pcat < calcStruct->npcats - 1 && adjpen > pcats[pcat])
        pcat = pcat + 1;

        if (pcat > 0)
        pcat = pcat - 1;

        initialpcat[age - 50] = pcat + (adjpen - pcats[pcat]) / (pcats[pcat + 1] - pcats[pcat]);

        if (initialpcat[age - 50] > calcStruct->npcats - 1)
        initialpcat[age - 50] = calcStruct->npcats - 1;
        }
      else initialpcat[age - 50] = 0;
      }
    }


  /* calculate vector of dc contributions */

  memset(dccontrib, 0, sizeof(double[45]));

  contribstartage = 99;

  for (age = 50; age < 70; age++)
  { dccontrib[age - 25] = plumpsum[(age + 1) - 50] / 1.022 - plumpsum[age - 50];

    if (dccontrib[age - 25] < 0)
    dccontrib[age - 25] = 0;

    if (dccontrib[age - 25] > 0 && age < contribstartage)
    contribstartage = age;
    }

  if (plumpsum[50 - 50] > 0)
  { contribrate = 0;

    for (age = 50; age < 70; age++)
    if (contribrate == 0 && dccontrib[age - 25] > 0 && mainwage[age - 25] > 0)
    contribrate = dccontrib[age - 25] / mainwage[age - 25];

    if (contribrate > 0)
    { accum = plumpsum[50 - 50] / 1.022;
      adjfactor = 0;

      for (age = 49; age >= 25; age--)
      if (accum > 0)
      { if (mainwage[age - 25] > 0)
        { if (accum > contribrate * mainwage[age - 25])
          { dccontrib[age - 25] = contribrate * mainwage[age - 25];
            accum = (accum - dccontrib[age - 25]) / 1.022;
            }
          else
          { dccontrib[age - 25] = accum;
            accum = 0;
            }

          contribstartage = age;
          }
        else
        { adjfactor = 1 + (accum * pow(1.022, 50 - age)) / plumpsum[50 - 50];
          accum = 0;
          }
        }

      if (adjfactor > 0)
      { for (age = 25; age < 50; age++)
        { dccontrib[age - 25] = dccontrib[age - 25] * adjfactor;

          if (dccontrib[age - 25] > 0.25 * mainwage[age - 25])
          dccontrib[age - 25] = 0.25 * mainwage[age - 25];
          }
        }
      }
    }

  calcStruct->contribstartage = contribstartage;
  calcStruct->penjobend = data->penjobend;


  /* set up categories for dc accumulations and transition matrices */

  if (contribstartage < 99)
  { maxaccum = 0;

    for (age = 25; age < lastage; age++)
    { if (dccontrib[age - 25] > 0)
      maxaccum = maxaccum + dccontrib[age - 25] * pow(1 + ramean + 2.5 * rstddev,
        sqrt((double) (lastage - age)));
      }

    lcats[calcStruct->nlcats - 1] = maxaccum;

    for (lcat = calcStruct->nlcats - 2; lcat >= 4; lcat--)
    lcats[lcat] = lcats[lcat + 1] / 1.4;

    for (lcat = 3; lcat >= 0; lcat--)
    lcats[lcat] = lcat * lcats[4] / 4;

    /* set up transitions among dc accumulation categories */

    for (age = contribstartage + 1; age <= lastage; age++)
    for (lcat = 0; lcat < calcStruct->nlcats; lcat++)
    for (rcat = 0; rcat < calcStruct->nrcats; rcat++)
    { accum = (lcats[lcat] + dccontrib[(age - 1) - 25]) * (1 + ramean + rnorm[rcat] * rstddev);

      tcat = 0;

      while(tcat < calcStruct->nlcats - 1 && accum > lcats[tcat])
      tcat = tcat + 1;

      if (tcat > 0)
      tcat = tcat - 1;

      rtransl[age - 25][lcat][rcat] = tcat + (accum - lcats[tcat]) / (lcats[tcat + 1] - lcats[tcat]);
      }
    }
  else
  { calcStruct->nlcats = 1;
    lcats[0] = 0;

    for (age = contribstartage + 1; age <= lastage; age++)
    for (lcat = 0; lcat <= calcStruct->nlcats; lcat++)
    for (rcat = 0; rcat <= calcStruct->nrcats; rcat++)
    rtransl[age - 25][lcat][rcat] = lcat;
    }


  /* early out window amounts */

  for (age = 50; age <= 70; age++)
  eowamount[age - 50] = data->eowamount[age - 50];


  /* set up categories and benefit amounts for social security */

  sscalc(threadno);


  /* calculate spouse earnings, pension, and other income */

  memset(othincome, 0, sizeof(double[75][3]));

  if (spbirthyear > 0)
  for (age = 25; age < 100; age++)
  { spage = age + (birthyear - spbirthyear);

    if (25 <= spage && spage < 70)
    { othincome[age - 25][0] = othincome[age - 25][0] + data->spwage[spage - 25];
      othincome[age - 25][2] = othincome[age - 25][2] + data->spwage[spage - 25];
      }

    if (data->sppenstart < 99)
    { if (spage == data->sppenstart)
      sppensben = data->sppensben;

      if (data->sppenstart <= spage && spage < 100)
      { othincome[age - 25][0] = othincome[age - 25][0] + sppensben;
        othincome[age - 25][2] = othincome[age - 25][2] + sppensben;
        }

      sppensben = sppensben / (1 + inflrate * (1 - penadjrate));

      if (spage == data->sppenstart)
      { othincome[age - 25][0] = othincome[age - 25][0] + data->spplumpsum;
        othincome[age - 25][2] = othincome[age - 25][2] + data->spplumpsum;
        }
      }
    }

  for (age = 25; age < 100; age++)
  for (vcat = 0; vcat < 3; vcat++)
  othincome[age - 25][vcat] = othincome[age - 25][vcat] + data->othincome[age - 25];


  /* calculate utility of retirement */

  beta0 = calcStruct->beta0;
  beta1 = calcStruct->beta1;
  beta2 = calcStruct->beta2;
  sige  = calcStruct->sige;

  for (age = 50; age < 70; age++)
  { bX = beta0 + beta1 * (age - 62);

    if (age >= data->healthage)
    bX = bX + beta2;

    for (ecat = 0; ecat < necatlim; ecat++)
    utilr[ecat][age - 50] = exp(bX + sige * ecats[ecat]);
    }

  /* calculate relative utility of partial retirement */

  gamma0 = calcStruct->gamma0;
  gamma1 = calcStruct->gamma1;

  for (tcat = 0; tcat < ntcatlim; tcat++)
  { switch(tcat)
    { case  0:  percentile =  5;           break;
      case  1:  percentile = 20;           break;
      case  2:  percentile = 40;           break;
      case  3:  percentile = 60;           break;
      case  4:  percentile = 80;           break;
      case  5:  percentile = 95;           break;
      }

    for (age = 50; age < 70; age++)
    { exponent = gamma0 + gamma1 * (age - 65);

      delta = log((percentile / 100) * (exp(exponent) - 1) + 1) / exponent;

      if (data->gender == 1 && data->healthage <= age)
      delta = 0;

      tcats[tcat][age - 50] = 0.5 + 0.5 * delta;
      }
    }

  }


/*---------------------------------------------------------------------------
			 ADJ BEN routine
			 actualrially adjusts pension benefits to a new starting age
  ---------------------------------------------------------------------------*/

double adjben(
  int    origstart,
  int    newstart,
  CALCSTRUCT *calcStruct)

{ int    age, start;
  double inflrate, realrate, penadjrate;
  double pv0, pv1;

  double (*srate)[75];

  inflrate = calcStruct->inflrate;
  realrate = calcStruct->realrateg;
  penadjrate = calcStruct->penadjrate;

  srate = calcStruct->srate;

  start = min(origstart, newstart);

  pv0 = 0;
  pv1 = 0;

  for (age = 99; age >= start; age--)
  { if (age >= origstart)
    pv0 = pv0 + 1;

    if (age >= newstart)
    pv1 = pv1 + 1;

    if (age >= start)
    { if (age > origstart)
      pv0 = pv0 * srate[0][age - 25] / (1 + realrate + inflrate * (1 - penadjrate));
      else if (age > start)
      pv0 = pv0 * srate[0][age - 25] / (1 + realrate);

      if (age > newstart)
      pv1 = pv1 * srate[0][age - 25] / (1 + realrate + inflrate * (1 - penadjrate));
      else if (age > start)
      pv1 = pv1 * srate[0][age - 25] / (1 + realrate);
      }
    }

  return(pv0 / pv1);

  }


/*---------------------------------------------------------------------------
			 SS CALC routine
			 Calculates social security categories and benefit amounts
  ---------------------------------------------------------------------------*/

void sscalc(
  int    threadno)            /* thread number */

{ int    obs;
  int    jcat, scat, scati, nscats;
  int    age, retage, spage;
  int    firstage, lastage;
  int    nrage, spnrage, sserage;
  int    year, birthyear, spbirthyear;
  int    earntest, spearntest;
  double alpha;
  double adjown, adjsp, spadjown, spadjsp, adjfactor;
  double delayedret, spdelayedret;
  double pia, sppia, ncpens, spncpens;
  double rednrate, sprednrate, earningstest, spearningstest;
  double ownben, spownben, benefitsp, spbenefitsp;
  double survben, spsurvben;
  double earnings, spearnings;
  double pctlost, sppctlost, pctlostsp, sppctlostsp, survpctlost, spsurvpctlost;
  double pctchange, pctchangesp, sppctchange, sppctchangesp;
  double ssben, benchange;

  double fbenefit[21];
  double adjownv[4], adjspv[4], spadjownv[4], spadjspv[4];
  double adjownm[21][4], adjspm[21][4], spadjownm[21][4], spadjspm[21][4];

  double *mainwage, *secwage, *prwage;
  double *initialscat, *ssbenf;
  double (*benchanges)[2][nscatlim], (*benchangep)[2][nscatlim];
  double (*ssbenm)[2], (*ssbens)[2][nscatlim], (*ssbenp)[2][nscatlim], (*ssbenr)[3][nscatlim];

  DATA       *data;
  CALCSTRUCT *calcStruct;


  calcStruct = &calcStructv[threadno - localthreadmin];
  obs = calcStruct->obs;
  data = &datav[obs];

  alpha = calcStruct->alpha;

  nscats = calcStruct->nscats;

  mainwage = calcStruct->mainwage;
  secwage  = calcStruct->secwage;
  prwage   = calcStruct->prwage;

  initialscat = calcStruct->initialscat;

  ssbenf = calcStruct->ssbenf;
  ssbenm = calcStruct->ssbenm;
  ssbens = calcStruct->ssbens;
  ssbenp = calcStruct->ssbenp;
  ssbenr = calcStruct->ssbenr;

  benchanges  = calcStruct->benchanges;
  benchangep  = calcStruct->benchangep;

  birthyear = data->birthyear;
  spbirthyear = data->spbirthyear;

  sserage = 62;

  if (birthyear <= 1939)
  nrage = 65;
  else if (birthyear <= 1956)
  nrage = 66;
  else nrage = 67;

  if (birthyear <= 1924)
  delayedret = 0.03;
  else if (birthyear >= 1943)
  delayedret = 0.08;
  else delayedret = 0.03 + 0.005 * ((birthyear - 1923) / 2);

  if (spbirthyear > 0)
  { if (spbirthyear <= 1939)
	  spnrage = 65;
	  else if (spbirthyear <= 1956)
	  spnrage = 66;
	  else spnrage = 67;

    if (spbirthyear <= 1924)
    spdelayedret = 0.03;
    else if (spbirthyear >= 1943)
    spdelayedret = 0.08;
    else spdelayedret = 0.03 + 0.005 * ((spbirthyear - 1923) / 2);
    }
  else
  { spnrage = 65;
    spdelayedret = 0.08;
    }

  calcStruct->sserage = sserage;
  calcStruct->nrage   = nrage;
  calcStruct->spnrage = spnrage;
  calcStruct->delayedret   = delayedret;
  calcStruct->spdelayedret = spdelayedret;

  /* calculate current benefits if working in main job */

  adjown = 1 - (0.2 / 3) * min(nrage - sserage, 3) - 0.05 * max(nrage - sserage - 3, 0);
  adjsp  = 1 - (0.25 / 3) * min(nrage - sserage, 3) - 0.05 * max(nrage - sserage - 3, 0);
  spadjown = 1 - (0.2 / 3) * min(spnrage - sserage, 3) - 0.05 * max(spnrage - sserage - 3, 0);
  spadjsp  = 1 - (0.25 / 3) * min(spnrage - sserage, 3) - 0.05 * max(spnrage - sserage - 3, 0);

  for (age = 25; age < 70; age++)
  { year = age + birthyear;
    spage = year - spbirthyear;

    if (age < 50)
    { pia    = data->pia[50 - 50];
      ncpens = data->ncpens[50 - 50];
      }
    else
    { pia    = data->pia[age - 50];
      ncpens = data->ncpens[age - 50];
      }

    if (50 <= age)
    fbenefit[age - 50] = adjown * pia;

    if (age >= sserage || spage >= sserage)
    { if (spage < 50)
      { sppia    = data->sppia[50 - 50];
        spncpens = data->spncpens[50 - 50];
        }
      else
      { if (spage < 70)
        { sppia = data->sppia[spage - 50];
          spncpens = data->spncpens[spage - 50];
          }
        else
        { sppia = data->sppia[70 - 50];
          spncpens = data->spncpens[70 - 50];
          }
        }

      if (age >= sserage)
      ownben = adjown * pia;
      else ownben = 0;

      if (spage >= sserage)
      spownben = spadjown * sppia;
      else spownben = 0;

      if (age >= sserage && spage >= sserage)
      { benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0);
        spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0);
        }
      else
      { benefitsp = 0;
        spbenefitsp = 0;
        }

      adjfactor = pow(1.01, year - 1992);

      if (age < nrage || (age < 70 && year < 2001))
      { earntest = 1;

        if (age < nrage)
        { earningstest = 7440 * adjfactor;
          rednrate = 0.5;
          }
        else
        { earningstest = 10200 * adjfactor;
          rednrate = 0.333333;
          }
        }
      else earntest = 0;

      if (spage < spnrage || (spage < 70 && year < 2001))
      { spearntest = 1;
         if (spage < spnrage)
        { spearningstest = 7440 * adjfactor;
          sprednrate = 0.5;
          }
        else
        { spearningstest = 10200 * adjfactor;
          sprednrate = 0.333333;
          }
        }
      else spearntest = 0;

      earnings = mainwage[age - 25];

      if (25 <= spage && spage < 70)
      spearnings = data->spwage[spage - 25];
      else spearnings = 0;

      /* both spouses survive (vcat = 0) */

      if (earntest == 1)
      { if (ownben + spbenefitsp > 0)
        { pctlost = rednrate * (earnings - earningstest) / (ownben + spbenefitsp);

          if (pctlost < 0)
          pctlost = 0;
          else if (pctlost > 1)
          pctlost = 1;
          }
        else pctlost = 1;
        }
      else pctlost = 0;

      if (spearntest == 1)
      { if (spownben + benefitsp > 0)
        { sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);

          if (sppctlost < 0)
          sppctlost = 0;
          else if (sppctlost > 1)
          sppctlost = 1;
          }
        else sppctlost = 1;
        }
      else sppctlost = 0;

      if (benefitsp > 0)                                  /* husband collecting spouse benefits */
      { if (sppctlost < pctlost)                          /* additional loss of spouse benefits */
        { sppctlost = sppctlost - (pctlost - sppctlost) * benefitsp / spownben;
          pctlostsp = pctlost;

          if (sppctlost < 0)
          sppctlost = 0;
          }
        else pctlostsp = sppctlost;
        }
      else pctlostsp = 1;

      if (spbenefitsp > 0)                                /* wife collecting spouse benefits */
      { if (pctlost < sppctlost)                          /* additional loss of spouse benefits */
        { pctlost = pctlost - (sppctlost - pctlost) * spbenefitsp / ownben;
          sppctlostsp = sppctlost;

          if (pctlost < 0)
          pctlost = 0;
          }
        else sppctlostsp = pctlost;
        }
      else sppctlostsp = 1;

      ssbenm[age - 25][0] = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp
                           + (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;

      /* husband only survives (vcat = 1) */

      if (age >= sserage)
      { survben = max(ownben, sppia * max(0.825, spadjown));

        if (survben > 0)
        { if (earntest == 1)
          { survpctlost = rednrate * (earnings - earningstest) / survben;

            if (survpctlost < 0)
            survpctlost = 0;
            else if (survpctlost > 1)
            survpctlost = 1;
            }
          else survpctlost = 0;

          ssben = (1 - survpctlost) * survben;
          }
        else ssben = 0;
        }
      else ssben = 0;

      ssbenm[age - 25][1] = ssben;

      /* update adjustment factors */

      if (sserage <= age && age < 70)
      { if (age < nrage - 3)
        { pctchange = 0.05 * pctlost;
          pctchangesp = 0.05 * pctlostsp;
          }
        else if (age < nrage)
        { pctchange = (0.2 / 3) * pctlost;
          pctchangesp = (0.25 / 3) * pctlostsp;
          }
        else
        { pctchange = delayedret * pctlost;
          pctchangesp = 0;
          }
        }
      else
      { pctchange = 0;
        pctchangesp = 0;
        }

      if (sserage <= spage && spage < 70)
      { if (spage < spnrage - 3)
        { sppctchange = 0.05 * sppctlost;
          sppctchangesp = 0.05 * sppctlostsp;
          }
        else if (spage < spnrage)
        { sppctchange = (0.2 / 3) * sppctlost;
          sppctchangesp = (0.25 / 3) * sppctlostsp;
          }
        else
        { sppctchange = spdelayedret * sppctlost;
          sppctchangesp = 0;
          }
        }
      else
      { sppctchange = 0;
        sppctchangesp = 0;
        }

      adjown   = adjown   + pctchange;
      adjsp    = adjsp    + pctchangesp;
      spadjown = spadjown + sppctchange;
      spadjsp  = spadjsp  + sppctchangesp;

      if (age + 1 == 70)
      fbenefit[70 - 50] = adjown * data->pia[70 - 50];
      }
    else
    { ssbenm[age - 25][0] = 0;
      ssbenm[age - 25][1] = 0;
      }

    /* collect adjustment factors at ages corresponding to scat categories */

    if (age + 1 == 50)
    scati = 0;
    else if (age + 1 == sserage)
    scati = 1;
    else if (age + 1 == nrage)
    scati = 2;
    else if (age + 1 == 70)
    scati = 3;
    else scati = -1;
        
    if (scati >= 0)
    { adjownv[scati]   = adjown;
      adjspv[scati]    = adjsp;
      spadjownv[scati] = spadjown;
      spadjspv[scati]  = spadjsp;
      }
    }

  /* calculate benefits if retired at age corresponding to scat categories */

  for (scat = 0; scat < nscats; scat++)
  { switch(scat)
    { case  0:   retage = 50;         break;
      case  1:   retage = sserage;    break;
      case  2:   retage = nrage;      break;
      case  3:   retage = 70;         break;
      }

    pia    = data->pia[retage - 50];
    ncpens = data->ncpens[retage - 50];

    for (age = 50; age <= 99; age++)
    { year = age + birthyear;
      spage = year - spbirthyear;

      if (age < retage)
      { adjown = adjownv[scat] - (adjownm[retage - 50][scat - 1] - adjownm[age - 50][scat - 1]);
        adjsp = adjspv[scat] - (adjspm[retage - 50][scat - 1] - adjspm[age - 50][scat - 1]);
        spadjown = spadjownv[scat] - (spadjownm[retage - 50][scat - 1] - spadjownm[age - 50][scat - 1]);
        spadjsp = spadjspv[scat] - (spadjspm[retage - 50][scat - 1] - spadjspm[age - 50][scat - 1]);
        }
      else if (age == retage)
      { adjown   = adjownv[scat];
        adjsp    = adjspv[scat];
        spadjown = spadjownv[scat];
        spadjsp  = spadjspv[scat];
        }

      if (age <= 70)
      { adjownm[age - 50][scat]   = adjown;
        adjspm[age - 50][scat]    = adjsp;
        spadjownm[age - 50][scat] = spadjown;
        spadjspm[age - 50][scat]  = spadjsp;
        }

      if (age >= sserage || spage >= sserage)
      { if (spage < 50)
        { sppia    = data->sppia[50 - 50];
          spncpens = data->spncpens[50 - 50];
          }
        else if (spage < 70)
        { sppia = data->sppia[spage - 50];
          spncpens = data->spncpens[spage - 50];
          }
        else
        { sppia = data->sppia[70 - 50];
          spncpens = data->spncpens[70 - 50];
          }

        if (age >= sserage)
        ownben = adjown * pia;
        else ownben = 0;

        if (spage >= sserage)
        spownben = spadjown * sppia;
        else spownben = 0;

        if (age >= sserage && spage >= sserage)
        { benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0);
          spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0);
          }
        else
        { benefitsp = 0;
          spbenefitsp = 0;
          }

        adjfactor = pow(1.01, year - 1992);

        if (spage < spnrage || (spage < 70 && year < 2001))
        { spearntest = 1;

          if (spage < spnrage)
          { spearningstest = 7440 * adjfactor;
            sprednrate = 0.5;
            }
          else
          { spearningstest = 10200 * adjfactor;
            sprednrate = 0.333333;
            }
          }
        else spearntest = 0;

        if (25 <= spage && spage < 70)
        spearnings = data->spwage[spage - 25];
        else spearnings = 0;

        /* both spouses survive (vcat = 0) */

        pctlost = 0;

        if (spearntest == 1)
        { if (spownben + benefitsp > 0)
          { sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);

            if (sppctlost < 0)
            sppctlost = 0;
            else if (sppctlost > 1)
            sppctlost = 1;
            }
          else sppctlost = 1;
          }
        else sppctlost = 0;

        if (benefitsp > 0)                                  /* husband collecting spouse benefits */
        pctlostsp = sppctlost;
        else pctlostsp = 1;

        if (spbenefitsp > 0)                                /* wife collecting spouse benefits */
        sppctlostsp = sppctlost;
        else sppctlostsp = 1;

        ssbenr[age - 50][0][scat] = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp
                                   + (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;

        /* husband only survives (vcat = 1) */

        if (age >= sserage)
        survben = max(ownben, sppia * max(0.825, spadjown));
        else survben = 0;

        ssbenr[age - 50][1][scat] = survben;
 
        /* wife only survives (vcat = 2) */

        if (spage >= sserage)
        { spsurvben = max(spownben, pia * max(0.825, adjown));

          if (spsurvben > 0)
          { if (spearntest == 1)
            { spsurvpctlost = sprednrate * (spearnings - spearningstest) / spsurvben;

              if (spsurvpctlost < 0)
              spsurvpctlost = 0;
              else if (spsurvpctlost > 1)
              spsurvpctlost = 1;
              }
            else spsurvpctlost = 0;

            ssben = (1 - spsurvpctlost) * spsurvben;
            }
          else ssben = 0;
          }
        else ssben = 0;

        ssbenr[age - 50][2][scat] = ssben;

        /* update adjustment factors */

        if (sserage <= age && age < 70)
        { if (age < nrage - 3)
          pctchangesp = 0.05 * pctlostsp;
          else if (age < nrage)
          pctchangesp = (0.25 / 3) * pctlostsp;
          else pctchangesp = 0;
          }
        else pctchangesp = 0;

        if (sserage <= spage && spage < 70)
        { if (spage < spnrage - 3)
          { sppctchange = 0.05 * sppctlost;
            sppctchangesp = 0.05 * sppctlostsp;
            }
          else if (spage < spnrage)
          { sppctchange = (0.2 / 3) * sppctlost;
            sppctchangesp = (0.25 / 3) * sppctlostsp;
            }
          else
          { sppctchange = spdelayedret * sppctlost;
            sppctchangesp = 0;
            }
          }
        else
        { sppctchange = 0;
          sppctchangesp = 0;
          }

        adjsp    = adjsp    + pctchangesp;
        spadjown = spadjown + sppctchange;
        spadjsp  = spadjsp  + sppctchangesp;
        }
      }
    }

  /* calculate benefits and adjustments if in secondary or partial retirement jobs */

  for (jcat = 1; jcat < 3; jcat++)
  for (scat = 0; scat < nscats; scat++)
  { switch(scat)
    { case  0:   retage = 50;         break;
      case  1:   retage = sserage;    break;
      case  2:   retage = nrage;      break;
      case  3:   retage = 70;         break;
      }

    pia    = data->pia[retage - 50];
    ncpens = data->ncpens[retage - 50];

    for (age = 50; age < 70; age++)
    { year = age + birthyear;
      spage = year - spbirthyear;

      adjown   = adjownm[age - 50][scat];
      adjsp    = adjspm[age - 50][scat];
      spadjown = spadjownm[age - 50][scat];
      spadjsp  = spadjspm[age - 50][scat];

      if (age >= sserage || spage >= sserage)
      { if (spage < 50)
        { sppia    = data->sppia[50 - 50];
          spncpens = data->spncpens[50 - 50];
          }
        else if (spage < 70)
        { sppia = data->sppia[spage - 50];
          spncpens = data->spncpens[spage - 50];
          }
        else
        { sppia = data->sppia[70 - 50];
          spncpens = data->spncpens[70 - 50];
          }

        if (age >= sserage)
        ownben = adjown * pia;
        else ownben = 0;

        if (spage >= sserage)
        spownben = spadjown * sppia;
        else spownben = 0;

        if (age >= sserage && spage >= sserage)
        { benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0);
          spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0);
          }
        else
        { benefitsp = 0;
          spbenefitsp = 0;
          }

        adjfactor = pow(1.01, year - 1992);

        if (age < nrage || (age < 70 && year < 2001))
        { earntest = 1;

          if (age < nrage)
          { earningstest = 7440 * adjfactor;
            rednrate = 0.5;
            }
          else
          { earningstest = 10200 * adjfactor;
            rednrate = 0.333333;
            }
          }
        else earntest = 0;

        if (spage < spnrage || (spage < 70 && year < 2001))
        { spearntest = 1;

          if (spage < spnrage)
          { spearningstest = 7440 * adjfactor;
            sprednrate = 0.5;
            }
          else
          { spearningstest = 10200 * adjfactor;
            sprednrate = 0.333333;
            }
          }
        else spearntest = 0;

        if (jcat == 1)
        earnings = secwage[age - 50];
        else earnings = prwage[age - 50];

        if (25 <= spage && spage < 70)
        spearnings = data->spwage[spage - 25];
        else spearnings = 0;

        /* both spouses survive (vcat = 0) */

        if (earntest == 1)
        { if (ownben + spbenefitsp > 0)
          { pctlost = rednrate * (earnings - earningstest) / (ownben + spbenefitsp);

            if (pctlost < 0)
            pctlost = 0;
            else if (pctlost > 1)
            pctlost = 1;
            }
          else pctlost = 1;
          }
        else pctlost = 0;

        if (spearntest == 1)
        { if (spownben + benefitsp > 0)
          { sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);

            if (sppctlost < 0)
            sppctlost = 0;
            else if (sppctlost > 1)
            sppctlost = 1;
            }
          else sppctlost = 1;
          }
        else sppctlost = 0;

        if (benefitsp > 0)                                  /* husband collecting spouse benefits */
        { if (sppctlost < pctlost)                          /* additional loss of spouse benefits */
          { sppctlost = sppctlost - (pctlost - sppctlost) * benefitsp / spownben;
            pctlostsp = pctlost;

            if (sppctlost < 0)
            sppctlost = 0;
            }
          else pctlostsp = sppctlost;
          }
        else pctlostsp = 1;

        if (spbenefitsp > 0)                                /* wife collecting spouse benefits */
        { if (pctlost < sppctlost)                          /* additional loss of spouse benefits */
          { pctlost = pctlost - (sppctlost - pctlost) * spbenefitsp / ownben;
            sppctlostsp = sppctlost;

            if (pctlost < 0)
            pctlost = 0;
            }
          else sppctlostsp = pctlost;
          }
        else sppctlostsp = 1;

        ssben = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp
               + (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;

        if (jcat == 1)
        ssbens[age - 50][0][scat] = ssben;
        else ssbenp[age - 50][0][scat] = ssben;

        /* husband only survives (vcat = 1) */

        if (age >= sserage)
        { survben = max(ownben, sppia * max(0.825, spadjown));

          if (survben > 0)
          { if (earntest == 1)
            { survpctlost = rednrate * (earnings - earningstest) / survben;

              if (survpctlost < 0)
              survpctlost = 0;
              else if (survpctlost > 1)
              survpctlost = 1;
              }
            else survpctlost = 0;
            }
          else survpctlost = 1;
          }
        else survpctlost = 0;

        ssben = (1 - survpctlost) * survben;

        if (jcat == 1)
        ssbens[age - 50][1][scat] = ssben;
        else ssbenp[age - 50][1][scat] = ssben;

        /* calculate increase in future benefits if in secondary or partial retirement jobs */

        if (age >= sserage)
        { if (sserage <= age && age < 70)
          { if (age < nrage - 3)
            pctchange = 0.05 * pctlost;
            else if (age < nrage)
            pctchange = (0.2 / 3) * pctlost;
            else pctchange = delayedret * pctlost;
            }
          else pctchange = 0;

          benchange = pctchange * ownben;

          if (jcat == 1)
          benchanges[age - 62][0][scat] = benchange;
          else benchangep[age - 62][0][scat] = benchange;

          if (sserage <= age && age < 70)
          { if (age < nrage - 3)
            pctchange = 0.05 * survpctlost;
            else if (age < nrage)
            pctchange = (0.2 / 3) * survpctlost;
            else pctchange = delayedret * survpctlost;
            }
          else pctchange = 0;

          benchange = pctchange * ownben;

          if (jcat == 1)
          benchanges[age - 62][1][scat] = benchange;
          else benchangep[age - 62][1][scat] = benchange;
          }
        }
      }
    }

  /* calculate initial scat values for each age of leaving main job */
  
  for (age = 50; age <= 70; age++)
  { if (age < sserage)
    { scat = 0;
      firstage = 50;
      lastage = sserage;
      }
    else if (age < nrage)
    { scat = 1;
      firstage = sserage;
      lastage = nrage;
      }
    else
    { scat = 2;
      firstage = nrage;
      lastage = 70;
      }

    if (fbenefit[lastage - 50] - fbenefit[firstage - 50] > 0)
    { initialscat[age - 50] = scat + (fbenefit[age - 50] - fbenefit[firstage - 50])
                               / (fbenefit[lastage - 50] - fbenefit[firstage - 50]);

      if (initialscat[age - 50] < scat)
      initialscat[age - 50] = scat;
      else if (initialscat[age - 50] > scat + 1)
      { if (scat < 2)
        initialscat[age - 50] = scat + 1;
        else initialscat[age - 50] = scat + 0.999;
        }
      }
    else initialscat[age - 50] = scat;
    }

  ssbenf[0] = fbenefit[50 - 50];
  ssbenf[1] = fbenefit[sserage - 50];
  ssbenf[2] = fbenefit[nrage - 50];
  ssbenf[3] = fbenefit[70 - 50];

  }


/*---------------------------------------------------------------------------
			 RHO CALC
			 caluclates rho consistent with initial wealth and retirement
  ---------------------------------------------------------------------------*/

double rhocalc(
  int    threadno)            /* thread number */

{ int    obs, iter, row;
  int    lastftage, firstprage, lastprage, firstretage;
  int    age, retage;
  double earnings, earntemp;
  double alpha, rho, rho1, rho2;
  double wealthdiff, wealthdiff1, wealthdiff2;

  double wagevec[5];

  DATA       *data;
  CALCSTRUCT *calcStruct;


  calcStruct = &calcStructv[threadno - localthreadmin];
  obs = calcStruct->obs;
  data = &datav[obs];

  alpha = calcStruct->alpha;


  /* calculate approximate retirement dates */
  
  lastftage = 0;
  firstprage = 99;
  lastprage = 0;
  firstretage = 99;

  for (age = 50; age < 70; age++)
  { if (data->retv[age - 50] == 5)       /* working full time */
    { if (firstprage == 99 && firstretage == 99)
      lastftage = age;

      firstretage = 99;
      }
    else if (data->retv[age - 50] == 3)  /* partially retired */ 
    { if (firstprage == 99)
      firstprage = age;

      lastprage = age;
      firstretage = 99;
      }
    else if (data->retv[age - 50] == 1)  /* completely retired */
    { if (firstretage == 99)
      firstretage = age;
      }
    }

  if (lastftage > 0)           /* working during survey */
  { if (firstprage < 99)       /* partial retirement observed */
    retage = ((lastftage + 1) + firstprage) / 2;
    else
    { if (firstretage < 99)    /* complete retirement observed */
      retage = ((lastftage + 1) + firstretage) / 2;
      else retage = lastftage + 1;
      }
    }
  else
  { if (firstprage < 99)       /* partial retirement observed */
    retage = firstprage;
    else if (firstretage < 99) /* complete retirement observed */
    retage = firstretage;
    else retage = 62;
    }

  if (retage < 50)
  retage = 50;
  else if (retage > 70)
  retage = 70;

  /* if low assets, assign rho a high value */

  memset(wagevec, 0, sizeof(double[5]));

  for (age = max(25, retage - 10); age < retage; age++)
  { earnings = data->ftwage[age - 25];

    for (row = 4; row >= 0; row--)
    if (earnings > wagevec[row])
    { earntemp = wagevec[row];
      wagevec[row] = earnings;
      earnings = earntemp;
      }
    }

  if (data->assetwealth <= 0.25 * wagevec[2]      /* 25% of third highest year */
   && data->plumpsum[retage - 50] < wagevec[2])
  return(3.0);

  incomecalc(retage, calcStruct, data);

  rho2 = 0.2;

  conscalc(rho2, calcStruct, alpha);
  wealthdiff2 = wealthcalc(calcStruct, data);

  if (wealthdiff2 >= 0)
  { rho1 = 0;
    wealthdiff1 = -1;    /* actual wealth - simulated wealth in 1992 */
    }
  else
  { for (iter = 0; iter < 3 && wealthdiff2 < 0; iter++)
    { rho1 = rho2;
      wealthdiff1 = wealthdiff2;

      rho2 = 2 * rho1;

      conscalc(rho2, calcStruct, alpha);
      wealthdiff2 = wealthcalc(calcStruct, data);
      }
    }

  if (rho1 == 0 || wealthdiff2 >= 0)
  { for (iter = 0; iter < 20; iter++)
    { rho = (rho1 + rho2) / 2;

      conscalc(rho, calcStruct, alpha);
      wealthdiff = wealthcalc(calcStruct, data);

      if (wealthdiff >= 0)
      { rho2 = rho;
        wealthdiff2 = wealthdiff;
        }
      else
      { rho1 = rho;
        wealthdiff1 = wealthdiff;
        }
      }

    rho = (rho1 * wealthdiff2 - rho2 * wealthdiff1) / (wealthdiff2 - wealthdiff1);
    }
  else rho = rho2;

  return(rho);

  }


/*---------------------------------------------------------------------------
			 INCOME CALC routine
			 calculates conditional income for the three survival outcomes
  ---------------------------------------------------------------------------*/

void incomecalc(
  int        retage,          /* retirement age        */
  CALCSTRUCT *calcStruct,     /* calculation structure */
  DATA       *data)           /* data structure        */

{ int    scat, nscats, vcat;
  int    age, spage;
  int    sserage, nrage, nragew;
  int    birthyear, spbirthyear;
  double fs;
  double p, pben, sppensben;
  double inflrate, penadjrate;
  double ssbenefit;

  int    *penstart;
  double *ftwage, *spwage;
  double *pensben, *plumpsum;
  double *initialscat;
  double *othincome;
  double (*income)[75];
  double (*ssbenm)[2];
  double (*ssbenr)[3][nscatlim];


  /* set up income vector if both survive */

  birthyear   = data->birthyear;
  spbirthyear = data->spbirthyear;

  ftwage      = data->ftwage;      /* full-time wages at ages 25-69                      */
  spwage      = data->spwage;      /* wages of spouse at spouse ages 25-69               */
  pensben     = data->pensben;     /* annual pension benefit for retirement at 50-70     */
  penstart    = data->penstart;    /* starting age of pension for retirement at 50-70    */
  plumpsum    = data->plumpsum;    /* lump-sum pension benefit for retirement at 50-70   */
  othincome   = data->othincome;   /* inheritances for ages 25-99                        */

  sserage = calcStruct->sserage;
  nrage   = calcStruct->nrage;
  nragew  = calcStruct->spnrage;

  nscats = calcStruct->nscats;
  initialscat = calcStruct->initialscat;

  income = calcStruct->income;
  ssbenm = calcStruct->ssbenm;
  ssbenr = calcStruct->ssbenr;

  memset(income, 0, sizeof(double[3][75]));

  inflrate = calcStruct->inflrate;
  penadjrate = calcStruct->penadjrate;

  income = calcStruct->income;

  for (age = 25; age < retage && age < 70; age++)
  { income[0][age - 25] = ftwage[age - 25];
    income[1][age - 25] = ftwage[age - 25];
    }

  if (spbirthyear > 0)
  for (age = 25; age < 100; age++)
  { spage = age + (birthyear - spbirthyear);

    if (25 <= spage && spage < 70)
    { income[0][age - 25] = income[0][age - 25] + spwage[spage - 25];
      income[2][age - 25] = spwage[spage - 25];
      }
    }

  p = (1 - penadjrate) * inflrate;

  pben = pensben[retage - 50];

  for (age = penstart[retage - 50]; age < 100; age++)
  { income[0][age - 25] = income[0][age - 25] + pben;
    income[1][age - 25] = income[1][age - 25] + pben;
    pben = pben / (1 + p);
    }

  income[0][retage - 25] = income[0][retage - 25] + plumpsum[retage - 50];
  income[1][retage - 25] = income[1][retage - 25] + plumpsum[retage - 50];

  if (data->sppenstart < 99)
  { sppensben = data->sppensben;

    for (spage = data->sppenstart; spage < 100; spage++)
    { age = spage + (spbirthyear - birthyear);

      if (25 < age && age < 100)
      { income[0][age - 25] = income[0][age - 25] + sppensben;
        income[2][age - 25] = income[2][age - 25] + sppensben;
        }

      pben = pben / (1 + p);
      }

    age = data->sppenstart + (spbirthyear - birthyear);

    if (25 <= age && age < 100)
    { income[0][age - 25] = income[0][age - 25] + data->spplumpsum;
      income[2][age - 25] = income[2][age - 25] + data->spplumpsum;
      }
    }

  scat = (int) initialscat[retage - 50];

  if (scat >= nscats - 1)
  scat = nscats - 2;

  fs = initialscat[retage - 50] - scat;

  for (age = 25; age < 100; age++)
  { spage = (age + birthyear) - spbirthyear;

    for (vcat = 0; vcat < 3; vcat++)
    { if (vcat < 2 && age < retage)
      ssbenefit = ssbenm[age - 25][vcat];
      else
      ssbenefit = (1 - fs) * ssbenr[age - 50][vcat][scat    ]
                 +  fs     * ssbenr[age - 50][vcat][scat + 1];
      
      income[vcat][age - 25] = income[vcat][age - 25] +  ssbenefit;
      }
    }

  for (vcat = 0; vcat < 3; vcat++)
  for (age = 25; age < 100; age++)
  income[vcat][age - 25] = income[vcat][age - 25] + othincome[age - 25];

  for (age = 25; age < 100; age++)
  { if (income[0][age - 25] < 10) income[0][age - 25] = 10;
    if (income[1][age - 25] < 10) income[1][age - 25] = 10;
    if (income[2][age - 25] < 10) income[2][age - 25] = 10;
    }

  }


/*---------------------------------------------------------------------------
			 CONS CALC routine
			 calculates conditional consumption streams
  ---------------------------------------------------------------------------*/

void conscalc(
  double     rho,             /* value of time preference */
  CALCSTRUCT *calcStruct,     /* calculation structure    */
  double     alpha)           /* consumption parameter    */

{ int    k, age;
  int    acat, acatf, rcat, vcat, vcatt;
  int    nacats, nrcats;
  double r, tp, consadj, sqrt2;
  double f, fa, y, mu, mur, murv;
  double mu0, mu1, dmu, mufact;
  double c, nassets;

  double *acats, *rprob;
  double (*rtransa)[nrcatlim];
  double (*cons)[3][nacatlim];
  double (*income)[75], (*srate)[75], (*transrate)[75];

  double cv[nacatlim], yac[nacatlim];
  double margu[3][nacatlim];

  tp = 1 + rho;
  r  = 1 + calcStruct->realrateg;

  sqrt2 = sqrt(2.0);
  consadj = pow(2, 0.5 * (1 - alpha));

  nacats = calcStruct->nacats;
  nrcats = calcStruct->nrcats;

  cons  = calcStruct->cons;

  acats = calcStruct->acats;

  income = calcStruct->income;
  srate = calcStruct->srate;
  transrate = calcStruct->transrate;

  rprob = calcStruct->rprob;
  rtransa = calcStruct->rtransa;

  /* calculate utilities for age 99 */

  for (vcat = 0; vcat < 3; vcat++)
  for (acat = 0; acat < nacats;  acat++)
  { y = acats[acat] + income[vcat][99 - 25];
    mu = pow(y, alpha - 1);

    if (vcat == 0)                          /* married */
    mu = consadj * mu;

    margu[vcat][acat] = mu;
    }

  /* calculate utilities for ages 25-98 */

  for (age = 98; age >= 25; age--)
  for (vcat = 0; vcat < 3; vcat++)
  { checkMessages();

    for (acat = 0; acat < nacats; acat++)
    { mu = 0;

      for (rcat = 0; rcat < nrcats; rcat++)
      { acatf = (int) rtransa[acat][rcat];

        if (acatf >= nacats - 1)
        acatf = nacats - 2;

        fa = rtransa[acat][rcat] - acatf;

        mur = 0;

        for (vcatt = 0; vcatt < 3; vcatt++)
        if (vcatt == 0 || (vcat == 0 && age >= 62))
        { mu0 = margu[vcatt][acatf];
          mu1 = margu[vcatt][acatf + 1];

          if (mu0 <= 0 || mu1 <= 0)
          { wnprintf("\nBad mu in rho section: %2d %2d %14.6e %14.6e %2d %2d %2d",
              calcStruct->obs, age, mu0, mu1, vcat, acat, rcat);
            pause();
            }

          if (fa <= 1)
          murv = (1 - fa) * mu0 + fa * mu1;
          else
          { if (mu0 > mu1)
            { dmu = mu0 - mu1;
              mufact = dmu / mu0;

              murv = mu1 / (1 + (fa - 1) * mufact);
              }
            else murv = mu1;
            }

          if (vcat == 0 && age >= 62)
          mur = mur + transrate[vcatt][age - 25] * murv;
          else mur = mur + murv;
          }

        mu = mu + rprob[rcat] * mur;
        }

      if (vcat == 0)
      cv[acat] = sqrt2 * pow((mu / tp), 1 / (alpha - 1));
      else cv[acat] = pow(mu * srate[vcat - 1][age - 25] / tp, 1 / (alpha - 1));
      }

    y = income[vcat][age - 25];

    for (k = 0; k < nacats; k++)
    yac[k] = y - acats[k] - cv[k];

    k = 0;

    for (acat = 0; acat < nacats; acat++)
    { nassets = - acats[acat];

      if (yac[k] < nassets)
      { k = k - 1;

        while(k >= 0 && yac[k] < nassets)
        k = k - 1;

        if (k < 0)          /* optimal to leave no assets to next period */
        { f = 0;
          k = 0;
          }
        else f = (yac[k] - nassets) / (yac[k] - yac[k + 1]);
        }
      else
      { while(k < nacats - 1 && yac[k + 1] >= nassets)
        k = k + 1;

        if (k > nacats - 2)    /* requires extrapolation */
        { k = nacats - 2;

          if (cv[k + 1] > cv[k])
          f = (yac[k] - nassets) / (yac[k] - yac[k + 1]);
          else f = 1 + (yac[k + 1] - nassets) / (acats[k + 1] - acats[k]);
          }
        else f = (yac[k] - nassets) / (yac[k] - yac[k + 1]);
        }

      c = y + acats[acat] - ((1 - f) * acats[k] + f * acats[k + 1]);

      if (vcat == 0)                         /* married */
      mu = consadj * pow(c, alpha - 1);
      else mu = pow(c, alpha - 1);

      cons[age - 25][vcat][acat] = c;
      margu[vcat][acat] = mu;
      }
    }

  }


/*---------------------------------------------------------------------------
			 WEALTH CALC routine
			 calculates differences between calculated and observed wealth
  ---------------------------------------------------------------------------*/

double wealthcalc(
  CALCSTRUCT *calcStruct,     /* calculation structure */
  DATA       *data)           /* data structure        */

{ int    age, age92, acat, birthyear, year;
  double r, c, wealth, wealthdiff;

  double *acats, *realrate;
  double (*income)[75];
  double (*cons)[3][nacatlim];

  r = calcStruct->realrateg;

  realrate = calcStruct->realrate;

  cons = calcStruct->cons;
  acats = calcStruct->acats;
  income = calcStruct->income;

  birthyear = data->birthyear;
  age92 = 1992 - birthyear;

  wealth = 0;

  for (age = 25; age < age92; age++)
  { year = age + birthyear;

    if (1926 < year)
    wealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * wealth;
    else wealth = (1 + r) * wealth;

    acat = 0;

    while (acat < 39 && wealth > acats[acat])
    acat = acat + 1;

    if (acat > 0)
    acat = acat - 1;

    c = ((acats[acat + 1] - wealth) * cons[age - 25][0][acat]
         + (wealth - acats[acat]) * cons[age - 25][0][acat + 1])
         / (acats[acat + 1] - acats[acat]);

    wealth = wealth + income[0][age - 25] - c;
    }

  if (data->assetwealth > 0 || wealth > 0.1)
  wealthdiff = data->assetwealth - (1 + 0.01 * realrate[1991 - 1926]) * wealth;
  else wealthdiff = 0.1;

  return(wealthdiff);

  }


/*---------------------------------------------------------------------------
			 CALC MODEL routine
			 calculates utility, savings, and job choices for the model
  ---------------------------------------------------------------------------*/

void calcModel(
  int    threadno,            /* thread number */
  int    tcat)                /* utility of partial retirement category */

{ int    obs;
  int    age, agew, firstage, lastage;
  int    earlypenage, delayedpenage, contribstartage, penjobend;
  int    sserage, nrage, nragew;
  int    nacats, necats, nlcats, npcats, nrcats, nscats;
  int    acat, dcat, ecat, lacat, lcat, pcat, rcat, scat, vcat;
  int    acati, lcati, pcati, scati;
  int    reversal, choice, job, jobmax, k, utype;
  double alpha, rho;
  double y, c, cmax, mu, mumax, mur, tu, tur, tumax;
  double r, tp, consadj, sqrt2;
  double f, fa, fl, fla, fp, fs;
  double mu0, mu1, tu0, tu1, temp, util;
  double dmu, dtu, muratio, mufact;
  double marguf0, marguf1, marguf2, totuf0, totuf1, totuf2;
  double margufw0, margufw1, margufr2, totufw0, totufw1, totufr2;
  double margufr0, margufr1, totufr0, totufr1;
  double nassets, pben, pwealth, eowamt, wealth;
  double sben, sbenf, benchange, delayedret, delayedretw;
  double inflrate, penadjrate;
  double yacmin, yacmax;

  double tempp[2], temps[2];
  double muvbuf[nacatlim], cvbuf[nacatlim], tuvbuf[nacatlim], yac[nacatlim];
  double mula[nlcatlim][nacatlim], tula[nlcatlim][nacatlim];
  double mular[nlcatlim][nacatlim][nrcatlim], tular[nlcatlim][nacatlim][nrcatlim];
  double lacats[nlcatlim][nacatlim];
  double lacatsr[nlcatlim][nacatlim], lacatsw[nlcatlim][nacatlim];
  double tuj[4], muj[4];


  double *cv, *tuv, *saving, *margu, *totu, *totutil;
  double *acats, *lcats, *pcats, *rprob;
  double *mainwage, *secwage, *prwage, *eowamount;
  double (*tcats)[20];
  double *initialpcat, *initialscat, *ssbenf;
  double (*benchanges)[2][nscatlim];
  double (*benchangep)[2][nscatlim];
  double (*othincome)[3];
  double (*ssbenm)[2];
  double (*ssbens)[2][nscatlim];
  double (*ssbenp)[2][nscatlim];
  double (*ssbenr)[3][nscatlim];
  double (*srate)[75], (*transrate)[75];
  double (*etrans)[necatlim], (*rtransa)[nrcatlim];
  double (*rtransl)[nlcatlim][nrcatlim];
  double (*utilr)[20];
  double (*marguc)[2][npcatlim][nscatlim][nacatlim];
  double (*totuc)[2][npcatlim][nscatlim][nacatlim];
  double (*ufm)[nscatlim][nacatlim];
  double (*uft)[nscatlim][nacatlim];
  double (*marguf)[2][npcatlim][nscatlim][nacatlim];
  double (*totuf)[2][npcatlim][nscatlim][nacatlim];
  double (*margufw)[nlcatlim][necatlim][nacatlim];
  double (*totufw)[nlcatlim][necatlim][nacatlim];
  double (*margufr)[2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*totufr)[2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*cmatw)[nlcatlim][necatlim][nacatlim];
  double (*tumatw)[nlcatlim][necatlim][nacatlim];
  double (*cmatr)[2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*tumatr)[2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*margucw)[nlcatlim][necatlim][nacatlim];
  double (*totucw)[nlcatlim][necatlim][nacatlim];
  double (*ucrm)[nscatlim][necatlim][nacatlim];
  double (*ucrt)[nscatlim][necatlim][nacatlim];
  double (*margucr)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*totucr)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*totuw)[nlcatlim][necatlim][nacatlim];
  double (*totud)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*savw)[nlcatlim][necatlim][nacatlim];
  double (*savd)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*savr)[2][npcatlim][nscatlim][nacatlim];
  int    (*mjchoice)[nlcatlim][necatlim][nacatlim];
  int    (*retchoice)[2][npcatlim][nscatlim][necatlim][nacatlim];

  DATA       *data;
  CALCSTRUCT *calcStruct;


  calcStruct = &calcStructv[threadno - localthreadmin];
  obs = calcStruct->obs;
  data = &datav[obs];

  alpha = calcStruct->alpha;
  rho = arglist.rvec[obs];

  firstage = calcStruct->firstage;
  lastage = calcStruct->lastage;

  earlypenage     = calcStruct->earlypenage;
  delayedpenage   = calcStruct->delayedpenage;
  contribstartage = calcStruct->contribstartage;
  penjobend       = calcStruct->penjobend;
  sserage         = calcStruct->sserage;
  nrage           = calcStruct->nrage;
  nragew          = calcStruct->spnrage;
  delayedret      = calcStruct->delayedret;
  delayedretw     = calcStruct->spdelayedret;

  nacats = calcStruct->nacats;
  necats = calcStruct->necats;
  nlcats = calcStruct->nlcats;
  npcats = calcStruct->npcats;
  nrcats = calcStruct->nrcats;
  nscats = calcStruct->nscats;

  acats = calcStruct->acats;
  lcats = calcStruct->lcats;
  pcats = calcStruct->pcats;
  tcats = calcStruct->tcats;

  mainwage = calcStruct->mainwage;
  secwage  = calcStruct->secwage;
  prwage   = calcStruct->prwage;

  ssbenf = calcStruct->ssbenf;
  ssbenm = calcStruct->ssbenm;
  ssbens = calcStruct->ssbens;
  ssbenp = calcStruct->ssbenp;
  ssbenr = calcStruct->ssbenr;
  eowamount = calcStruct->eowamount;
  othincome = calcStruct->othincome;

  initialpcat = calcStruct->initialpcat;
  initialscat = calcStruct->initialscat;

  benchanges  = calcStruct->benchanges;
  benchangep  = calcStruct->benchangep;

  srate = calcStruct->srate;
  transrate = calcStruct->transrate;
  etrans = calcStruct->etrans;
  rprob = calcStruct->rprob;
  rtransa = calcStruct->rtransa;
  rtransl = calcStruct->rtransl;

  utilr = calcStruct->utilr;

  marguc = calcStruct->marguc;
  totuc = calcStruct->totuc;

  marguf = calcStruct->marguf;
  totuf = calcStruct->totuf;

  margufw = calcStruct->margufw;
  totufw = calcStruct->totufw;

  margufr = calcStruct->margufr;
  totufr = calcStruct->totufr;

  cmatw = calcStruct->cmatw;
  tumatw = calcStruct->tumatw;

  cmatr = calcStruct->cmatr;
  tumatr = calcStruct->tumatr;

  margucw = calcStruct->margucw;
  totucw = calcStruct->totucw;
  margucr = calcStruct->margucr;
  totucr = calcStruct->totucr;

  totuw = calcStruct->totuw;
  
  totud = calcStruct->totud;
  savw = calcStruct->savw;
  savd = calcStruct->savd;
  savr = calcStruct->savr;

  mjchoice = calcStruct->mjchoice;
  retchoice = calcStruct->retchoice;
  /* wnprintf("m"); */

  tp = 1 + rho;
  r = 1 + calcStruct->realrateg;

  inflrate = calcStruct->inflrate;
  penadjrate = calcStruct->penadjrate;

  sqrt2 = sqrt(2.0);
  consadj = pow(2, 0.5 * (1 - alpha));

  k = 0;

  /* calculate consumption and utility at age 99 */

  agew = (calcStruct->birthyear + 99) - calcStruct->spbirthyear;

  for (vcat = 0; vcat < 3; vcat++)           /* survival categories */
  for (pcat = 0; pcat < npcats; pcat++)
  if (pcat == 0 || vcat < 2)
  { if (vcat < 2 && npcats > 1)
    pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (99 - 50));
    else pben = 0;

    for (scat = 0; scat < nscats; scat++)
    { sben = ssbenr[99 - 50][vcat][scat];

      for (acat = 0; acat < nacats; acat++)
      { y = acats[acat] + pben + sben + othincome[99 - 25][vcat];

        if (y < 10)
        y = 10;

        mu = pow(y, alpha - 1);
        tu = y * mu / alpha;

        if (vcat == 0)                         /* married */
        { mu = consadj * mu;
          tu = consadj * tu;
          }

        marguf[vcat][0][pcat][scat][acat] = mu;
        totuf[vcat][0][pcat][scat][acat] = tu;

        if (vcat == 0)
        savr[99 - 70][0][pcat][scat][acat] = - acats[acat];
        }
      }
    }

  /* calculate consumption and utility for ages lastage to 99 */

  for (age = 98; age > lastage; age--)
  { /* calculate expectations with respect to interest rates */

    margu = muvbuf;
    totu  = tuvbuf;
    cv    = cvbuf;
    tuv   = tuvbuf;

    for (vcat = 0; vcat < 3; vcat++)
    if (vcat == 0 || age > 62)
    { for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        if (pcat == 0 || vcat < 2)
        { for (scat = 0; scat < nscats; scat++)
          { for (acat = 0; acat < nacats; acat++)
            { mur = 0;
              tur = 0;

              for (rcat = 0; rcat < nrcats; rcat++)
              { acati = (int) rtransa[acat][rcat];

                if (acati >= nacats - 1)
                acati = nacats - 2;

                fa = rtransa[acat][rcat] - acati;

                mu0 = marguf[vcat][dcat][pcat][scat][acati    ];
                mu1 = marguf[vcat][dcat][pcat][scat][acati + 1];
                tu0 = totuf[vcat][dcat][pcat][scat][acati    ];
                tu1 = totuf[vcat][dcat][pcat][scat][acati + 1];

                if (mu0 <= 0 || mu1 <= 0)
                { wnprintf("\nBad mu in post-retirement section: %2d %2d %14.6e %14.6e %2d %2d %2d %2d %2d %2d",
                    obs, age, mu0, mu1, vcat, dcat, pcat, scat, acat, acati);
                  pause();
                  }

                if (fa <= 1)
                { mu = (1 - fa) * mu0 + fa * mu1;
                  tu = (1 - fa) * tu0 + fa * tu1;
                  }
                else
                { if (mu0 > mu1 && tu1 > tu0)
                  { dmu = mu0 - mu1;
                    dtu = tu1 - tu0;
                    muratio = mu0 / mu1;
                    mufact = dmu / mu0;

                    mu = mu1 / (1 + (fa - 1) * mufact);
                    tu = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                    }
                  else
                  { mu = mu1;

                    if (tu1 > tu0)
                    tu = (1 - fa) * tu0 + fa * tu1;
                    else tu = tu1;
                    }
                  }

                mur = mur + rprob[rcat] * mu;
                tur = tur + rprob[rcat] * tu;
                }

              margu[acat] = mur;
              totu[acat] = tur;
              }

            for (acat = 0; acat < nacats; acat++)
            { marguf[vcat][dcat][pcat][scat][acat] = margu[acat];
              totuf[vcat][dcat][pcat][scat][acat] = totu[acat];
              }
            }
          }
        }
      }

    /* calculate expectations with respect to survival */

    if (age >= 62)                                   /* vcat = 0 */
    { for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        for (scat = 0; scat < nscats; scat++)
        for (acat = 0; acat < nacats; acat++)
        { marguf0 = marguf[0][dcat][pcat][scat][acat];
          totuf0 = totuf[0][dcat][pcat][scat][acat];

          marguf1 = marguf[1][dcat][pcat][scat][acat];
          totuf1 = totuf[1][dcat][pcat][scat][acat];

          marguf2 = marguf[2][0][0][scat][acat];
          totuf2 = totuf[2][0][0][scat][acat];

          marguf[0][dcat][pcat][scat][acat]
           =  transrate[0][age - 25] * marguf0
            + transrate[1][age - 25] * marguf1
            + transrate[2][age - 25] * marguf2;

          totuf[0][dcat][pcat][scat][acat]
           =  transrate[0][age - 25] * totuf0
            + transrate[1][age - 25] * totuf1
            + transrate[2][age - 25] * totuf2;
          }
        }
      }

    /* calculate consumption and utility */

    for (vcat = 0; vcat < 3; vcat++)
    if (vcat == 0 || age > 62)
    { for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        if (pcat == 0 || vcat < 2)
        { if (vcat < 2 && dcat == 0 && earlypenage <= age)
          pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (age - 50));
          else pben = 0;

          for (scat = 0; scat < nscats; scat++)
          { checkMessages();

            for (acat = 0; acat < nacats; acat++)
            { mu = marguf[vcat][dcat][pcat][scat][acat];
              tu = totuf[vcat][dcat][pcat][scat][acat];

              if (mu <= 0)
              { wnprintf("\nBad mu for post-retirement decision: %2d %14.6e %2d %2d %2d %2d %2d",
                  age, mu0, mu1, vcat, dcat, pcat, scat, acat);
                pause();
                }

              if (vcat == 0)
              { cv[acat] = sqrt2 * pow((mu / tp), 1 / (alpha - 1));
                tuv[acat] = tu / tp;
                }
              else
              { cv[acat] = pow(mu * srate[vcat - 1][age - 25] / tp, 1 / (alpha - 1));
                tuv[acat] = tu * srate[vcat - 1][age - 25] / tp;
                }
              }

            margu = marguc[vcat][dcat][pcat][scat];
            totu = totuc[vcat][dcat][pcat][scat];
            totutil = totuc[vcat][dcat][pcat][scat];

            if (age < 70)
            saving = savd[age - 50][2][dcat][pcat][scat][0];
            else saving = savr[age - 70][dcat][pcat][scat];

            sben = ssbenr[age - 50][vcat][scat];

            y = pben + sben + othincome[age - 25][vcat];

            #include "GSModel.inc"
            }
          }
        }

      /* allow for delayed pension receipt */

      if (vcat < 2)
      { if (earlypenage < age && age == delayedpenage)
        { memcpy(marguc[vcat][1], marguc[vcat][0], sizeof(double[npcatlim][nscatlim][nacatlim]));
          memcpy(totuc[vcat][1], totuc[vcat][0], sizeof(double[npcatlim][nscatlim][nacatlim]));
          }
        else if (earlypenage == age && age < delayedpenage)
        { memcpy(marguc[vcat][0], marguc[vcat][1], sizeof(double[npcatlim][nscatlim][nacatlim]));
          memcpy(totuc[vcat][0], totuc[vcat][1], sizeof(double[npcatlim][nscatlim][nacatlim]));
          }
        }
      }

 
    /* transfer margu & totu to future matrices */

    memcpy(marguf, marguc, 3 * 2 * npcatlim * nscatlim * nacatlim * sizeof(double));
    memcpy(totuf, totuc, 3 * 2 * npcatlim * nscatlim * nacatlim * sizeof(double));
    }

  /* force retirement at lastage + 1 */

  pcat = (int) initialpcat[(lastage + 1) - 50];
  scat = (int) initialscat[(lastage + 1) - 50];

  fp = initialpcat[(lastage + 1) - 50] - pcat;
  fs = initialscat[(lastage + 1) - 50] - scat;

  eowamt = eowamount[(lastage + 1) - 50];

  for (lcat = 0; lcat < nlcats; lcat++)
  { if ((lcat > 0 && penjobend + 1 >= (lastage + 1) && (lastage + 1) > contribstartage)
     || (lcat == 0 && eowamt > 0))
    { for (acat = 0; acat < nacats; acat++)
      { pwealth = acats[acat] + lcats[lcat] + eowamt;

        if (acat < nacats - 1)
        lacat = acat + 1;
        else lacat = nacats - 1;

        while (lacat < nacats - 1 && pwealth > acats[lacat])
        lacat = lacat + 1;

        lacat = lacat - 1;

        lacats[lcat][acat] = lacat + (pwealth - acats[lacat]) / (acats[lacat + 1] - acats[lacat]);
        }
      }
    else if (lcat == 0)
    { for (acat = 0; acat < nacats; acat++)
      lacats[lcat][acat] = acat;
      }
    }

  for (vcat = 0; vcat < 2; vcat++)
  if (vcat == 0 || (lastage + 1) > 62)
  { for (lcat = 0; lcat < nlcats; lcat++)
    if (lcat == 0 || ((penjobend + 1) >= (lastage + 1) && (lastage + 1) > contribstartage))
    { for (acat = 0; acat < nacats; acat++)
      { lacat = (int) lacats[lcat][acat];

        if (lacat > nacats - 2)
        lacat = nacats - 2;

        fla = lacats[lcat][acat] - lacat;

        ufm = marguf[vcat][0];
        uft = totuf[vcat][0];

        for (utype = 0; utype <= 1; utype++)
        { for (pcati = pcat; pcati <= pcat + 1; pcati++)
          if (pcati == pcat || fp != 0)
          { for (scati = scat; scati <= scat + 1; scati++)
            if (scati == scat || fs != 0)
            { if (fla != 0)
              { if (utype == 0)
                { mu0 = ufm[pcati][scati][lacat    ];
                  mu1 = ufm[pcati][scati][lacat + 1];

                  if (fla <= 1)
                  temp = (1 - fla) * mu0 + fla * mu1;
                  else if (mu0 > mu1)
                  { dmu = mu0 - mu1;
                    mufact = dmu / mu0;

                    temp = mu1 / (1 + (fla - 1) * mufact);
                    }
                  else temp = mu1;
                  }
                else
                { tu0 = uft[pcati][scati][lacat    ];
                  tu1 = uft[pcati][scati][lacat + 1];

                  if (fla <= 1)
                  temp = (1 - fla) * tu0 + fla * tu1;
                  else
                  { mu0 = ufm[pcati][scati][lacat    ];
                    mu1 = ufm[pcati][scati][lacat + 1];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      temp = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                      }
                    else if (tu1 > tu0)
                    temp = (1 - fla) * tu0 + fla * tu1;
                    else temp = tu1;
                    }
                  }
                }
              else
              { if (utype == 0)
                temp = ufm[pcati][scati][lacat];
                else temp = uft[pcati][scati][lacat];
                }

              if (scati == scat)
              temps[0] = temp;
              else temps[1] = temp;
              }

            if (fs != 0)
            util = (1 - fs) * temps[0] + fs * temps[1];
            else util = temps[0];

            if (pcati == pcat)
            tempp[0] = temp;
            else tempp[1] = temp;
            }

          if (fp != 0)
          util = (1 - fp) * tempp[0] + fp * tempp[1];
          else util = tempp[0];

          if (utype == 0)
          { for (ecat = 0; ecat < necats; ecat++)
            margufw[vcat][lcat][ecat][acat] = util;
            }
          else
          { for (ecat = 0; ecat < necats; ecat++)
            totufw[vcat][lcat][ecat][acat] = util;
            }
          }
        }
      }
    }

  /* set consumption and utility for all eps categories at lastage + 1 */

  for (vcat = 0; vcat < 3; vcat++)
  if (vcat == 0 || (lastage + 1) > 62)
  { for (dcat = 0; dcat < 2; dcat++)
    if (dcat == 0 || (vcat < 2 && earlypenage < (lastage + 1) && (lastage + 1) <= delayedpenage))
    { for (pcat = 0; pcat < npcats; pcat++)
      if (pcat == 0 || vcat < 2)
      { for (scat = 0; scat < nscats; scat++)
        for (ecat = 0; ecat < necats; ecat++)
        if (ecat == 0 || vcat < 2)
        { for (acat = 0; acat < nacats; acat++)
          { margufr[vcat][dcat][pcat][scat][ecat][acat] = marguf[vcat][dcat][pcat][scat][acat];
            totufr[vcat][dcat][pcat][scat][ecat][acat] = totuf[vcat][dcat][pcat][scat][acat];

            if (margufr[vcat][dcat][pcat][scat][ecat][acat] <= 0) {
            wnprintf("\ndd %2d %d %d %d %d %d %d", lastage + 1, vcat, dcat, pcat, scat, ecat, acat); pause(); }
            }
          }
        }
      }
    }
  
  /* calculate utility and consumption for firstage to lastage */

  for (age = lastage; age >= firstage; age--)
  { /* calculate expectations with respect to returns, given exit from main job */

    margu = muvbuf;
    totu = tuvbuf;

    for (vcat = 0; vcat < 3; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        if (pcat == 0 || vcat < 2)
        { for (scat = 0; scat < nscats; scat++)
          for (ecat = 0; ecat < necats; ecat++)
          if (ecat == 0 || vcat < 2)
          { for (acat = 0; acat < nacats; acat++)
            { mu = 0;
              tu = 0;

              for (rcat = 0; rcat < nrcats; rcat++)
              { acati = (int) rtransa[acat][rcat];

                if (acati >= nacats - 1)
                acati = nacats - 2;

                fa = rtransa[acat][rcat] - acati;

                mu0 = margufr[vcat][dcat][pcat][scat][ecat][acati    ];
                mu1 = margufr[vcat][dcat][pcat][scat][ecat][acati + 1];
                tu0 = totufr[vcat][dcat][pcat][scat][ecat][acati    ];
                tu1 = totufr[vcat][dcat][pcat][scat][ecat][acati + 1];

                if (mu0 <= 0 || mu1 <= 0)
                { wnprintf("\nBad mu in decision-retired section: %2d %14.6e %14.6e %2d %2d %2d %2d %2d %2d %2d %2d",
                   age, mu0, mu1, vcat, dcat, pcat, scat, ecat, acat, acati, rcat);
                  pause();
                  }

                if (fa <= 1)
                { mur = (1 - fa) * mu0 + fa * mu1;
                  tur = (1 - fa) * tu0 + fa * tu1;
                  }
                else
                { if (mu0 > mu1 && tu1 > tu0)
                  { dmu = mu0 - mu1;
                    dtu = tu1 - tu0;
                    muratio = mu0 / mu1;
                    mufact = dmu / mu0;

                    mur = mu1 / (1 + (fa - 1) * mufact);
                    tur = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                    }
                  else
                  { mur = mu1;

                    if (tu1 > tu0)
                    tur = (1 - fa) * tu0 + fa * tu1;
                    else tur = tu1;
                    }
                  }

                mu = mu + rprob[rcat] * mur;
                tu = tu + rprob[rcat] * tur;
                }

              margu[acat] = mu;
              totu[acat] = tu;
              }

            for (acat = 0; acat < nacats; acat++)
            { margufr[vcat][dcat][pcat][scat][ecat][acat] = margu[acat];
              totufr[vcat][dcat][pcat][scat][ecat][acat] = totu[acat];
              }
            }
          }
        }
      }

    /* calculate expectations with respect to returns, given current main job */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (ecat = 0; ecat < necats; ecat++)
      { for (lcat = 0; lcat < nlcats; lcat++)
        if (lcat == 0 || (penjobend >= age && age > contribstartage))
        { for (acat = 0; acat < nacats; acat++)
          { mu = 0;
            tu = 0;

            for (rcat = 0; rcat < nrcats; rcat++)
            { acati = (int) rtransa[acat][rcat];

              if (acati >= nacats - 1)
              acati = nacats - 2;

              fa = rtransa[acat][rcat] - acati;

              if (nlcats > 1 && penjobend + 1 > age && age > contribstartage)
              { lcati = (int) rtransl[age - 25][lcat][rcat];

                if (lcati >= nlcats - 1)
                lcati = nlcats - 2;

                fl = rtransl[age - 25][lcat][rcat] - lcati;

                if (fa <= 1 && fl <= 1)
                { mur = (1 - fl) * ((1 - fa) * margufw[vcat][lcati    ][ecat][acati]
                                  +   fa     * margufw[vcat][lcati    ][ecat][acati + 1])
                       +  fl     * ((1 - fa) * margufw[vcat][lcati + 1][ecat][acati]
                                  +   fa     * margufw[vcat][lcati + 1][ecat][acati + 1]);

                  tur = (1 - fl) * ((1 - fa) * totufw[vcat][lcati    ][ecat][acati]
                                  +   fa     * totufw[vcat][lcati    ][ecat][acati + 1])
                        + fl     * ((1 - fa) * totufw[vcat][lcati + 1][ecat][acati]
                                   +  fa     * totufw[vcat][lcati + 1][ecat][acati + 1]);

                  if (mur <= 0)
                  { wnprintf("\nBad mu in decision-main job section: %2d %14.6e %14.6e %14.6e %14.6e",
                      age, margufw[vcat][lcati    ][ecat][acati],
                           margufw[vcat][lcati    ][ecat][acati + 1],
                           margufw[vcat][lcati + 1][ecat][acati],
                           margufw[vcat][lcati + 1][ecat][acati + 1]);
                    wnprintf(" %2d %2d %2d %2d %2d %2d %2d", vcat, lcat, lcati, ecat, acat, acati, rcat);
                    pause();
                    }
                  }
                else
                { if (rcat >= 2)
                  { mu0 = mular[lcat][acat][rcat - 2];
                    mu1 = mular[lcat][acat][rcat - 1];
                    tu0 = tular[lcat][acat][rcat - 2];
                    tu1 = tular[lcat][acat][rcat - 1];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      mur = mu1 / (1 + mufact);
                      tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                      }
                    else
                    { mur = mu1;

                      if (tu1 > tu0)
                      tur = 2 * tu1 - tu0;
                      else tur = tu1;
                      }
                    }
                  else if (lcat > nlcats - 5)
                  { mu0 = mular[lcat - 2][acat][rcat];
                    mu1 = mular[lcat - 1][acat][rcat];
                    tu0 = tular[lcat - 2][acat][rcat];
                    tu1 = tular[lcat - 1][acat][rcat];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      mur = mu1 / (1 + (fl - 1) * mufact);
                      tur = tu1 + (dtu / log(muratio)) * log(1 + (fl - 1) * mufact);
                      }
                    else
                    { mur = mu1;

                      if (tu1 > tu0)
                      tur = (1 - fl) * tu0 + fl * tu1;
                      else tur = tu1;
                      }
                    }
                  else
                  { wnprintf("\nfl problem: age %d rcat %d lcat %d %d %6.3f acat %d %d %6.3f rtransl %6.3f",
                      age, rcat, lcat, lcati, fl, acat, acati, fa, rtransl[age - 25][lcat][rcat]);
                    pause();
                    }

                  if (rcat > 0)
                  if (mur > mular[lcat][acat][rcat - 1])
                  mur = mular[lcat][acat][rcat - 1];

                  if (lcat > 0)
                  if (mur > mular[lcat - 1][acat][rcat])
                  mur = mular[lcat - 1][acat][rcat];

                  if (rcat > 0)
                  if (tur < tular[lcat][acat][rcat - 1])
                  tur = tular[lcat][acat][rcat - 1];

                  if (lcat > 0)
                  if (tur < tular[lcat - 1][acat][rcat])
                  tur = tular[lcat - 1][acat][rcat];
                  }
                }
              else
              { if (fa <= 1)
                { mur = (1 - fa) * margufw[vcat][0][ecat][acati]
                       +  fa     * margufw[vcat][0][ecat][acati + 1];
                  tur = (1 - fa) * totufw[vcat][0][ecat][acati]
                       +  fa     * totufw[vcat][0][ecat][acati + 1];
                  }
                else
                { if (rcat >= 2)
                  { mu0 = mular[0][acat][rcat - 2];
                    mu1 = mular[0][acat][rcat - 1];
                    tu0 = tular[0][acat][rcat - 2];
                    tu1 = tular[0][acat][rcat - 1];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      mur = mu1 / (1 + mufact);
                      tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                      }
                    else
                    { mur = mu1;

                      if (tu1 > tu0)
                      tur = 2 * tu1 - tu0;
                      else tur = tu1;
                      }
                    }
                  else
                  { wnprintf("\nfl problem: age %d rcat %d lcat %d %d %6.3f acat %d %d %6.3f rtransl %6.3f",
                      age, rcat, lcat, lcati, fl, acat, acati, fa, rtransl[age - 25][lcat][rcat]);
                    pause();
                    }

                  if (rcat > 0)
                  if (mur > mular[lcat][acat][rcat - 1])
                  mur = mular[lcat][acat][rcat - 1];

                  if (rcat > 0)
                  if (tur < tular[lcat][acat][rcat - 1])
                  tur = tular[lcat][acat][rcat - 1];
                  }
                }

              mular[lcat][acat][rcat] = mur;
              tular[lcat][acat][rcat] = tur;

              mu = mu + rprob[rcat] * mur;
              tu = tu + rprob[rcat] * tur;
              }

            mula[lcat][acat] = mu;
            tula[lcat][acat] = tu;
            } 
          }

        for (lcat = 0; lcat < nlcats; lcat++)
        if (lcat == 0 || (penjobend + 1 > age && age > contribstartage))
        { for (acat = 0; acat < nacats; acat++)
          { margufw[vcat][lcat][ecat][acat] = mula[lcat][acat];
            totufw[vcat][lcat][ecat][acat] = tula[lcat][acat];
            }
          }
        }
      }

    /* calculate expectations with respect to survival, given exit from main job */

    if (age >= 62)                            /* vcat == 0 */
    for (dcat = 0; dcat < 2; dcat++)
    if (dcat == 0 || (earlypenage <= age && age < delayedpenage))
    { for (pcat = 0; pcat < npcats; pcat++)
      for (scat = 0; scat < nscats; scat++)
      for (ecat = 0; ecat < necats; ecat++)
      for (acat = 0; acat < nacats; acat++)
      { margufr0 = margufr[0][dcat][pcat][scat][ecat][acat];
        totufr0 = totufr[0][dcat][pcat][scat][ecat][acat];

        margufr1 = margufr[1][dcat][pcat][scat][ecat][acat];
        totufr1 = totufr[1][dcat][pcat][scat][ecat][acat];

        margufr2 = margufr[2][0][0][scat][0][acat];
        totufr2 = totufr[2][0][0][scat][0][acat];

        margufr[0][dcat][pcat][scat][ecat][acat]
         =  transrate[0][age - 25] * margufr0
          + transrate[1][age - 25] * margufr1
          + transrate[2][age - 25] * margufr2;

        totufr[0][dcat][pcat][scat][ecat][acat]
         =  transrate[0][age - 25] * totufr0
          + transrate[1][age - 25] * totufr1
          + transrate[2][age - 25] * totufr2;
        }
      }

    /* calculate expectations with respect to survival, given current main job */
    
    if (age >= 62)                          /* vcat = 0 */
    { scat = (int) initialscat[(lastage + 1) - 50];
      fs = initialscat[(lastage + 1) - 50] - scat;

      for (ecat = 0; ecat < necats; ecat++)
      for (lcat = 0; lcat < nlcats; lcat++)
      if (lcat == 0 || (penjobend >= age && age > contribstartage))
      { for (acat = 0; acat < nacats; acat++)
        { wealth = acats[acat] + lcats[lcat];

          if (acat < nacats - 1)
          acati = acat + 1;
          else acati = nacats - 1;

          while (acati < nacats - 1 && wealth > acats[acati])
          acati = acati + 1;

          acati = acati - 1;

          fa = (wealth - acats[acati]) / (acats[acati + 1] - acats[acati]);

          margufw0 = margufw[0][lcat][ecat][acat];
          totufw0 = totufw[0][lcat][ecat][acat];

          margufw1 = margufw[1][lcat][ecat][acat];
          totufw1 = totufw[1][lcat][ecat][acat];

          mu0 = (1 - fs) * margufr[2][0][0][scat    ][0][acati    ]
               +  fs     * margufr[2][0][0][scat + 1][0][acati    ];
          mu1 = (1 - fs) * margufr[2][0][0][scat    ][0][acati + 1]
               +  fs     * margufr[2][0][0][scat + 1][0][acati + 1];
          tu0 = (1 - fs) * totufr[2][0][0][scat    ][0][acati    ]
               +  fs     * totufr[2][0][0][scat + 1][0][acati    ];
          tu1 = (1 - fs) * totufr[2][0][0][scat    ][0][acati + 1]
               +  fs     * totufr[2][0][0][scat + 1][0][acati + 1];

          if (fa <= 1)
          { margufr2 = (1 - fa) * mu0 + fa * mu1;
            totufr2 = (1 - fa) * tu0 + fa * tu1;
            }
          else
          { if (mu0 > mu1 && tu1 > tu0)
            { dmu = mu0 - mu1;
              dtu = tu1 - tu0;
              muratio = mu0 / mu1;
              mufact = dmu / mu0;

              margufr2 = mu1 / (1 + (fa - 1) * mufact);
              totufr2 = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
              }
            else
            { margufr2 = mu1;

              if (tu1 > tu0)
              totufr2 = (1 - fa) * tu0 + fa * tu1;
              else totufr2 = tu1;
              }
            }

          margufw[0][lcat][ecat][acat]
            =  transrate[0][age - 25] * margufw0
             + transrate[1][age - 25] * margufw1
             + transrate[2][age - 25] * margufr2;

          totufw[0][lcat][ecat][acat]
            =  transrate[0][age - 25] * totufw0
             + transrate[1][age - 25] * totufw1
             + transrate[2][age - 25] * totufr2;
          }
        }
      }

    /* caculate consumption equivalents of future marginal utility, given exit from main job */

    for (vcat = 0; vcat < 3; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        if (pcat == 0 || vcat < 2)
        { for (scat = 0; scat < nscats; scat++)
          for (ecat = 0; ecat < necats; ecat++)
          if (ecat == 0 || vcat < 2)
          { for (acat = 0; acat < nacats; acat++)
            { mu = margufr[vcat][dcat][pcat][scat][ecat][acat];
              tu = totufr[vcat][dcat][pcat][scat][ecat][acat];

              if (mu <= 0)
              { wnprintf("\nBad mu for decision-retired consumption: %2d %14.6e %2d %2d %2d %2d %2d %2d",
                  age, mu0, mu1, vcat, dcat, pcat, scat, ecat, acat);
                pause();
                }

              if (vcat == 0)
              { cmatr[vcat][dcat][pcat][scat][ecat][acat] = sqrt2 * pow((mu / tp), 1 / (alpha - 1));
                tumatr[vcat][dcat][pcat][scat][ecat][acat] = tu / tp;
                }
              else
              { cmatr[vcat][dcat][pcat][scat][ecat][acat] = pow(mu * srate[vcat - 1][age - 25] / tp, 1 / (alpha - 1));
                tumatr[vcat][dcat][pcat][scat][ecat][acat] = tu * srate[vcat - 1][age - 25] / tp;
                }
              }
            }
          }
        }
      }

    /* caculate consumption equivalents of future marginal utility, given current main job */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (ecat = 0; ecat < necats; ecat++)
      { for (lcat = 0; lcat < nlcats; lcat++)
        if (lcat == 0 || (penjobend >= age && age > contribstartage))
        { /* if (age == 67 && vcat == 0 && (ecat == 86 || ecat == 87)) { wnprintf("\n %2d", ecat); for (acat = 30; acat < 35; acat++) wnprintf(" %d %14.6e", acat, margufw[vcat][lcat][ecat][acat]); } */
          for (acat = 0; acat < nacats; acat++)
          { mu = margufw[vcat][lcat][ecat][acat];
            tu = totufw[vcat][lcat][ecat][acat];

            if (mu <= 0)
            { wnprintf("\nBad mu for decision-main job consumption: %2d %14.6e %2d %2d %2d %2d",
                age, mu0, mu1, vcat, lcat, ecat, acat);
              pause();
              }

            if (vcat == 0)
            { cmatw[vcat][lcat][ecat][acat] = sqrt2 * pow((mu / tp), 1 / (alpha - 1));
              tumatw[vcat][lcat][ecat][acat] = tu / tp;
              }
            else
            { cmatw[vcat][lcat][ecat][acat] = pow(mu * srate[vcat - 1][age - 25] / tp, 1 / (alpha - 1));
              tumatw[vcat][lcat][ecat][acat] = tu * srate[vcat - 1][age - 25] / tp;
              }
            }
          }
        }
      }


    /* calculate consumption, saving, and utility for staying in main job */

    if (npcats > 1 && age > penjobend)
    { pcat = (int) initialpcat[(max(penjobend, 49) + 1) - 50];
      fp = initialpcat[(max(penjobend, 49) + 1) - 50] - pcat;

      if (fp != 0)
      pben = ((1 - fp) * pcats[pcat] + fp * pcats[pcat + 1])
             * exp(- inflrate * (1 - penadjrate) * (age - 50));
      else pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (age - 50));
      }
    else pben = 0;

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      sben = ssbenm[age - 25][vcat];

      for (lcat = 0; lcat < nlcats; lcat++)
      if (lcat == 0 || (penjobend >= age && age > contribstartage))
      { for (ecat = 0; ecat < necats; ecat++)
        { cv = cmatw[vcat][lcat][ecat];
          tuv = tumatw[vcat][lcat][ecat];

          margu = margucw[vcat][lcat][ecat];
          totu = totucw[vcat][lcat][ecat];

          saving = savw[age - 25][lcat][ecat];
          totutil = totuw[age - 25][lcat][ecat];

          y = mainwage[age - 25] + pben + sben + othincome[age - 25][vcat];

          #include "GSModel.inc"
          }
        }
      }

    /* calculate consumption, saving, and utility for retirement */

    for (vcat = 0; vcat < 3; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (vcat < 2 && earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        if (pcat == 0 || vcat < 2)
        { if (vcat < 2 && dcat == 0 && earlypenage <= age)
          pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (age - 50));
          else pben = 0;

          for (scat = 0; scat < nscats; scat++)
          { sben = ssbenr[age - 50][vcat][scat];

            for (ecat = 0; ecat < necats; ecat++)
            if (ecat == 0 || vcat < 2)
            { cv = cmatr[vcat][dcat][pcat][scat][ecat];
              tuv = tumatr[vcat][dcat][pcat][scat][ecat];

              margu = margucr[2][vcat][dcat][pcat][scat][ecat];
              totu = totucr[2][vcat][dcat][pcat][scat][ecat];

              saving = savd[age - 50][2][dcat][pcat][scat][ecat];
              totutil = totud[age - 50][2][dcat][pcat][scat][ecat];

              y = pben + sben + othincome[age - 25][vcat];

              #include "GSModel.inc"

              for (acat = 0; acat < nacats; acat++)
              { totu[acat] = totu[acat] + utilr[ecat][age - 50];

                if (vcat == 0)
                totutil[acat] = totutil[acat] + utilr[ecat][age - 50];
                }
              }
            }
          }
        }
      }

    /* calculate consumption, saving, and utility for partial retirement */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        { if (dcat == 0 && earlypenage <= age)
          pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (age - 50));
          else pben = 0;

          for (scat = 0; scat < nscats; scat++)
          { if (sserage <= age && age < 70)
            { benchange = benchangep[age - 62][vcat][scat];

              if (benchange > 0)
              { sbenf = ssbenf[scat] + benchangep[age - 62][vcat][scat];

                fs = -1;

                for (scati = scat + 1; scati < nscats; scati++)
                { if (fs == -1 && sbenf < ssbenf[scati])
                  fs = (scati - 1) + (sbenf - ssbenf[scati - 1])
                      / (ssbenf[scati] - ssbenf[scati - 1]);
                  }

                if (fs == -1)
                fs = (scati - 2) + 0.99;

                scati = (int) fs;
                fs = fs - scati;
                }
              else
              { scati = scat;
                fs = 0;
                }
              }
            else
            { scati = scat;
              fs = 0;
              }

            sben = ssbenp[age - 50][vcat][scat];

            for (ecat = 0; ecat < necats; ecat++)
            { if (fs != 0)
              { cv = cvbuf;
                tuv = tuvbuf;

                for (acat = 0; acat < nacats; acat++)
                { cv[acat] = (1 - fs) * cmatr[vcat][dcat][pcat][scati    ][ecat][acat]
                            +  fs     * cmatr[vcat][dcat][pcat][scati + 1][ecat][acat];
                  tuv[acat] = (1 - fs) * tumatr[vcat][dcat][pcat][scati    ][ecat][acat]
                             +  fs     * tumatr[vcat][dcat][pcat][scati + 1][ecat][acat];
                  }
                }
              else
              { cv = cmatr[vcat][dcat][pcat][scati][ecat];
                tuv = tumatr[vcat][dcat][pcat][scati][ecat];
                }

              margu = margucr[1][vcat][dcat][pcat][scat][ecat];
              totu = totucr[1][vcat][dcat][pcat][scat][ecat];

              saving = savd[age - 50][1][dcat][pcat][scat][ecat];
              totutil = totud[age - 50][1][dcat][pcat][scat][ecat];

              y = prwage[age - 50] + pben + sben + othincome[age - 25][vcat];

              #include "GSModel.inc"

              for (acat = 0; acat < nacats; acat++)
              { totu[acat] = totu[acat] + tcats[tcat][age - 50] * utilr[ecat][age - 50];

                if (vcat == 0)
                totutil[acat] = totutil[acat] + tcats[tcat][age - 50] * utilr[ecat][age - 50];
                }
              }
            }
          }
        }
      }


    /* calculate consumption, saving, and utility for return to work */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        { if (dcat == 0 && earlypenage <= age)
          pben = pcats[pcat] * exp(- inflrate * (1 - penadjrate) * (age - 50));
          else pben = 0;

          for (scat = 0; scat < nscats; scat++)
          { if (sserage <= age && age < 70)
            { benchange = benchanges[age - 62][vcat][scat];

              if (benchange > 0)
              { sbenf = ssbenf[scat] + benchange;

                fs = -1;

                for (scati = scat + 1; scati < nscats; scati++)
                { if (fs == -1 && sbenf < ssbenf[scati])
                  fs = (scati - 1) + (sbenf - ssbenf[scati - 1])
                        / (ssbenf[scati] - ssbenf[scati - 1]);
                  }

                if (fs == -1)
                fs = (scati - 2) + 0.99;

                scati = (int) fs;
                fs = fs - scati;
                }
              else
              { scati = scat;
                fs = 0;
                }
              }
            else
            { scati = scat;
              fs = 0;
              }

            sben = ssbens[age - 50][vcat][scat];

            for (ecat = 0; ecat < necats; ecat++)
            { if (fs != 0)
              { cv = cvbuf;
                tuv = tuvbuf;

                for (acat = 0; acat < nacats; acat++)
                { cv[acat] = (1 - fs) * cmatr[vcat][dcat][pcat][scati    ][ecat][acat]
                            +  fs     * cmatr[vcat][dcat][pcat][scati + 1][ecat][acat];
                  tuv[acat] = (1 - fs) * tumatr[vcat][dcat][pcat][scati    ][ecat][acat]
                             +  fs     * tumatr[vcat][dcat][pcat][scati + 1][ecat][acat];
                  }
                }
              else
              { cv = cmatr[vcat][dcat][pcat][scati][ecat];
                tuv = tumatr[vcat][dcat][pcat][scati][ecat];
                }

              margu = margucr[0][vcat][dcat][pcat][scat][ecat];
              totu = totucr[0][vcat][dcat][pcat][scat][ecat];

              saving = savd[age - 50][0][dcat][pcat][scat][ecat];
              totutil = totud[age - 50][0][dcat][pcat][scat][ecat];

              y = secwage[age - 50] + pben + sben + othincome[age - 25][vcat];

              #include "GSModel.inc"
              }
            }
          }
        }
      }

    /* choose between work and retirement if left main job previously */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (dcat = 0; dcat < 2; dcat++)
      if (dcat == 0 || (earlypenage <= age && age < delayedpenage))
      { for (pcat = 0; pcat < npcats; pcat++)
        for (scat = 0; scat < nscats; scat++)
        for (ecat = 0; ecat < necats; ecat++)
        for (acat = 0; acat < nacats; acat++)
        { for (job = 0; job < 3; job++)
          { muj[job] = margucr[job][vcat][dcat][pcat][scat][ecat][acat];
            tuj[job] = totucr[job][vcat][dcat][pcat][scat][ecat][acat];
            }

          jobmax = -1;
          tumax = 0;

          for (job = 0; job < 3; job++)
          if (jobmax == -1 || tuj[job] > tumax)
          { tumax = tuj[job];
            jobmax = job;
            }

          margufr[vcat][dcat][pcat][scat][ecat][acat] = muj[jobmax];
          totufr[vcat][dcat][pcat][scat][ecat][acat] = tuj[jobmax];

          if (vcat == 0)
          retchoice[age - 50][dcat][pcat][scat][ecat][acat] = jobmax + 1;
          }
        }

      if (earlypenage < age && age == delayedpenage)
      { memcpy(margufr[vcat][1], margufr[vcat][0], sizeof(double[npcatlim][nscatlim][necatlim][nacatlim]));
        memcpy(totufr[vcat][1], totufr[vcat][0], sizeof(double[npcatlim][nscatlim][necatlim][nacatlim]));
        }
      else if (earlypenage == age && age < delayedpenage)
      { memcpy(margufr[vcat][0], margufr[vcat][1], sizeof(double[npcatlim][nscatlim][necatlim][nacatlim]));
        memcpy(totufr[vcat][0], totufr[vcat][1], sizeof(double[npcatlim][nscatlim][necatlim][nacatlim]));
        }
      }

    if (age > 62)                                 /* vcat = 2 */
    { if (sserage <= agew && agew < 70)
      { for (scat = 0; scat < nscats; scat++)
        for (acat = 0; acat < nacats; acat++)
        { margufr[2][0][0][scat][0][acat] = margucr[2][2][0][0][scat][0][acat];
          totufr[2][0][0][scat][0][acat] = totucr[2][2][0][0][scat][0][acat];
          }
        }
      }

    /* chooose between work and retirement if previously in main job */

    pcat = (int) initialpcat[age - 50];
    scat = (int) initialscat[age - 50];

    fp = initialpcat[age - 50] - pcat;
    fs = initialscat[age - 50] - scat;

    eowamt = eowamount[age - 50];

    for (lcat = 0; lcat < nlcats; lcat++)
    { if (lcat > 0 && penjobend + 1 == age)
      { for (acat = 0; acat < nacats; acat++)
        { pwealth = acats[acat] + lcats[lcat];

          if (acat < nacats - 1)
          lacat = acat + 1;
          else lacat = nacats - 1;

          while (lacat < nacats - 1 && pwealth > acats[lacat])
          lacat = lacat + 1;

          lacat = lacat - 1;

          lacatsw[lcat][acat] = lacat + (pwealth - acats[lacat])
                               / (acats[lacat + 1] - acats[lacat]);
          }
        }
      else if (lcat == 0 || (penjobend <= age && age < contribstartage))
      { for (acat = 0; acat < nacats; acat++)
        lacatsw[lcat][acat] = acat;
        }
      }
         
    for (lcat = 0; lcat < nlcats; lcat++)
    { if ((lcat > 0 && penjobend + 1 >= age && age > contribstartage)
       || (lcat == 0 && eowamt > 0))
      { for (acat = 0; acat < nacats; acat++)
        { pwealth = acats[acat] + lcats[lcat] + eowamt;

          if (acat < nacats - 1)
          lacat = acat + 1;
          else lacat = nacats - 1;

          while (lacat < nacats - 1 && pwealth > acats[lacat])
          lacat = lacat + 1;

          lacat = lacat - 1;

          lacatsr[lcat][acat] = lacat + (pwealth - acats[lacat]) / (acats[lacat + 1] - acats[lacat]);
          }
        }
      else if (lcat == 0)
      { for (acat = 0; acat < nacats; acat++)
        lacatsr[lcat][acat] = acat;
        }
      }

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();
      
      for (lcat = 0; lcat < nlcats; lcat++)
      if (lcat == 0 || (penjobend + 1 >= age && age > contribstartage))
      { for (ecat = 0; ecat < necats; ecat++)
        for (acat = 0; acat < nacats; acat++)
        { lacat = (int) lacatsw[lcat][acat];

          if (lacat > nacats - 2)
          lacat = nacats - 2;

          fla = lacatsw[lcat][acat] - lacat;

          if (penjobend + 1 == age)
          { if (fla > 0)
            { mu0 = margucw[vcat][0][ecat][lacat    ];
              mu1 = margucw[vcat][0][ecat][lacat + 1];
              tu0 = totucw[vcat][0][ecat][lacat    ];
              tu1 = totucw[vcat][0][ecat][lacat + 1];

              if (fla <= 1)
              { muj[0] = (1 - fla) * mu0 + fla * mu1;
                tuj[0] = (1 - fla) * tu0 + fla * tu1;
                }
              else
              { if (mu0 > mu1 && tu1 > tu0)
                { dmu = mu0 - mu1;
                  dtu = tu1 - tu0;
                  muratio = mu0 / mu1;
                  mufact = dmu / mu0;

                  muj[0] = mu1 / (1 + (fla - 1) * mufact);
                  tuj[0] = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                  }
                else
                { muj[0] = mu1;

                  if (tu1 > tu0)
                  tuj[0] = (1 - fla ) * tu0 + fla * tu1;
                  else tuj[0] = tu1;
                  }
                }
              }
            else
            { muj[0] = margucw[vcat][0][ecat][lacat];
              tuj[0] = totucw[vcat][0][ecat][lacat];
              }
            }
          else
          { muj[0] = margucw[vcat][lcat][ecat][acat];
            tuj[0] = totucw[vcat][lcat][ecat][acat];
            }

          lacat = (int) lacatsr[lcat][acat];

          if (lacat > nacats - 2)
          lacat = nacats - 2;

          fla = lacatsr[lcat][acat] - lacat;

          for (choice = 1; choice <= 3; choice++)
          if (vcat < 2 || choice == 3)
          { ucrm = margucr[choice - 1][vcat][0];
            ucrt = totucr[choice - 1][vcat][0];

            for (utype = 0; utype <= 1; utype++)
            { for (pcati = pcat; pcati <= pcat + 1; pcati++)
              if (pcati == pcat || fp != 0)
              { for (scati = scat; scati <= scat + 1; scati++)
                if (scati == scat || fs != 0)
                { if (fla != 0)
                  { if (utype == 0)
                    { mu0 = ucrm[pcati][scati][ecat][lacat    ];
                      mu1 = ucrm[pcati][scati][ecat][lacat + 1];

                      if (fla <= 1)
                      temp = (1 - fla) * mu0 + fla * mu1;
                      else if (mu0 > mu1)
                      { dmu = mu0 - mu1;
                        mufact = dmu / mu0;

                        temp = mu1 / (1 + (fla - 1) * mufact);
                        }
                      else temp = mu1;
                      }
                    else
                    { tu0 = ucrt[pcati][scati][ecat][lacat    ];
                      tu1 = ucrt[pcati][scati][ecat][lacat + 1];

                      if (fla <= 1)
                      temp = (1 - fla) * tu0 + fla * tu1;
                      else
                      { mu0 = ucrm[pcati][scati][ecat][lacat    ];
                        mu1 = ucrm[pcati][scati][ecat][lacat + 1];

                        if (mu0 > mu1 && tu1 > tu0)
                        { dmu = mu0 - mu1;
                          dtu = tu1 - tu0;
                          muratio = mu0 / mu1;
                          mufact = dmu / mu0;

                          temp = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                          }
                        else if (tu1 > tu0)
                        temp = (1 - fla) * tu0 + fla * tu1;
                        else temp = tu1;
                        }
                      }
                    }
                  else
                  { if (utype == 0)
                    temp = ucrm[pcati][scati][ecat][lacat];
                    else temp = ucrt[pcati][scati][ecat][lacat];
                    }

                  if (scati == scat)
                  temps[0] = temp;
                  else temps[1] = temp;
                  }

                if (fs != 0)
                temp = (1 - fs) * temps[0] + fs * temps[1];
                else temp = temps[0];

                if (pcati == pcat)
                tempp[0] = temp;
                else tempp[1] = temp;
                }

              if (fp != 0)
              util = (1 - fp) * tempp[0] + fp * tempp[1];
              else util = tempp[0];

              if (utype == 0)
              muj[choice] = util;
              else tuj[choice] = util;
              }
            }


          jobmax = -1;
          tumax = 0;

          for (job = 0; job < 4; job++)
          if (jobmax == -1 || tuj[job] > tumax)
          { tumax = tuj[job];
            jobmax = job;
            }

          margufw[vcat][lcat][ecat][acat] = muj[jobmax];
          totufw[vcat][lcat][ecat][acat] = tuj[jobmax];

          if (vcat == 0)
          mjchoice[age - 50][lcat][ecat][acat] = jobmax;
          }
        }
      }
    }

  /* calculate consumption and saving for age 25 to firstage - 1 */

  cv = cvbuf;
  tuv = tuvbuf;

  for (age = firstage - 1; age >= 25; age--)
  { /* expectations with regard to returns, husband living */

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (ecat = 0; ecat < necats; ecat++)
      { for (lcat = 0; lcat < nlcats; lcat++)
        if (lcat == 0 || (penjobend >= age && age > contribstartage))
        { for (acat = 0; acat < nacats; acat++)
          { mu = 0;
            tu = 0;

            for (rcat = 0; rcat < nrcats; rcat++)
            { acati = (int) rtransa[acat][rcat];

              if (acati >= nacats - 1)
              acati = nacats - 2;

              fa = rtransa[acat][rcat] - acati;

              if (nlcats > 1 && penjobend >= age && age > contribstartage)
              { lcati = (int) rtransl[age - 25][lcat][rcat];

                if (lcati >= nlcats - 1)
                lcati = nlcats - 2;

                fl = rtransl[age - 25][lcat][rcat] - lcati;

                if (fa <= 1 && fl <= 1)
                { mur = (1 - fl) * ((1 - fa) * margufw[vcat][lcati    ][ecat][acati]
                                  +   fa     * margufw[vcat][lcati    ][ecat][acati + 1])
                       +  fl     * ((1 - fa) * margufw[vcat][lcati + 1][ecat][acati]
                                  +   fa     * margufw[vcat][lcati + 1][ecat][acati + 1]);

                  tur = (1 - fl) * ((1 - fa) * totufw[vcat][lcati    ][ecat][acati]
                                  +   fa     * totufw[vcat][lcati    ][ecat][acati + 1])
                       +  fl     * ((1 - fa) * totufw[vcat][lcati + 1][ecat][acati]
                                  +   fa     * totufw[vcat][lcati + 1][ecat][acati + 1]);

                  if (margufw[vcat][lcati    ][ecat][acati    ] <= 0 || margufw[vcat][lcati    ][ecat][acati + 1] <= 0
                   || margufw[vcat][lcati + 1][ecat][acati    ] <= 0 || margufw[vcat][lcati + 1][ecat][acati + 1] <= 0)
                  { wnprintf("\nBad mu in work section-living: %5d %2d %14.6e %14.6e %14.6e %14.6e",
                      obs, age, margufw[vcat][lcati    ][ecat][acati    ], margufw[vcat][lcati    ][ecat][acati + 1],
                                margufw[vcat][lcati + 1][ecat][acati    ], margufw[vcat][lcati + 1][ecat][acati + 1]);
                    wnprintf(" %2d %2d %2d %2d %2d %2d %2d", vcat, lcat, lcati, ecat, acat, acati, rcat);
                    pause();
                    }

                  }
                else
                { if (rcat >= 2)
                  { mu0 = mular[lcat][acat][rcat - 2];
                    mu1 = mular[lcat][acat][rcat - 1];
                    tu0 = tular[lcat][acat][rcat - 2];
                    tu1 = tular[lcat][acat][rcat - 1];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      mur = mu1 / (1 + mufact);
                      tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                      }
                    else
                    { mur = mu1;

                      if (tu1 > tu0)
                      tur = 2 * tu1 - tu0;
                      else tur = tu1;
                      }
                    }
                  else if (lcat > nlcats - 5)
                  { mu0 = mular[lcat - 2][acat][rcat];
                    mu1 = mular[lcat - 1][acat][rcat];
                    tu0 = tular[lcat - 2][acat][rcat];
                    tu1 = tular[lcat - 1][acat][rcat];

                    if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      mur = mu1 / (1 + (fl - 1) * mufact);
                      tur = tu1 + (dtu / log(muratio)) * log(1 + (fl - 1) * mufact);
                      }
                    else
                    { mur = mu1;

                      if (tu1 > tu0)
                      tur = (1 - fl) * tu0 + fl * tu1;
                      else tur = tu1;
                      }
                    }
                  else
                  { wnprintf("\nfl problem: age %d rcat %d lcat %d", age, rcat, lcat);
                    pause();
                    }

                  if (rcat > 0)
                  { if (mur > mular[lcat][acat][rcat - 1])
                    mur = mular[lcat][acat][rcat - 1];

                    if (tur < tular[lcat][acat][rcat - 1])
                    tur = tular[lcat][acat][rcat - 1];
                    }

                  if (lcat > 0)
                  { if (mur > mular[lcat - 1][acat][rcat])
                    mur = mular[lcat - 1][acat][rcat];

                    if (tur < tular[lcat - 1][acat][rcat])
                    tur = tular[lcat - 1][acat][rcat];
                    }
                  }

                mular[lcat][acat][rcat] = mur;
                tular[lcat][acat][rcat] = tur;
                }
              else
              { if (fa <= 1)
                { mur = (1 - fa) * margufw[vcat][0][ecat][acati]
                       +  fa     * margufw[vcat][0][ecat][acati + 1];

                  tur = (1 - fa) * totufw[vcat][0][ecat][acati]
                       +  fa     * totufw[vcat][0][ecat][acati + 1];
                  }
                else
                { mu0 = mular[0][acat][rcat - 2];
                  mu1 = mular[0][acat][rcat - 1];
                  tu0 = tular[0][acat][rcat - 2];
                  tu1 = tular[0][acat][rcat - 1];

                  if (mu0 > mu1 && tu1 > tu0)
                  { dmu = mu0 - mu1;
                    dtu = tu1 - tu0;
                    muratio = mu0 / mu1;
                    mufact = dmu / mu0;

                    mur = mu1 / (1 + mufact);
                    tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                    }
                  else
                  { mur = mu1;

                    if (tu1 > tu0)
                    tur = 2 * tu1 - tu0;
                    else tur = tu1;
                    }
                  }
                }

              mu = mu + rprob[rcat] * mur;
              tu = tu + rprob[rcat] * tur;
              }

            mula[lcat][acat] = mu;
            tula[lcat][acat] = tu;
            }
          }

        for (lcat = 0; lcat < nlcats; lcat++)
        if (lcat == 0 || (penjobend >= age && age > contribstartage))
        { for (acat = 0; acat < nacats; acat++)
          { margufw[vcat][lcat][ecat][acat] = mula[lcat][acat];
            totufw[vcat][lcat][ecat][acat] = tula[lcat][acat];
            }
          }
        }
      }

    /* expectations with respect to returns, husband deceased */

    if (age > 62)
    { for (scat = 0; scat < nscats; scat++)
      { for (acat = 0; acat < nacats; acat++)
        { mu = 0;
          tu = 0;

          for (rcat = 0; rcat < nrcats; rcat++)
          { acati = (int) rtransa[acat][rcat];

            if (acati >= nacats - 1)
            acati = nacats - 2;

            fa = rtransa[acat][rcat] - acati;

            mu0 = margufr[2][0][0][scat][0][acati    ];
            mu1 = margufr[2][0][0][scat][0][acati + 1];
            tu0 = totufr[2][0][0][scat][0][acati    ];
            tu1 = totufr[2][0][0][scat][0][acati + 1];

            if (mu0 <= 0 || mu1 <= 0)
            { wnprintf("\nBad mu in work section-deceased: %2d %14.6e %14.6e %2d %2d %2d %2d",
               age, mu0, mu1, scat, acat, acati, rcat);
              pause();
              }

            if (fa <= 1)
            { mu = mu + rprob[rcat] * ((1 - fa) * mu0 + fa * mu1);
              tu = tu + rprob[rcat] * ((1 - fa) * tu0 + fa * mu1);
              }
            else
            { if (mu0 > mu1 && tu1 > tu0)
              { dmu = mu0 - mu1;
                dtu = tu1 - tu0;
                muratio = mu0 / mu1;
                mufact = dmu / mu0;

                mu = mu + rprob[rcat] * mu1 / (1 + (fa - 1) * mufact);
                tu = tu + rprob[rcat] * (tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact));
                }
              else
              { mu = mu + rprob[rcat] * mu1;

                if (tu1 > tu0)
                tu = tu + rprob[rcat] * ((1 - fa) * tu0 + fa * tu1);
                else tu = tu + rprob[rcat] * tu1;
                }
              }
            }

          margu[acat] = mu;
          totu[acat] = tu;
          }

        for (acat = 0; acat < nacats; acat++)
        { margufr[2][0][0][scat][0][acat] = margu[acat];
          totufr[2][0][0][scat][0][acat] = totu[acat];
          }
        }
      }

    /* expectations with respect to survival, husband living */

    if (age >= 62)                              /* vcat = 0 */
    { scat = (int) initialscat[age - 50];
      fs = initialscat[age - 50] - scat;
    
      for (lcat = 0; lcat < nlcats; lcat++)
      if (lcat == 0 || (penjobend >= age && age > contribstartage))
      { for (ecat = 0; ecat < necats; ecat++)
        for (acat = 0; acat < nacats; acat++)
        { wealth = acats[acat] + lcats[lcat];

          if (acat < nacats - 1)
          acati = acat + 1;
          else acati = nacats - 1;

          while (acati < nacats - 1 && wealth > acats[acati])
          acati = acati + 1;

          acati = acati - 1;

          fa = (wealth - acats[acati]) / (acats[acati + 1] - acats[acati]);

          margufw0 = margufw[0][lcat][ecat][acat];
          totufw0 = totufw[0][lcat][ecat][acat];

          margufw1 = margufw[1][lcat][ecat][acat];
          totufw1 = totufw[1][lcat][ecat][acat];

          mu0 = (1 - fs) * margufr[2][0][0][scat    ][0][acati    ]
               +  fs     * margufr[2][0][0][scat + 1][0][acati    ];
          mu1 = (1 - fs) * margufr[2][0][0][scat    ][0][acati + 1]
               +  fs     * margufr[2][0][0][scat + 1][0][acati + 1];
          tu0 = (1 - fs) * totufr[2][0][0][scat    ][0][acati    ]
               +  fs     * totufr[2][0][0][scat + 1][0][acati    ];
          tu1 = (1 - fs) * totufr[2][0][0][scat    ][0][acati + 1]
               +  fs     * totufr[2][0][0][scat + 1][0][acati + 1];

          if (fa <= 1)
          { margufr2 = (1 - fa) * mu0 + fa * mu1;
            totufr2 = (1 - fa) * tu0 + fa * tu1;
            }
          else
          { if (mu0 > mu1 && tu1 > tu0)
            { dmu = mu0 - mu1;
              dtu = tu1 - tu0;
              muratio = mu0 / mu1;
              mufact = dmu / mu0;

              margufr2 = mu1 / (1 + (fa - 1) * mufact);
              totufr2 = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
              }
            else
            { margufr2 = mu1;

              if (tu1 > tu0)
              totufr2 = (1 - fa) * tu0 + fa * tu1;
              else totufr2 = tu1;
              }
            }

          margufw[0][lcat][ecat][acat]
            =  transrate[0][age - 25] * margufw0
             + transrate[1][age - 25] * margufw1
             + transrate[2][age - 25] * margufr2;

          totufw[0][lcat][ecat][acat]
            =  transrate[0][age - 25] * totufw0
             + transrate[1][age - 25] * totufw1
             + transrate[2][age - 25] * totufr2;
          }
        }
      }

    /* current consumption and saving, husband living */

    cv = cvbuf;
    tuv = tuvbuf;

    for (vcat = 0; vcat < 2; vcat++)
    if (vcat == 0 || age > 62)
    { checkMessages();

      for (lcat = 0; lcat < nlcats; lcat++)
      if (lcat == 0 || (penjobend >= age && age > contribstartage))
      { checkMessages();

        for (ecat = 0; ecat < necats; ecat++)
        { for (acat = 0; acat < nacats; acat++)
          { mu = margufw[vcat][lcat][ecat][acat];
            tu = totufw[vcat][lcat][ecat][acat];

            if (mu <= 0)
            { wnprintf("\nBad mu for work section-living consumption: %5d %2d %14.6e %2d %2d %2d %2d",
               obs, age, mu, vcat, lcat, ecat, acat);
              wnprintf("\n fa %d", firstage);
              pause();
              }

            if (vcat == 0)
            { cv[acat] = sqrt2 * pow((mu / tp), 1 / (alpha - 1));
              tuv[acat] = tu / tp;
              }
            else
            { cv[acat] = pow(mu * srate[vcat - 1][age - 25] / tp, 1 / (alpha - 1));
              tuv[acat] = tu * srate[vcat - 1][age - 25] / tp;
              }
            }

          sben = ssbenm[age - 25][vcat];

          margu = margufw[vcat][lcat][ecat];
          totu = totufw[vcat][lcat][ecat];

          saving = savw[age - 25][lcat][ecat];
          totutil = totuw[age - 25][lcat][ecat];

          y = mainwage[age - 25] + sben + othincome[age - 25][vcat];

          #include "GSModel.inc"

          /* at end of penjob, transfer dc assets to unrestricted assets */

          if (penjobend + 1 == age)
          { for (lcati = 1; lcati < nlcats; lcati++)
            { margu = margufw[vcat][lcati][ecat];
              totu = totufw[vcat][lcati][ecat];

              for (acat = 0; acat < nacats; acat++)
              { pwealth = acats[acat] + lcats[lcat];

                if (acat < nacats - 1)
                lacat = acat + 1;
                else lacat = nacats - 1;

                while (lacat < nacats - 1 && pwealth > acats[lacat])
                lacat = lacat + 1;

                lacat = lacat - 1;

                fla = (pwealth - acats[lacat]) / (acats[lacat + 1] - acats[lacat]);

                if (fla == 0)
                { margu[acat] = margufw[vcat][0][ecat][lacat];
                  totu[acat] = totufw[vcat][0][ecat][lacat];
                  }
                else
                { mu0 = margufw[vcat][0][ecat][lacat    ];
                  mu1 = margufw[vcat][0][ecat][lacat + 1];
                  tu0 = totufw[vcat][0][ecat][lacat    ];
                  tu1 = totufw[vcat][0][ecat][lacat + 1];

                  if (fla <= 1)
                  { margu[acat] = (1 - fla) * mu0 + fla * mu1;
                    totu[acat] = (1 - fla) * tu0 + fla * tu1;
                    }
                  else
                  { if (mu0 > mu1 && tu1 > tu0)
                    { dmu = mu0 - mu1;
                      dtu = tu1 - tu0;
                      muratio = mu0 / mu1;
                      mufact = dmu / mu0;

                      margu[acat] = mu1 / (1 + (fla - 1) * mufact);
                      totu[acat] = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                      }
                    else
                    { margu[acat] = mu1;

                      if (tu1 > tu0)
                      totu[acat] = (1 - fla) * tu0 + fla * tu1;
                      else totu[acat] = tu1;
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }

    /* current consumption and saving, husband deceased */

    vcat = 2;

    if (age > 62)
    { for (scat = 0; scat < nscats; scat++)
      { for (acat = 0; acat < nacats; acat++)
        { mu = margufr[2][0][0][scat][0][acat];
          tu = totufr[2][0][0][scat][0][acat];

          if (mu <= 0)
          { wnprintf("\nBad mu for work section-deceased consumption: %2d %14.6e %14.6e %2d %2d",
             age, mu, scat, acat);
            pause();
            }

          cv[acat] = pow(mu * srate[1][age - 25] / tp, 1 / (alpha - 1));
          tuv[acat] = tu * srate[vcat - 1][age - 25] / tp;
          }

        margu = margufr[2][0][0][scat][0];
        totu = totufr[2][0][0][scat][0];

        y = ssbenr[age - 50][2][scat] + othincome[age - 25][vcat];

        #include "GSModel.inc"
        }
      }
    }

  }


/*---------------------------------------------------------------------------
			 GET MOMENTS routine
			 simulates and calculates moments for the observation
  ---------------------------------------------------------------------------*/

void getMoments(
  int    threadno,            /* thread number */
  int    tcat)                /* utility of partial retirement category */

{ int    k, l, l1, l2;
  int    obs, mode, survey;
  int    simcount, nsims, llim, ulim;
  int    firstage, lastage;
  int    sserage, nrage, nragew;
  int    year, birthyear;
  int    age, age0, age1;
  int    ret, ret0, ret1;
  int    delayedpen, earlypenage, delayedpenage;
  int    contribstartage, penjobend;
  int    nacats, necats, nlcats, npcats, nscats, ntcats;
  int    acat, dcat, ecat, lcat, lacat, pcat, scat;
  int    acati, ecati, lcati, pcati, scati;
  int    job, mainjob, priorret;
  int    choice, choicecount;
  int    epscat, epscatlim;
  int    simeverreturnft, simeverreturnpt, simeverreturn;
  double erho;
  double eps, epsilon, epschange;
  double fa, fe, fl, fla, fp, fs, m1, m2;
  double changesdev, saving, wealth, dcwealth, rgmean, eowamt;
  double fepscat, fecat, fscat;
  double temp, pwealth, sbenf, totutil, totalutil;

  int    retv[21], jobchoice[4];
  double templ[2], tempp[2], temps[2];
  double epsilonvec[10 * necatlim + 1];
  double wealthvec[10 * necatlim + 1], dcwealthvec[10 * necatlim + 1];

  char   *mobs;
  double *mvec, *wmat;
  double *cumret0mat, *cumret1mat, *cumret2mat;
  double *cumreturnft, *cumreturnpt, *cumnewpt;
  double *everreturnft, *everreturnpt, *everreturn;
  double *cumeverreturnft, *cumeverreturnpt, *cumeverreturn;
  double *acats, *ecats, *lcats;
  double *initialpcat, *initialscat, *ssbenf;
  double *savvec;
  double *ret0vec, *ret1vec, *ret2vec, *reversal;
  double *returnft, *returnpt, *newpt;
  double *realrate, *dccontrib, *eowamount;
  double (*tuea)[nacatlim];
  double (*savlea)[necatlim][nacatlim];
  double (*savea)[nacatlim];
  double (*benchanges)[2][nscatlim];
  double (*benchangep)[2][nscatlim];
  double (*totuw)[nlcatlim][necatlim][nacatlim];
  double (*totud)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*savw)[nlcatlim][necatlim][nacatlim];
  double (*savd)[3][2][npcatlim][nscatlim][necatlim][nacatlim];
  double (*savr)[2][npcatlim][nscatlim][nacatlim];

  int    (*mjchoice)[nlcatlim][necatlim][nacatlim];
  int    (*retchoice)[2][npcatlim][nscatlim][necatlim][nacatlim];

  DATA *data;
  CALCSTRUCT *calcStruct;
  MOMENTSTRUCT moments;
  MOMENTSTRUCT *momentStruct;
  double *momentvec;

  calcStruct = &calcStructv[threadno - localthreadmin];
  obs = calcStruct->obs;
  data = &datav[obs];

  erho = calcStruct->erho;

  firstage = calcStruct->firstage;
  lastage  = calcStruct->lastage;

  rgmean = calcStruct->realrateg;
  realrate = calcStruct->realrate;

  earlypenage     = calcStruct->earlypenage;
  delayedpenage   = calcStruct->delayedpenage;
  contribstartage = calcStruct->contribstartage;
  penjobend       = calcStruct->penjobend;
  sserage         = calcStruct->sserage;
  nrage           = calcStruct->nrage;
  nragew          = calcStruct->spnrage;
  eowamount       = calcStruct->eowamount;

  nacats = calcStruct->nacats;
  necats = calcStruct->necats;
  nlcats = calcStruct->nlcats;
  npcats = calcStruct->npcats;
  nscats = calcStruct->nscats;
  ntcats = calcStruct->ntcats;

  acats = calcStruct->acats;
  ecats = calcStruct->ecats;
  lcats = calcStruct->lcats;

  totuw = calcStruct->totuw;
  totud = calcStruct->totud;

  savw = calcStruct->savw;
  savd = calcStruct->savd;
  savr = calcStruct->savr;

  mjchoice  = calcStruct->mjchoice;
  retchoice = calcStruct->retchoice;

  initialpcat = calcStruct->initialpcat;
  initialscat = calcStruct->initialscat;
  dccontrib = calcStruct->dccontrib;

  ssbenf = calcStruct->ssbenf;

  benchanges  = calcStruct->benchanges;
  benchangep  = calcStruct->benchangep;

  ret0vec      = calcStruct->ret0vec;
  ret1vec      = calcStruct->ret1vec;
  ret2vec      = calcStruct->ret2vec;
  returnft     = calcStruct->returnft;
  returnpt     = calcStruct->returnpt;
  newpt        = calcStruct->newpt;

  everreturnft = &(calcStruct->everreturnft);
  everreturnpt = &(calcStruct->everreturnpt);
  everreturn   = &(calcStruct->everreturn);

  reversal = calcStruct->reversal;

  mode = calcStruct->mode;

  if (mode == 1 || mode == 2)
  { mobs     = arglist.nmat[obs];             /* indicator of observed moments */
    mvec     = arglist.mvec[threadno];        /* cumulative momemt vector */
    wmat     = arglist.wmat[threadno];        /* cumulative w matrix */
    }

  if (mode != 2)
  { cumret0mat      = arglist.ret0mat[threadno];
    cumret1mat      = arglist.ret1mat[threadno];
    cumret2mat      = arglist.ret2mat[threadno];
    cumreturnft     = arglist.returnft[threadno];
    cumreturnpt     = arglist.returnpt[threadno];
    cumnewpt        = arglist.newpt[threadno];

    cumeverreturnft = &(arglist.everreturnft[threadno]);
    cumeverreturnpt = &(arglist.everreturnpt[threadno]);
    cumeverreturn   = &(arglist.everreturn[threadno]);
    }

  nsims = 10000;
  wealth = 0;


  /* do calculations of wealth at firstage as functions of epsilon */

  birthyear = data->birthyear;

  epscatlim = 10 * (necats - 1);

  for (epscat = 0; epscat <= epscatlim; epscat++)
  { eps = ((epscatlim - epscat) * ecats[0] + epscat * ecats[necats - 1]) / epscatlim;

    ecat = epscat / 10;
    fe = (double) (epscat % 10) / 10.0;

    wealth = 0;
    dcwealth = 0;

    for (age = 25; age < firstage; age++)
    { year = age + birthyear;

      if (1926 < year && year <= 2001)
      { wealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * wealth;
        dcwealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * dcwealth;
        }
      else
      { wealth = (1 + rgmean) * wealth;
        dcwealth = (1 + rgmean) * dcwealth;
        }

      acat = 0;

      while (acat < nacats - 1 && wealth > acats[acat])
      acat = acat + 1;

      if (acat > 0)
      acat = acat - 1;

      fa = (wealth - acats[acat]) / (acats[acat + 1] - acats[acat]);

      if (nlcats > 1 && penjobend >= age && age > contribstartage)
      { lcat = 0;

        while (lcat < nlcats - 1 && dcwealth > lcats[lcat])
        lcat = lcat + 1;

        if (lcat > 0)
        lcat = lcat - 1;

        fl = (dcwealth - lcats[lcat]) / (lcats[lcat + 1] - lcats[lcat]);
        }
      else
      { lcat = 0;
        fl = 0;
        }

      /* choose savings level */

      for (lcati = lcat; lcati <= lcat + 1; lcati++)
      if (lcati == lcat || fl != 0)
      { if (fe != 0)
        temp  = (1 - fe) * ((1 - fa) * savw[age - 25][lcati][ecat    ][acat    ]
                           +  fa     * savw[age - 25][lcati][ecat    ][acat + 1])
               +  fe     * ((1 - fa) * savw[age - 25][lcati][ecat + 1][acat    ]
                           +  fa     * savw[age - 25][lcati][ecat + 1][acat + 1]);
        else temp =         (1 - fa) * savw[age - 25][lcati][ecat    ][acat    ]
                           +  fa     * savw[age - 25][lcati][ecat    ][acat + 1];

        if (lcati == lcat)
        templ[0] = temp;
        else templ[1] = temp;
        }

      if (fl != 0)
      saving = (1 - fl) * templ[0] + fl * templ[1];
      else saving = templ[0];

      if (penjobend == age)
      saving = saving + dcwealth + dccontrib[age - 25];

      wealth = wealth + saving;

      if (penjobend > age && age >= contribstartage)
      dcwealth = dcwealth + dccontrib[age - 25];
      else dcwealth = 0;
      }

    epsilonvec[epscat] = eps;
    wealthvec[epscat] = wealth;
    dcwealthvec[epscat] = dcwealth;
    }

  changesdev = sqrt(1 - erho * erho);

  if (tcat == 0)
  { memset(ret0vec, 0, sizeof(double[21]));
    memset(ret1vec, 0, sizeof(double[21]));
    memset(ret2vec, 0, sizeof(double[21]));
    memset(returnft, 0, sizeof(double[21]));
    memset(returnpt, 0, sizeof(double[21]));
    memset(newpt, 0, sizeof(double[21]));
    memset(reversal, 0, sizeof(double[5]));
    
    *everreturnft = 0;
    *everreturnpt = 0;
    *everreturn   = 0;
    }

  switch(tcat)
  { case  0:  llim = ( 0 * nsims) / 100;    ulim = ( 10 * nsims) / 100;     break;
    case  1:  llim = (10 * nsims) / 100;    ulim = ( 30 * nsims) / 100;     break;
    case  2:  llim = (30 * nsims) / 100;    ulim = ( 50 * nsims) / 100;     break;
    case  3:  llim = (50 * nsims) / 100;    ulim = ( 70 * nsims) / 100;     break;
    case  4:  llim = (70 * nsims) / 100;    ulim = ( 90 * nsims) / 100;     break;
    case  5:  llim = (90 * nsims) / 100;    ulim = (100 * nsims) / 100;     break;
    }

  if (tcat == 0)
  randreset(threadno, obs);

  for (simcount = llim; simcount < ulim; simcount++)
  { if (simcount % 100 == 0)
    checkMessages();

    /* get epsilon and wealth at firstage */

    epsilon = gaussdev(threadno);

    if (epsilon < ecats[0])
    epsilon = ecats[0];
    else if (epsilon > ecats[necats - 1])
    epsilon = ecats[necats - 1];

    if (epsilon < ecats[necats - 1])
    fepscat = 10 * (epsilon - ecats[0]) / (ecats[1] - ecats[0]);
    else fepscat = 10 * (necats - 1);

    epscat = (int) fepscat;
    fe = fepscat - epscat;

    if (fe != 0)
    { wealth   = (1 - fe) * wealthvec[epscat]   + fe * wealthvec[epscat + 1];
      dcwealth = (1 - fe) * dcwealthvec[epscat] + fe * dcwealthvec[epscat + 1];
      }
    else
    { wealth   = wealthvec[epscat];
      dcwealth = dcwealthvec[epscat];
      }

    mainjob = 1;

    acat = 0;
    lcat = 0;

    if (firstage < earlypenage && earlypenage < delayedpenage)
    delayedpen = 1;
    else delayedpen = 0;

    fecat = (epsilon - ecats[0]) / (ecats[1] - ecats[0]);

    if (fecat < necats - 1)
    { ecat = (int) fecat;
      fe = fecat - ecat;
      }
    else
    { ecat = necats - 1;
      fe = 0;
      }

    pcat = (int) initialpcat[firstage - 50];
    scat = (int) initialscat[firstage - 50];

    fp = initialpcat[firstage - 50] - pcat;
    fs = initialscat[firstage - 50] - scat;

    for (age = firstage; age <= lastage; age++)
    { year = age + birthyear;

      if (1926 < year && year <= 2001)
      wealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * wealth;
      else wealth = (1 + rgmean) * wealth;

      if (mainjob == 1)
      { if (1926 < year && year <= 2001)
        dcwealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * dcwealth;
        else dcwealth = (1 + rgmean) * dcwealth;
        }

      if (delayedpen == 1 && earlypenage <= age && age < delayedpenage)
      dcat = 1;
      else dcat = 0;

      /* get categories and offsets for eps, wealth, and dcwealth */

      if (mainjob == 0)
      { fecat = (epsilon - ecats[0]) / (ecats[1] - ecats[0]);

        ecat = (int) fecat;
        fe = fecat - ecat;
        }

      if (wealth > acats[acat])
      { acat = acat + 1;

        while (acat < nacats - 1 && wealth > acats[acat])
        acat = acat + 1;

        if (acat > 0)
        acat = acat - 1;
        }
      else
      { while (acat > 0 && wealth < acats[acat])
        acat = acat - 1;
        }

      fa = (wealth - acats[acat]) / (acats[acat + 1] - acats[acat]);

      eowamt = eowamount[age - 50];

      if (mainjob == 1 && (nlcats > 1 && penjobend + 1 > age && age > contribstartage))
      { if (dcwealth > lcats[lcat])
        { lcat = lcat + 1;

          while (lcat < nlcats - 1 && dcwealth > lcats[lcat])
          lcat = lcat + 1;

          if (lcat > 0)
          lcat = lcat - 1;
          }
        else
        { while (lcat > 0 && dcwealth < lcats[lcat])
          lcat = lcat - 1;
          }

        fl = (dcwealth - lcats[lcat]) / (lcats[lcat + 1] - lcats[lcat]);
        }
      else
      { lcat = 0;
        fl = 0;
        }

      if (mainjob == 1 && ((nlcats > 1 && penjobend + 1 > age && age > contribstartage)
       || eowamt > 0))
      { pwealth = wealth + dcwealth + eowamt;

        lacat = acat + 1;

        while (lacat < nacats - 1 && pwealth > acats[lacat])
        lacat = lacat + 1;

        lacat = lacat - 1;
        fla = (pwealth - acats[lacat]) / (acats[lacat + 1] - acats[lacat]);
        }
      else
      { lacat = acat;
        fla = fa;
        }

      /* need to decide whether to work or retire */

      for (choice = 0; choice < 4; choice++)
      jobchoice[choice] = 0;

      choicecount = 0;

      if (mainjob == 1)
      { for (lcati = lcat; lcati <= lcat + 1; lcati++)
        if (lcati == lcat || fl != 0)
        { for (ecati = ecat; ecati <= ecat + 1; ecati++)
          if (ecati == ecat || fe != 0)
          { for (acati = acat; acati <= acat + 1; acati++)
            { choice = mjchoice[age - 50][lcati][ecati][acati];

              if (jobchoice[choice] == 0)
              { choicecount = choicecount + 1;
                jobchoice[choice] = 1;
                }
              }
            }
          }
        }
      else
      { for (pcati = pcat; pcati <= pcat + 1; pcati++)
        if (pcati == pcat || fp != 0)
        { for (ecati = ecat; ecati <= min(ecat + 1, necats - 1); ecati++)
          if (ecati == ecat || fe != 0)
          { for (acati = acat; acati <= acat + 1; acati++)
            for (scati = scat; scati <= scat + 1; scati++)
            if (scati == scat || fs != 0)
            { choice = retchoice[age - 50][dcat][pcati][scati][ecati][acati];

              if (jobchoice[choice] == 0)
              { choicecount = choicecount + 1;
                jobchoice[choice] = 1;
                }
              }
            }
          }
        }

      job = -1;

      if (choicecount > 1)
      { for (choice = 0; choice < 4; choice++)
        if (jobchoice[choice] == 1)
        { if (choice == 0)
          { for (lcati = lcat; lcati <= lcat + 1; lcati++)
            if (lcati == lcat || fl != 0)
            { if (fe != 0)
              temp  = (1 - fe) * ((1 - fa) * totuw[age - 25][lcati][ecat    ][acat    ]
                                 +  fa     * totuw[age - 25][lcati][ecat    ][acat + 1])
                     +  fe     * ((1 - fa) * totuw[age - 25][lcati][ecat + 1][acat    ]
                                 +  fa     * totuw[age - 25][lcati][ecat + 1][acat + 1]);
              else temp =         (1 - fa) * totuw[age - 25][lcati][ecat    ][acat    ]
                                 +  fa     * totuw[age - 25][lcati][ecat    ][acat + 1];

              if (lcati == lcat)
              templ[0] = temp;
              else templ[1] = temp;
              }

            if (fl != 0)
            totutil = (1 - fl) * templ[0] + fl * templ[1];
            else totutil = templ[0];
            }
          else
          { for (pcati = pcat; pcati <= pcat + 1; pcati++)
            if (pcati == pcat || fp != 0)
            { for (scati = scat; scati <= scat + 1; scati++)
              if (scati == scat || fs != 0)
              { tuea = totud[age - 50][choice - 1][dcat][pcati][scati];

                if (fe != 0)
                temp  = (1 - fe) * ((1 - fla) * tuea[ecat    ][lacat    ]
                                   +  fla     * tuea[ecat    ][lacat + 1])
                       +  fe     * ((1 - fla) * tuea[ecat + 1][lacat    ]
                                   +  fla     * tuea[ecat + 1][lacat + 1]);
                else temp =         (1 - fla) * tuea[ecat    ][lacat    ]
                                   +  fla     * tuea[ecat    ][lacat + 1];

                if (scati == scat)
                temps[0] = temp;
                else temps[1] = temp;
                }

              if (fs != 0)
              temp = (1 - fs) * temps[0] + fs * temps[1];
              else temp = temps[0];

              if (pcati == pcat)
              tempp[0] = temp;
              else tempp[1] = temp;
              }

            if (fp != 0)
            totutil = (1 - fp) * tempp[0] + fp * tempp[1];
            else totutil = tempp[0];
            }

          if (job < 0)
          { job = choice;
            totalutil = totutil;
            }
          else if (totutil > totalutil)
          { job = choice;
            totalutil = totutil;
            }
          }
        }
      else
      { for (choice = 0; choice < 4; choice++)
        if (jobchoice[choice] == 1)
        job = choice;
        }

      /* calculate savings, depending on retirement */

      if (job == 0)
      { savlea = savw[age - 25];

        for (lcati = lcat; lcati <= lcat + 1; lcati++)
        if (lcati == lcat || fl != 0)
        { if (fe != 0)
          temp  = (1 - fe) * ((1 - fa) * savw[age - 25][lcati][ecat    ][acat    ]
                             +  fa     * savw[age - 25][lcati][ecat    ][acat + 1])
                 +  fe     * ((1 - fa) * savw[age - 25][lcati][ecat + 1][acat    ]
                             +  fa     * savw[age - 25][lcati][ecat + 1][acat + 1]);
          else temp =         (1 - fa) * savw[age - 25][lcati][ecat    ][acat    ]
                             +  fa     * savw[age - 25][lcati][ecat    ][acat + 1];

          if (lcati == lcat)
          templ[0] = temp;
          else templ[1] = temp;
          }

        if (fl != 0)
        saving = (1 - fl) * templ[0] + fl * templ[1];
        else saving = templ[0];

        if (penjobend == age)
        saving = saving + dcwealth + dccontrib[age - 25];

        /* assign values for dcat, pcat, scat, and fp */

        if (age + 1 < earlypenage && earlypenage < delayedpenage)
        delayedpen = 1;
        else delayedpen = 0;

        pcat = (int) initialpcat[(age + 1) - 50];
        scat = (int) initialscat[(age + 1) - 50];

        fp = initialpcat[(age + 1) - 50] - pcat;
        fs = initialscat[(age + 1) - 50] - scat;
        }
      else
      { for (pcati = pcat; pcati <= pcat + 1; pcati++)
        if (pcati == pcat || fp != 0)
        { for (scati = scat; scati <= scat + 1; scati++)
          if (scati == scat || fs != 0)
          { savea = savd[age - 50][job - 1][dcat][pcati][scati];

            if (fe != 0)
            temp = (1 - fe) * ((1 - fla) * savea[ecat][lacat    ]
                              +  fla     * savea[ecat][lacat + 1])
                  +  fe     * ((1 - fla) * savea[ecat + 1][lacat    ]
                              +  fla     * savea[ecat + 1][lacat + 1]);
            else temp =        (1 - fla) * savea[ecat][lacat    ]
                              +  fla     * savea[ecat][lacat + 1];

            if (scati == scat)
            temps[0] = temp;
            else temps[1] = temp;
            }

          if (fs != 0)
          temp = (1 - fs) * temps[0] + fs * temps[1];
          else temp = temps[0];

          if (pcati == pcat)
          tempp[0] = temp;
          else tempp[1] = temp;
          }

        if (fp != 0)
        saving = (1 - fp) * tempp[0] + fp * tempp[1];
        else saving = tempp[0];

        if (mainjob == 1)
        { if (penjobend + 1 > age && age > contribstartage)
          saving = saving + dcwealth;

          saving = saving + eowamt;

          mainjob = 0;
          }
        }

      retv[age - 50] = job;

      /* get epsilon for next period */

      epschange = gaussdev(threadno);

      if (mainjob == 0)
      { epsilon = erho * epsilon + epschange * changesdev;

        if (epsilon < ecats[0])
        epsilon = ecats[0];
        else if (epsilon > ecats[necats - 1])
        epsilon = ecats[necats - 1];
        }

      /* update wealth, including dc wealth */

      wealth = wealth + saving;

      if (job == 0)
      { if (penjobend > age && age >= contribstartage)
        dcwealth = dcwealth + dccontrib[age - 25];
        else dcwealth = 0;
        }

      if (sserage <= age && age < 70)
      if (job == 1 || job == 2)
      { if (job == 1)
        { if (fs != 0)
          sbenf = (1 - fs) * (ssbenf[scat    ] + benchanges[age - 62][0][scat    ])
                 +  fs     * (ssbenf[scat + 1] + benchanges[age - 62][0][scat + 1]);
          else sbenf = ssbenf[scat] + benchanges[age - 62][0][scat];
          }
        else
        { if (fs != 0)
          sbenf = (1 - fs) * (ssbenf[scat    ] + benchangep[age - 62][0][scat    ])
                 +  fs     * (ssbenf[scat + 1] + benchangep[age - 62][0][scat + 1]);
          else sbenf = ssbenf[scat] + benchangep[age - 62][0][scat];
          }

        fscat = -1;

        for (scati = scat + 1; scati < nscats; scati++)
        { if (fscat == -1 && sbenf < ssbenf[scati])
          fscat = (scati - 1) + (sbenf - ssbenf[scati - 1])
                 / (ssbenf[scati] - ssbenf[scati - 1]);
          }

        if (fscat == -1)
        fscat = (scati - 2) + 0.99;

        scat = (int) fscat;
        fs = fscat - scat;
        }
      }

    priorret = 0;
    simeverreturnft = 0;
    simeverreturnpt = 0;
    simeverreturn = 0;

    for (age = firstage; age <= lastage; age++)
    { job = retv[age - 50];

      if (job == 1 && priorret == 1)         /* returned to ft work */
      { ret0vec[age - 50] = ret0vec[age - 50] + 1;

        if (age > firstage)
        if (retv[(age - 1) - 50] >= 2)
        { returnft[age - 50] = returnft[age - 50] + 1;
          simeverreturnft = 1;
          simeverreturn = 1;
          }
        }
      else if (job == 2)                     /* partially retired */
      { ret0vec[age - 50] = ret0vec[age - 50] + 1;
        ret1vec[age - 50] = ret1vec[age - 50] + 1;

        if (age > firstage)
        { if (retv[(age - 1) - 50] == 3)
          { returnpt[age - 50] = returnpt[age - 50] + 1;
            simeverreturnpt = 1;
            simeverreturn = 1;
            }

          if (retv[(age - 1) - 50] != 2)
          newpt[age - 50] = newpt[age - 50] + 1;
          }
          
        priorret = 1;
        }
      else if (job == 3)                     /* retired */
      { ret0vec[age - 50] = ret0vec[age - 50] + 1;
        ret1vec[age - 50] = ret1vec[age - 50] + 1;
        ret2vec[age - 50] = ret2vec[age - 50] + 1;
          
        priorret = 1;
        }
      }

    *everreturnft = *everreturnft + simeverreturnft;
    *everreturnpt = *everreturnpt + simeverreturnpt;
    *everreturn   = *everreturn + simeverreturn;

    for (survey = 0; survey < 5; survey++)
    { age0 = data->age[survey];
      ret0 = data->ret[survey];
      age1 = data->age[survey + 1];
      ret1 = data->ret[survey + 1];

      if (54 <= age0 && age0 <= 66 && 1 <= ret0 && ret0 <= 5
       && 54 <= age1 && age1 <= 66 && 1 <= ret1 && ret1 <= 5
       && age1 <= lastage)
      if (retv[age0 - 50] >= 2 && retv[age1 - 50] <= 1)
      reversal[survey] = reversal[survey] + 1;
      }

    /* calculate wealth patterns after lastage if necessary */

    if (mainjob == 1)
    wealth = wealth + dcwealth + eowamount[lastage + 1 - 50];

    for (age = lastage + 1; age <= 99; age++)
    { year = age + birthyear;

      if (1926 < year && year <= 2001)
      wealth = (1 + 0.01 * realrate[(year - 1) - 1926]) * wealth;
      else wealth = (1 + rgmean) * wealth;

      if (delayedpen == 1 && earlypenage <= age && age < delayedpenage)
      dcat = 1;
      else dcat = 0;

      /* get categories and offsets for eps and wealth */

      if (wealth > acats[acat])
      { acat = acat + 1;

        while (acat < nacats - 1 && wealth > acats[acat])
        acat = acat + 1;

        if (acat > 0)
        acat = acat - 1;
        }
      else
      { while (acat > 0 && wealth < acats[acat])
        acat = acat - 1;
        }

      fa = (wealth - acats[acat]) / (acats[acat + 1] - acats[acat]);

      for (pcati = pcat; pcati <= pcat + 1; pcati++)
      if (pcati == pcat || fp != 0)
      { for (scati = scat; scati <= scat + 1; scati++)
        if (scati == scat || fs != 0)
        { if (age < 70)
          savvec = savd[age - 50][2][dcat][pcati][scati][0];
          else savvec = savr[age - 70][dcat][pcati][scati];

          temp = (1 - fa) * savvec[acat] +  fa  * savvec[acat + 1];

          if (scati == scat)
          temps[0] = temp;
          else temps[1] = temp;
          }

        if (fs != 0)
        temp = (1 - fs) * temps[0] + fs * temps[1];
        else temp = temps[0];

        if (pcati == pcat)
        tempp[0] = temp;
        else tempp[1] = temp;
        }

      if (fp != 0)
      saving = (1 - fp) * tempp[0] + fp * tempp[1];
      else saving = tempp[0];

      wealth = wealth + saving;
      }

    /* end of computations after lastage */
    }

  /* return if not at last tcat category */

  if (tcat < ntcats - 1)
  return;

  for(age = 50; age <= lastage; age++)
  { ret0vec[age - 50] = ret0vec[age - 50] / nsims;
    ret1vec[age - 50] = ret1vec[age - 50] / nsims;
    ret2vec[age - 50] = ret2vec[age - 50] / nsims;
    }

  if (mode == 1 || mode == 2)
  { /* calculate and update the moments for this respondent */

    for (survey = 0; survey < 5; survey++)
    reversal[survey] = reversal[survey] / nsims;

    memset(&moments, 0, sizeof(MOMENTSTRUCT));
    momentStruct = &moments;

    for (survey = 0; survey < 6; survey++)
    { age = data->age[survey];
      ret = data->ret[survey];

      if (54 <= age && age <= 66 && (1 <= ret && ret <= 5))
      { if (ret == 1 || ret == 3)
        m1 = 1 - ret1vec[age - 50];
        else m1 = - ret1vec[age - 50];

        if (ret == 1)
        m2 = 1 - ret2vec[age - 50];
        else m2 = - ret2vec[age - 50];

        momentStruct->age[age - 54] = m1;

             if (age == 55)      k =  0;
        else if (age == 58)      k =  1;
        else if (age == 60)      k =  2;
        else if (age == 62)      k =  3;
        else if (age == 65)      k =  4;
        else                     k = -1;

        if (k >= 0)
        {                              momentStruct->pr[k]       = m2;
          if (data->inccat == 1)       momentStruct->lowinc[k]   = m1;
          if (data->inccat == 3)       momentStruct->highinc[k]  = m1;
          if (data->healthage <= age)  momentStruct->health[k]   = m1;
          if (data->healthage <= age)  momentStruct->healthpr[k] = m2;
          }
        }
      }

    for (survey = 0; survey < 5; survey++)
    { age0 = data->age[survey];
      ret0 = data->ret[survey];
      age1 = data->age[survey + 1];
      ret1 = data->ret[survey + 1];

      if (54 <= age0 && age0 <= 66 && 1 <= ret0 && ret0 <= 5
       && 54 <= age1 && age1 <= 66 && 1 <= ret1 && ret1 <= 5
       && age1 <= lastage)
      { if ((ret0 == 1 || ret0 == 3) && ret1 == 5)
        momentStruct->reversal[survey] = 1 - reversal[survey];
        else momentStruct->reversal[survey] = - reversal[survey];
        }
      }

    /* update moment matrices */

    momentvec = (double *) &moments;

    for (l = 0; l < nmoments; l++)
    if (mobs[l] > 0)
    mvec[l] = mvec[l] + momentvec[l];


    /* update w matrix */

    for (l1 = 0; l1 < nmoments; l1++)
    if (mobs[l1] > 0)
    { for (l2 = 0; l2 <= l1; l2++)
      if (mobs[l2] > 0)
      { l = l1 * (l1 + 1) / 2 + l2;

        wmat[l] = wmat[l] + momentvec[l1] * momentvec[l2];
        }
      }
    }

  if (mode != 2)
  { /* update tabs for this respondent */
  
    for (age = 50; age <= 69; age++)
    { cumret0mat[age - 50]  = cumret0mat[age - 50] + ret0vec[age - 50];
      cumret1mat[age - 50]  = cumret1mat[age - 50] + ret1vec[age - 50];
      cumret2mat[age - 50]  = cumret2mat[age - 50] + ret2vec[age - 50];
      cumreturnft[age - 50] = cumreturnft[age - 50] + returnft[age - 50] / nsims;
      cumreturnpt[age - 50] = cumreturnpt[age - 50] + returnpt[age - 50] / nsims;
      cumnewpt[age - 50]    = cumnewpt[age - 50] + newpt[age - 50] / nsims;
      }

    *cumeverreturnft = *cumeverreturnft + *everreturnft / nsims;
    *cumeverreturnpt = *cumeverreturnpt + *everreturnpt / nsims;
    *cumeverreturn   = *cumeverreturn + *everreturn / nsims;
    }

  }


/*---------------------------------------------------------------------------
			 GET SURVIVAL RATES routine
			 gets birthyear and gender specific survival rates
  ---------------------------------------------------------------------------*/

void getSurvivalRates(
  int    birthyear,            /* year of birth            */
  int    gender,               /* sex of respondent        */
  double *surv)                /* vector of survival rates */

{ int fileno;
  int age, coh, gen, line;
  int agecheck, survprob;

  char filename[60];
  char stringline[151];

  static int init = 1;
  static float survtable[91][2][120];

  static int startcohort[3] = { 0, 16, 56};
  static int endcohort[3]   = {15, 55, 90};
  static char infilename[3][12] = {"coh0015.95t", "coh1655.95t", "coh5690.95t"};

  FILE *infile, *outfile;
  int inhandle, outhandle;


  strcpy(filename, "SurvTabs.dat");

  if (init == 1)
  { if (_access(filename, 0) < 0)
    { for (fileno = 0; fileno < 3; fileno++)
      { strcpy(filename, infilename[fileno]);
        infile = fopen(filename, "r");

        if (infile == NULL)
        error("File not found in getSurvivalRates");

        for (coh = startcohort[fileno]; coh <= endcohort[fileno]; coh++)
        for (gen = 0; gen < 2; gen++)
        for (age = 0; age < 120; age++)
        { if (age % 40 == 0)
          for (line = 0; line < 8; line++)
          fgets(stringline, 150, infile);

          fgets(stringline, 150, infile);

          sscanf(&stringline[1], "%3d", &agecheck);
          if (age != agecheck)
          error("Bad input in getSurvivalRates");

          sscanf(&stringline[17], "%6d", &survprob);
          survtable[coh][gen][age] = (float) survprob;

          if (age % 5 == 4)
          fgets(stringline, 150, infile);
          }

        fclose(infile);
        }

      strcpy(filename, "SurvTabs.dat");

      outfile = fopen(filename, "wb");
      outhandle = _fileno(outfile);
      _write(outhandle, survtable, sizeof(survtable));
      fclose(outfile);
      }
    else
    { infile = fopen(filename, "r");
      inhandle = _fileno(infile);
      _read(inhandle, survtable, sizeof(survtable));
      fclose(infile);
      }

    init = 0;
    return;
    }

  if (birthyear < 1900 || birthyear > 1990 || gender < 1 || gender > 2)
  { wnprintf("\nBad birthyear or gender: birthyear %d  gender %d", birthyear, gender);
    error("Bad birthyear or gender in getSurvivalRates");
    }

  for (age = 0; age < 120; age++)
  surv[age] = survtable[birthyear - 1900][gender - 1][age];

  }


/*---------------------------------------------------------------------------
			 ERF
			 Calculates the Cumulative Normal Probability Distribution
  ---------------------------------------------------------------------------*/

double erf(
  double x)               /* standardized normal variable */

{ double ax, d, p, t;

  if (x > 0)
  ax = x;
  else ax = -x;

  if (ax <= 7)
  { t = 1 / (1 + .2316419 * ax);
	 d = .3989423 * exp(- x * x / 2);
	 p = 1 - d * t * ((((1.330274 * t - 1.821256) * t + 1.781478) * t
	  - .3565638) * t + .3193815);
	 }
  else p = 1;

  if (x < 0)
  p = 1 - p;

  return(p);

  }


/*---------------------------------------------------------------------------
          FRAND routine
			 Obtains a random number in the [0:1] interval
  ---------------------------------------------------------------------------*/

int randseed[48] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* maximum 48 threads */
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                    0, 0, 0, 0, 0, 0, 0, 0};
int randresetflag[48] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                         1, 1, 1, 1, 1, 1, 1, 1};

double frand(
  int    threadno)

{ int k;
  double rannum;

  int    x1, x2, x3;
  double *r;

  static int seed = 13698763;
  static int m1 = 259200;
  static int a1 = 7141;
  static int c1 = 54773;
  static int m2 = 134456;
  static int a2 = 8121;
  static int c2 = 28411;
  static int m3 = 243000;
  static int a3 = 4561;
  static int c3 = 51349;
  static double rm1 = 1 / (double) 259200;  /* m1 */
  static double rm2 = 1 / (double) 134456;  /* m2 */

  typedef struct
  { int    x1, x2, x3;
    double r[97];
    } RANDSTRUCT;

  static RANDSTRUCT randstruct[nthreads];

  
  r = randstruct[threadno].r;

  if (randresetflag[threadno] == 1)
  { randresetflag[threadno] = 0;

    x1 = (c1 + seed) % m1;
    x1 = (a1 * x1) % m1;

    for (k = 0; k < randseed[threadno]; k++)
    x1 = (a1 * x1) % m1;

    x2 = x1 % m2;
    x1 = (a1 * x1) % m1;
    x3 = x1 % m3;

    for (k = 0; k < 97; k++)
    { x1 = (a1 * x1) % m1;
      x2 = (a2 * x2) % m2;
      r[k] = rm1 * (x1 + rm2 * x2);
      }
    }
  else
  { x1 = randstruct[threadno].x1;
    x2 = randstruct[threadno].x2;
    x3 = randstruct[threadno].x3;
    }

  x1 = (a1 * x1 + c1) % m1;
  x2 = (a2 * x2 + c2) % m2;
  x3 = (a3 * x3 + c3) % m3;

  k = (97 * x3) / m3;

  rannum = r[k];
  r[k] = rm1 * (x1 + rm2 * x2);

  randstruct[threadno].x1 = x1;
  randstruct[threadno].x2 = x2;
  randstruct[threadno].x3 = x3;

  return(rannum);

  }


/*---------------------------------------------------------------------------
			 GAUSSDEV routine
			 Obtains a normal gaussian deviate
  ---------------------------------------------------------------------------*/

int gaussdevreset[nthreads];

double gaussdev(
  int   threadno)

{ double factor, r, v1, v2;

  static int    alternate[nthreads];
  static double gaussdev1[nthreads], gaussdev2[nthreads];

  if (gaussdevreset[threadno] == 1)
  { alternate[threadno] = 0;
    gaussdevreset[threadno] = 0;
    }

  if (alternate[threadno] == 0)
  { alternate[threadno] = 1;
    r = 2;

    while (r >= 1)
    { v1 = 2 * frand(threadno) - 1;
      v2 = 2 * frand(threadno) - 1;
      r = v1 * v1 + v2 * v2;
      }

    factor = sqrt(-2 * log(r) / r);

    gaussdev1[threadno] = v1 * factor;
    gaussdev2[threadno] = v2 * factor;

    return(gaussdev1[threadno]);
    }
  else
  { alternate[threadno] = 0;
    return(gaussdev2[threadno]);
    }

  }


/*---------------------------------------------------------------------------
          RAND RESET routine
			 Resets the random number generator of frand
  ---------------------------------------------------------------------------*/

void randreset(
  int   threadno,
  int   seed)

{ randseed[threadno] = seed;
  randresetflag[threadno] = 1;
  gaussdevreset[threadno] = 1;

  }
